@@ -1,6 +1,6 @@
 // Copyright (C) 2004 John Farrell
 // Copyright (C) 2004, 2005, 2006, 2007 Ulf Lorenz
-// Copyright (C) 2008, 2009, 2010 Ben Asselstine
+// Copyright (C) 2008, 2009, 2010, 2014, 2015 Ben Asselstine
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -44,10 +44,9 @@
 #include "QKillHero.h"
 #include "QEnemyArmies.h"
 #include "QEnemyArmytype.h"
+#include "rnd.h"
 
-using namespace std;
-
-#define debug(x) {cerr<<__FILE__<<": "<<__LINE__<<": "<<x<<flush<<endl;}
+#define debug(x) {std::cerr<<__FILE__<<": "<<__LINE__<<": "<<x<<std::flush<<std::endl;}
 //#define debug(x)
 
 AI_Allocation* AI_Allocation::s_instance = 0;
@@ -171,9 +170,9 @@ int AI_Allocation::continueQuests()
   if (GameScenarioOptions::s_play_with_quests == GameParameters::NO_QUESTING)
     return count;
 
-  std::vector<Quest*> quest = 
+  std::vector<Quest*> quests =
     QuestsManager::getInstance()->getPlayerQuests(d_owner);
-  for (std::vector<Quest*>::iterator i = quest.begin(); i != quest.end(); i++)
+  for (std::vector<Quest*>::iterator i = quests.begin(); i != quests.end(); i++)
     {
       Quest *quest = *i;
       if (quest == NULL)
@@ -421,7 +420,7 @@ bool AI_Allocation::emptyOutCities()
               if (!s)
                 continue;
               if ((s->getMoves() > 3 && s->size() >= 4 &&
-                   (num_defenders - s->size()) >= 3) || (rand() % 10) == 0)
+                   (num_defenders - s->size()) >= 3) || (Rnd::rand() % 10) == 0)
                 {
                   City *target = cl->getNearestEnemyCity(s->getPos());
                   if (target)
@@ -565,8 +564,8 @@ int AI_Allocation::oldPickupItems()
   for (std::vector<Vector<int> >::iterator i = items.begin(); i != items.end();
        i++)
     {
-      std::list<Stack*> s = GameMap::getNearbyFriendlyStacks(*i, 8);
-      for (std::list<Stack*>::iterator j = s.begin(); j != s.end(); j++)
+      std::list<Stack*> stks = GameMap::getNearbyFriendlyStacks(*i, 8);
+      for (std::list<Stack*>::iterator j = stks.begin(); j != stks.end(); j++)
         {
           Stack *s = *j;
           if (s->hasHero() == false)
@@ -619,7 +618,7 @@ int AI_Allocation::oldPickupItems()
 
 int AI_Allocation::move(City *first_city, bool take_neutrals)
 {
-  int temple_moved = 0, ruin_moved = 0, pickup_moved = 0, attack_moved = 0, quest_moved = 0, immediate_moved = 0, defensive_moved = 0, capacity_moved = 0, offensive_moved = 0, default_moved= 0;
+  int attack_moved = 0, defensive_moved = 0, capacity_moved = 0, offensive_moved = 0, default_moved= 0;
   int temple_alloc = 0, ruin_alloc = 0, pickup_alloc = 0, attack_alloc = 0, quest_alloc = 0, immediate_alloc = 0, defensive_alloc = 0, capacity_alloc = 0, offensive_alloc = 0, default_alloc= 0;
   int moved;
   // move stacks
@@ -637,7 +636,6 @@ int AI_Allocation::move(City *first_city, bool take_neutrals)
   // go on a quest
   quest_alloc = d_stacks->size();
   moved = continueQuests();
-  quest_moved = moved;
   quest_alloc -= d_stacks->size();
   debug("Player " << d_owner->getName() << " still has " << d_stacks->size() << " stacks after allocating stacks to fulfilling quests");
   debug("Player " << d_owner->getName() << " moved " << moved << " stacks in quest mode.");
@@ -649,7 +647,6 @@ int AI_Allocation::move(City *first_city, bool take_neutrals)
   //move stacks to temples for blessing, or ones with heroes for a quest.
   temple_alloc = d_stacks->size();
   moved = visitTemples(GameScenarioOptions::s_play_with_quests != GameParameters::NO_QUESTING);
-  temple_moved = moved;
   temple_alloc -= d_stacks->size();
   debug("Player " << d_owner->getName() << " still has " << d_stacks->size() << " stacks after allocating stacks to visiting temples");
   debug("Player " << d_owner->getName() << " moved " << moved << " stacks in temple-visiting mode.");
@@ -660,7 +657,6 @@ int AI_Allocation::move(City *first_city, bool take_neutrals)
   //move hero stacks to ruins for searching.
   ruin_alloc = d_stacks->size();
   moved = visitRuins();
-  ruin_moved = moved;
   ruin_alloc -= d_stacks->size();
   debug("Player " << d_owner->getName() << " still has " << d_stacks->size() << " stacks after allocating stacks to visiting ruins");
   debug("Player " << d_owner->getName() << " moved " << moved << " stacks in ruin-visiting mode.");
@@ -671,7 +667,6 @@ int AI_Allocation::move(City *first_city, bool take_neutrals)
   //if we're near a bag of stuff, go pick it up.
   pickup_alloc = d_stacks->size();
   moved = pickupItems();
-  pickup_moved = moved;
   pickup_alloc -= d_stacks->size();
   debug("Player " << d_owner->getName() << " still has " << d_stacks->size() << " stacks after allocating stacks to picking up items");
   debug("Player " << d_owner->getName() << " moved " << moved << " stacks in pickup-items mode.");
@@ -693,7 +688,6 @@ int AI_Allocation::move(City *first_city, bool take_neutrals)
   // if a stack is 2 tiles away from another enemy city, then attack it.
   immediate_alloc = d_stacks->size();
   moved = attackNearbyEnemies();
-  immediate_moved = moved;
   immediate_alloc -= d_stacks->size();
   debug("Player " << d_owner->getName() << " still has " << d_stacks->size() << " stacks after allocating stacks to attacking nearby stacks");
   debug("Player " << d_owner->getName() << " moved " << moved << " stacks in attack-nearby-stacks mode.");
@@ -711,7 +705,6 @@ int AI_Allocation::move(City *first_city, bool take_neutrals)
       debug("Player " << d_owner->getName() << " moved " << moved << " stacks in capacity building mode.");
       count+=moved;
     }
-  checkAmbiguities();
 
   sbusy.emit();
   if (d_owner->abortRequested())
@@ -724,7 +717,6 @@ int AI_Allocation::move(City *first_city, bool take_neutrals)
   debug("Player " << d_owner->getName() << " has " << d_stacks->size() << " stacks after assigning defenders");
   debug("Player " << d_owner->getName() << " moved " << count << " stacks in defensive mode.");
 
-  checkAmbiguities();
   if (d_stacks->size() == 0)
     {
       delete d_stacks;
@@ -742,7 +734,6 @@ int AI_Allocation::move(City *first_city, bool take_neutrals)
   debug("Player " << d_owner->getName() << " still has " << d_stacks->size() << " stacks after allocating stacks to threats")
     debug("Player " << d_owner->getName() << " moved " << moved << " stacks in offensive mode.");
 
-  checkAmbiguities();
   if (d_stacks->size() == 0)
     {
       delete d_stacks;
@@ -764,7 +755,6 @@ int AI_Allocation::move(City *first_city, bool take_neutrals)
     return count;
   //empty out the cities damnit.
   emptyOutCities();
-  checkAmbiguities();
   debug("Player " << d_owner->getName() << " moved totals: " << attack_moved << "," << capacity_moved <<"," <<defensive_moved<<"," << offensive_moved <<"," << default_moved <<".");
   debug("Player " << d_owner->getName() << " alloc totals: " << attack_alloc << "," << capacity_alloc <<"," <<defensive_alloc<<"," << offensive_alloc <<"," << default_alloc <<" (" << total <<").");
   delete d_stacks;
@@ -795,7 +785,7 @@ int AI_Allocation::allocateDefensiveStacksToCity(City *city)
           shuffleStacksWithinCity(city, d, Vector<int>(0,0));
       }
   std::vector<Stack*> defenders = city->getDefenders();
-  vector<Stack*>::iterator it;
+  std::vector<Stack*>::iterator it;
   for (it = defenders.begin(); it != defenders.end(); it++)
     {
       Stack *defender = *it;
@@ -1026,11 +1016,11 @@ Vector<int> AI_Allocation::getFreeOtherSpotInCity(City *city, Stack *stack)
         std::list<Stack*> f = GameMap::getFriendlyStacks(pos);
         if (f.size() > 0)
           {
-            for (std::list<Stack*>::iterator i = f.begin(); i != f.end(); i++)
+            for (std::list<Stack*>::iterator k = f.begin(); k != f.end(); k++)
               {
-                if ((*i)->size() > size)
+                if ((*k)->size() > size)
                   {
-                    size = (*i)->size();
+                    size = (*k)->size();
                     best = pos;
                   }
               }
@@ -1059,7 +1049,6 @@ Vector<int> AI_Allocation::getFreeSpotInCity(City *city, int stackSize)
 
 Stack *AI_Allocation::findClosestStackToEnemyCity(City *city, bool try_harder)
 {
-  Vector<int> pos = city->getPos();
   Stack *best = 0;
   int lowest_mp = -1;
   for (StackReflist::iterator it = d_stacks->begin(); it != d_stacks->end(); ++it)
@@ -1095,7 +1084,6 @@ Stack *AI_Allocation::findClosestStackToEnemyCity(City *city, bool try_harder)
 
 Stack *AI_Allocation::findClosestStackToCity(City *city)
 {
-  Vector<int> pos = city->getPos();
   Stack *best = 0;
   int lowest_mp = -1;
   for (StackReflist::iterator it = d_stacks->begin(); it != d_stacks->end(); ++it)
@@ -1254,22 +1242,6 @@ int AI_Allocation::defaultStackMovements()
               // enemy city. Let's iterator through all cities and attack the first
               // one we can lay our hands on.
               debug("Mmmh, did not work.")
-                //sleep (10);
-                /*
-                   MoveResult *result = 0;
-                   for (Citylist::iterator cit = allCities->begin(); cit != allCities->end(); cit++)
-                   if ((*cit)->getOwner() != d_owner)
-                   {
-                   debug("Let's try "<<(*cit).getName() <<" instead.")
-                   result = moveStack(s, (*cit)->getPos());
-                   if (result && result->moveSucceeded())
-                   {
-                   debug("Worked")
-                   count++;
-                   break;
-                   }
-                   }
-                 */
             }
         }
       else
@@ -1410,7 +1382,7 @@ bool AI_Allocation::shuffleStacksWithinCity(City *city, Stack *stack,
   if (f.size() > 1)
     {
       printf("i am stack %d at %d,%d\n", stack->getId(), stack->getPos().x, stack->getPos().y);
-      printf("crap.  there are %d stacks at %d,%d\n", f.size(), target.x, target.y);
+      printf("crap.  there are %lu stacks at %d,%d\n", f.size(), target.x, target.y);
       for (std::list<Stack*>::iterator it = f.begin(); it != f.end(); it++)
         {
           Stack *n = *it;
@@ -1588,35 +1560,4 @@ bool AI_Allocation::groupStacks(Stack *stack)
   return true;
 }
 
-bool AI_Allocation::checkAmbiguities()
-{
-  //Stack *bobo = d_owner->getStacklist()->getStackById(4416);
-  //if (bobo)
-    //{
-      //printf ("stack 4416 parked? %d\n", bobo->getParked());
-    //}
-  return false;
-    int count = 0;
-    Stacklist *sl = d_owner->getStacklist();
-    for (Stacklist::iterator it = sl->begin(); it != sl->end(); it++)
-      {
-        if (GameMap::getFriendlyStacks((*it)->getPos()).size() > 1)
-          {
-            printf("stack id %d at %d,%d sits on the same tile as:\n", 
-                   (*it)->getId(),
-                   (*it)->getPos().x, (*it)->getPos().y);
-            std::list<Stack*> o = GameMap::getFriendlyStacks((*it)->getPos());
-            for (std::list<Stack*>::iterator i = o.begin(); i != o.end(); i++)
-              {
-                if ((*i)->getId() == (*it)->getId())
-                  continue;
-                printf("stack %d\n", (*i)->getId());
-              }
-            count++;
-          }
-      }
-    if (count > 0)
-      exit(0);
-    return count != 0;
-}
 // End of file
