@@ -1,4 +1,4 @@
-//  Copyright (C) 2007, 2008 Ben Asselstine
+//  Copyright (C) 2007, 2008, 2014 Ben Asselstine
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -18,13 +18,11 @@
 #ifndef AI_DIPLOMACY_H
 #define AI_DIPLOMACY_H
 
-#include <string>
 #include <list>
 
 class Player;
 
-using namespace std;
-
+//! Artificial intelligence for managing diplomatic relations.
 class AI_Diplomacy
 {
     public:
