@@ -2,7 +2,7 @@
 // Copyright (C) 2003 Michael Bartl
 // Copyright (C) 2004 John Farrell
 // Copyright (C) 2004, 2005, 2006 Andrea Paternesi
-// Copyright (C) 2006, 2007, 2008, 2009, 2010 Ben Asselstine
+// Copyright (C) 2006, 2007, 2008, 2009, 2010, 2012, 2014, 2015 Ben Asselstine
 // Copyright (C) 2007 Ole Laursen
 //
 //  This program is free software; you can redistribute it and/or modify
@@ -20,6 +20,7 @@
 //  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 
 //  02110-1301, USA.
 
+#include <iostream>
 #include <sstream>
 #include <sigc++/functors/mem_fun.h>
 
@@ -35,6 +36,7 @@
 #include "city.h"
 #include "ruinlist.h"
 #include "ruin.h"
+#include "SightMap.h"
 #include "rewardlist.h"
 #include "Itemlist.h"
 #include "templelist.h"
@@ -52,6 +54,8 @@
 #include "tilesetlist.h"
 #include "shieldsetlist.h"
 #include "real_player.h"
+#include "AI_Analysis.h"
+#include "AI_Diplomacy.h"
 #include "ai_fast.h"
 #include "ai_smart.h"
 #include "ai_dummy.h"
@@ -61,10 +65,10 @@
 #include "Configuration.h"
 #include "FogMap.h"
 #include "history.h"
+#include "game-parameters.h"
+#include "rnd.h"
 
-using namespace std;
-
-#define debug(x) {cerr<<__FILE__<<": "<<__LINE__<<": "<<x<<endl<<flush;}
+#define debug(x) {std::cerr<<__FILE__<<": "<<__LINE__<<": "<<x<<std::endl<<std::flush;}
 //#define debug(x)
 
 CreateScenario::CreateScenario(int width, int height)
@@ -91,7 +95,7 @@ CreateScenario::CreateScenario(int width, int height)
     setHeight(height);
 
     d_generator = new MapGenerator();
-    d_generator->progress.connect (sigc::mem_fun(*this, &CreateScenario::on_progress));
+    d_generator->progress.connect (sigc::hide(sigc::hide(sigc::mem_fun(*this, &CreateScenario::on_progress))));
 }
 
 CreateScenario::~CreateScenario()
@@ -105,7 +109,7 @@ CreateScenario::~CreateScenario()
         delete d_scenario;
 }
 
-void CreateScenario::on_progress(double percent, std::string description)
+void CreateScenario::on_progress()
 {
   progress.emit();
 }
@@ -134,25 +138,25 @@ void CreateScenario::setPercentages(int pgrass, int pwater, int pforest,
     d_generator->setPercentages(pwater, pforest, pswamp, phills, pmountains);
 }
 
-void CreateScenario::setMapTiles(std::string tilesname)
+void CreateScenario::setMapTiles(Glib::ustring tilesname)
 {
     debug("CreateScenario::setMapTiles")
     d_tilesname = tilesname;
     GameMap::getInstance()->setTileset(tilesname);
 }
 
-void CreateScenario::setShieldset(std::string shieldset)
+void CreateScenario::setShieldset(Glib::ustring shieldset)
 {
     debug("CreateScenario::setShieldset")
     d_shieldsname = shieldset;
     GameMap::getInstance()->setShieldset(shieldset);
 }
 
-void CreateScenario::setCityset(std::string citysetname)
+void CreateScenario::setCityset(Glib::ustring citysetname)
 {
     debug("CreateScenario::setCityset")
     d_citysetname = citysetname;
-    Cityset *cs = Citysetlist::getInstance()->getCityset(citysetname);
+    Cityset *cs = Citysetlist::getInstance()->get(citysetname);
     d_generator->setCityset(cs);
     GameMap::getInstance()->setCityset(citysetname);
 }
@@ -216,8 +220,8 @@ void CreateScenario::setHeight(int height)
     GameMap::setHeight(height);
 }
 
-Player* CreateScenario::addPlayer(std::string name, guint32 armyset,
-                                Gdk::Color color, int type)
+Player* CreateScenario::addPlayer(Glib::ustring name, guint32 armyset,
+                                Gdk::RGBA color, int type)
 {
     debug("CreateScenario::addPlayer")
 
@@ -228,8 +232,8 @@ Player* CreateScenario::addPlayer(std::string name, guint32 armyset,
     return p;
 }
 
-bool CreateScenario::addNeutral(std::string name, guint32 armyset,
-                                Gdk::Color color, int type)
+bool CreateScenario::addNeutral(Glib::ustring name, guint32 armyset,
+                                Gdk::RGBA color, int type)
 {
     // for consistency, we only allow exactly one neutral player
     if (Playerlist::getInstance()->getNeutral() != 0)
@@ -240,49 +244,6 @@ bool CreateScenario::addNeutral(std::string name, guint32 armyset,
     return true;
 }
 
-int CreateScenario::getNoPlayers() const
-{
-    return Playerlist::getInstance()->size();
-}
-
-Player* CreateScenario::getPlayer(int number) const
-{
-    debug("CreateScenario::getPlayer")
-
-    if (number >= static_cast<int>(Playerlist::getInstance()->size()))
-        return 0;
-
-    Playerlist::iterator it;
-    for (it = Playerlist::getInstance()->begin(); number > 0; number--)
-        it++;
-
-    return (*it);
-}
-
-int CreateScenario::getNoCities() const
-{
-    if (!d_generator)
-        return -1;
-
-    return d_generator->getNoCities();
-}
-
-int CreateScenario::getNoRuins() const
-{
-    if (!d_generator)
-        return -1;
-
-    return d_generator->getNoRuins();
-}
-
-int CreateScenario::getNoTemples() const
-{
-    if (!d_generator)
-        return -1;
-
-    return d_generator->getNoTemples();
-}
-
 bool CreateScenario::create(const GameParameters &g)
 {
     debug("CreateScenario::create")
@@ -354,7 +315,7 @@ bool CreateScenario::create(const GameParameters &g)
     return true;
 }
 
-bool CreateScenario::dump(std::string filename) const
+bool CreateScenario::dump(Glib::ustring filename) const
 {
     debug("CreateScenario::dump")
 
@@ -381,7 +342,7 @@ bool CreateScenario::createMap()
 
     //...and create cities, temples, ruins ,signposts
     map = d_generator->getBuildings(d_width, d_height);
-    Cityset *cityset = Citysetlist::getInstance()->getCityset(d_citysetname);
+    Cityset *cityset = Citysetlist::getInstance()->get(d_citysetname);
     
     for (int y = 0; y < d_height; y++)
         for (int x = 0; x < d_width; x++)
@@ -471,7 +432,7 @@ bool CreateScenario::distributePlayers()
 	  continue;
 	while (1)
 	  {
-	    Vector<int> pos = Vector<int>(rand() % d_width, rand() % d_height);
+	    Vector<int> pos = Vector<int>(Rnd::rand() % d_width, Rnd::rand() % d_height);
 	    City *city = Citylist::getInstance()->getNearestCity(pos);
 	    if (city->isBurnt() == false && city->isCapital() == false)
 	      {
@@ -519,7 +480,6 @@ bool CreateScenario::setupCities(bool cities_can_produce_allies,
     return true;
 }
 
-
 bool CreateScenario::setupRoads()
 {
   Roadlist* rl = Roadlist::getInstance();
@@ -542,13 +502,14 @@ bool CreateScenario::setupTemples()
     for (Templelist::iterator it = tl->begin(); it != tl->end(); it++)
     {
         // set a random temple type
-        int type= (int) ((TEMPLE_TYPES*1.0) * (rand() / (RAND_MAX + 1.0)));
+        int type= (int) ((TEMPLE_TYPES*1.0) * (Rnd::rand() / (RAND_MAX + 1.0)));
         (*it)->setType(type);
 
     }
 
     return true;
 }
+
 bool CreateScenario::setupRuins(bool strongholds_invisible, int sage_factor,
 				int no_guardian_factor, int stronghold_factor)
 {
@@ -561,7 +522,7 @@ bool CreateScenario::setupRuins(bool strongholds_invisible, int sage_factor,
         it != Ruinlist::getInstance()->end(); it++)
     {
         // set a random ruin type
-        if (rand() % stronghold_factor == 0) //one in six ruins is a stronghold
+        if (Rnd::rand() % stronghold_factor == 0) //one in six ruins is a stronghold
           {
             (*it)->setType(Ruin::STRONGHOLD);
             if (strongholds_invisible == true)
@@ -574,7 +535,7 @@ bool CreateScenario::setupRuins(bool strongholds_invisible, int sage_factor,
           (*it)->setType(Ruin::RUIN);
 
         //one in twenty ruins is a sage
-        if (rand() % sage_factor == 0 && (*it)->getType() == Ruin::RUIN) 
+        if (Rnd::rand() % sage_factor == 0 && (*it)->getType() == Ruin::RUIN) 
           {
             (*it)->setSage (true);
             continue;
@@ -582,7 +543,7 @@ bool CreateScenario::setupRuins(bool strongholds_invisible, int sage_factor,
 
 
         //one in ten ruins doesn't have a guardian
-        if (rand() % no_guardian_factor == 0 && (*it)->getType() == Ruin::RUIN) 
+        if (Rnd::rand() % no_guardian_factor == 0 && (*it)->getType() == Ruin::RUIN) 
           continue;
 
         // and set a guardian
@@ -619,7 +580,7 @@ bool CreateScenario::setupSignposts(int ratio)
 	if (randomSignpostsEmpty())
 	    randno = dynamicPercent;
 	else
-	    randno = rand() % 100;
+	    randno = Rnd::rand() % 100;
 	if (randno < dynamicPercent)
 	{
             // set up a signpost from the list of signposts
@@ -698,7 +659,7 @@ void CreateScenario::getRuinDifficulty (int difficulty, int *sage_factor,
       *stronghold_factor = 3;
     }
 }
-    
+
 void CreateScenario::getSignpostDifficulty (int difficulty, bool hidden_map, 
 					    int *signpost_ratio)
 {
@@ -721,7 +682,6 @@ void CreateScenario::getSignpostDifficulty (int difficulty, bool hidden_map,
     *signpost_ratio = 6;
 }
 
-    
 void CreateScenario::getCityDifficulty(int difficulty, 
 				       int *number_of_armies_factor)
 {
@@ -734,7 +694,7 @@ void CreateScenario::getCityDifficulty(int difficulty,
   else 
     *number_of_armies_factor = 0;
 }
-    
+
 int CreateScenario::calculateRoadType (Vector<int> t)
 {
     Roadlist *rl = Roadlist::getInstance();
