@@ -1,7 +1,7 @@
 // Copyright (C) 2003 Michael Bartl
 // Copyright (C) 2003, 2004, 2005, 2006 Ulf Lorenz
 // Copyright (C) 2003, 2005, 2006 Andrea Paternesi
-// Copyright (C) 2006, 2007, 2008, 2009, 2010, 2011 Ben Asselstine
+// Copyright (C) 2006, 2007, 2008, 2009, 2010, 2011, 2014, 2015 Ben Asselstine
 // Copyright (C) 2007 Ole Laursen
 // Copyright (C) 2008 Janek Kozicki
 //
@@ -22,6 +22,7 @@
 
 #include <sstream>
 #include <iostream>
+#include <string>
 #include <iomanip>
 #include <assert.h>
 #include <sigc++/functors/mem_fun.h>
@@ -29,6 +30,7 @@
 #include <math.h>
 
 #include "ucompose.hpp"
+#include "army.h"
 #include "GameMap.h"
 #include "citylist.h"
 #include "bridgelist.h"
@@ -50,7 +52,6 @@
 #include "tilesetlist.h"
 #include "shieldsetlist.h"
 #include "citysetlist.h"
-#include "GraphicsCache.h"
 #include "MapBackpack.h"
 #include "stacktile.h"
 #include "armyprodbase.h"
@@ -58,12 +59,12 @@
 #include "armyset.h"
 #include "armysetlist.h"
 #include "CreateScenario.h"
+#include "SightMap.h"
 #include "reward.h"
 #include "rewardlist.h"
 
-std::string GameMap::d_tag = "map";
-std::string GameMap::d_itemstack_tag = "itemstack";
-using namespace std;
+Glib::ustring GameMap::d_tag = "map";
+Glib::ustring GameMap::d_itemstack_tag = "itemstack";
 
 //#define debug(x) {std::cerr<<__FILE__<<": "<<__LINE__<<": "<<x<<std::endl<<flush;}
 #define debug(x)
@@ -76,18 +77,10 @@ Tileset* GameMap::s_tileset = 0;
 Cityset* GameMap::s_cityset = 0;
 Shieldset* GameMap::s_shieldset = 0;
 
-GameMap* GameMap::getInstance()
-{
-    if (s_instance != 0)
-      return s_instance;
-    else
-      return getInstance("", "", "");
-}
 
-
-GameMap* GameMap::getInstance(std::string TilesetName, 
-			      std::string ShieldsetName, 
-			      std::string CitysetName)
+GameMap* GameMap::getInstance(Glib::ustring TilesetName, 
+			      Glib::ustring ShieldsetName, 
+			      Glib::ustring CitysetName)
 {
     if (s_instance == 0)
     {
@@ -114,8 +107,8 @@ void GameMap::deleteInstance()
     s_instance = 0;
 }
 
-GameMap::GameMap(std::string TilesetName, std::string ShieldsetName,
-		 std::string CitysetName)
+GameMap::GameMap(Glib::ustring TilesetName, Glib::ustring ShieldsetName,
+		 Glib::ustring CitysetName)
 {
   s_tileset = 0;
   s_cityset = 0;
@@ -142,8 +135,9 @@ bool GameMap::offmap(int x, int y)
   return false;
 }
 
-void GameMap::processStyles(std::string styles, int chars_per_style)
+void GameMap::processStyles(Glib::ustring s, int chars_per_style)
 {
+  std::string styles = s.raw();
   Tileset *tileset = GameMap::getTileset();
   int c = chars_per_style;
     int offset = 0;
@@ -178,8 +172,8 @@ void GameMap::processStyles(std::string styles, int chars_per_style)
         }
     }
 }
-    
-int GameMap::determineCharsPerStyle(std::string styles)
+
+int GameMap::determineCharsPerStyle(Glib::ustring styles)
 {
   return styles.length() / (s_width * s_height);
 }
@@ -189,11 +183,11 @@ GameMap::GameMap(XML_Helper* helper)
     s_tileset = 0;
     s_cityset = 0;
     s_shieldset = 0;
-    std::string types;
-    std::string styles;
-    std::string t_dir;
-    std::string s_dir;
-    std::string c_dir;
+    Glib::ustring types;
+    Glib::ustring styles;
+    Glib::ustring t_dir;
+    Glib::ustring s_dir;
+    Glib::ustring c_dir;
 
     helper->getData(s_width, "width");
     helper->getData(s_height, "height");
@@ -207,11 +201,11 @@ GameMap::GameMap(XML_Helper* helper)
     d_shieldset = s_dir;
     d_cityset = c_dir;
 
-    Tileset *tileset = Tilesetlist::getInstance()->getTileset(d_tileset);
+    Tileset *tileset = Tilesetlist::getInstance()->get(d_tileset);
     s_tileset = tileset;
-    Cityset *cityset = Citysetlist::getInstance()->getCityset(d_cityset);
+    Cityset *cityset = Citysetlist::getInstance()->get(d_cityset);
     s_cityset = cityset;
-    Shieldset *shieldset = Shieldsetlist::getInstance()->getShieldset(d_shieldset);
+    Shieldset *shieldset = Shieldsetlist::getInstance()->get(d_shieldset);
     s_shieldset = shieldset;
     Vector<int>::setMaximumWidth(s_width);
     //create the map
@@ -237,7 +231,7 @@ GameMap::GameMap(XML_Helper* helper)
             //the chars now hold the ascii representation of the numbers, which
             //we don't want
             type -= '0';
-            d_map[j*s_width + i] = new Maptile(tileset, i, j, type, NULL);
+            d_map[j*s_width + i] = new Maptile(i, j, type);
         }
     }
 
@@ -274,10 +268,10 @@ bool GameMap::fill(MapGenerator* generator)
 
     //the sizes should definitely match, else we have a problem here
     if (width != s_width || height != s_height)
-    {
-        std::cerr << "Error in GameMap::fillMap: sizes don't match!! Exiting.\n";
+      {
+        std::cerr << String::ucompose(_("Error!  Map Generator and Game Map tile sizes differ.  %1 != %2 || %3 != %4.  Exiting."), width, s_width, height, s_height) << std::endl;
         exit(-1);
-    }
+      }
 
     // create tiles; there is a hack here: The map generator outputs tile types,
     // but we supply the index of the tile types in the tileset to Maptile. Was
@@ -287,8 +281,7 @@ bool GameMap::fill(MapGenerator* generator)
         {
             int index = tileset->getIndex(terrain[j*width + i]);
 	    if (index != -1)
-	      d_map[j*s_width + i] = new Maptile(tileset, i, j, 
-						 (guint32)index, NULL);
+	      d_map[j*s_width + i] = new Maptile(i, j, (guint32)index);
         }
 
     applyTileStyles(0, 0, height, width, true);
@@ -297,15 +290,12 @@ bool GameMap::fill(MapGenerator* generator)
 
 bool GameMap::fill(guint32 type)
 {
-    Tileset *tileset = GameMap::getTileset();
-    for (int i = 0; i < s_width; i++)
-        for (int j = 0; j < s_height; j++)
-	  {
-            d_map[j*s_width + i] = new Maptile(tileset, i, j, type, NULL);
-	  }
+  for (int i = 0; i < s_width; i++)
+    for (int j = 0; j < s_height; j++)
+      d_map[j*s_width + i] = new Maptile(i, j, type);
 
-    applyTileStyles(0, 0, s_height, s_width, false);
-    return true;
+  applyTileStyles(0, 0, s_height, s_width, false);
+  return true;
 }
 
 bool GameMap::save(XML_Helper* helper) const
@@ -315,16 +305,19 @@ bool GameMap::save(XML_Helper* helper) const
 
     std::stringstream types;
 
-    types <<endl;
+    types <<std::endl;
     for (int i = 0; i < s_height; i++)
     {
         for (int j = 0; j < s_width; j++)
-            types << getTile(j, i)->getIndex();
-        types <<endl;
+          {
+            guint32 tile_type = getTile(j, i)->getIndex();
+            types << (guint32) tile_type;
+          }
+        types <<std::endl;
     }
 
     std::stringstream styles;
-    styles <<endl;
+    styles <<std::endl;
 	    
     int largest_style_id = tileset->getLargestTileStyleId();
     guint32 num_digits = TileStyle::calculateHexDigits(largest_style_id);
@@ -338,7 +331,7 @@ bool GameMap::save(XML_Helper* helper) const
             hexstr = TileStyle::idToString(style->getId(), num_digits);
 	    styles << hexstr;
 	  }
-        styles <<endl;
+        styles <<std::endl;
     }
 
 
@@ -361,7 +354,7 @@ bool GameMap::save(XML_Helper* helper) const
     return retval;
 }
 
-bool GameMap::loadItems(std::string tag, XML_Helper* helper)
+bool GameMap::loadItems(Glib::ustring tag, XML_Helper* helper)
 {
     if (tag == MapBackpack::d_tag)
       {
@@ -380,23 +373,12 @@ void GameMap::setTile(int x, int y, Maptile *tile)
     applyTileStyle (y, x);
 }
 
-Maptile* GameMap::getTile(int x, int y) const
-{
-  if (x < 0 && x >= s_width && y < 0 && y >= s_height)
-    {
-      printf("%d,%d > or < %d,%d\n", x, y, s_width, s_height);
-    }
-  assert(x >= 0 && x < s_width && y >= 0 && y < s_height);
-
-  return d_map[y*s_width + x];
-}
-
 Stack* GameMap::addArmy(Vector<int> pos, Army *a)
 {
   City *c = getCity(pos);
   if (c)
     {
-      if (c->isBurnt())
+      if (c->isBurnt() || c->getOwner() != a->getOwner())
         return addArmyAtPos(pos, a);
       else
         return addArmy(c, a);
@@ -424,10 +406,13 @@ Stack* GameMap::addArmyAtPos(Vector<int> pos, Army *a)
     max = s_width;
   max--;
 
-  Location l(pos, 1);
-  s = l.addArmy(a);
-  if (s)
-    return s;
+  if (getBuilding(pos) == Maptile::NONE)
+    {
+      Location l(pos, 1);
+      s = l.addArmy(a);
+      if (s)
+        return s;
+    }
 
   // we couldn't add the army to the square(s) identified by location,
   // so the idea is to go around in ever widening boxes until we find a
@@ -440,12 +425,14 @@ Stack* GameMap::addArmyAtPos(Vector<int> pos, Army *a)
   //d is the distance from Pos where our box starts
   for (d = 1; d < max; d++)
     {
-      for (i = 0; i < (d * 2) + 1; i++)
+      guint32 imax = (d * 2) + 1;
+      guint32 jmax = (d * 2) + 1;
+      for (i = 0; i < imax; i++)
         {
-          for (j = 0; j < (d * 2) + 1; j++)
+          for (j = 0; j < jmax; j++)
             {
-              if ((i == 0 || i == (d * 2) + 1) && 
-                  (j == 0 || j == (d * 2) + 1))
+              if ((i == 0 || i == imax - 1) && 
+                  (j == 0 || j == jmax - 1))
                 {
                   x = pos.x + (i - d);
                   y = pos.y + (j - d);
@@ -463,8 +450,6 @@ Stack* GameMap::addArmyAtPos(Vector<int> pos, Army *a)
                     continue;
                   if (!land && getTile(x, y)->getType() != Tile::WATER)
                     continue;
-                  if (land && getTile(x, y)->getType() == Tile::VOID)
-		    continue;
                   if (land && getTile(x, y)->getType() == Tile::MOUNTAIN &&
 		      (a->getStat(Army::MOVE_BONUS) & Tile::MOUNTAIN) == 0)
                     continue;
@@ -479,13 +464,13 @@ Stack* GameMap::addArmyAtPos(Vector<int> pos, Army *a)
                     }
                   else 
                     {
-                      Vector<int> pos(x, y);
+                      Vector<int> p(x, y);
 		      //hmm. no nifty stacks here.  anybody else's?
-		      s = getEnemyStack(pos);
+		      s = getEnemyStack(p);
 		      if (s)
 			continue;
 		      //okay, no stacks here at all.  make one.
-                      s = new Stack(a->getOwner(), pos);
+                      s = new Stack(a->getOwner(), p);
 		      s->add(a);
                       a->getOwner()->addStack(s);
                     }
@@ -579,11 +564,6 @@ bool GameMap::isBlockedAvenue(int x, int y, int destx, int desty)
       if (from->getType() == Tile::MOUNTAIN &&
 	  getRoad(Vector<int>(x, y)) == NULL)
         return true;
-
-      if (from->getType() == Tile::VOID)
-	return true;
-      if (to->getType() == Tile::VOID)
-	return true;
     }
  return false;
 }
@@ -623,26 +603,6 @@ void GameMap::calculateBlockedAvenues()
       calculateBlockedAvenue(i, j);
 }
 
-Vector<int> GameMap::findStack(guint32 id)
-{
-    bool found = false;
-    Vector<int> pos = Vector<int>(-1,-1);
-    for (int x = 0; x < getWidth(); x++)
-      {
-        for (int y = 0; y < getHeight(); y++)
-          {
-	    StackTile *stile = getTile(x,y)->getStacks();
-	    if (stile->contains(id) == true)
-	      {
-		pos = Vector<int>(x,y);
-		found = true;
-		break;
-	      }
-          }
-      }
-  return pos;
-}
-
 Vector<int> GameMap::findPlantedStandard(Player *p)
 {
     bool found = false;
@@ -905,14 +865,15 @@ void GameMap::demote_lone_tile(int minx, int miny, int maxx, int maxy,
 	  {
 	    //if we're not connected in a square of
 	    //same types, then we're a lone tile.
+            if (getTile(j,i)->getBuilding())
+              continue;
 	    if (tile_is_connected_to_other_like_tiles(tile, i, j) == 0)
 	      {
 		//okay, this is a lone tile.
 		//downgrade it
 		int idx = tileset->getIndex(outtype);
 		if (idx != -1)
-		  setTile(j, i, new Maptile (tileset, j, i, 
-                                             (guint32)idx, NULL));
+		  setTile(j, i, new Maptile (j, i, (guint32)idx));
 	      }
 	  }
       }
@@ -981,16 +942,20 @@ void GameMap::surroundMountains(int minx, int miny, int maxx, int maxy)
 		  if(getTile((j+J), (i+I))->getType() != Tile::WATER)
 		    {
 		      if (idx != -1)
-			setTile(j+J, i+I, 
-				new Maptile (tileset, j+J, i+I, 
-					     (guint32)idx, NULL));
+                        {
+                          Maptile::Building b =
+                            getTile(j+J, i+I)->getBuilding();
+                          setTile(j+J, i+I,
+                                  new Maptile (j+J, i+I, (guint32)idx));
+                          if (b)
+                            setBuilding(Vector<int>(j+J,i+I), b);
+                        }
 		    }
 		  else 
 		    {
 		    // water has priority here, there was some work done to conenct bodies of water
 		    // so don't break those connections.
-		      setTile(j, i, 
-			    new Maptile (tileset, j, i, (guint32)idx, NULL));
+		      setTile(j, i, new Maptile (j, i, (guint32)idx));
 		    }
 		}
       }
@@ -1137,12 +1102,6 @@ Stack* GameMap::getFriendlyStack(Vector<int> pos)
   return getStacks(pos)->getFriendlyStack(Playerlist::getActiveplayer());
 }
 
-//StackReflist GameMap::getFriendlyStacks(Vector<int> pos, Player *player)
-//{
-  //if (player == NULL)
-    //player = Playerlist::getActiveplayer();
-  //return StackReflist(getStacks(pos)->getFriendlyStacks(player));
-//}
 std::list<Stack*> GameMap::getFriendlyStacks(Vector<int> pos, Player *player)
 {
   if (player == NULL)
@@ -1170,29 +1129,78 @@ std::list<Stack*> GameMap::getEnemyStacks(std::list<Vector<int> > positions)
 
 std::list<Stack*> GameMap::getEnemyStacks(Vector<int> pos, Player *player)
 {
+  if (!getStacks(pos))
+    {
+      std::list<Stack*> empty;
+      return empty;
+    }
+
   if (player == NULL)
     player = Playerlist::getActiveplayer();
   return getStacks(pos)->getEnemyStacks(player);
 }
 
+
+bool GameMap::compareStackStrength(Stack *lhs, Stack *rhs)
+{
+  Army *lhero = lhs->getStrongestHero();
+  Army *rhero = rhs->getStrongestHero();
+  if (lhero && rhero)
+    return 
+      lhero->getStat(Army::STRENGTH) > rhero->getStat(Army::STRENGTH);
+  else if (!lhero && rhero)
+    return false;
+  else if (lhero && !rhero)
+    return true;
+
+  Army *larmy = lhs->getStrongestArmy();
+  Army *rarmy = rhs->getStrongestArmy();
+  return larmy->getStat(Army::STRENGTH) > rarmy->getStat(Army::STRENGTH);
+}
+
+Stack* GameMap::getStrongestStack(Vector<int> pos)
+{
+  StackTile *s = getStacks(pos);
+  std::list<Stack*> stacks = s->getStacks();
+  if (stacks.empty())
+    return NULL;
+  stacks.sort(compareStackStrength);
+  return stacks.front();
+}
+
 Stack* GameMap::getStack(Vector<int> pos)
 {
-  return getStacks(pos)->getStack();
+  if (getStacks(pos))
+    return getStacks(pos)->getStack();
+  else
+    return NULL;
 }
 	
 StackTile* GameMap::getStacks(Vector<int> pos)
 {
-  return getInstance()->getTile(pos)->getStacks();
+  if (getInstance()->getTile(pos))
+    return getInstance()->getTile(pos)->getStacks();
+  else
+    return NULL;
 }
 
 Stack *GameMap::groupStacks(Vector<int> pos)
 {
-  return getStacks(pos)->group(Playerlist::getActiveplayer());
+  return getInstance()->groupStacks(pos, Playerlist::getActiveplayer());
+}
+
+Stack *GameMap::groupStacks(Vector<int> pos, Player *player)
+{
+  if (getStacks(pos))
+    return getStacks(pos)->group(player);
+  else
+    return NULL;
 }
   
 void GameMap::groupStacks(Stack *stack)
 {
-  getStacks(stack->getPos())->group(Playerlist::getActiveplayer(), stack);
+  if (getStacks(stack->getPos()))
+    return getStacks(stack->getPos())->group(Playerlist::getActiveplayer(), stack);
 }
   
 void GameMap::clearStackPositions()
@@ -1202,18 +1210,24 @@ void GameMap::clearStackPositions()
     {
       Stacklist *sl = (*i)->getStacklist();
       for (Stacklist::iterator s = sl->begin(); s != sl->end(); s++)
-	getStacks((*s)->getPos())->clear();
+        {
+          Vector<int> pos = (*s)->getPos();
+          StackTile *st = getStacks(pos);
+          st->clear();
+        }
     }
 }
 void GameMap::updateStackPositions()
 {
   Playerlist *plist = Playerlist::getInstance();
-  //okay, clear them all first.
   for (Playerlist::iterator i = plist->begin(); i != plist->end(); i++)
     {
       Stacklist *sl = (*i)->getStacklist();
       for (Stacklist::iterator s = sl->begin(); s != sl->end(); s++)
-	getStacks((*s)->getPos())->add(*s);
+        {
+          Vector<int> pos = (*s)->getPos();
+          getStacks(pos)->add(*s);
+        }
     }
 }
 
@@ -1242,30 +1256,41 @@ bool GameMap::canAddArmies(Vector<int> dest, guint32 stackSize)
 void GameMap::switchTileset(Tileset *tileset)
 {
   d_tileset = tileset->getBaseName();
-  s_tileset = Tilesetlist::getInstance()->getTileset(d_tileset);
+  s_tileset = Tilesetlist::getInstance()->get(d_tileset);
+  for (int i = 0; i < s_width; i++)
+    for (int j = 0; j < s_height; j++)
+      {
+        //there is also the problem of the index being kept in the maptile.
+        //perhaps we need to get a new index, right? e.g. when we switch
+        //the tileset for another one in the editor.
+        //because "Grass" won't always be in the 0th spot in the tileset.
+        d_map[j*s_width + i]->setIndex(d_map[j*s_width + i]->getIndex());
+      }
   applyTileStyles (0, 0, s_width, s_height,  false);
 }
 
 void GameMap::reloadTileset()
 {
-  bool broken = false;
   Tileset *tileset = GameMap::getTileset();
-  tileset->reload(broken);
+  if (tileset)
+    Tilesetlist::getInstance()->reload(tileset->getId());
 }
 
 void GameMap::reloadShieldset()
 {
-  bool broken = false;
   Shieldset *shieldset = GameMap::getShieldset();
-  shieldset->reload(broken);
-  Playerlist::getInstance()->setNewColours(shieldset);
+  if (shieldset)
+    {
+      Shieldsetlist::getInstance()->reload(shieldset->getId());
+      Playerlist::getInstance()->setNewColours(shieldset);
+    }
 }
 
 void GameMap::switchShieldset(Shieldset *shieldset)
 {
   Playerlist::getInstance()->setNewColours(shieldset);
   d_shieldset = shieldset->getBaseName();
-  s_shieldset = Shieldsetlist::getInstance()->getShieldset(d_shieldset);
+  s_shieldset = Shieldsetlist::getInstance()->get(d_shieldset);
 }
 
 Vector<int> GameMap::findNearestAreaForBuilding(Maptile::Building building_type, Vector<int> pos, guint32 width)
@@ -1282,7 +1307,7 @@ Vector<int> GameMap::findNearestAreaForBuilding(Maptile::Building building_type,
 
 void GameMap::switchCityset(Cityset *cityset)
 {
-  d_cityset = cityset->getBaseName();
+  setCityset(cityset->getBaseName());
 
   if (Templelist::getInstance()->size())
     {
@@ -1293,8 +1318,8 @@ void GameMap::switchCityset(Cityset *cityset)
       if (old_tile_width != cityset->getTempleTileWidth())
         Templelist::getInstance()->resizeLocations
           (Maptile::TEMPLE, cityset->getTempleTileWidth(), old_tile_width, 
-           (void (*)(Temple*, Maptile::Building, guint32)) changeFootprintToSmallerCityset, 
-           (void (*)(Temple*, Maptile::Building, guint32)) relocateLocation);
+           (void (*)(Location*, Maptile::Building, guint32)) changeFootprintToSmallerCityset, 
+           (void (*)(Location*, Maptile::Building, guint32)) relocateLocation);
     }
     
   if (Ruinlist::getInstance()->size())
@@ -1305,8 +1330,8 @@ void GameMap::switchCityset(Cityset *cityset)
       if (old_tile_width != cityset->getRuinTileWidth())
         Ruinlist::getInstance()->resizeLocations
           (Maptile::RUIN, cityset->getRuinTileWidth(), old_tile_width, 
-           (void (*)(Ruin*, Maptile::Building, guint32)) changeFootprintToSmallerCityset, 
-           (void (*)(Ruin*, Maptile::Building, guint32)) relocateLocation);
+           (void (*)(Location*, Maptile::Building, guint32)) changeFootprintToSmallerCityset, 
+           (void (*)(Location*, Maptile::Building, guint32)) relocateLocation);
     }
   if (Citylist::getInstance()->size())
     {
@@ -1316,8 +1341,8 @@ void GameMap::switchCityset(Cityset *cityset)
       if (old_tile_width != cityset->getCityTileWidth())
         Citylist::getInstance()->resizeLocations
           (Maptile::CITY, cityset->getCityTileWidth(), old_tile_width, 
-           (void (*)(City*, Maptile::Building, guint32)) changeFootprintToSmallerCityset, 
-           (void (*)(City*, Maptile::Building, guint32)) relocateLocation);
+           (void (*)(Location*, Maptile::Building, guint32)) changeFootprintToSmallerCityset, 
+           (void (*)(Location*, Maptile::Building, guint32)) relocateLocation);
     }
 }
 
@@ -1338,10 +1363,12 @@ guint32 GameMap::countBuildings(Maptile::Building building_type)
 
 void GameMap::reloadCityset()
 {
-  bool broken = false;
   Cityset *cityset = GameMap::getCityset();
-  cityset->reload(broken);
-  switchCityset(cityset);
+  if (cityset)
+    {
+      Citysetlist::getInstance()->reload(cityset->getId());
+      switchCityset(cityset);  //is this still needed?
+    }
 }
 
 void GameMap::switchArmysets(Armyset *armyset)
@@ -1355,21 +1382,18 @@ void GameMap::switchArmysets(Armyset *armyset)
       Stack *s = (*i)->getOccupant();
       if (s == NULL)
 	continue;
+      s->removeArmiesWithoutArmyType(armyset->getId());
       for (Stack::iterator j = s->begin(); j != s->end(); j++)
 	Armyset::switchArmysetForRuinKeeper(*j, armyset);
     }
   for (Playerlist::iterator i = pl->begin(); i != pl->end(); i++)
     {
-      Armyset *a = 
-	Armysetlist::getInstance()->getArmyset((*i)->getArmyset());
-      if (armyset == a)
-	continue;
-
       //change the armyprodbases in cities.
       for (Citylist::iterator j = cl->begin(); j != cl->end(); j++)
 	{
 	  City *c = *j;
-	  for (unsigned int k = 0; c->getSize(); k++)
+          c->removeArmyProdBasesWithoutAType(armyset->getId());
+	  for (unsigned int k = 0; k < c->getSize(); k++)
 	    {
 	      ArmyProdBase *prodbase = (*c)[k]->getArmyProdBase();
 	      if (prodbase)
@@ -1382,8 +1406,17 @@ void GameMap::switchArmysets(Armyset *armyset)
       for (Stacklist::iterator j = sl->begin(); j != sl->end(); j++)
 	{
 	  Stack *s = (*j);
-	  for (Stack::iterator k = s->begin(); k != s->end(); k++)
-	    Armyset::switchArmyset(*k,armyset);
+          s->removeArmiesWithoutArmyType(armyset->getId());
+          if (s->size() == 0)
+            {
+              GameMap::getInstance()->getStacks(s->getPos())->leaving(s);
+              j=sl->flErase(j);//this doesn't remove the stack from the map of id->stack pointer in stacklist. XXX XXX XXX
+              if (sl->size() > 0)
+                j--;
+              continue;
+            }
+          for (Stack::iterator k = s->begin(); k != s->end(); k++)
+            Armyset::switchArmyset(*k,armyset);
 	}
 
       //finally, change the player's armyset.
@@ -1394,8 +1427,7 @@ void GameMap::switchArmysets(Armyset *armyset)
 
 void GameMap::reloadArmyset(Armyset *armyset)
 {
-  bool broken = false;
-  armyset->reload(broken);
+  Armysetlist::getInstance()->reload(armyset->getId());
 }
 
 bool GameMap::canPutBuilding(Maptile::Building bldg, guint32 size, Vector<int> to, bool making_islands)
@@ -1422,9 +1454,6 @@ bool GameMap::canPutBuilding(Maptile::Building bldg, guint32 size, Vector<int> t
   switch (bldg)
     {
       case Maptile::CITY: 
-      case Maptile::RUIN: 
-      case Maptile::TEMPLE: 
-      case Maptile::SIGNPOST:
 	//gotta be on grass.
 	  {
 	    if (making_islands)
@@ -1438,10 +1467,24 @@ bool GameMap::canPutBuilding(Maptile::Building bldg, guint32 size, Vector<int> t
 		}
 	  }
 	break;
+      case Maptile::RUIN: 
+      case Maptile::TEMPLE: 
+      case Maptile::SIGNPOST:
+	  {
+	    if (making_islands)
+	      return true;
+	    for (unsigned int i = 0; i < size; i++)
+	      for (unsigned int j = 0; j < size; j++)
+		{
+		  Vector<int> pos = to + Vector<int>(i, j);
+		  if (getTerrainType(pos) == Tile::WATER)
+		    return false;
+		}
+	  }
+	break;
       case Maptile::ROAD: 
 	//can't be in the water
-	if (getTerrainType(to) == Tile::WATER ||
-	    getTerrainType(to) == Tile::VOID)
+	if (getTerrainType(to) == Tile::WATER)
 	  return false;
 	break;
       case Maptile::PORT: 
@@ -1567,23 +1610,16 @@ bool GameMap::moveBuilding(Vector<int> from, Vector<int> to, guint32 new_width)
   return moved;
 }
 
-Tile::Type GameMap::getTerrainType(Vector<int> tile)
-{
-  return getTile(tile)->getType();
-}
-
-Maptile::Building GameMap::getBuilding(Vector<int> tile)
-{
-  return getTile(tile)->getBuilding();
-}
-
 void GameMap::setBuilding(Vector<int> tile, Maptile::Building building)
 {
-  getTile(tile)->setBuilding(building);
+  if (getTile(tile))
+    getTile(tile)->setBuilding(building);
 }
 
 guint32 GameMap::getBuildingSize(Vector<int> tile)
 {
+  if (getTile(tile) == NULL)
+    return 0;
   switch (getTile(tile)->getBuilding())
     {
     case Maptile::CITY: return getCity(tile)->getSize(); break;
@@ -1621,20 +1657,10 @@ bool GameMap::moveStack(Stack *stack, Vector<int> to)
 
   getStacks(stack->getPos())->leaving(stack);
   stack->setPos(to);
-  //Stack *new_stack = new Stack(*stack);
-  //new_stack->setPos(to);
-  //getStacks(stack->getPos())->leaving(stack);
-  //delete stack;
-  //if we dropped it on a city, then change the ownership.
   City *c = GameMap::getCity(to);
   if (c != NULL && stack->getOwner() != c->getOwner())
     stack = Stacklist::changeOwnership(stack, c->getOwner());
   getStacks(stack->getPos())->arriving(stack);
-  bool ship = false;
-  if (getTerrainType(stack->getPos()) == Tile::WATER &&
-      getBuilding(stack->getPos()) != Maptile::PORT &&
-      getBuilding(stack->getPos()) != Maptile::BRIDGE)
-    ship = true;
   updateShips(stack->getPos());
 
   return moved;
@@ -1642,7 +1668,10 @@ bool GameMap::moveStack(Stack *stack, Vector<int> to)
 	
 MapBackpack *GameMap::getBackpack(Vector<int> pos)
 {
-  return getInstance()->getTile(pos)->getBackpack();
+  if (getInstance()->getTile(pos))
+    return getInstance()->getTile(pos)->getBackpack();
+  else
+    return NULL;
 }
 		    
 void GameMap::moveBackpack(Vector<int> from, Vector<int> to)
@@ -1662,10 +1691,21 @@ bool GameMap::removeRuin(Vector<int> pos)
     }
   return false;
 }
+
+bool GameMap::containsWater(Rectangle rect)
+{
+  for (int y = rect.y; y < rect.y + rect.h; y++)
+    for (int x = rect.x; x < rect.x + rect.w; x++)
+      if (getTile(x, y)->getType() == Tile::WATER)
+        return true;
+  return false;
+}
+
 bool GameMap::putRuin(Ruin *r)
 {
   Ruinlist::getInstance()->add(r);
-  putTerrain(r->getArea(), Tile::GRASS);
+  if (containsWater(r->getArea()))
+    putTerrain(r->getArea(), Tile::GRASS);
   putBuilding(r, Maptile::RUIN);
   return true;
 }
@@ -1702,7 +1742,8 @@ bool GameMap::removeTemple(Vector<int> pos)
 bool GameMap::putTemple(Temple *t)
 {
   Templelist::getInstance()->add(t);
-  putTerrain(t->getArea(), Tile::GRASS);
+  if (containsWater(t->getArea()))
+    putTerrain(t->getArea(), Tile::GRASS);
   putBuilding(t, Maptile::TEMPLE);
   return true;
 }
@@ -1744,7 +1785,8 @@ bool GameMap::removeSignpost(Vector<int> pos)
 bool GameMap::putSignpost(Signpost *s)
 {
   Signpostlist::getInstance()->add(s);
-  putTerrain(s->getArea(), Tile::GRASS);
+  if (containsWater(s->getArea()))
+    putTerrain(s->getArea(), Tile::GRASS);
   putBuilding(s, Maptile::SIGNPOST);
   return true;
 }
@@ -1770,11 +1812,15 @@ bool GameMap::putNewRoad(Vector<int> tile)
     return false;
 }
 
-bool GameMap::putRoad(Road *r)
+bool GameMap::putRoad(Road *r, bool smooth)
 {
+  if (containsWater(r->getArea()))
+    putTerrain(r->getArea(), Tile::GRASS);
   Roadlist::getInstance()->add(r);
   setBuilding(r->getPos(), Maptile::ROAD);
 
+  if (smooth == false)
+    return true;
   // now reconfigure all roads in the surroundings
   Vector<int> tile = r->getPos();
   for (int x = tile.x - 1; x <= tile.x + 1; ++x)
@@ -1784,8 +1830,9 @@ bool GameMap::putRoad(Road *r)
           continue;
 
 	Vector<int> pos(x, y);
-	if (Road *r = Roadlist::getInstance()->getObjectAt(pos))
+	if (Roadlist::getInstance()->getObjectAt(pos))
 	  {
+            r = Roadlist::getInstance()->getObjectAt(pos);
 	    int newtype = CreateScenario::calculateRoadType(pos);
 	    r->setType(newtype);
 	  }
@@ -1800,6 +1847,7 @@ bool GameMap::removeBridge(Vector<int> pos)
     {
       removeBuilding(b);
       Bridgelist::getInstance()->subtract(b);
+      updateShips(pos);
       return true;
     }
   return false;
@@ -1826,10 +1874,19 @@ Rectangle GameMap::putTerrain(Rectangle r, Tile::Type type, int tile_style_id, b
       {
 	if (offmap(x,y))
 	  continue;
+        
 	Maptile* t = getTile(Vector<int>(x, y));
+        if (t->hasLandBuilding() && type == Tile::WATER)
+          continue;
+        if (t->hasWaterBuilding() && type != Tile::WATER)
+          continue;
 	if (t->getType() != type)
           {
-            t->setIndex(index);
+            //it's always grass under cities.
+            if (t->getBuilding() == Maptile::CITY)
+              t->setIndex(tileset->getIndex(Tile::GRASS));
+            else
+              t->setIndex(index);
             updateShips(Vector<int>(x,y));
             replaced = true;
           }
@@ -2047,7 +2104,9 @@ void GameMap::removeStack(Stack *s)
 	
 guint32 GameMap::countArmyUnits(Vector<int> pos)
 {
-  return getStacks(pos)->countNumberOfArmies(Playerlist::getActiveplayer());
+  if (getStacks(pos))
+    return getStacks(pos)->countNumberOfArmies(Playerlist::getActiveplayer());
+  return 0;
 }
 
 std::list<Stack*> GameMap::getNearbyFriendlyStacks(Vector<int> pos, int dist)
@@ -2182,6 +2241,7 @@ int GameMap::calculateTilesPerOverviewMapTile()
 void GameMap::changeFootprintToSmallerCityset(Location *location, Maptile::Building building_type, guint32 old_tile_width)
 {
   GameMap::getInstance()->clearBuilding(location->getPos(), (guint32)old_tile_width);
+
   GameMap::getInstance()->putBuilding (location, building_type);
 }
 
@@ -2207,30 +2267,36 @@ guint32 GameMap::getTileSize() const
 
 guint32 GameMap::getTilesetId() const
 {
-  return GameMap::getTileset()->getId();
+  if (GameMap::getTileset())
+    return GameMap::getTileset()->getId();
+  else
+    return 0;
 }
 
 guint32 GameMap::getCitysetId() const
 {
-  return GameMap::getCityset()->getId();
+  if (GameMap::getCityset())
+    return GameMap::getCityset()->getId();
+  else
+    return 0;
 }
 
 guint32 GameMap::getShieldsetId() const
 {
-  return Shieldsetlist::getInstance()->getShieldsetId(d_shieldset);
+  return Shieldsetlist::getInstance()->getSetId(d_shieldset);
 }
 
-std::string GameMap::getTilesetName() const
+Glib::ustring GameMap::getTilesetBaseName() const
 {
   return d_tileset;
 }
 
-std::string GameMap::getCitysetName() const
+Glib::ustring GameMap::getCitysetBaseName() const
 {
   return d_cityset;
 }
 
-std::string GameMap::getShieldsetName() const
+Glib::ustring GameMap::getShieldsetBaseName() const
 {
   return d_shieldset;
 }
@@ -2305,7 +2371,7 @@ bool GameMap::eraseTile(Vector<int> tile)
 Tileset* GameMap::getTileset()
 {
   if (s_tileset == 0)
-    s_tileset = Tilesetlist::getInstance()->getTileset(GameMap::getInstance()->getTilesetName());
+    s_tileset = Tilesetlist::getInstance()->get(GameMap::getInstance()->getTilesetBaseName());
     
   return s_tileset;
 }
@@ -2313,7 +2379,7 @@ Tileset* GameMap::getTileset()
 Cityset* GameMap::getCityset()
 {
   if (s_cityset == 0)
-    s_cityset = Citysetlist::getInstance()->getCityset(GameMap::getInstance()->getCitysetName());
+    s_cityset = Citysetlist::getInstance()->get(GameMap::getInstance()->getCitysetBaseName());
     
   return s_cityset;
 }
@@ -2321,27 +2387,27 @@ Cityset* GameMap::getCityset()
 Shieldset* GameMap::getShieldset()
 {
   if (s_shieldset == 0)
-    s_shieldset = Shieldsetlist::getInstance()->getShieldset(GameMap::getInstance()->getShieldsetName());
+    s_shieldset = Shieldsetlist::getInstance()->get(GameMap::getInstance()->getShieldsetBaseName());
     
   return s_shieldset;
 }
 
-void GameMap::setTileset(std::string tileset)
+void GameMap::setTileset(Glib::ustring tileset)
 {
   d_tileset = tileset;
-  s_tileset = Tilesetlist::getInstance()->getTileset(tileset);
+  s_tileset = Tilesetlist::getInstance()->get(tileset);
 }
 
-void GameMap::setCityset(std::string cityset)
+void GameMap::setCityset(Glib::ustring cityset)
 {
   d_cityset = cityset;
-  s_cityset = Citysetlist::getInstance()->getCityset(cityset);
+  s_cityset = Citysetlist::getInstance()->get(cityset);
 }
 
-void GameMap::setShieldset(std::string shieldset)
+void GameMap::setShieldset(Glib::ustring shieldset)
 {
   d_shieldset = shieldset;
-  s_shieldset = Shieldsetlist::getInstance()->getShieldset(shieldset);
+  s_shieldset = Shieldsetlist::getInstance()->get(shieldset);
 }
 
 bool GameMap::can_search(Stack *stack)
@@ -2416,3 +2482,144 @@ bool GameMap::can_plant_flag(Stack *stack)
   return false;
 }
 
+bool GameMap::burnBridge(Vector<int> pos)
+{
+  bool burned = false;
+  Bridge *bridge = GameMap::getBridge(pos);
+  if (bridge)
+    {
+      Bridge *other = Bridgelist::getInstance()->getOtherSide(bridge);
+      Vector<int> src = bridge->getPos();
+      GameMap::getInstance()->removeBridge(src);
+      Vector<int> dest = Vector<int>(-1, -1);
+      std::list<Stack*> stacks;
+      if (other)
+        {
+          dest = other->getPos();
+          GameMap::getInstance()->removeBridge(dest);
+          std::list<Stack*> s = GameMap::getStacks(src)->getStacks();
+          stacks.merge(s);
+        }
+      std::list<Stack*> s = 
+        GameMap::getFriendlyStacks(src, Playerlist::getActiveplayer());
+      stacks.merge(s);
+      for (std::list<Stack*>::iterator i = stacks.begin(); 
+           i != stacks.end(); i++)
+        {
+          (*i)->setDefending(false);
+          (*i)->setParked(false);
+          (*i)->clearPath();
+          (*i)->drainMovement();
+        }
+      std::list<Vector<int> > r = 
+        Bridgelist::getInstance()->getRoadEntryPoints(bridge);
+      for (std::list<Vector<int> >::iterator i = r.begin(); i != r.end(); i++)
+        {
+          Road *rd = GameMap::getInstance()->getRoad(*i);
+          if (rd)
+            rd->setType(Roadlist::getInstance()->calculateType(rd->getPos()));
+        }
+      burned = true;
+    }
+  return burned;
+}
+
+bool GameMap::friendlyCitiesPresent()
+{
+  return Citylist::getInstance()->countCities(Playerlist::getActiveplayer());
+}
+
+bool GameMap::enemyCitiesPresent()
+{
+  Playerlist *plist = Playerlist::getInstance();
+  for (Playerlist::iterator i = plist->begin(); i != plist->end(); i++)
+    {
+      if ((*i) == plist->getNeutral())
+        continue;
+      if ((*i) == plist->getActiveplayer())
+        continue;
+      if ((*i)->isDead())
+        continue;
+      if (Citylist::getInstance()->countCities(*i) > 0)
+        return true;
+    }
+  return false;
+}
+
+bool GameMap::neutralCitiesPresent()
+{
+  return Citylist::getInstance()->countCities
+    (Playerlist::getInstance()->getNeutral());
+}
+
+void GameMap::addArmies(const ArmyProto *a, guint32 num_allies, Vector<int> pos)
+{
+  for (unsigned int i = 0; i < num_allies; i++)
+    {
+      Army *army = new Army(*a, Playerlist::getActiveplayer());
+      if (army)
+        addArmy(pos, army);
+    }
+}
+      
+//we can't defend on cities, ruins, temples, ports, or water.
+bool GameMap::can_defend(Stack *stack)
+{
+  Tile::Type type = getInstance()->getTile(stack->getPos())->getType();
+  Maptile::Building building = getInstance()->getBuilding(stack->getPos());
+  if (type == Tile::WATER && building == Maptile::NONE)
+    return false;
+  if (building == Maptile::CITY || building == Maptile::RUIN ||
+      building == Maptile::TEMPLE || building == Maptile::PORT)
+    return false;
+  return true;
+}
+        
+bool GameMap::checkBuildingTerrain(Maptile::Building b, bool land)
+{
+  bool found = false;
+  GameMap *gm = GameMap::getInstance();
+  for (int i = 0; i < s_width; i++)
+    {
+      for (int j = 0; j < s_height; j++)
+        {
+          if (gm->getBuilding(Vector<int>(i, j)) == b)
+            {
+              if (land)
+                {
+                  if (gm->getTerrainType(Vector<int>(i, j)) != Tile::WATER)
+                    found = true;
+                }
+              else
+                {
+                  if (gm->getTerrainType(Vector<int>(i, j)) == Tile::WATER)
+                    found = true;
+                }
+              if (found)
+                break;
+            }
+        }
+    }
+
+  return found;
+}
+        
+City* GameMap::getCity(Movable *m)
+{
+  return getCity(m->getPos());
+}
+        
+Ruin* GameMap::getRuin(Movable *m)
+{
+  return getRuin(m->getPos());
+}
+        
+Temple* GameMap::getTemple(Movable *m)
+{
+  return getTemple(m->getPos());
+}
+        
+Signpost* GameMap::getSignpost(Movable *m)
+{
+  return getSignpost(m->getPos());
+}
