@@ -3,7 +3,7 @@
 // Copyright (C) 2004 David Barnsdale
 // Copyright (C) 2003 Michael Bartl
 // Copyright (C) 2004, 2005 Andrea Paternesi
-// Copyright (C) 2006, 2007, 2008, 2009, 2010 Ben Asselstine
+// Copyright (C) 2006, 2007, 2008, 2009, 2010, 2014, 2015 Ben Asselstine
 // Copyright (C) 2008 Janek Kozicki
 //
 //  This program is free software; you can redistribute it and/or modify
@@ -21,16 +21,12 @@
 //  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 
 //  02110-1301, USA.
 
-#include <stdlib.h>
 #include <iostream>
 #include <math.h>  
-#include <algorithm>
 #include <set>
-#include <string.h>
-
-//#include <boost/foreach.hpp>
 
 #include "MapGenerator.h"
+#include "army.h"
 #include "GameMap.h"
 #include "stack.h"
 #include "path.h"
@@ -49,18 +45,16 @@
 #include "bridge.h"
 #include "armysetlist.h"
 #include "tilesetlist.h"
-#include "army.h"
 #include "vector.h"
 #include "RoadPathCalculator.h"
 #include "cityset.h"
 #include "overviewmap.h"
+#include "rnd.h"
 
-#define debug(x) {std::cerr<<__FILE__<<": "<<__LINE__<<": "<<x<<std::endl<<std::flush;}
-//#define debug(x)
+//#define debug(x) {std::cerr<<__FILE__<<": "<<__LINE__<<": "<<x<<std::endl<<std::flush;}
+#define debug(x)
 #define offmap(bx,by) (by<0)||(by>=d_height)||(bx<0)||(bx>=d_width)
 
-using namespace std;
-
 //-------------------------------------------------------------------
 
 MapGenerator::MapGenerator()
@@ -84,7 +78,7 @@ MapGenerator::~MapGenerator()
 
 int MapGenerator::setNoCities(int nocities)
 {
-    if (nocities <= 0)
+    if (nocities < 0)
         return -1;
 
     int tmp = d_nocities;
@@ -139,6 +133,20 @@ void MapGenerator::setPercentages(int pwater, int pforest, int pswamp,
     d_pmountains = pmountains;
 }
 
+void MapGenerator::cleanupRoads()
+{
+  /*
+   * in rare cases, we can make roads on water.  this routine gets rid of them.
+   */
+  for (int i = 0; i < d_height; i++)
+    for (int j = 0; j < d_width; j++)
+      {
+        if (d_building[i*d_width + j] == Maptile::ROAD &&
+            d_terrain[i*d_width + j] == Tile::WATER)
+          d_building[i*d_width + j] = Maptile::NONE;
+      }
+}
+
 /** 
  * Generates a random map . The map is stored as a char array of size 
  * 100x100. Each character stands for something. We use :
@@ -172,7 +180,7 @@ void MapGenerator::makeMap(int width, int height, bool roads)
     debug("Making random map:");
    
     // create the terrain
-    debug("flatening plains");
+    debug("flattening plains");
     progress.emit(.090, _("flattening plains..."));
     makePlains();
     debug("raining water");
@@ -226,15 +234,14 @@ void MapGenerator::makeMap(int width, int height, bool roads)
         progress.emit(.950, _("building bridges..."));
         makeBridges();
       }
+    cleanupRoads();
     debug("raising signs");
     progress.emit(.990, _("raising signs..."));
     makeBuildings(Maptile::SIGNPOST,d_nosignposts);
 
     debug("Done making map.");
-
-//    printMap();
 }
-        
+
 #define  NORTH_SOUTH_BRIDGE 1
 #define  EAST_WEST_BRIDGE 2
 
@@ -286,9 +293,9 @@ bool MapGenerator::canPlaceBridge(Vector<int> pos, int type, Vector<int> &src, V
 
 void MapGenerator::makeBridges()
 {
-  std::string orig_tileset = GameMap::getInstance()->getTilesetName();
-  std::string orig_shieldset = GameMap::getInstance()->getShieldsetName();
-  std::string orig_cityset = GameMap::getInstance()->getCitysetName();
+  Glib::ustring orig_tileset = GameMap::getInstance()->getTilesetBaseName();
+  Glib::ustring orig_shieldset = GameMap::getInstance()->getShieldsetBaseName();
+  Glib::ustring orig_cityset = GameMap::getInstance()->getCitysetBaseName();
   GameMap::deleteInstance();
   Citylist::deleteInstance();
   Roadlist::deleteInstance();
@@ -329,9 +336,9 @@ void MapGenerator::makeBridges()
   GameMap::getInstance()->calculateBlockedAvenues();
 
   Vector<int> src, dest;
-  std::vector<pair<int , Vector<int> > >  bridges;
+  std::vector<std::pair<int , Vector<int> > >  bridges;
   bridges = findBridgePlaces();
-  for (std::vector<pair<int, Vector<int> > >::iterator it = bridges.begin();
+  for (std::vector<std::pair<int, Vector<int> > >::iterator it = bridges.begin();
        it != bridges.end(); it++)
     {
       Vector<int> pos = (*it).second + Vector<int>(1,1);
@@ -393,7 +400,6 @@ void MapGenerator::printMap(int j, int i)
         case Tile::FOREST  :  ch=adom_convention ? '&' : '$';break; // forest
         case Tile::GRASS   :  ch=adom_convention ? '.' : '.';break; // plains
         case Tile::SWAMP   :  ch=adom_convention ? '"' : '_';break; // swamps
-        case Tile::VOID    :  ch=adom_convention ? '?' : '?';break;
 
             // cannot print those, actually because they don't exist in Tile::Type
             //     ch='C';break; // city/castle
@@ -444,15 +450,15 @@ void MapGenerator::connectWithWater(Vector<int> from, Vector<int> to)
         // we don't want to mess up whole map with straight lines
         return;
 
-    int kind(rand()%4);
+    int kind(Rnd::rand()%4);
     delta /= length(delta)*2;
     for(Vector<float>path = Vector<float>(from)+delta*4 ; dist<float>(path,Vector<float>(to)-delta*4) > 0.5 ; path -= delta)
     {
         int j = (int)(path.x);
         int i = (int)(path.y);
 
-        if(rand()%3 == 0) 
-            kind = rand()%4;
+        if(Rnd::rand()%3 == 0) 
+            kind = Rnd::rand()%4;
         switch(kind)
         {
             case 0:
@@ -586,7 +592,7 @@ void MapGenerator::verifyIslands()
     // largest are good. Also one/third of all others is good:
     std::set<int> good(largest);
     for(size_t i=0 ; i<counts.size() ; ++i)
-        if(rand()%3 == 0) // that's one/third here
+        if(Rnd::rand()%3 == 0) // that's one/third here
             good.insert(counts[i]);
 
     // now, eliminate all land that is not good
@@ -604,7 +610,7 @@ void MapGenerator::makeRivers()
     //  1 - plenty of short rivers and islands
     //  2 - longer rivers, less islands
     //  3 - even longer rivers, even less islands
-    int river_style=rand()%3+1;
+    int river_style=Rnd::rand()%3+1;
 
     // count how many separate bodies of water were found
     int how_many;
@@ -650,8 +656,6 @@ void MapGenerator::makeRivers()
                 if(box[J][I] != h)
                 // center doesn't necessarily fall on water tile, so fix this.
                 {
-                    //      // for debugging...
-                    //      box[J][I]+=5000;
                     int i_up=0,i_dn=0,j_up=0,j_dn=0;
                     while((I+i_up <  d_width-1 ) && (box[J     ][I+i_up] != h)) ++i_up;
                     while((I-i_dn >  0         ) && (box[J     ][I-i_dn] != h)) ++i_dn;
@@ -679,8 +683,6 @@ void MapGenerator::makeRivers()
                                     //MessageBox("Message from author: this is algorithmically a very interesting map, please make screenshot and send to cosurgi@gmail.com");
                                 }
                 }
-                //      // for debugging...
-                //      box[(int)(centers[h].x)][(int)(centers[h].y)]+=4000;
             }
         }
 
@@ -695,7 +697,7 @@ void MapGenerator::makeRivers()
         }
 
         // find shortest distances between areas
-        std::vector<std::vector<std::pair<float, std::pair<Vector<int>, Vector<int> > > > > distances; // I would prefer boost::tuple, but oh well...
+        std::vector<std::vector<std::pair<float, std::pair<Vector<int>, Vector<int> > > > > distances;
         distances.resize(how_many+2);
         for(int h = 0; h < how_many+2; ++h)
         {
@@ -780,42 +782,9 @@ void MapGenerator::makeRivers()
                 distances[closest_h][closest_k].first = d_height*d_height;
             }
         }
-        //          // for debugging...   print whole box
-        //          std::cerr << how_many << " separate bodies of water found.\n";
-        //          std::cerr << the_biggest_area << " is the biggest\n";
-        //          std::cerr << second_biggest_area << " is second in size\n";
-        //          std::vector<int> a;
-        //          BOOST_FOREACH(a,box)
-        //          {
-        //              BOOST_FOREACH(int i,a)
-        //              {
-        //                  if(i<4000)
-        //                      std::cout << i << " ";
-        //                  else
-        //                      if(i<5000)
-        //                      std::cout << "X" << " ";
-        //                      else
-        //                      if(i<7000)
-        //                      std::cout << "%" << " ";
-        //                      else
-        //                      if(i<14000)
-        //                      std::cout << "!" << " ";
-        //                      else
-        //                      std::cout << "|" << " ";
-        //              }
-        //              std::cout << "\n";
-        //          }
-        //          std::cout << "\n";
-
-    };
-    //if(how_many>1)
-        //std::cout << "There are " << how_many << (how_many<4?(std::string(" seas")):(std::string(" lakes"))) << " on this map.\n";
-    //else
-        //std::cout << "There is 1 sea on this map.\n";
-    //std::cout << "River style was: " << river_style << "\n";
+    }
 }
 
-
 /**
  * Makes Terrains.
  * The algorithm is as follows :
@@ -835,6 +804,7 @@ void MapGenerator::makeRivers()
  */
 void MapGenerator::makeTerrain(Tile::Type t, int percent, bool contin)
 {
+    int tries = 0;
     // calculate the total number of tiles for this terrain
     int terrain = d_width*d_height*percent / 100;
     int placed = 0;  // total of current terrain placed so far 
@@ -842,10 +812,15 @@ void MapGenerator::makeTerrain(Tile::Type t, int percent, bool contin)
     while(placed != terrain)
     {
         // find a random starting position
-        int x = rand() % d_width;
-        int y = rand() % d_height;
+        int x = Rnd::rand() % d_width;
+        int y = Rnd::rand() % d_height;
         if (seekPlain(x, y) == false)
+        {
+          tries++;
+          if (tries > 30000)
+            break;
           continue;
+        }
 
 
         // now go on until we hit a dead end
@@ -861,7 +836,7 @@ void MapGenerator::makeTerrain(Tile::Type t, int percent, bool contin)
             
             // from a random direction, check all directions for further progress
             int loop = 0;
-            for (int dir = rand()%8; loop < 8; loop++, dir = (dir+1)%8)
+            for (int dir = Rnd::rand()%8; loop < 8; loop++, dir = (dir+1)%8)
             {
                 int tmpx = x + d_xdir[dir];
                 int tmpy = y + d_ydir[dir];
@@ -924,11 +899,11 @@ void MapGenerator::makeStreamer(Tile::Type t, int percent, int thick)
     while(placed < terrain)
     {
         // find a random starting position
-        int x = rand() % d_width;
-        int y = rand() % d_height;
+        int x = Rnd::rand() % d_width;
+        int y = Rnd::rand() % d_height;
         if (seekPlain(x, y) == false)
           continue;
-        dir = rand()%8; // pick a random direction
+        dir = Rnd::rand()%8; // pick a random direction
         // now go on until we hit a dead end
         while (placed < terrain)
         {
@@ -940,9 +915,9 @@ void MapGenerator::makeStreamer(Tile::Type t, int percent, int thick)
                 continue;
             }
             
-            if (rand() % 2 == 0)
+            if (Rnd::rand() % 2 == 0)
               {
-                if (rand() % 2 == 0)
+                if (Rnd::rand() % 2 == 0)
                   {
                     dir++;
                     if (dir > 7)
@@ -995,10 +970,8 @@ void MapGenerator::makeStreamer(Tile::Type t, int percent, int thick)
                       break;
                   }
             }
-
         }
     }
-
 }
 
 bool MapGenerator::seekPlain(int& x, int& y)
@@ -1022,7 +995,7 @@ bool MapGenerator::seekPlain(int& x, int& y)
     
     // fill the list with initial values; the rand is there to avoid a bias
     // (i.e. prefer a certain direction)
-    for (int dir = rand() % 8, i = 0; i < 8; i++, dir = (dir+1)%8)
+    for (int dir = Rnd::rand() % 8, i = 0; i < 8; i++, dir = (dir+1)%8)
         tiles.push_back(Vector<int>(x + d_xdir[dir], y + d_ydir[dir]));
     
     // now loop until all tiles were checked (should hardly happen)
@@ -1113,8 +1086,8 @@ void MapGenerator::makeCities(int cities)
     // place the cities
     while(city_count < cities)
     {
-        int x = rand()%(d_width-2);
-        int y = rand()%(d_height-2);
+        int x = Rnd::rand()%(d_width-2);
+        int y = Rnd::rand()%(d_height-2);
         if (inhospitableTerrain(x, y, cityset->getCityTileWidth()) && (iterations < 1000))
         {
             iterations++;
@@ -1193,8 +1166,8 @@ void MapGenerator::makeBuildings(Maptile::Building b, int building)
     {        
 	for (j = 0; j < iterations; j++)
 	{
-             x = rand()%d_width;
-             y = rand()%d_height;
+             x = Rnd::rand()%d_width;
+             y = Rnd::rand()%d_height;
         
 	     found_place = true;
 	     for (unsigned int k = 0; k < width; k++)
@@ -1281,7 +1254,7 @@ void MapGenerator::normalize()
     Tile::Type curTer=Tile::NONE, ajTer=Tile::NONE;
 
     // that was 40 before. Now with rivers, the smaller the value - the more connected rivers we got.
-    int center_tiles = rand()%40;
+    int center_tiles = Rnd::rand()%40;
     //std::cerr << center_tiles << "\% chance of disconnecting rivers.\n";
 
     // Go through every tile bar the outer edge
@@ -1313,20 +1286,20 @@ void MapGenerator::normalize()
             {
                 if (ajacentTer[curTer]==0)
                     d_terrain[globy*d_width +globx] = Tile::GRASS;
-                else if ((ajacentTer[curTer]==1) && (rand()%100 < 95 ))
+                else if ((ajacentTer[curTer]==1) && (Rnd::rand()%100 < 95 ))
                     d_terrain[globy*d_width +globx] = Tile::GRASS;
-                else if ((ajacentTer[curTer]==2) && (rand()%100 < 70 ))
+                else if ((ajacentTer[curTer]==2) && (Rnd::rand()%100 < 70 ))
                     d_terrain[globy*d_width +globx] = Tile::GRASS;
-                else if ((ajacentTer[curTer]==3) && (rand()%100 < center_tiles ))
+                else if ((ajacentTer[curTer]==3) && (Rnd::rand()%100 < center_tiles ))
                     d_terrain[globy*d_width +globx] = Tile::GRASS;
             }
             else 
             {
                 if (ajacentTer[Tile::WATER]==8)
                     d_terrain[globy*d_width +globx] = Tile::WATER;
-                else if ((ajacentTer[Tile::WATER]==7) && (rand()%100 < 70 ))
+                else if ((ajacentTer[Tile::WATER]==7) && (Rnd::rand()%100 < 70 ))
                     d_terrain[globy*d_width +globx] = Tile::WATER;
-                else if ((ajacentTer[Tile::WATER]==6) && (rand()%100 < 40 ))
+                else if ((ajacentTer[Tile::WATER]==6) && (Rnd::rand()%100 < 40 ))
                     d_terrain[globy*d_width +globx] = Tile::WATER;
              }
         }
@@ -1470,7 +1443,6 @@ bool MapGenerator::makeAccessible(int src_x, int src_y, int dest_x, int dest_y)
 {
   bool retval = true;
   GameMap *gm = GameMap::getInstance();
-  Tileset *ts = Tilesetlist::getInstance()->getTileset(gm->getTilesetName());
   Vector<int> src(src_x, src_y);
   Vector<int> dest(dest_x, dest_y);
 
@@ -1487,9 +1459,9 @@ bool MapGenerator::makeAccessible(int src_x, int src_y, int dest_x, int dest_y)
     {
       s.clear();
       delete a;
-      ArmyProto *basearmy = ArmyProto::createBat();
-      a = Army::createNonUniqueArmy(*basearmy);
-      delete basearmy;
+      ArmyProto *bat = ArmyProto::createBat();
+      a = Army::createNonUniqueArmy(*bat);
+      delete bat;
       s.push_back(a);
       moves = p->calculate(&s, dest, false);
     }
@@ -1508,7 +1480,7 @@ bool MapGenerator::makeAccessible(int src_x, int src_y, int dest_x, int dest_y)
 	  if (d_terrain[y*d_width + x] == Tile::MOUNTAIN)
 	    {
 	      d_terrain[y*d_width +x] = Tile::HILLS;
-	      Maptile *t = new Maptile(ts, x, y, Tile::HILLS, NULL);
+	      Maptile *t = new Maptile(x, y, Tile::HILLS);
 	      gm->setTile(x, y, t);
 	      calculateBlockedAvenue(x, y);
 	    }
@@ -1546,9 +1518,9 @@ bool MapGenerator::makeAccessible(int src_x, int src_y, int dest_x, int dest_y)
   return retval;
 }
 
-std::vector<pair<int , Vector<int> > > MapGenerator::findBridgePlaces()
+std::vector<std::pair<int , Vector<int> > > MapGenerator::findBridgePlaces()
 {
-    std::vector<pair<int , Vector<int> > > result;
+    std::vector<std::pair<int , Vector<int> > > result;
     result.clear();
 
     for(int j = 1; j < d_height-5; j++)
@@ -1631,7 +1603,7 @@ std::vector<pair<int , Vector<int> > > MapGenerator::findBridgePlaces()
         for(size_t s = r+1; s<result.size() ; ++s)
             if(dist(Vector<float>(result[r].second),Vector<float>(result[s].second)) < 4.5)
                 bad.insert(r);
-    std::vector<pair<int , Vector<int> > > filter;filter.clear();
+    std::vector<std::pair<int , Vector<int> > > filter;filter.clear();
     for(size_t r = 0; r<result.size() ; ++r)
         if(bad.find(r) == bad.end())
             filter.push_back(result[r]);
@@ -1642,9 +1614,9 @@ std::vector<pair<int , Vector<int> > > MapGenerator::findBridgePlaces()
 
 void MapGenerator::makeRoads()
 {
-  std::string orig_tileset = GameMap::getInstance()->getTilesetName();
-  std::string orig_shieldset = GameMap::getInstance()->getShieldsetName();
-  std::string orig_cityset = GameMap::getInstance()->getCitysetName();
+  Glib::ustring orig_tileset = GameMap::getInstance()->getTilesetBaseName();
+  Glib::ustring orig_shieldset = GameMap::getInstance()->getShieldsetBaseName();
+  Glib::ustring orig_cityset = GameMap::getInstance()->getCitysetBaseName();
   GameMap::deleteInstance();
   Citylist::deleteInstance();
   Roadlist::deleteInstance();
@@ -1672,7 +1644,7 @@ void MapGenerator::makeRoads()
   Citylist *cl = Citylist::getInstance();
   for (Citylist::iterator it = cl->begin(); it != cl->end(); it++)
     {
-      if (rand() % 2 == 0)
+      if (Rnd::rand() % 2 == 0)
 	continue;
       City *c = cl->getNearestCityPast((*it)->getPos(), 13);
       Vector<int> dest = c->getPos();
@@ -1828,11 +1800,11 @@ void MapGenerator::rescueLoneTiles(Tile::Type FIND_THIS, Tile::Type REPLACE, boo
                     if ( box[0][2] && !box[1][2] &&  box[2][2] &&  
                          box[0][1] &&  box[1][1] &&  box[2][1] && 
                          box[0][0] && !box[1][0] &&  box[2][0])
-                            d_terrain[j*d_width + i+(rand()%2?+1:-1)] = FIND_THIS;
+                            d_terrain[j*d_width + i+(Rnd::rand()%2?+1:-1)] = FIND_THIS;
                     if ( box[0][2] &&  box[1][2] &&  box[2][2] &&  
                         !box[0][1] &&  box[1][1] && !box[2][1] && 
                          box[0][0] &&  box[1][0] &&  box[2][0])
-                            d_terrain[(j+(rand()%2?+1:-1))*d_width + i] = FIND_THIS;
+                            d_terrain[(j+(Rnd::rand()%2?+1:-1))*d_width + i] = FIND_THIS;
 
                     if ( box[0][2] && !box[1][2] && !box[2][2] &&  
                          box[0][1] &&  box[1][1] && !box[2][1] && 
