@@ -19,8 +19,8 @@
 
 #include "xmlhelper.h"
 
-NamedLocation::NamedLocation(Vector<int> pos, guint32 size, std::string name,
-			     std::string desc)
+NamedLocation::NamedLocation(Vector<int> pos, guint32 size, Glib::ustring name,
+			     Glib::ustring desc)
   :Location(pos, size), Namable(name), d_description(desc)
 {
 }
@@ -40,8 +40,3 @@ NamedLocation::NamedLocation(XML_Helper* helper, guint32 size)
 {
   helper->getData(d_description, "description");
 }
-
-NamedLocation::~NamedLocation()
-{
-}
-
