@@ -1,5 +1,5 @@
 // Copyright (C) 2003, 2004, 2005, 2006 Ulf Lorenz
-// Copyright (C) 2007, 2008 Ben Asselstine
+// Copyright (C) 2007, 2008, 2014 Ben Asselstine
 // Copyright (C) 2007, 2008 Ole Laursen
 //
 //  This program is free software; you can redistribute it and/or modify
@@ -22,9 +22,8 @@
 
 #include <sigc++/trackable.h>
 #include <sigc++/connection.h>
-#include "playerlist.h"
-#include "hero.h"
 
+class Player;
 /**
    \brief The class to pass turns around the players
  
@@ -52,7 +51,7 @@ class NextTurn: public sigc::trackable
         /**
            \brief destructor
          */
-        ~NextTurn();
+        ~NextTurn() {};
         
         /**
            \brief start a new game
