@@ -1,4 +1,4 @@
-// Copyright (C) 2009, 2010 Ben Asselstine
+// Copyright (C) 2009, 2010, 2014, 2015 Ben Asselstine
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -19,6 +19,7 @@
 #include <string.h>
 #include <queue>
 #include "PathCalculator.h"
+#include "army.h"
 #include "GameMap.h"
 #include "path.h"
 #include "stack.h"
@@ -28,14 +29,13 @@
 #include "armysetlist.h"
 #include "armyprodbase.h"
 
-#define ENEMY_CITY_WEIGHT 10
-#define ENEMY_STACK_WEIGHT 10
-using namespace std;
-//#define debug(x) {cerr<<__FILE__<<": "<<__LINE__<<": "<<x<<flush<<endl;}
+//#define debug(x) {std::cerr<<__FILE__<<": "<<__LINE__<<": "<<x<<std::flush<<std::endl;}
 #define debug(x)
 
 void PathCalculator::populateNodeMap()
 {
+  load_unload_stack = Stack::createNonUniqueStack(stack->getOwner(), 
+                                                  stack->getPos());
   Vector<int> start = stack->getPos();
   int width = GameMap::getWidth();
   int height = GameMap::getHeight();
@@ -115,10 +115,10 @@ PathCalculator::PathCalculator(Player *p, Vector<int> src, const ArmyProdBase *p
   Army *army;
   if (!prodbase)
     {
-      ArmyProto *proto = Armysetlist::getInstance()->getScout(p->getArmyset());
-      if (!proto)
-	return;
+      //make a scout
+      ArmyProto *proto = ArmyProto::createScout();
       army = Army::createNonUniqueArmy (*proto, p);
+      delete proto;
     }
   else
     army = Army::createNonUniqueArmy (*prodbase, p);
@@ -154,7 +154,6 @@ PathCalculator::PathCalculator(const Stack &s, bool zig, int city_avoidance, int
   populateNodeMap();
 }
 
-
 PathCalculator::PathCalculator(const PathCalculator &p)
 :stack(new Stack(*p.stack)), flying(p.flying), d_bonus(p.d_bonus),
     land_reset_moves(p.land_reset_moves),
@@ -182,10 +181,6 @@ bool PathCalculator::calcFinalMoves(Vector<int> pos, Vector<int> next)
   printf("moves of source is %d\n", dxy);
   if (!flying && is_blocked_dir)
     return false;
-  //flyers can't go through the void
-  if (flying && is_blocked_dir &&
-      GameMap::getInstance()->getTile(next)->getType() == Tile::VOID)
-    return false;
 
   int dsxy = nodes[next.toIndex()].moves;
   printf("moves of dest is %d\n", dsxy);
@@ -238,17 +233,10 @@ bool PathCalculator::calcFinalMoves(Vector<int> pos, Vector<int> next)
 
 bool PathCalculator::calcMoves(Vector<int> pos, Vector<int> next)
 {
-  bool traversable = false;
-  int mp;
-  int dxy = nodes[pos.toIndex()].moves;   // always >= 0
   //am i blocked from entering sx,sy from pos?
   bool is_blocked_dir = isBlockedDir(pos, next);
   if (!flying && is_blocked_dir)
     return false;
-  //flyers can't go through the void
-  if (flying && is_blocked_dir &&
-      GameMap::getInstance()->getTile(next)->getType() == Tile::VOID)
-    return false;
 
   int dsxy = nodes[next.toIndex()].moves;
   if (dsxy < -1)
@@ -260,8 +248,9 @@ bool PathCalculator::calcMoves(Vector<int> pos, Vector<int> next)
       if (diff.x && diff.y)
 	return false;
     }
+  int dxy = nodes[pos.toIndex()].moves;   // always >= 0
   int newDsxy = dxy;
-  mp = pointsToMoveTo(pos, next);
+  int mp = pointsToMoveTo(pos, next);
   if (mp < 0)
     mp = 0;
   if (!flying && load_or_unload(pos, next, on_ship) == true)
@@ -284,9 +273,9 @@ bool PathCalculator::calcMoves(Vector<int> pos, Vector<int> next)
 	}
 
       // append the item to the queue
-      traversable = true;
+      return true;
     }
-  return traversable;
+  return false;
 }
 
 bool PathCalculator::calcFinalMoves(Vector<int> pos)
@@ -345,30 +334,30 @@ std::list<Vector<int> > PathCalculator::calcMoves(Vector<int> pos)
   return process;
 }
 
-bool PathCalculator::load_or_unload(Vector<int> src, Vector<int> dest, bool &on_ship)
+bool PathCalculator::load_or_unload(Vector<int> src, Vector<int> dest, bool &ship)
 {
-  Stack *new_stack = Stack::createNonUniqueStack(stack->getOwner(), src);
-  //do we load or unload if we step from SRC to DEST?
-  bool retval = new_stack->isMovingToOrFromAShip(dest, on_ship);
-  delete new_stack;
+  load_unload_stack->setPos(src);
+  bool retval = load_unload_stack->isMovingToOrFromAShip(dest, ship);
   return retval;
 }
+
 int PathCalculator::pointsToMoveTo(Vector<int> pos, Vector<int> next) const
 {
-  guint32 moves;
   const Maptile* tile = GameMap::getInstance()->getTile(next);
-
   if (pos == next) //probably shouldn't happen
     return 0;
 
-  moves = tile->getMoves();
+  guint32 moves = tile->getMoves();
 
   if (enemy_city_avoidance >= 1)
     {
-      //We will still try to avoid enemy cities a little.
-      City *enemy = GameMap::getEnemyCity(pos);
-      if (enemy && enemy->isBurnt() == false)
-	moves += enemy_city_avoidance;
+      if (GameMap::getInstance()->getBuilding(pos) == Maptile::CITY)
+        {
+          //We will still try to avoid enemy cities a little.
+          City *enemy = GameMap::getEnemyCity(pos);
+          if (enemy && enemy->isBurnt() == false)
+            moves += enemy_city_avoidance;
+        }
     }
   if (enemy_stack_avoidance >= 1)
     {
@@ -386,10 +375,13 @@ int PathCalculator::pointsToMoveTo(Vector<int> pos, Vector<int> next) const
 
 PathCalculator::~PathCalculator()
 {
+  if (load_unload_stack)
+    delete load_unload_stack;
   free (nodes);
   if (delete_stack)
     delete stack;
 }
+
 //am i blocked from entering destx,desty from x,y when i'm not flying?
 bool PathCalculator::isBlockedDir(Vector<int> pos, Vector<int> next)
 {
@@ -397,26 +389,13 @@ bool PathCalculator::isBlockedDir(Vector<int> pos, Vector<int> next)
   int diffy = next.y - pos.y;
   if (diffx >= -1 && diffx <= 1 && diffy >= -1 && diffy <= 1) 
     {
-      int idx = 0;
-      if (diffx == -1 && diffy == -1)
-	idx = 0;
-      else if (diffx == -1 && diffy == 0)
-	idx = 1;
-      else if (diffx == -1 && diffy == 1)
-	idx = 2;
-      else if (diffx == 0 && diffy == 1)
-	idx = 3;
-      else if (diffx == 0 && diffy == -1)
-	idx = 4;
-      else if (diffx == 1 && diffy == -1)
-	idx = 5;
-      else if (diffx == 1 && diffy == 0)
-	idx = 6;
-      else if (diffx == 1 && diffy == 1)
-	idx = 7;
-      else
-	return false;
-      return GameMap::getInstance()->getTile(pos)->d_blocked[idx];
+      int idxs[3][3] =
+        {
+            { 0, 1, 2 },
+            { 4, 0, 3 },
+            { 5, 6, 7 },
+        };
+      return GameMap::getInstance()->getTile(pos)->d_blocked[idxs[diffx+1][diffy+1]];
     }
 
   return false;
@@ -455,29 +434,11 @@ bool PathCalculator::isBlocked(Vector<int> pos)
   return isBlocked(stack, pos, enemy_city_avoidance < 0, enemy_stack_avoidance < 0);
 }
 
-bool PathCalculator::canMoveThere(Vector<int> dest)
-{
-  Vector<int> pos = stack->getPos();
-  if (flying && isBlockedDir(pos, dest) &&
-      GameMap::getInstance()->getTile(dest)->getType() == Tile::VOID)
-    return false;
-  if (isBlocked(pos))
-    {
-      //psst. if it's our last step we can step into cities.
-      //psst. if it's our last step we can step onto enemy stacks
-      //psst.  if it's our last step and we're a computer player who was diligently walking around friendly stacks, we can merge on our last step
-      return false;
-    }
-  if (isBlockedDir(pos, dest) && !flying)
-    return false;
-  return true;
-}
-
-int PathCalculator::calculate(Vector<int> dest, bool zigzag)
+int PathCalculator::calculate(Vector<int> dest, bool zig)
 {
   int retval = 0;
   guint32 moves = 0, turns = 0, left = 0;
-  Path *p = calculate(dest, moves, turns, left, zigzag);
+  Path *p = calculate(dest, moves, turns, left, zig);
   if (p->size() == 0)
     retval = -1;
   delete p;
@@ -486,7 +447,7 @@ int PathCalculator::calculate(Vector<int> dest, bool zigzag)
   return retval;
 }
 
-Path* PathCalculator::calculate(Vector<int> dest, guint32 &moves, guint32 &turns, guint32 &left, bool zigzag)
+Path* PathCalculator::calculate(Vector<int> dest, guint32 &moves, guint32 &turns, guint32 &left, bool zig)
 {
   Path *path = new Path();
   int width = GameMap::getWidth();
@@ -531,7 +492,7 @@ Path* PathCalculator::calculate(Vector<int> dest, guint32 &moves, guint32 &turns
   // choose the order in which we process directions so as to favour
   // diagonals over straight lines
   std::list<Vector<int> > diffs;
-  if (zigzag)
+  if (zig)
     {
       diffs.push_back(Vector<int>(-1, -1));
       diffs.push_back(Vector<int>(-1, 1));
@@ -584,9 +545,9 @@ Path* PathCalculator::calculate(Vector<int> dest, guint32 &moves, guint32 &turns
   guint32 moves_left = stack->getMoves();
   for (Path::iterator it = path->begin(); it != path->end(); it++)
     {
-      guint32 moves = stack->calculateTileMovementCost(*it);
-      if (moves_left >= moves)
-	moves_left -= moves;
+      guint32 tile_moves = stack->calculateTileMovementCost(*it);
+      if (moves_left >= tile_moves)
+	moves_left -= tile_moves;
       else
 	break;
       pathcount++;
@@ -603,6 +564,7 @@ Path* PathCalculator::calculate(Vector<int> dest, guint32 &moves, guint32 &turns
   nodes[idx] = orig_dest;
   return path;
 }
+
 void PathCalculator::dumpNodeMap(Vector<int> dest)
 {
   int width = GameMap::getWidth();
@@ -628,30 +590,14 @@ void PathCalculator::dumpNodeMap(Vector<int> dest)
       }
   printf ("=======================================\n");
   return;
-} 
-bool PathCalculator::compareNodeMaps(void *map)
-{
-  bool found = false;
-  int width = GameMap::getWidth();
-  int height = GameMap::getHeight();
-  struct node * othernodes = (struct node *) map;
-  for (int i = 0; i < width; i++)
-    for (int j = 0; j < height; j++)
-      {
-	if (othernodes[j*width+i].moves != nodes[j*width+i].moves)
-	  {
-	  found = true;
-	  break;
-	  }
-      }
-  return found;
 }
+
 bool PathCalculator::isReachable(Vector<int> pos)
 {
   return nodes[pos.toIndex()].moves >= 0;
 }
 
-Path *PathCalculator::calculateToCity (City *c, guint32 &moves, guint32 &turns, guint32 &left, bool zigzag)
+Path *PathCalculator::calculateToCity (City *c, guint32 &moves, guint32 &turns, guint32 &left, bool zig)
 {
   int min_dist = -1;
   Vector<int> shortest = c->getPos();
@@ -676,7 +622,7 @@ Path *PathCalculator::calculateToCity (City *c, guint32 &moves, guint32 &turns,
 	      }
 	  }
       }
-  Path *p = calculate(shortest, moves, turns, left, zigzag);
+  Path *p = calculate(shortest, moves, turns, left, zig);
   if (p->size() > 0)
     return p;
   delete p;
@@ -692,8 +638,7 @@ Path *PathCalculator::calculateToCity (City *c, guint32 &moves, guint32 &turns,
 	    if (other_stack && GameMap::canJoin(stack, other_stack) == false)
 	      continue;
 	  }
-	p = calculate(c->getPos() + Vector<int>(i,j), moves, turns, left,
-                      zigzag);
+	p = calculate(c->getPos() + Vector<int>(i,j), moves, turns, left, zig);
 	int dist = (int) moves;
 	delete p;
 	if (dist > 0)
@@ -705,7 +650,7 @@ Path *PathCalculator::calculateToCity (City *c, guint32 &moves, guint32 &turns,
 	      }
 	  }
       }
-  return calculate(shortest, moves, turns, left, zigzag);
+  return calculate(shortest, moves, turns, left, zig);
 }
 
 std::list<Vector<int> > PathCalculator::getReachablePositions(int mp)
