@@ -1,4 +1,4 @@
-//  Copyright (C) 2007, 2008 Ben Asselstine
+//  Copyright (C) 2007, 2008, 2014, 2015 Ben Asselstine
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -20,17 +20,18 @@
 #include <assert.h>
 #include <sigc++/functors/mem_fun.h>
 #include "ucompose.hpp"
-
+#include "army.h"
+#include "city.h"
 #include "QCityRaze.h"
 #include "QuestsManager.h"
 #include "citylist.h"
 #include "playerlist.h"
 #include "stack.h"
 #include "xmlhelper.h"
+#include "hero.h"
+#include "rnd.h"
 
-using namespace std;
-
-//#define debug(x) {cerr<<__FILE__<<": "<<__LINE__<<": "<<x<<endl<<flush;}
+//#define debug(x) {std::cerr<<__FILE__<<": "<<__LINE__<<": "<<x<<std::endl<<std::flush;}
 #define debug(x)
 
 QuestCityRaze::QuestCityRaze (QuestsManager& mgr, guint32 hero) 
@@ -82,17 +83,17 @@ bool QuestCityRaze::save(XML_Helper* helper) const
   return retval;
 }
 
-std::string QuestCityRaze::getProgress() const
+Glib::ustring QuestCityRaze::getProgress() const
 {
   return _("You aren't afraid of doing it, are you?");
 }
 
-void QuestCityRaze::getSuccessMsg(std::queue<std::string>& msgs) const
+void QuestCityRaze::getSuccessMsg(std::queue<Glib::ustring>& msgs) const
 {
   msgs.push(_("The priests thank you for razing this evil place."));
 }
 
-void QuestCityRaze::getExpiredMsg(std::queue<std::string>& msgs) const
+void QuestCityRaze::getExpiredMsg(std::queue<Glib::ustring>& msgs) const
 {
   const City* c = getCity();
   msgs.push(String::ucompose
@@ -134,18 +135,25 @@ City* QuestCityRaze::chooseToRaze(Player *p)
   if (cities.empty())
     return 0;
 
-  return cities[rand() % cities.size()];
+  return cities[Rnd::rand() % cities.size()];
 }
 
 void QuestCityRaze::armyDied(Army *a, bool heroIsCulprit)
 {
-  ;
+  if (a || heroIsCulprit)
+    {
+      ;
+    }
   //this quest does nothing when an army dies
 }
 
 void QuestCityRaze::cityAction(City *c, CityDefeatedAction action, 
 			       bool heroIsCulprit, int gold)
 {
+  if (gold)
+    {
+      ;
+    }
   if (isPendingDeletion())
     return;
   Hero *h = getHero();
