@@ -1,4 +1,4 @@
-//  Copyright (C) 2007, 2008, 2009 Ben Asselstine
+//  Copyright (C) 2007, 2008, 2009, 2014, 2015 Ben Asselstine
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -19,20 +19,23 @@
 #include <sigc++/functors/mem_fun.h>
 #include "ucompose.hpp"
 
+#include "army.h"
 #include "QPillageGold.h"
 #include "QuestsManager.h"
 #include "playerlist.h"
+#include "city.h"
+#include "xmlhelper.h"
+#include "hero.h"
+#include "rnd.h"
 
-using namespace std;
-
-//#define debug(x) {cerr<<__FILE__<<": "<<__LINE__<<": "<<x<<endl<<flush;}
+//#define debug(x) {std::cerr<<__FILE__<<": "<<__LINE__<<": "<<x<<std::endl<<std::flush;}
 #define debug(x)
 
 QuestPillageGold::QuestPillageGold(QuestsManager& q_mgr, guint32 hero)
   : Quest(q_mgr, hero, Quest::PILLAGEGOLD), d_pillaged(0)
 {
   //pick an amount of gold to sack and pillage
-  d_to_pillage = 850 + (rand() % 630);
+  d_to_pillage = 850 + (Rnd::rand() % 630);
 
   initDescription();
 }
@@ -66,19 +69,23 @@ bool QuestPillageGold::save(XML_Helper *helper) const
   return retval;
 }
 
-std::string QuestPillageGold::getProgress() const
+Glib::ustring QuestPillageGold::getProgress() const
 {
   return String::ucompose(_("You have already stolen %1 gold pieces."), d_pillaged);
 }
 
-void QuestPillageGold::getSuccessMsg(std::queue<std::string>& msgs) const
+void QuestPillageGold::getSuccessMsg(std::queue<Glib::ustring>& msgs) const
 {
   msgs.push(String::ucompose(_("You have managed to sack and pillage %1 gold."), d_pillaged));
   msgs.push(_("Well done!"));
 }
 
-void QuestPillageGold::getExpiredMsg(std::queue<std::string>& msgs) const
+void QuestPillageGold::getExpiredMsg(std::queue<Glib::ustring>& msgs) const
 {
+  if (msgs.size())
+    {
+      ;
+    }
     // This quest should never expire, so this is just a dummy function
 }
 
@@ -89,13 +96,19 @@ void QuestPillageGold::initDescription()
 	
 void QuestPillageGold::armyDied(Army *a, bool heroIsCulprit)
 {
-  ;
-  //this quest does nothing when an army dies
+  if (a || heroIsCulprit)
+    {
+      ; //this quest does nothing when an army dies
+    }
 }
 
 void QuestPillageGold::cityAction(City *c, CityDefeatedAction action, 
 				  bool heroIsCulprit, int gold)
 {
+  if (c || gold)
+    {
+      ;
+    }
   if (isPendingDeletion())
     return;
   Hero *h = getHero();
