@@ -19,7 +19,7 @@
 
 #include "xmlhelper.h"
 
-Renamable::Renamable(std::string name)
+Renamable::Renamable(Glib::ustring name)
   :Named(name)
 {
 }
@@ -33,8 +33,3 @@ Renamable::Renamable(XML_Helper* helper)
   :Named(helper)
 {
 }
-
-Renamable::~Renamable()
-{
-}
-
