@@ -1,4 +1,4 @@
-// Copyright (C) 2009 Ben Asselstine
+// Copyright (C) 2009, 2014 Ben Asselstine
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -15,10 +15,10 @@
 //  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 
 //  02110-1301, USA.
 
+#include "SightMap.h"
 #include "Sage.h"
 #include "rewardlist.h"
-
-using namespace std;
+#include "rnd.h"
 
 Sage::Sage()
 {
@@ -35,10 +35,5 @@ Sage::Sage()
     }
   //this covers, the one-time rewards of items and maps
   //but now we put in gold too
-  push_back(new Reward_Gold(500 + rand() % 1000));
+  push_back(new Reward_Gold(500 + Rnd::rand() % 1000));
 }
-
-Sage::~Sage()
-{
-}
-
