@@ -1,6 +1,6 @@
 // Copyright (C) 2001, 2002, 2003 Michael Bartl
 // Copyright (C) 2002, 2003, 2004, 2005 Ulf Lorenz
-// Copyright (C) 2007, 2008, 2010, 2011 Ben Asselstine
+// Copyright (C) 2007, 2008, 2010, 2011, 2014, 2015 Ben Asselstine
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -20,15 +20,13 @@
 #include "Tile.h"
 #include "SmallTile.h"
 #include <iostream>
-#include <algorithm>
 #include "File.h"
 #include "tileset.h"
 #include "tarhelper.h"
+#include "xmlhelper.h"
+#include "rnd.h"
 
-std::string Tile::d_tag = "tile";
-
-using namespace std;
-
+Glib::ustring Tile::d_tag = "tile";
 
 Tile::Tile()
 {
@@ -37,13 +35,27 @@ Tile::Tile()
   d_smalltile = new SmallTile();
 }
 
+Tile::Tile(Tile::Type type, Glib::ustring name, guint32 moves, SmallTile*small)
+ : d_name(name), d_moves(moves), d_type(type), d_smalltile(small)
+{
+}
+
+Tile::Tile(const Tile &t)
+ : std::list<TileStyleSet*>(), d_name(t.d_name), d_moves(t.d_moves), 
+    d_type(t.d_type)
+{
+  d_smalltile = new SmallTile (*t.d_smalltile);
+  for (Tile::const_iterator i = t.begin(); i != t.end(); ++i)
+    push_back(new TileStyleSet(*(*i)));
+}
+
 Tile::Tile(XML_Helper* helper)
 {
     helper->getData(d_name, "name");
     helper->getData(d_moves, "moves");
-    std::string type_str;
+    Glib::ustring type_str;
     helper->getData(type_str, "type");
-    d_type = tileTypeFromString(type_str);
+    d_type = Tile::Type(tileTypeFromString(type_str));
 }
 
 bool Tile::save(XML_Helper *helper) const
@@ -53,7 +65,7 @@ bool Tile::save(XML_Helper *helper) const
   retval &= helper->openTag(d_tag);
   retval &= helper->saveData("name", d_name);
   retval &= helper->saveData("moves", d_moves);
-  std::string type_str = tileTypeToString(Tile::Type(d_type));
+  Glib::ustring type_str = tileTypeToString(Tile::Type(d_type));
   retval &= helper->saveData("type", type_str);
   retval &= d_smalltile->save(helper);
   for (Tile::const_iterator i = begin(); i != end(); ++i)
@@ -66,9 +78,11 @@ bool Tile::save(XML_Helper *helper) const
 Tile::~Tile()
 {
   for (iterator it = begin(); it != end(); it++)
-      delete *it;
+    delete *it;
+  clear();
   if (d_smalltile)
     delete d_smalltile;
+  d_smalltile = NULL;
 }
 
 void Tile::setTypeByIndex(int idx)
@@ -81,7 +95,6 @@ void Tile::setTypeByIndex(int idx)
     case 3: setType(HILLS); break;
     case 4: setType(MOUNTAIN); break;
     case 5: setType(SWAMP); break;
-    case 6: setType(VOID); break;
     }
 }
 
@@ -95,7 +108,6 @@ int Tile::getTypeIndexForType(Tile::Type type)
     case HILLS: return 3; break;
     case MOUNTAIN: return 4; break;
     case SWAMP: return 5; break;
-    case VOID: return 6; break;
     }
   return 0;
 }
@@ -116,49 +128,47 @@ TileStyle *Tile::getRandomTileStyle (TileStyle::Type style) const
 
   if (tilestyles.empty() == true)
     return NULL;
-  return tilestyles[rand() % tilestyles.size()];
+  return tilestyles[Rnd::rand() % tilestyles.size()];
 }
 
-std::string Tile::tileTypeToString(const Tile::Type type)
+Glib::ustring Tile::tileTypeToString(const Tile::Type type)
 {
   switch (type)
     {
-    case Tile::GRASS:
-      return "Tile::GRASS";
-    case Tile::WATER:
-      return "Tile::WATER";
-    case Tile::FOREST:
-      return "Tile::FOREST";
-    case Tile::HILLS:
-      return "Tile::HILLS";
-    case Tile::MOUNTAIN:
-      return "Tile::MOUNTAIN";
-    case Tile::SWAMP:
-      return "Tile::SWAMP";
-    case Tile::VOID:
-      return "Tile::VOID";
+    case Tile::GRASS: return "Tile::GRASS";
+    case Tile::WATER: return "Tile::WATER";
+    case Tile::FOREST: return "Tile::FOREST";
+    case Tile::HILLS: return "Tile::HILLS";
+    case Tile::MOUNTAIN: return "Tile::MOUNTAIN";
+    case Tile::SWAMP: return "Tile::SWAMP";
     }
   return "Tile::GRASS";
 }
 
-Tile::Type Tile::tileTypeFromString(const std::string str)
+Glib::ustring Tile::tileTypeToFriendlyName(const Tile::Type type)
+{
+  switch (type)
+    {
+    case Tile::GRASS: return _("Grass");
+    case Tile::WATER: return _("Water");
+    case Tile::FOREST: return _("Forest");
+    case Tile::HILLS: return _("Hills");
+    case Tile::MOUNTAIN: return _("Mountain");
+    case Tile::SWAMP: return _("Swamp");
+    }
+  return _("Grass");
+}
+
+guint32 Tile::tileTypeFromString(const Glib::ustring str)
 {
   if (str.size() > 0 && isdigit(str.c_str()[0]))
     return Tile::Type(atoi(str.c_str()));
-  if (str == "Tile::GRASS")
-    return Tile::GRASS;
-  else if (str == "Tile::WATER")
-    return Tile::WATER;
-  else if (str == "Tile::FOREST")
-    return Tile::FOREST;
-  else if (str == "Tile::HILLS")
-    return Tile::HILLS;
-  else if (str == "Tile::MOUNTAIN")
-    return Tile::MOUNTAIN;
-  else if (str == "Tile::SWAMP")
-    return Tile::SWAMP;
-  else if (str == "Tile::VOID")
-    return Tile::VOID;
+  if (str == "Tile::GRASS") return Tile::GRASS;
+  else if (str == "Tile::WATER") return Tile::WATER;
+  else if (str == "Tile::FOREST") return Tile::FOREST;
+  else if (str == "Tile::HILLS") return Tile::HILLS;
+  else if (str == "Tile::MOUNTAIN") return Tile::MOUNTAIN;
+  else if (str == "Tile::SWAMP") return Tile::SWAMP;
   return Tile::GRASS;
 }
 
@@ -194,9 +204,10 @@ bool Tile::consistsOnlyOfLoneAndOtherStyles() const
     (*i)->getUniqueTileStyleTypes(types);
       return validateGrass(types);
 }
+
 bool Tile::validate() const
 {
-  if (size() == 0)
+  if (empty())
     return false;
 
   for (Tile::const_iterator i = begin(); i != end(); ++i)
@@ -217,7 +228,7 @@ bool Tile::validate() const
 	return false;
       break;
     case Tile::FOREST: case Tile::WATER: case Tile::HILLS: 
-    case Tile::SWAMP: case Tile::VOID: case Tile::MOUNTAIN:
+    case Tile::SWAMP: case Tile::MOUNTAIN:
       if (validateFeature(types) == false)
 	{
 	  if (validateGrass(types) == false)
@@ -241,7 +252,7 @@ void Tile::instantiateImages(int tilesize, Tar_Helper *t, bool &broken)
   broken = false;
   for (iterator it = begin(); it != end(); it++)
     {
-      std::string file = "";
+      Glib::ustring file = "";
       if ((*it)->getName().empty() == false && !broken)
         {
           file = t->getFile((*it)->getName() + ".png", broken);
@@ -281,4 +292,40 @@ std::list<TileStyle*> Tile::getTileStyles(TileStyle::Type type) const
         styles.push_back((*j));
   return styles;
 }
+
+Tile* Tile::get_default_grass()
+{
+  return new Tile(Tile::GRASS, Tile::tileTypeToFriendlyName(Tile::GRASS), 2, 
+                  SmallTile::get_default_grass());
+}
+
+Tile* Tile::get_default_water()
+{
+  return new Tile(Tile::WATER, Tile::tileTypeToFriendlyName(Tile::WATER), 2, 
+                  SmallTile::get_default_water());
+}
+
+Tile* Tile::get_default_forest()
+{
+  return new Tile(Tile::FOREST, Tile::tileTypeToFriendlyName(Tile::FOREST), 3, 
+                  SmallTile::get_default_forest());
+}
+
+Tile* Tile::get_default_hills()
+{
+  return new Tile(Tile::HILLS, Tile::tileTypeToFriendlyName(Tile::HILLS), 4, 
+                  SmallTile::get_default_hills());
+}
+
+Tile* Tile::get_default_mountains()
+{
+  return new Tile(Tile::MOUNTAIN, Tile::tileTypeToFriendlyName(Tile::MOUNTAIN),
+                  6, SmallTile::get_default_mountains());
+}
+
+Tile* Tile::get_default_swamp()
+{
+  return new Tile(Tile::SWAMP, Tile::tileTypeToFriendlyName(Tile::SWAMP), 8, 
+                  SmallTile::get_default_swamp());
+}
 // End of file
