@@ -1,7 +1,7 @@
 // Copyright (C) 2001, 2002, 2003 Michael Bartl
 // Copyright (C) 2004, 2005 Ulf Lorenz
 // Copyright (C) 2004, 2006 Andrea Paternesi
-// Copyright (C) 2007, 2008, 2009 Ben Asselstine
+// Copyright (C) 2007, 2008, 2009, 2014, 2015 Ben Asselstine
 // Copyright (C) 2007 Ole Laursen
 //
 //  This program is free software; you can redistribute it and/or modify
@@ -23,8 +23,6 @@
 #define UNIQUELYIDENTIFIED_H
 
 #include <gtkmm.h>
-#include "vector.h"
-#include "rectangle.h"
 
 class XML_Helper;
 
@@ -45,7 +43,7 @@ class UniquelyIdentified
     UniquelyIdentified(XML_Helper* helper);
 
     //! Destructor.
-    virtual ~UniquelyIdentified();
+    virtual ~UniquelyIdentified() {};
 
 
     // Get Methods
@@ -62,6 +60,7 @@ class UniquelyIdentified
     //! Go get a new unique identifier for this object.
     void assignNewId();
         
+    bool isUnique() {return d_unique;}
  protected:
 
     //! A unique numeric identifier for an object in the game.
