@@ -1,7 +1,7 @@
 // Copyright (C) 2002, 2003, 2004, 2005 Ulf Lorenz
 // Copyright (C) 2003 Michael Bartl
 // Copyright (C) 2006 Andrea Paternesi
-// Copyright (C) 2007, 2008, 2009, 2010 Ben Asselstine
+// Copyright (C) 2007, 2008, 2009, 2010, 2014, 2015 Ben Asselstine
 // Copyright (C) 2007, 2008 Ole Laursen
 //
 //  This program is free software; you can redistribute it and/or modify
@@ -19,8 +19,6 @@
 //  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 
 //  02110-1301, USA.
 
-#include <stdlib.h>
-#include <algorithm>
 #include "ai_dummy.h"
 #include "playerlist.h"
 #include "armysetlist.h"
@@ -33,14 +31,14 @@
 #include "xmlhelper.h"
 #include "history.h"
 #include "GameScenarioOptions.h"
+#include "SightMap.h"
 #include "Sage.h"
+#include "rnd.h"
 
-#define debug(x) {std::cerr<<__FILE__<<": "<<__LINE__<<": "<<x<<endl<<flush;}
-//#define debug(x)
+//#define debug(x) {std::cerr<<__FILE__<<": "<<__LINE__<<": "<<x<<endl<<flush;}
+#define debug(x)
 
-using namespace std;
-
-AI_Dummy::AI_Dummy(std::string name, guint32 armyset, Gdk::Color color, int width, int height, int player_no)
+AI_Dummy::AI_Dummy(Glib::ustring name, guint32 armyset, Gdk::RGBA color, int width, int height, int player_no)
     :RealPlayer(name, armyset, color, width, height, Player::AI_DUMMY, player_no)
 {
 }
@@ -56,10 +54,6 @@ AI_Dummy::AI_Dummy(XML_Helper* helper)
 {
 }
 
-AI_Dummy::~AI_Dummy()
-{
-}
-
 void AI_Dummy::abortTurn()
 {
   abort_requested = true;
@@ -72,9 +66,9 @@ void AI_Dummy::setDefensiveProduction(City *city)
   if (city->getActiveProductionSlot() == -1 &&
       city->countDefenders() < MAX_ARMIES_PRODUCED_IN_NEUTRAL_CITY)
     {
-      if (rand() % 2 == 0)
+      if (Rnd::rand() % 2 == 0)
 	{
-	  int idx = rand() % city->getMaxNoOfProductionBases();
+	  int idx = Rnd::rand() % city->getMaxNoOfProductionBases();
 	  cityChangeProduction(city, idx);
 	}
     }
@@ -145,69 +139,105 @@ void AI_Dummy::heroGainsLevel(Hero * a)
 {
   Army::Stat stat = Army::STRENGTH;
   doHeroGainsLevel(a, stat);
-
-  Action_Level* item = new Action_Level();
-  item->fillData(a, stat);
-  addAction(item);
+  addAction(new Action_Level(a, stat));
 }
 
 bool AI_Dummy::chooseHero(HeroProto *hero, City *city, int gold)
 {
+  if (hero || city || gold)
+    {
+      ;
+    }
   //neutral players never accept heroes.
   return false;
 }
         
 Reward *AI_Dummy::chooseReward(Ruin *ruin, Sage *sage, Stack *stack)
 {
+  if (ruin || stack)
+    {
+      ;
+    }
   //neutrals don't search ruins, but let's not return null.
   return sage->front();
 }
 
 bool AI_Dummy::chooseTreachery (Stack *stack, Player *player, Vector <int> pos)
 {
+  if (stack || player || pos != Vector<int>(-1,-1))
+    {
+      ;
+    }
   //neutrals don't leave the castle.
-  bool performTreachery = true;
-  return performTreachery;
+  return true;
 }
 
 Army::Stat AI_Dummy::chooseStat(Hero *hero)
 {
+  if (hero)
+    {
+      ;
+    }
   //neutrals don't have heroes.
   return Army::STRENGTH;
 }
 
 bool AI_Dummy::chooseQuest(Hero *hero)
 {
+  if (hero)
+    {
+      ;
+    }
   //neutrals don't have heroes.
   return true;
 }
 
 bool AI_Dummy::computerChooseVisitRuin(Stack *stack, Vector<int> dest, guint32 moves, guint32 turns)
 {
+  if (stack || dest != Vector<int>(-1,-1) || moves || turns)
+    {
+      ;
+    }
   //neutrals don't have heroes.
   return true;
 }
 
 bool AI_Dummy::computerChoosePickupBag(Stack *stack, Vector<int> dest, guint32 moves, guint32 turns)
 {
+  if (stack || dest != Vector<int>(-1,-1) || moves || turns)
+    {
+      ;
+    }
   //neutrals don't have heroes.
   return true;
 }
 
 bool AI_Dummy::computerChooseVisitTempleForBlessing(Stack *stack, Vector<int> dest, guint32 moves, guint32 turns)
 {
+  if (stack || dest != Vector<int>(-1,-1) || moves || turns)
+    {
+      ;
+    }
   //neutral army units don't go afield.
   return true;
 }
 
 bool AI_Dummy::computerChooseVisitTempleForQuest(Stack *stack, Vector<int> dest, guint32 moves, guint32 turns)
 {
+  if (stack || dest != Vector<int>(-1,-1) || moves || turns)
+    {
+      ;
+    }
   //neutrals don't have heroes
   return true;
 }
 
 bool AI_Dummy::computerChooseContinueQuest(Stack *stack, Quest *quest, Vector<int> dest, guint32 moves, guint32 turns)
 {
+  if (stack || quest || dest != Vector<int>(0,0) || moves || turns)
+    {
+      ;
+    }
   //neutrals don't have heroes
   return true;
 }
