@@ -2,7 +2,7 @@
 // Copyright (C) 2003 Michael Bartl
 // Copyright (C) 2004, 2006 Andrea Paternesi
 // Copyright (C) 2004 John Farrell
-// Copyright (C) 2006, 2007, 2008, 2009 Ben Asselstine
+// Copyright (C) 2006, 2007, 2008, 2009, 2014, 2015 Ben Asselstine
 // Copyright (C) 2007, 2008 Ole Laursen
 //
 //  This program is free software; you can redistribute it and/or modify
@@ -20,11 +20,10 @@
 //  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 
 //  02110-1301, USA.
 
-#include <stdlib.h>
-#include <algorithm>
 #include <fstream>
-#include <assert.h>
 
+#include "AI_Diplomacy.h"
+#include "AI_Analysis.h"
 #include "ai_fast.h"
 
 #include "playerlist.h"
@@ -39,7 +38,6 @@
 #include "Threatlist.h"
 #include "action.h"
 #include "xmlhelper.h"
-#include "AI_Diplomacy.h"
 #include "stack.h"
 #include "GameScenarioOptions.h"
 #include "hero.h"
@@ -49,14 +47,14 @@
 #include "armyprodbase.h"
 #include "QuestsManager.h"
 #include "Quest.h"
+#include "SightMap.h"
 #include "Sage.h"
+#include "rnd.h"
 
-using namespace std;
-
-#define debug(x) {cerr<<__FILE__<<": "<<__LINE__<<": "<<x<<flush<<endl;}
+#define debug(x) {std::cerr<<__FILE__<<": "<<__LINE__<<": "<<x<<std::flush<<std::endl;}
 //#define debug(x)
 
-AI_Fast::AI_Fast(string name, guint32 armyset, Gdk::Color color, int width, int height, int player_no)
+AI_Fast::AI_Fast(Glib::ustring name, guint32 armyset, Gdk::RGBA color, int width, int height, int player_no)
     :RealPlayer(name, armyset, color, width, height, Player::AI_FAST, player_no), d_join(true),
     d_maniac(false), d_analysis(0), d_diplomacy(0)
 {
@@ -113,7 +111,7 @@ bool AI_Fast::startTurn()
     sbusy.emit();
     if (getStacklist()->getHeroes().size() == 0 &&
         Citylist::getInstance()->countCities(this) == 1)
-      AI_maybeBuyScout(Citylist::getInstance()->getFirstCity(this));
+      AI_maybeBuyScout(getFirstCity());
     sbusy.emit();
 
     debug(getName() << ": AI_Fast::start_turn")
@@ -332,10 +330,7 @@ void AI_Fast::heroGainsLevel(Hero * a)
     // increase the strength attack (uninnovative, but enough here)
     Army::Stat stat = Army::STRENGTH;
     doHeroGainsLevel(a, stat);
-
-    Action_Level* item = new Action_Level();
-    item->fillData(a, stat);
-    addAction(item);
+    addAction(new Action_Level(a, stat));
 }
 
 Stack *AI_Fast::findNearOwnStackToJoin(Stack *s, int max_distance)
@@ -536,7 +531,7 @@ bool AI_Fast::computerTurn()
 
 		if (target->contains(s->getPos()) == false)
 		  {
-		    debug("Stack is not in " << target->getName() << " yet" <<endl);
+		    debug("Stack is not in " << target->getName() << " yet" <<std::endl);
 		    int mp = s->getPath()->calculateToCity(s, target);
 		    if (mp > 0)
 		      {
@@ -555,7 +550,7 @@ bool AI_Fast::computerTurn()
 		  }
 		else if (s->getPos() != target->getPos())
 		  {
-		    debug("Stack is inside " << target->getName() << endl);
+		    debug("Stack is inside " << target->getName() << std::endl);
 		    //if we're not in the upper right corner
 		    s->getPath()->calculate(s, target->getPos());
 		    //go there, and take as many as we can
@@ -591,7 +586,7 @@ bool AI_Fast::computerTurn()
 		Path *target2_path = NULL;
 		Citylist *cl = Citylist::getInstance();
 		City *target1;
-                if (rand() % 3 == 0)
+                if (Rnd::rand() % 3 == 0)
                   target1 = cl->getClosestEnemyCity(s);
                 else
                   target1 = cl->getNearestEnemyCity(s->getPos());
@@ -632,7 +627,7 @@ bool AI_Fast::computerTurn()
 
 		if (!target)    // strange situation
 		  {
-		    cerr << "yet another bad situation!!\n";
+                    std::cerr << "yet another bad situation!!" << std::endl;
                     stackPark(s);
 		    return true;
 		  }
@@ -799,17 +794,28 @@ bool AI_Fast::computerTurn()
 
 bool AI_Fast::chooseTreachery (Stack *stack, Player *player, Vector <int> pos)
 {
-  bool performTreachery = true;
-  return performTreachery;
+  if (stack || player || pos != Vector<int>(-1,-1))
+    {
+      ;
+    }
+  return true;
 }
 
 bool AI_Fast::chooseHero(HeroProto *hero, City *city, int gold)
 {
+  if (hero || city || gold)
+    {
+      ;
+    }
   return true;
 }
 
 Reward *AI_Fast::chooseReward(Ruin *ruin, Sage *sage, Stack *stack)
 {
+  if (ruin || stack)
+    {
+      ;
+    }
   //always pick the money.
   for (Sage::iterator it = sage->begin(); it != sage->end(); it++)
     if ((*it)->getType() == Reward::GOLD)
@@ -819,16 +825,28 @@ Reward *AI_Fast::chooseReward(Ruin *ruin, Sage *sage, Stack *stack)
 
 Army::Stat AI_Fast::chooseStat(Hero *hero)
 {
+  if (hero)
+    {
+      ;
+    }
   return Army::STRENGTH;
 }
 
 bool AI_Fast::chooseQuest(Hero *hero)
 {
+  if (hero)
+    {
+      ;
+    }
   return true;
 }
 
 bool AI_Fast::computerChooseVisitRuin(Stack *stack, Vector<int> dest, guint32 moves, guint32 turns)
 {
+  if (turns)
+    {
+      ;
+    }
   if (stack->getPos() == dest)
     return true;
   if (moves < stack->getMoves() + 15)
@@ -839,6 +857,10 @@ bool AI_Fast::computerChooseVisitRuin(Stack *stack, Vector<int> dest, guint32 mo
 
 bool AI_Fast::computerChoosePickupBag(Stack *stack, Vector<int> dest, guint32 moves, guint32 turns)
 {
+  if (turns)
+    {
+      ;
+    }
   if (stack->getPos() == dest)
     return true;
   if (moves < stack->getMoves() + 7)
@@ -849,6 +871,10 @@ bool AI_Fast::computerChoosePickupBag(Stack *stack, Vector<int> dest, guint32 mo
 
 bool AI_Fast::computerChooseVisitTempleForBlessing(Stack *stack, Vector<int> dest, guint32 moves, guint32 turns)
 {
+  if (turns)
+    {
+      ;
+    }
   if (stack->isOnCity() == true)
     return false;
   if (stack->getPos() == dest)
@@ -861,6 +887,10 @@ bool AI_Fast::computerChooseVisitTempleForBlessing(Stack *stack, Vector<int> des
 
 bool AI_Fast::computerChooseVisitTempleForQuest(Stack *stack, Vector<int> dest, guint32 moves, guint32 turns)
 {
+  if (turns)
+    {
+      ;
+    }
   if (stack->isOnCity() == true)
     return false;
   if (stack->getPos() == dest)
@@ -873,6 +903,10 @@ bool AI_Fast::computerChooseVisitTempleForQuest(Stack *stack, Vector<int> dest,
 
 bool AI_Fast::computerChooseContinueQuest(Stack *stack, Quest *quest, Vector<int> dest, guint32 moves, guint32 turns)
 {
+  if (stack || quest || dest != Vector<int>(-1,-1) || moves || turns)
+    {
+      ;
+    }
   return true;
 }
 // End of file
