@@ -1,7 +1,7 @@
 // Copyright (C) 2004 John Farrell
 // Copyright (C) 2004, 2005, 2006 Ulf Lorenz
 // Copyright (C) 2004, 2005, 2006 Andrea Paternesi
-// Copyright (C) 2007, 2008, 2009, 2010 Ben Asselstine
+// Copyright (C) 2007, 2008, 2009, 2010, 2014, 2015 Ben Asselstine
 // Copyright (C) 2007, 2008 Ole Laursen
 //
 //  This program is free software; you can redistribute it and/or modify
@@ -19,8 +19,6 @@
 //  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 
 //  02110-1301, USA.
 
-#include <stdlib.h>
-#include <algorithm>
 #include <map>
 
 #include "ai_smart.h"
@@ -38,14 +36,14 @@
 #include "history.h"
 #include "citylist.h"
 #include "city.h"
+#include "SightMap.h"
 #include "Sage.h"
+#include "GameMap.h"
 
-using namespace std;
-
-#define debug(x) {cerr<<__FILE__<<": "<<__LINE__<<": "<<x<<flush<<endl;}
+#define debug(x) {std::cerr<<__FILE__<<": "<<__LINE__<<": "<<x<<std::flush<<std::endl;}
 //#define debug(x)
 
-AI_Smart::AI_Smart(string name, unsigned int armyset, Gdk::Color color, int width, int height, int player_no)
+AI_Smart::AI_Smart(Glib::ustring name, unsigned int armyset, Gdk::RGBA color, int width, int height, int player_no)
   :RealPlayer(name, armyset, color, width, height, Player::AI_SMART, player_no),
    d_mustmakemoney(0)
 {
@@ -62,17 +60,13 @@ AI_Smart::AI_Smart(XML_Helper* helper)
 {
 }
 
-AI_Smart::~AI_Smart()
-{
-}
-
 bool AI_Smart::startTurn()
 {
   sbusy.emit();
 
   if (getStacklist()->getHeroes().size() == 0 &&
       Citylist::getInstance()->countCities(this) == 1)
-    AI_maybeBuyScout(Citylist::getInstance()->getFirstCity(this));
+    AI_maybeBuyScout(getFirstCity());
   
     debug("Player " << getName() << " starts a turn.")
 
@@ -95,7 +89,7 @@ bool AI_Smart::startTurn()
         
     AI_Analysis *analysis = new AI_Analysis(this);
     const Threatlist *threats = analysis->getThreatsInOrder();
-    City *first_city = Citylist::getInstance()->getFirstCity(this);
+    City *first_city = getFirstCity();
     bool build_capacity = false;
     if (first_city)
       {
@@ -188,10 +182,7 @@ void AI_Smart::heroGainsLevel(Hero * a)
 {
     Army::Stat stat = Army::STRENGTH;
     doHeroGainsLevel(a, stat);
-
-    Action_Level* item = new Action_Level();
-    item->fillData(a, stat);
-    addAction(item);
+    addAction(new Action_Level(a, stat));
 }
 
 int AI_Smart::maybeBuyProduction(City *c, bool quick)
@@ -295,23 +286,19 @@ int AI_Smart::setBestProduction(City *c)
 
 int AI_Smart::chooseArmyTypeToBuy(City *c, bool quick)
 {
-    int bestScore, bestIndex;
-    guint32 size = 0;
+    int bestScore, bestTypeId;
 
     const Armysetlist* al = Armysetlist::getInstance();
 
-    size = al->getSize(getArmyset());
-        
     bestScore = -1;
-    bestIndex = -1;
+    bestTypeId = -1;
     
-    debug("size " << size)
-
-    for (unsigned int i = 0; i < size; i++)
+    Armyset *as = al->get(getArmyset());
+    for (Armyset::iterator i = as->begin(); i != as->end(); ++i)
     {
         const ArmyProto *proto = NULL;
 
-        proto=al->getArmy(getArmyset(), i);
+        proto=al->getArmy(getArmyset(), (*i)->getId());
 
 	if (proto->getNewProductionCost() == 0)
 	  continue;
@@ -319,7 +306,7 @@ int AI_Smart::chooseArmyTypeToBuy(City *c, bool quick)
         if ((int)proto->getNewProductionCost() > d_gold)
           continue;
         
-       if (c->hasProductionBase(proto->getTypeId(), getArmyset())==false)
+       if (c->hasProductionBase(proto->getId(), getArmyset())==false)
        {
          int score;
          if (quick)
@@ -328,13 +315,13 @@ int AI_Smart::chooseArmyTypeToBuy(City *c, bool quick)
            score = scoreBestArmyType(proto);
          if (score >= bestScore)
          {
-            bestIndex = i;
+            bestTypeId = (*i)->getId();
             bestScore = score;
          }
        }
     }
 
-    return bestIndex;
+    return bestTypeId;
 }
 
 int AI_Smart::scoreQuickArmyType(const ArmyProdBase *a)
@@ -479,17 +466,29 @@ void AI_Smart::examineCities()
 
 bool AI_Smart::chooseTreachery (Stack *stack, Player *player, Vector <int> pos)
 {
-  bool performTreachery = true;
-  return performTreachery;
+  if (stack || player || pos != Vector<int>(-1,-1))
+    {
+      ;
+    }
+  return true;
 }
 
 bool AI_Smart::chooseHero(HeroProto *hero, City *city, int gold)
 {
+  if (hero || city || gold)
+    {
+      ;
+    }
+
   return true;
 }
 
 Reward *AI_Smart::chooseReward(Ruin *ruin, Sage *sage, Stack *stack)
 {
+  if (ruin || stack)
+    {
+      ;
+    }
   //always pick the money.
   for (Sage::iterator it = sage->begin(); it != sage->end(); it++)
     if ((*it)->getType() == Reward::GOLD)
@@ -506,11 +505,19 @@ Army::Stat AI_Smart::chooseStat(Hero *hero)
 
 bool AI_Smart::chooseQuest(Hero *hero)
 {
+  if (hero)
+    {
+      ;
+    }
   return true;
 }
 
 bool AI_Smart::computerChooseVisitRuin(Stack *stack, Vector<int> dest, guint32 moves, guint32 turns)
 {
+  if (turns)
+    {
+      ;
+    }
   if (stack->isOnCity() == true)
     {
       if (moves <= stack->getMoves())
@@ -530,6 +537,10 @@ bool AI_Smart::computerChooseVisitRuin(Stack *stack, Vector<int> dest, guint32 m
 
 bool AI_Smart::computerChoosePickupBag(Stack *stack, Vector<int> dest, guint32 moves, guint32 turns)
 {
+  if (turns)
+    {
+      ;
+    }
   City *enemy = GameMap::getEnemyCity(dest);
   if (enemy != NULL && enemy->isBurnt() == false)
     return false;
@@ -572,6 +583,10 @@ bool AI_Smart::computerChoosePickupBag(Stack *stack, Vector<int> dest, guint32 m
 
 bool AI_Smart::computerChooseVisitTempleForBlessing(Stack *stack, Vector<int> dest, guint32 moves, guint32 turns)
 {
+  if (turns)
+    {
+      ;
+    }
   if (stack->isOnCity() == true)
     {
       if (moves * 2 <= stack->getMoves())
@@ -596,6 +611,10 @@ bool AI_Smart::computerChooseVisitTempleForBlessing(Stack *stack, Vector<int> de
 
 bool AI_Smart::computerChooseVisitTempleForQuest(Stack *stack, Vector<int> dest, guint32 moves, guint32 turns)
 {
+  if (turns)
+    {
+      ;
+    }
   if (stack->size() == 1)
     return false;
 
@@ -616,6 +635,10 @@ bool AI_Smart::computerChooseVisitTempleForQuest(Stack *stack, Vector<int> dest,
 
 bool AI_Smart::computerChooseContinueQuest(Stack *stack, Quest *quest, Vector<int> dest, guint32 moves, guint32 turns)
 {
+  if (stack || quest || dest != Vector<int>(-1,-1) || moves)
+    {
+      ;
+    }
   if (turns > 4)
     return false;
   return true;
