@@ -1,4 +1,4 @@
-//  Copyright (C) 2007, 2008, 2009 Ben Asselstine
+//  Copyright (C) 2007, 2008, 2009, 2012, 2014 Ben Asselstine
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -21,7 +21,6 @@
 #include <sigc++/signal.h>
 
 #include "overviewmap.h"
-#include "input-events.h"
 
 //! Display a miniature map that shows where Stack objects are on the game map.
 /** 
@@ -31,14 +30,17 @@
 class ArmyMap : public OverviewMap
 {
  public:
-     //! Default constructor.  Make a new ArmyMap.
-     ArmyMap();
+    //! Default constructor.  Make a new ArmyMap.
+    ArmyMap();
+
+    //! Destructor.
+    //~ArmyMap() {};
 
     //! Emitted when the Army units are finished being drawn on the map surface.
     /**
      * Classes that use ArmyMap must catch this signal to display the map.
      */
-    sigc::signal<void, Glib::RefPtr<Gdk::Pixmap> > map_changed;
+    sigc::signal<void, Cairo::RefPtr<Cairo::Surface> > map_changed;
     
  private:
     
