@@ -1,7 +1,7 @@
 // Copyright (C) 2000, 2001, 2003 Michael Bartl
 // Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006 Ulf Lorenz
 // Copyright (C) 2004, 2005 Andrea Paternesi
-// Copyright (C) 2007, 2008, 2009, 2010, 2011 Ben Asselstine
+// Copyright (C) 2007, 2008, 2009, 2010, 2011, 2014, 2015 Ben Asselstine
 // Copyright (C) 2007, 2008 Ole Laursen
 //
 //  This program is free software; you can redistribute it and/or modify
@@ -23,21 +23,20 @@
 #define ARMY_PROTO_H
 
 #include <gtkmm.h>
-#include <string>
 #include "PixMask.h"
 #include "shield.h"
-#include "hero.h"
 #include "armyprotobase.h"
+#include "hero.h"
 
 class XML_Helper;
-class Armyset;
 class Tar_Helper;
 
+//! An army prototype object.  Appears in an armyset file.
 class ArmyProto : public ArmyProtoBase
 {
     public:
 	//! The xml tag of this object in an armyset configuration file.
-	static std::string d_tag; 
+	static Glib::ustring d_tag; 
 
 	//! Copy constructor.
         ArmyProto(const ArmyProto& armyproto);
@@ -49,13 +48,16 @@ class ArmyProto : public ArmyProtoBase
 	ArmyProto();
 
 	//! Destructor.
-        virtual ~ArmyProto();
+        ~ArmyProto();
 
 
         // Set Methods
         
+        //! Sets the Type Id of the Army.
+        void setId(guint32 id) {d_id = id;};
+
 	//! Sets the filename of the image.
-	void setImageName(Shield::Colour c,std::string name) {d_image_name[c] = name;}
+	void setImageName(Shield::Colour c,Glib::ustring name) {d_image_name[c] = name;}
 
         //! Set the basic image of the Army.
         void setImage(Shield::Colour c, PixMask* image) {d_image[c] = image;};
@@ -80,13 +82,16 @@ class ArmyProto : public ArmyProtoBase
 
         // Get Methods
         
+        //! Returns the Type Id of this Army prototype.
+        guint32 getId() const {return d_id;};
+
 	//! Returns the basename of the picture's filename
 	/**
 	 * Returns the filename that holds the image for this Army.
 	 * The filename does not have a path, and the filename does
 	 * not have an extension (e.g. .png).
 	 */
-	std::string getImageName(Shield::Colour c) const {return d_image_name[c];}
+	Glib::ustring getImageName(Shield::Colour c) const {return d_image_name[c];}
 
         //! Get the image of the army prototype. 
 	PixMask* getImage(Shield::Colour c) const {return d_image[c];};
@@ -117,7 +122,7 @@ class ArmyProto : public ArmyProtoBase
 	void instantiateImages(guint32 tilesize, Tar_Helper *t, bool &broken);
 
 	//! Load the ArmyProto image in the given filename.
-	void instantiateImages(int tilesize, Shield::Colour c, std::string image_filename, bool &broken);
+	void instantiateImages(int tilesize, Shield::Colour c, Glib::ustring image_filename, bool &broken);
 
 	//! Destroy the images associated with this ArmyProto object.
 	void uninstantiateImages();
@@ -142,6 +147,9 @@ class ArmyProto : public ArmyProtoBase
 
     private:
 
+        //! The Type Id of this Army prototype.
+        guint32 d_id;
+
 	//! The picture of the Army prototype.
 	/**
 	 * There is an image for each player, plus the neutral player.
@@ -179,7 +187,7 @@ class ArmyProto : public ArmyProtoBase
 	 *
 	 * There is an image filename for each player, plus the neutral player.
 	 */
-	std::string d_image_name[MAX_PLAYERS + 1];
+	Glib::ustring d_image_name[MAX_PLAYERS + 1];
 
 	//! The gender of this object.
 	/**
