@@ -2,7 +2,7 @@
 // Copyright (C) 2003, 2004, 2005, 2006, 2007 Ulf Lorenz
 // Copyright (C) 2004, 2005 Bryan Duff
 // Copyright (C) 2004, 2005, 2006 Andrea Paternesi
-// Copyright (C) 2006, 2007, 2008, 2009, 2010 Ben Asselstine
+// Copyright (C) 2006, 2007, 2008, 2009, 2010, 2014, 2015 Ben Asselstine
 // Copyright (C) 2007 Ole Laursen
 //
 //  This program is free software; you can redistribute it and/or modify
@@ -23,42 +23,32 @@
 #include <config.h>
 
 #include <assert.h>
-#include <stdlib.h>
 
 #include "bigmap.h"
 
 #include "army.h"
-#include "path.h"
+#include "Item.h"
 #include "stacklist.h"
 #include "stack.h"
-#include "citylist.h"
 #include "city.h"
-#include "ruinlist.h"
 #include "ruin.h"
-#include "signpostlist.h"
 #include "signpost.h"
-#include "templelist.h"
 #include "temple.h"
-#include "armysetlist.h"
-#include "portlist.h"
 #include "port.h"
-#include "bridgelist.h"
 #include "bridge.h"
-#include "roadlist.h"
 #include "road.h"
 #include "playerlist.h"
 #include "File.h"
+#include "stacktile.h"
 #include "GameMap.h"
-#include "Configuration.h"
-#include "GraphicsCache.h"
+#include "ImageCache.h"
 #include "MapRenderer.h"
 #include "FogMap.h"
 #include "MapBackpack.h"
 #include "GameScenarioOptions.h"
 
 #include <iostream>
-using namespace std;
-//#define debug(x) {cerr<<__FILE__<<": "<<__LINE__<<": "<<x<<endl<<flush;}
+//#define debug(x) {std::cerr<<__FILE__<<": "<<__LINE__<<": "<<x<<std::endl<<std::flush;}
 #define debug(x)
 
 BigMap::BigMap()
@@ -68,7 +58,7 @@ BigMap::BigMap()
     view.x = view.y = 0;
     view.w = 0;
     view.h = 0;
-
+    input_locked = false;
     d_grid_toggled = false;
 
     blank_screen = false;
@@ -90,10 +80,8 @@ void BigMap::set_view(Rectangle new_view)
 {
     int tilesize = GameMap::getInstance()->getTileSize();
     
-    int width = 0;
-    int height = 0;
-    if (outgoing == true)
-      outgoing->get_size(width, height);
+    int width = image.get_width();
+    int height = image.get_height();
     if (view.dim == new_view.dim && buffer && image.get_width() == width && image.get_height() == height)
     {
 	// someone wants us to move the view, not resize it, no need to
@@ -129,13 +117,14 @@ void BigMap::set_view(Rectangle new_view)
     
     buffer_view.dim = view.dim + Vector<int>(2, 2);
 
-    buffer = Gdk::Pixmap::create (Glib::RefPtr<Gdk::Drawable>(0), buffer_view.w * tilesize, buffer_view.h * tilesize, 24);
-    buffer_gc = Gdk::GC::create(buffer);
+    Cairo::RefPtr<Cairo::Surface> empty = Cairo::ImageSurface::create (Cairo::FORMAT_ARGB32, buffer_view.w *tilesize, buffer_view.h * tilesize);
+    buffer = Cairo::Surface::create (empty, Cairo::CONTENT_COLOR_ALPHA, buffer_view.w * tilesize, buffer_view.h * tilesize);
+    buffer_gc = Cairo::Context::create(buffer);
 
     //now create the part that will go out to the gtk::image
     if (outgoing == true)
       outgoing.clear();
-    outgoing = Gdk::Pixmap::create(Glib::RefPtr<Gdk::Drawable>(buffer), image.get_width(), image.get_height(), 24);
+    outgoing = Cairo::Surface::create(buffer, Cairo::CONTENT_COLOR_ALPHA, image.get_width(), image.get_height());
 
 
     if (d_renderer)
@@ -144,14 +133,18 @@ void BigMap::set_view(Rectangle new_view)
     d_renderer = new MapRenderer(buffer);
 }
 
-void BigMap::clip_viewable_buffer(Glib::RefPtr<Gdk::Pixmap> pixmap, Glib::RefPtr<Gdk::GC> gc, Vector<int> pos, Glib::RefPtr<Gdk::Pixmap> out)
+void BigMap::clip_viewable_buffer(Cairo::RefPtr<Cairo::Surface> pixmap, Vector<int> pos, Cairo::RefPtr<Cairo::Surface> out)
 {
-    //Glib::RefPtr<Gdk::Pixmap> outgoing = Gdk::Pixmap::create(Glib::RefPtr<Gdk::Drawable>(pixmap), image.get_width(), image.get_height(), 24) ;
-    int width = 0;
-    int height = 0;
-    pixmap->get_size(width,height);
-    out->draw_drawable(gc, pixmap, pos.x, pos.y, 0, 0, image.get_width(), image.get_height());
-    return;
+  Cairo::RefPtr<Cairo::Context> out_gc = Cairo::Context::create(out);
+  out_gc->rectangle(0, 0, image.get_width(), image.get_height());
+  out_gc->clip();
+  out_gc->save();
+  out_gc->set_source(pixmap, -pos.x, -pos.y);
+  out_gc->rectangle (0, 0, image.get_width(), image.get_height());
+  out_gc->clip();
+  out_gc->paint();
+  out_gc->restore();
+  return;
 }
 
 void BigMap::draw(Player *player, bool redraw_buffer)
@@ -172,31 +165,33 @@ void BigMap::draw(Player *player, bool redraw_buffer)
 
     // redraw the buffer
     if (redraw_buffer)
-	draw_buffer();
+      draw_buffer();
 
     // blit the visible part of buffer to the screen
     Vector<int> p = view_pos - (buffer_view.pos * tilesize * magnification_factor);
+    outgoing.clear();
+    outgoing = Cairo::Surface::create(buffer, Cairo::CONTENT_COLOR_ALPHA, image.get_width(), image.get_height());
     //Glib::RefPtr<Gdk::Pixmap> outgoing;
     if (magnification_factor != 1.0)
       {
 	if (magnified_buffer == true)
 	  magnified_buffer.clear();
 	magnified_buffer = magnify(buffer);
-	clip_viewable_buffer(magnified_buffer, buffer_gc, p, outgoing);
+	clip_viewable_buffer(magnified_buffer, p, outgoing);
       }
     else
       {
-	clip_viewable_buffer(buffer, buffer_gc, p, outgoing);
+	clip_viewable_buffer(buffer, p, outgoing);
       }
 
     if (blank_screen)
       {
-	int width = 0;
-	int height = 0;
-	outgoing->get_size(width, height);
-	Glib::RefPtr<Gdk::GC> outgoing_gc = Gdk::GC::create(outgoing);
-	outgoing_gc->set_rgb_fg_color(FOG_COLOUR);
-	outgoing->draw_rectangle(outgoing_gc, true, 0, 0, width, height);
+	int width = image.get_width();
+	int height = image.get_height();
+	Cairo::RefPtr<Cairo::Context> outgoing_gc = Cairo::Context::create(outgoing);
+	outgoing_gc->set_source_rgba(FOG_COLOUR.get_red(), FOG_COLOUR.get_green(), FOG_COLOUR.get_blue(), FOG_COLOUR.get_alpha());
+	outgoing_gc->rectangle(0, 0, width, height);
+	outgoing_gc->fill();
       }
     map_changed.emit(outgoing);
 }
@@ -247,14 +242,12 @@ Vector<int> BigMap::tile_to_buffer_pos(Vector<int> tile)
 Vector<int> BigMap::mouse_pos_to_tile(Vector<int> pos)
 {
     int ts = GameMap::getInstance()->getTileSize();
-
     return (view_pos + pos) / (ts * magnification_factor);
 }
 
 Vector<int> BigMap::mouse_pos_to_tile_offset(Vector<int> pos)
 {
     int ts = GameMap::getInstance()->getTileSize();
-
     return (view_pos + pos) % (int)rint(ts * magnification_factor);
 }
 
@@ -311,28 +304,27 @@ MapTipPosition BigMap::map_tip_position(Rectangle tile_area)
     return m;
 }
 
-void BigMap::blit_object(const Location &obj, Vector<int> tile, PixMask *image, Glib::RefPtr<Gdk::Pixmap> surface, Glib::RefPtr<Gdk::GC> surface_gc)
+void BigMap::blit_object(const Location &obj, Vector<int> tile, PixMask *im, Cairo::RefPtr<Cairo::Surface> surface)
 {
   Vector<int> diff = tile - obj.getPos();
   int tilesize = GameMap::getInstance()->getTileSize();
   Vector<int> p = tile_to_buffer_pos(tile);
-  image->blit(diff, tilesize, surface, p);
+  im->blit(diff, tilesize, surface, p);
 }
 
-void BigMap::draw_stack(Stack *s, Glib::RefPtr<Gdk::Pixmap> surface, Glib::RefPtr<Gdk::GC> surface_gc)
+void BigMap::draw_stack(Stack *s, Cairo::RefPtr<Cairo::Surface> surface)
 {
+  //this routine is for drawing the active stack.
+  //for all other stacks see ImageCache::draw_tile_pic
   GameMap *gm = GameMap::getInstance();
-  GraphicsCache *gc = GraphicsCache::getInstance();
+  ImageCache *gc = ImageCache::getInstance();
   Vector<int> p = s->getPos();
   Player *player = s->getOwner();
-  int army_tilesize;
 
   // check if the object lies in the viewed part of the map
   // otherwise we shouldn't draw it
   if (is_inside(buffer_view, p) && !s->getDeleting())
     {
-      Armysetlist *al = Armysetlist::getInstance();
-      army_tilesize = al->getTileSize(player->getArmyset());
       if (s->empty())
 	{
 	  std::cerr << "WARNING: empty stack found" << std::endl;
@@ -344,6 +336,7 @@ void BigMap::draw_stack(Stack *s, Glib::RefPtr<Gdk::Pixmap> surface, Glib::RefPt
       // draw stack
 
       bool show_army = true;
+      //we don't show the army or the flag if we're in fortified tent.
       if (s->hasShip())
 	{
 	  gc->getShipPic(player)->blit(surface, p);
@@ -372,18 +365,29 @@ void BigMap::draw_stack(Stack *s, Glib::RefPtr<Gdk::Pixmap> surface, Glib::RefPt
 	    }
 	}
 
-
       if (show_army)
-	{
-	  // draw flag
-	  gc->getFlagPic(s)->blit(surface, p);
-	}
+        {
+          //does our position have us on that stacktile?
+          /*
+           * sometimes the stack tile isn't updated right away.
+           */
+          StackTile *st = GameMap::getStacks(s->getPos());
+          guint32 stacksize;
+          if (!st->contains(s->getId()))
+            stacksize = st->countNumberOfArmies(player) + s->size();
+          else
+            stacksize = st->countNumberOfArmies(player);
+          if (stacksize > MAX_STACK_SIZE)
+            stacksize = MAX_STACK_SIZE;
+          if (stacksize > 0)
+            gc->getFlagPic(stacksize, player)->blit(surface, p);
+        }
     }
 }
 
 void BigMap::draw_buffer()
 {
-  draw_buffer (buffer_view, buffer, buffer_gc);
+  draw_buffer (buffer_view, buffer);
     
   //the idea here is that we want to show what happens when an AI-owned
   //stack moves through our area.  in lieu of that, we just block everything
@@ -391,55 +395,44 @@ void BigMap::draw_buffer()
   if (Playerlist::getViewingplayer()->getType() != Player::HUMAN &&
       GameScenarioOptions::s_hidden_map == true)
     {
-      int width = 0;
-      int height = 0;
-      buffer->get_size(width, height);
-      buffer_gc->set_rgb_fg_color(FOG_COLOUR);
-      buffer->draw_rectangle(buffer_gc, true, 0, 0, width, height);
+      int width = image.get_width();
+      int height = image.get_height();
+      buffer_gc->set_source_rgba(FOG_COLOUR.get_red(), FOG_COLOUR.get_green(), FOG_COLOUR.get_blue(), FOG_COLOUR.get_alpha());
+      buffer_gc->rectangle(0, 0, width, height);
+      buffer_gc->fill();
     }
   else
     after_draw();
 
 }
 
-bool BigMap::saveViewAsBitmap(std::string filename)
-{
-  int width;
-  int height;
-  buffer->get_size(width, height);
-  remove (filename.c_str());
-  Glib::RefPtr<Gdk::Pixbuf> pixbuf = Gdk::Pixbuf::create(Glib::RefPtr<Gdk::Drawable>(buffer), 0, 0, width, height);
-  pixbuf->save (filename, "png");
-  return true;
-}
-
-bool BigMap::saveUnderlyingMapAsBitmap(std::string filename)
+bool BigMap::saveUnderlyingMapAsBitmap(Glib::ustring filename)
 {
   return d_renderer->saveAsBitmap(filename);
 }
 
-bool BigMap::saveAsBitmap(std::string filename)
+bool BigMap::saveAsBitmap(Glib::ustring filename)
 {
   int tilesize = GameMap::getInstance()->getTileSize();
   int width = GameMap::getWidth() * tilesize;
   int height = GameMap::getHeight() * tilesize;
-  Glib::RefPtr<Gdk::Pixmap> surf = Gdk::Pixmap::create(Glib::RefPtr<Gdk::Drawable>(0), width, height, 24);
+  Cairo::RefPtr<Cairo::Surface> empty = Cairo::ImageSurface::create (Cairo::FORMAT_ARGB32, width, height);
+  Cairo::RefPtr<Cairo::Surface> surf = Cairo::Surface::create (empty, Cairo::CONTENT_COLOR_ALPHA, width, height);
   
   bool orig_grid = d_grid_toggled;
   d_grid_toggled = false;
-  draw_buffer(Rectangle (0, 0, GameMap::getWidth(), GameMap::getHeight()), surf,
-	      Gdk::GC::create(surf));
+  draw_buffer(Rectangle (0, 0, GameMap::getWidth(), GameMap::getHeight()), surf);
   d_grid_toggled = orig_grid;
-  Glib::RefPtr<Gdk::Pixbuf> pixbuf = Gdk::Pixbuf::create(Glib::RefPtr<Gdk::Drawable>(surf), 0, 0, width, height);
+  Glib::RefPtr<Gdk::Pixbuf> pixbuf = Gdk::Pixbuf::create(surf, 0, 0, width, height);
   pixbuf->save (filename, "png");
   return true;
 }
 
-void BigMap::draw_buffer_tile(Vector<int> tile, Glib::RefPtr<Gdk::Pixmap> surface, Glib::RefPtr<Gdk::GC> context)
+void BigMap::draw_buffer_tile(Vector<int> tile, Cairo::RefPtr<Cairo::Surface> surface)
 {
   guint32 tilesize = GameMap::getInstance()->getTileSize();
   Player *viewing = Playerlist::getViewingplayer();
-  GraphicsCache *gc = GraphicsCache::getInstance();
+  ImageCache *gc = ImageCache::getInstance();
   GameMap *gm = GameMap::getInstance();
   int tile_style_id = gm->getTile(tile)->getTileStyle()->getId();
   int fog_type_id = 0;
@@ -488,7 +481,7 @@ void BigMap::draw_buffer_tile(Vector<int> tile, Glib::RefPtr<Gdk::Pixmap> surfac
 	has_bag = true;
     }
 
-  Stack *stack = GameMap::getStack(tile);
+  Stack *stack = GameMap::getStrongestStack(tile);
   if (stack)
     {
       if (viewing->getFogMap()->isCompletelyObscuredFogTile(tile) == false)
@@ -504,14 +497,22 @@ void BigMap::draw_buffer_tile(Vector<int> tile, Glib::RefPtr<Gdk::Pixmap> surfac
 		  m->getBuilding() != Maptile::TEMPLE)
 		has_tower = true;
 	      else if (stack->hasShip() == true)
-		{
-		  has_ship = true;
-		  stack_size = stack->size();
-		}
+                {
+                  has_ship = true;
+                  stack_size = GameMap::getStacks(stack->getPos())->countNumberOfArmies(Playerlist::getInstance()->getPlayer(stack_player_id));
+                  if (stack_size > 0 && (guint)stack_size > MAX_STACK_SIZE)
+                    stack_size = stack->size();
+                  //here we show the number of armies on the tile.
+                  //instead of the number of armies in the stack.
+                  //so that stacks appear whole before we click on them.
+                  //and that a stack of 1 can't hide a stack of 7.
+                }
 	      else
 		{
 		  army_type_id = (*stack->begin())->getTypeId();
-		  stack_size = stack->size();
+                  stack_size = GameMap::getStacks(stack->getPos())->countNumberOfArmies(Playerlist::getInstance()->getPlayer(stack_player_id));
+                  if (stack_size > 0 && (guint)stack_size > MAX_STACK_SIZE)
+                    stack_size = stack->size();
 		}
 	    }
 	}
@@ -595,39 +596,34 @@ void BigMap::draw_buffer_tile(Vector<int> tile, Glib::RefPtr<Gdk::Pixmap> surfac
   pixmask->blit(surface, tile_to_buffer_pos(tile));
 }
 
-void BigMap::draw_buffer_tiles(Rectangle map_view, Glib::RefPtr<Gdk::Pixmap> surface, Glib::RefPtr<Gdk::GC> context)
+void BigMap::draw_buffer_tiles(Rectangle map_view, Cairo::RefPtr<Cairo::Surface> surface)
 {
   for (int i = map_view.x; i < map_view.x + map_view.w; i++)
     for (int j = map_view.y; j < map_view.y + map_view.h; j++)
       if (i < GameMap::getWidth() && j < GameMap::getHeight())
-	draw_buffer_tile(Vector<int>(i,j), surface, context);
+	draw_buffer_tile(Vector<int>(i,j), surface);
 }
 
-void BigMap::draw_buffer(Rectangle map_view, Glib::RefPtr<Gdk::Pixmap> surface, Glib::RefPtr<Gdk::GC> context)
+void BigMap::draw_buffer(Rectangle map_view, Cairo::RefPtr<Cairo::Surface> surface)
 {
-  draw_buffer_tiles(map_view, surface, context);
+  draw_buffer_tiles(map_view, surface);
 }
 
 //here we want to magnify the entire buffer, not a subset
-Glib::RefPtr<Gdk::Pixmap> BigMap::magnify(Glib::RefPtr<Gdk::Pixmap> orig)
+Cairo::RefPtr<Cairo::Surface> BigMap::magnify(Cairo::RefPtr<Cairo::Surface> orig)
 {
   //magnify the buffer into a buffer of the correct size
 
-  int width = 0;
-  int height = 0;
-  orig->get_size(width, height);
+  int width = image.get_width();
+  int height = image.get_height();
   if (width == 0 || height == 0)
     return orig;
-  Glib::RefPtr<Gdk::Pixmap> result = 
-    Gdk::Pixmap::create(Glib::RefPtr<Gdk::Drawable>(orig), 
-			width * magnification_factor,
-			height * magnification_factor);
+  Cairo::RefPtr<Cairo::Surface> result = 
+    Cairo::Surface::create(orig, Cairo::CONTENT_COLOR_ALPHA,
+                           width * magnification_factor,
+                           height * magnification_factor);
   Glib::RefPtr<Gdk::Pixbuf> unzoomed_buffer;
-  unzoomed_buffer = Gdk::Pixbuf::create(Glib::RefPtr<Gdk::Drawable>(orig), 0, 0, width, height);
-
-  //Glib::RefPtr<Gdk::Pixbuf> zoomed_buffer;
-  //zoomed_buffer = unzoomed_buffer->scale_simple(width * magnification_factor, height * magnification_factor, Gdk::INTERP_BILINEAR);
-  //result->draw_pixbuf(zoomed_buffer, 0, 0, 0, 0, zoomed_buffer->get_width(), zoomed_buffer->get_height(), Gdk::RGB_DITHER_NONE, 0, 0);
+  unzoomed_buffer = Gdk::Pixbuf::create(orig, 0, 0, width, height);
 
   return result;
 }
@@ -643,3 +639,45 @@ void BigMap::blank(bool on)
   blank_screen = on;
   draw (Playerlist::getViewingplayer());
 }
+
+bool BigMap::scroll(GdkEventScroll *event)
+{
+  if (input_locked)
+    return true;
+  Rectangle n = view;
+  int amt = 1;
+  int max_height = GameMap::getHeight() * GameMap::getInstance()->getTileSize();
+  int max_width = GameMap::getWidth() * GameMap::getInstance()->getTileSize();
+  switch (event->direction)
+    {
+    case GDK_SCROLL_UP:
+      if (n.y > amt)
+        n.y -= amt;
+      else
+        n.y = 0;
+      break;
+      break;
+    case GDK_SCROLL_DOWN:
+      if (n.y + amt < max_height)
+        n.y += amt;
+      else
+        n.y = max_height;
+      break;
+    case GDK_SCROLL_RIGHT:
+      if (n.x + amt < max_width)
+        n.x += amt;
+      else
+        n.x = max_width;
+      break;
+    case GDK_SCROLL_LEFT:
+      if (n.x > amt)
+        n.x -= amt;
+      else
+        n.x = 0;
+    default:
+      break;
+    }
+  set_view(n);
+  view_changed.emit(view);
+  return true;
+}
