@@ -1,4 +1,4 @@
-// Copyright (C) 2008 Ben Asselstine
+// Copyright (C) 2008, 2014 Ben Asselstine
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -20,16 +20,12 @@
 
 #include "chat-client.h"
 
-ChatClient::ChatClient(std::string nick)
+ChatClient::ChatClient(Glib::ustring nick)
 	: d_nickname(nick)
 {
 }
 
-ChatClient::~ChatClient()
-{
-}
-
-void ChatClient::gotChatMessage(const std::string nickname, const std::string &payload)
+void ChatClient::gotChatMessage(const Glib::ustring nickname, const Glib::ustring &payload)
 {
   chat_message_received.emit(nickname, payload);
 }
