@@ -2,7 +2,7 @@
 // Copyright (C) 2002 Mark L. Amidon
 // Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006 Ulf Lorenz
 // Copyright (C) 2005, 2006 Andrea Paternesi
-// Copyright (C) 2006, 2007, 2008, 2009, 2011 Ben Asselstine
+// Copyright (C) 2006, 2007, 2008, 2009, 2011, 2014 Ben Asselstine
 // Copyright (C) 2008 Ole Laursen
 //
 //  This program is free software; you can redistribute it and/or modify
@@ -24,21 +24,16 @@
 #define CITY_H
 
 #include <list>
-#include <string>
 #include <vector>
 #include "Location.h"
 #include "Ownable.h"
 #include "Renamable.h"
-#include "prodslot.h"
-#include "defs.h"
 #include "prodslotlist.h"
 
 class Player;
 class Stack;
 class Army;
 class ArmyProdBase;
-class ArmyProto;
-class Hero;
 
 #define DEFAULT_CITY_NAME "Noname"
 #define DEFAULT_CITY_INCOME 20
@@ -69,7 +64,7 @@ class City : public Ownable, public Location, public Renamable,
     public:
 
 	//! The xml tag of this object in a saved-game file.
-	static std::string d_tag; 
+	static Glib::ustring d_tag; 
 
 	//! Default constructor.
         /** 
@@ -82,7 +77,7 @@ class City : public Ownable, public Location, public Renamable,
 	  * @param numslots  The number of production slots for this city.
           */
         City(Vector<int> pos, guint32 width, 
-	     std::string name = DEFAULT_CITY_NAME, 
+	     Glib::ustring name = DEFAULT_CITY_NAME, 
 	     guint32 gold = DEFAULT_CITY_INCOME, 
 	     guint32 numslots = MAX_PRODUCTION_SLOTS_IN_A_CITY);
 
@@ -104,12 +99,12 @@ class City : public Ownable, public Location, public Renamable,
         City(XML_Helper* helper, guint32 width);
 
 	//! Destructor.
-        ~City();
+        ~City() {};
 
 	//Get Methods
 	
         //! Return the defense level of the city.
-        int getDefenseLevel() const {return calculateDefenseLevel();}
+        int getDefenseLevel() const;
 
         //! Return the income of the city per turn.
         guint32 getGold() const {return d_gold;}
@@ -185,14 +180,7 @@ class City : public Ownable, public Location, public Renamable,
         void produceWeakestProductionBase();
 
         //! Produces a scout in the city.
-	/**
-	 * A Scout is defined as the first Army in the Armyset.
-	 * It is used when neutral cities are "average".  
-	 * It is also used as a fallback when a city must have one Army unit.
-	 * @note The scout army doesn't have to be an Army production base in
-	 * the city for this method to operate.
-	 */
-        void produceScout();
+        void produceWeakestQuickestArmyInArmyset();
 
         //! Do everything neccessary for a new turn.
 	/**
@@ -221,6 +209,11 @@ class City : public Ownable, public Location, public Renamable,
 	//! Callback that makes the army show up.
 	const Army *armyArrives(Stack *& stack);
 
+        //! Kill a percentage of the army units in the city.
+        std::list<Stack*> diseaseDefenders(double percent_to_kill);
+
+        //! Change owner of city AND the stacks in it.
+        void persuadeDefenders(Player *new_owner);
 	
 	// Methods that operate on class data and do not modify the class.
 
@@ -263,12 +256,12 @@ class City : public Ownable, public Location, public Renamable,
 	// Static Methods
 	
 	//! Get the default name of any city.
-	static std::string getDefaultName() {return _(DEFAULT_CITY_NAME);};
+	static Glib::ustring getDefaultName() {return _(DEFAULT_CITY_NAME);};
 
     private:
 
 	//! Callback for loading city objects from a saved-game file.
-	bool load(std::string tag, XML_Helper *helper);
+	bool load(Glib::ustring tag, XML_Helper *helper);
 
         //! Produces the currently active Army production base.
         Army * produceArmy(Stack *& stack);
@@ -290,13 +283,6 @@ class City : public Ownable, public Location, public Renamable,
 	 */
 	void sortProduction();
 
-	//! Calculate defense level.
-	/**
-	 * The defense level is a function of how many production slots
-	 * the city has.
-	 */
-	guint32 calculateDefenseLevel() const;
-
         // DATA
 
 	//! The City gives the Player this much gold per turn.
@@ -359,4 +345,5 @@ class City : public Ownable, public Location, public Renamable,
 	Player *d_capital_owner;
 };
 
+bool armyCompareStrength (const ArmyProdBase *lhs, const ArmyProdBase *rhs);
 #endif // CITY_H
