@@ -1,4 +1,4 @@
-// Copyright (C) 2007 Ben Asselstine
+// Copyright (C) 2007, 2014 Ben Asselstine
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -15,17 +15,11 @@
 //  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 
 //  02110-1301, USA.
 
-#include "citymap.h"
-
-#include "city.h"
-#include "citylist.h"
-#include "playerlist.h"
-#include "stacklist.h"
-#include "stack.h"
-#include "GraphicsCache.h"
-#include "GameMap.h"
+#include <config.h>
 #include <assert.h>
 
+#include "citymap.h"
+
 CityMap::CityMap()
 {
 }
