@@ -1,4 +1,4 @@
-//  Copyright (C) 2007, 2008, 2009, 2010 Ben Asselstine
+//  Copyright (C) 2007, 2008, 2009, 2010, 2014 Ben Asselstine
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -23,25 +23,17 @@
 #include "armyset-info-dialog.h"
 #include "armysetlist.h"
 
-#include <string.h>
-#include <stdlib.h>
-#include <ctype.h>
-#include "glade-helpers.h"
 #include "ucompose.hpp"
 #include "defs.h"
 #include "File.h"
 
 
-ArmySetInfoDialog::ArmySetInfoDialog(Armyset *armyset, std::string dir, std::string file, bool readonly, std::string title)
+ArmySetInfoDialog::ArmySetInfoDialog(Gtk::Window &parent, Set *armyset, Glib::ustring dir, Glib::ustring file, bool readonly, Glib::ustring title)
+ : LwEditorDialog(parent, "armyset-info-dialog.ui")
 {
   d_armyset = armyset;
   d_readonly = readonly;
     
-    Glib::RefPtr<Gtk::Builder> xml
-	= Gtk::Builder::create_from_file(get_glade_path()
-				    + "/armyset-info-dialog.ui");
-
-    xml->get_widget("dialog", dialog);
     if (title != "")
       dialog->set_title(title);
 
@@ -61,10 +53,10 @@ ArmySetInfoDialog::ArmySetInfoDialog(Armyset *armyset, std::string dir, std::str
     else
       {
         guint32 num = 0;
-        std::string basename = Armysetlist::getInstance()->findFreeBaseName(_("untitled"), 100, num);
+        Glib::ustring basename = Armysetlist::getInstance()->findFreeBaseName(_("untitled"), 100, num);
         filename_entry->set_text(basename);
 
-        std::string name = String::ucompose("%1 %2", _("Untitled"), num);
+        Glib::ustring name = String::ucompose("%1 %2", _("Untitled"), num);
         name_entry->set_text(name);
       }
     if (readonly == false)
@@ -102,16 +94,6 @@ void ArmySetInfoDialog::on_name_changed()
   update_buttons();
 }
 
-ArmySetInfoDialog::~ArmySetInfoDialog()
-{
-  delete dialog;
-}
-void ArmySetInfoDialog::set_parent_window(Gtk::Window &parent)
-{
-    dialog->set_transient_for(parent);
-    //dialog->set_position(Gtk::WIN_POS_CENTER_ON_PARENT);
-}
-
 int ArmySetInfoDialog::run()
 {
     dialog->show_all();
@@ -139,8 +121,7 @@ void ArmySetInfoDialog::update_buttons()
       return;
     }
 
-  std::string dir = File::getUserArmysetDir() + filename_entry->get_text();
-  if (Armysetlist::getInstance()->getArmyset(filename_entry->get_text()))
+  if (Armysetlist::getInstance()->get(filename_entry->get_text()))
     {
       accept_button->set_sensitive(false);
       status_label->set_markup(String::ucompose("<b>%1</b>", 
