@@ -1,5 +1,5 @@
 //  Copyright (C) 2007 Ole Laursen
-//  Copyright (C) 2007, 2008, 2009 Ben Asselstine
+//  Copyright (C) 2007, 2008, 2009, 2014 Ben Asselstine
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -19,10 +19,8 @@
 #ifndef CITY_EDITOR_DIALOG_H
 #define CITY_EDITOR_DIALOG_H
 
-#include <memory>
-#include <sigc++/trackable.h>
 #include <gtkmm.h>
-
+#include "lw-editor-dialog.h"
 
 class CreateScenarioRandomize;
 class City;
@@ -30,20 +28,17 @@ class ArmyProdBase;
 class Player;
 
 //! Scenario editor.  Edits a City object.
-class CityEditorDialog: public sigc::trackable
+class CityEditorDialog: public LwEditorDialog
 {
  public:
-    CityEditorDialog(City *city, CreateScenarioRandomize *randomizer);
-    ~CityEditorDialog();
-
-    void set_parent_window(Gtk::Window &parent);
+    CityEditorDialog(Gtk::Window &parent, City *city, CreateScenarioRandomize *randomizer);
+    ~CityEditorDialog() {};
 
     int run();
     
  private:
     City *city;
     CreateScenarioRandomize *d_randomizer;
-    Gtk::Dialog* dialog;
     Gtk::ComboBoxText *player_combobox;
     Gtk::CheckButton *capital_checkbutton;
     Gtk::Entry *name_entry;
