@@ -1,4 +1,4 @@
-//  Copyright (C) 2007, 2008, 2009, 2010 Ben Asselstine
+//  Copyright (C) 2007, 2008, 2009, 2010, 2014 Ben Asselstine
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -22,26 +22,16 @@
 
 #include "cityset-info-dialog.h"
 #include "citysetlist.h"
-
-#include <string.h>
-#include <stdlib.h>
-#include <ctype.h>
-#include "glade-helpers.h"
 #include "ucompose.hpp"
 #include "defs.h"
 #include "File.h"
 
-
-CitySetInfoDialog::CitySetInfoDialog(Cityset *cityset, std::string dir, std::string file, bool readonly, std::string title)
+CitySetInfoDialog::CitySetInfoDialog(Gtk::Window &parent, Set *cityset, Glib::ustring dir, Glib::ustring file, bool readonly, Glib::ustring title)
+ : LwEditorDialog(parent, "cityset-info-dialog.ui")
 {
   d_cityset = cityset;
   d_readonly = readonly;
     
-    Glib::RefPtr<Gtk::Builder> xml
-	= Gtk::Builder::create_from_file(get_glade_path()
-				    + "/cityset-info-dialog.ui");
-
-    xml->get_widget("dialog", dialog);
     if (title != "")
       dialog->set_title(title);
 
@@ -61,10 +51,10 @@ CitySetInfoDialog::CitySetInfoDialog(Cityset *cityset, std::string dir, std::str
     else
       {
         guint32 num = 0;
-        std::string basename = Citysetlist::getInstance()->findFreeBaseName(_("untitled"), 100, num);
+        Glib::ustring basename = Citysetlist::getInstance()->findFreeBaseName(_("untitled"), 100, num);
         filename_entry->set_text(basename);
 
-        std::string name = String::ucompose("%1 %2", _("Untitled"), num);
+        Glib::ustring name = String::ucompose("%1 %2", _("Untitled"), num);
         name_entry->set_text(name);
       }
     if (readonly == false)
@@ -102,16 +92,6 @@ void CitySetInfoDialog::on_name_changed()
   update_buttons();
 }
 
-CitySetInfoDialog::~CitySetInfoDialog()
-{
-  delete dialog;
-}
-void CitySetInfoDialog::set_parent_window(Gtk::Window &parent)
-{
-    dialog->set_transient_for(parent);
-    //dialog->set_position(Gtk::WIN_POS_CENTER_ON_PARENT);
-}
-
 int CitySetInfoDialog::run()
 {
     dialog->show_all();
@@ -139,8 +119,7 @@ void CitySetInfoDialog::update_buttons()
       return;
     }
 
-  std::string dir = File::getUserCitysetDir() + filename_entry->get_text();
-  if (Citysetlist::getInstance()->getCityset(filename_entry->get_text()))
+  if (Citysetlist::getInstance()->get(filename_entry->get_text()))
     {
       accept_button->set_sensitive(false);
       status_label->set_markup(String::ucompose("<b>%1</b>", 
