@@ -1,4 +1,4 @@
-//  Copyright (C) 2007, 2008, 2009, 2010 Ben Asselstine
+//  Copyright (C) 2007, 2008, 2009, 2010, 2014 Ben Asselstine
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -18,26 +18,23 @@
 #ifndef CITYSET_INFO_DIALOG_H
 #define CITYSET_INFO_DIALOG_H
 
-#include <memory>
-#include <sigc++/trackable.h>
 #include <gtkmm.h>
 #include "cityset.h"
+#include "lw-editor-dialog.h"
 
 //! Cityset Editor.  Edit the description of the Cityset.
-class CitySetInfoDialog: public sigc::trackable
+class CitySetInfoDialog: public LwEditorDialog
 {
  public:
-    CitySetInfoDialog(Cityset *cityset, std::string dir, std::string file,
-                        bool readonly = false, std::string title = "");
-    ~CitySetInfoDialog();
-
-    void set_parent_window(Gtk::Window &parent);
+    CitySetInfoDialog(Gtk::Window &parent, Set *cityset, Glib::ustring dir, 
+                      Glib::ustring file, bool readonly = false, 
+                      Glib::ustring title = "");
+    ~CitySetInfoDialog() {};
 
     int run();
     
  private:
-    Gtk::Dialog* dialog;
-    Cityset *d_cityset;
+    Set *d_cityset;
     Gtk::Entry *name_entry;
     Gtk::TextView *copyright_textview;
     Gtk::TextView *license_textview;
