@@ -1,4 +1,4 @@
-//  Copyright (C) 2009, 2010, 2011 Ben Asselstine
+//  Copyright (C) 2009, 2010, 2011, 2012, 2014, 2015 Ben Asselstine
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -21,49 +21,43 @@
 #include <iomanip>
 #include <assert.h>
 #include <string.h>
+#include <errno.h>
 
 #include <sigc++/functors/mem_fun.h>
 #include <sigc++/functors/ptr_fun.h>
 
 #include <gtkmm.h>
 #include "cityset-window.h"
+#include "builder-cache.h"
 #include "cityset-info-dialog.h"
 #include "masked-image-editor-dialog.h"
 
-#include "image-helpers.h"
-#include "input-helpers.h"
-#include "error-utils.h"
-
 #include "defs.h"
 #include "File.h"
 
 #include "ucompose.hpp"
 
-#include "glade-helpers.h"
 #include "image-editor-dialog.h"
-#include "GraphicsCache.h"
+#include "ImageCache.h"
 #include "citysetlist.h"
-#include "recently-edited-file.h"
-#include "recently-edited-file-list.h"
 #include "tile-size-editor-dialog.h"
 #include "editor-quit-dialog.h"
-#include "editor-recover-dialog.h"
+#include "GameMap.h"
 
 
-CitySetWindow::CitySetWindow(std::string load_filename)
+CitySetWindow::CitySetWindow(Glib::ustring load_filename)
 {
-  autosave = File::getSavePath() + "autosave" + Cityset::file_extension;
   needs_saving = false;
   d_cityset = NULL;
-    Glib::RefPtr<Gtk::Builder> xml
-	= Gtk::Builder::create_from_file(get_glade_path() + "/cityset-window.ui");
+    Glib::RefPtr<Gtk::Builder> xml = 
+      BuilderCache::editor_get("cityset-window.ui");
 
     xml->get_widget("window", window);
-    window->set_icon_from_file(File::getMiscFile("various/castle_icon.png"));
+    window->set_icon_from_file(File::getVariousFile("castle_icon.png"));
     window->signal_delete_event().connect
       (sigc::mem_fun(*this, &CitySetWindow::on_window_closed));
 
-    xml->get_widget("cityset_frame", cityset_frame);
+    xml->get_widget("cityset_alignment", cityset_alignment);
     xml->get_widget("new_cityset_menuitem", new_cityset_menuitem);
     new_cityset_menuitem->signal_activate().connect
       (sigc::mem_fun(this, &CitySetWindow::on_new_cityset_activated));
@@ -93,23 +87,23 @@ CitySetWindow::CitySetWindow(std::string load_filename)
     city_tile_width_spinbutton->signal_changed().connect
       (sigc::mem_fun(this, &CitySetWindow::on_city_tile_width_changed));
     city_tile_width_spinbutton->signal_insert_text().connect
-      (sigc::mem_fun(this, &CitySetWindow::on_city_tile_width_text_changed));
+      (sigc::hide(sigc::hide(sigc::mem_fun(this, &CitySetWindow::on_city_tile_width_text_changed))));
     xml->get_widget("ruin_tile_width_spinbutton", ruin_tile_width_spinbutton);
     ruin_tile_width_spinbutton->set_range (1, 4);
     ruin_tile_width_spinbutton->signal_changed().connect
       (sigc::mem_fun(this, &CitySetWindow::on_ruin_tile_width_changed));
     ruin_tile_width_spinbutton->signal_insert_text().connect
-      (sigc::mem_fun(this, &CitySetWindow::on_ruin_tile_width_text_changed));
+      (sigc::hide(sigc::hide(sigc::mem_fun(this, &CitySetWindow::on_ruin_tile_width_text_changed))));
     xml->get_widget("temple_tile_width_spinbutton", 
 		    temple_tile_width_spinbutton);
     temple_tile_width_spinbutton->set_range (1, 4);
     temple_tile_width_spinbutton->signal_changed().connect
       (sigc::mem_fun(this, &CitySetWindow::on_temple_tile_width_changed));
     temple_tile_width_spinbutton->signal_insert_text().connect
-      (sigc::mem_fun(this, &CitySetWindow::on_temple_tile_width_text_changed));
+      (sigc::hide(sigc::hide(sigc::mem_fun(this, &CitySetWindow::on_temple_tile_width_text_changed))));
 
     window->signal_delete_event().connect
-      (sigc::mem_fun(*this, &CitySetWindow::on_delete_event));
+      (sigc::hide(sigc::mem_fun(*this, &CitySetWindow::on_delete_event)));
     xml->get_widget("change_citypics_button", change_citypics_button);
     change_citypics_button->signal_clicked().connect(
 	sigc::mem_fun(this, &CitySetWindow::on_change_citypics_clicked));
@@ -136,47 +130,6 @@ CitySetWindow::CitySetWindow(std::string load_filename)
       current_save_filename = load_filename;
     update_cityset_panel();
 
-    if (File::exists(autosave))
-      {
-        std::string m;
-        std::list<RecentlyEditedFile*> files = RecentlyEditedFileList::getInstance()->getFilesWithExtension(Cityset::file_extension);
-        if (files.size() == 0)
-          m = _("Do you want to recover the session?");
-        else
-          {
-            RecentlyEditedCitysetFile *r = dynamic_cast<RecentlyEditedCitysetFile*>(files.front());
-            if (r->getName() == "")
-              m = String::ucompose(_("Do you want to recover %1?"),
-                                   File::get_basename(r->getFileName(), true));
-            else if (r->getName() != "" && r->getImagesNeeded() > 0)
-              {
-                m = String::ucompose
-                  (_("Do you want to recover %1 (%2, %3 more images needed)?"),
-                                         File::get_basename(r->getFileName(), 
-                                                            true),
-                                         r->getName(),
-                                         r->getImagesNeeded());
-              }
-            else if (r->getName() != "" && r->getImagesNeeded() == 0)
-              {
-                m = String::ucompose(_("Do you want to recover %1 (%2)?"),
-                                     File::get_basename(r->getFileName(), true),
-                                     r->getName());
-              }
-          }
-        EditorRecoverDialog d(m);
-        int response = d.run();
-        d.hide();
-        //ask if we want to recover the autosave.
-        if (response == Gtk::RESPONSE_ACCEPT)
-          {
-            load_cityset (autosave);
-            update_cityset_menuitems();
-            update_cityset_panel();
-            return;
-          }
-      }
-
     if (load_filename.empty() == false)
       {
 	load_cityset (load_filename);
@@ -208,9 +161,9 @@ CitySetWindow::update_cityset_menuitems()
 void
 CitySetWindow::update_cityset_panel()
 {
-  cityset_frame->set_sensitive(d_cityset != NULL);
-  std::string no_image = _("no image set");
-  std::string s;
+  cityset_alignment->set_sensitive(d_cityset != NULL);
+  Glib::ustring no_image = _("no image set");
+  Glib::ustring s;
   if (d_cityset && d_cityset->getCitiesFilename().empty() == false)
     s = d_cityset->getCitiesFilename() + ".png";
   else
@@ -275,21 +228,20 @@ void CitySetWindow::hide()
   window->hide();
 }
 
-bool CitySetWindow::on_delete_event(GdkEventAny *e)
+bool CitySetWindow::on_delete_event()
 {
   hide();
-
   return true;
 }
 
 void CitySetWindow::on_new_cityset_activated()
 {
-  std::string name = "";
+  Glib::ustring name = "";
   int id = Citysetlist::getNextAvailableId(0);
   Cityset *cityset = new Cityset(id, name);
-  CitySetInfoDialog d(cityset, File::getUserCitysetDir(), "", false,
-                      _("Make a New Cityset"));
-  d.set_parent_window(*window);
+  CitySetInfoDialog d(*window, cityset, 
+                      File::getSetDir(Cityset::file_extension, false), "", 
+                      false, _("Make a New Cityset"));
   int response = d.run();
   if (response != Gtk::RESPONSE_ACCEPT)
     {
@@ -299,16 +251,17 @@ void CitySetWindow::on_new_cityset_activated()
   if (d_cityset)
     delete d_cityset;
   d_cityset = cityset;
-  std::string dir = File::getUserCitysetDir();
+  Glib::ustring dir = File::getSetDir(Cityset::file_extension, false);
   d_cityset->setDirectory(dir);
   current_save_filename = d_cityset->getConfigurationFile();
-  RecentlyEditedFileList *refl = RecentlyEditedFileList::getInstance();
-  refl->updateEntry(current_save_filename);
-  refl->save();
-  d_cityset->setDirectory(File::get_dirname(autosave));
-  d_cityset->setBaseName(File::get_basename(autosave));
 
-  d_cityset->save(autosave, Cityset::file_extension);
+  //here we put a copy into the citysetlist, and keep d_cityset as our
+  //current working cityset.
+  Cityset *copy = Cityset::copy (d_cityset);
+  Glib::ustring new_basename = copy->getBaseName();
+  guint32 new_id = copy->getId();
+  if (!Citysetlist::getInstance()->addToPersonalCollection(copy, new_basename, new_id))
+    delete copy;
   update_cityset_panel();
   update_cityset_menuitems();
   needs_saving = true;
@@ -319,10 +272,11 @@ void CitySetWindow::on_load_cityset_activated()
 {
   Gtk::FileChooserDialog chooser(*window, 
 				 _("Choose a Cityset to Load"));
-  Gtk::FileFilter sav_filter;
-  sav_filter.add_pattern("*" + CITYSET_EXT);
-  chooser.set_filter(sav_filter);
-  chooser.set_current_folder(File::getUserCitysetDir());
+  Glib::RefPtr<Gtk::FileFilter> lwc_filter = Gtk::FileFilter::create();
+  lwc_filter->set_name(_("LordsAWar Citysets (*.lwc)"));
+  lwc_filter->add_pattern("*" + CITYSET_EXT);
+  chooser.add_filter(lwc_filter);
+  chooser.set_current_folder(File::getSetDir(Cityset::file_extension, false));
 
   chooser.add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
   chooser.add_button(Gtk::Stock::OPEN, Gtk::RESPONSE_ACCEPT);
@@ -345,7 +299,7 @@ void CitySetWindow::on_load_cityset_activated()
 
 void CitySetWindow::on_validate_cityset_activated()
 {
-  std::list<std::string> msgs;
+  std::list<Glib::ustring> msgs;
   if (d_cityset == NULL)
     return;
   if (d_cityset->validateCitiesFilename() == false)
@@ -369,8 +323,8 @@ void CitySetWindow::on_validate_cityset_activated()
   if (d_cityset->validateTempleTileWidth() == false)
     msgs.push_back(_("The tile width for temples must be over zero."));
 
-  std::string msg = "";
-  for (std::list<std::string>::iterator it = msgs.begin(); it != msgs.end();
+  Glib::ustring msg = "";
+  for (std::list<Glib::ustring>::iterator it = msgs.begin(); it != msgs.end();
        it++)
     msg += (*it) + "\n";
 
@@ -386,86 +340,131 @@ void CitySetWindow::on_validate_cityset_activated()
 
 void CitySetWindow::on_save_as_activated()
 {
-  std::string orig_basename = d_cityset->getBaseName();
-  guint32 orig_id = d_cityset->getId();
-  d_cityset->setId(Citysetlist::getNextAvailableId(orig_id));
-  CitySetInfoDialog d(d_cityset, File::getUserCitysetDir(), "", false,
-                        _("Save a Copy of a Cityset"));
-  d.set_parent_window(*window);
-  int response = d.run();
-  if (response == Gtk::RESPONSE_ACCEPT)
-    {
-      std::string new_basename = d_cityset->getBaseName();
-      guint32 new_id = d_cityset->getId();
-      d_cityset->setId(orig_id);
-      d_cityset->setBaseName(orig_basename);
-      bool success = Citysetlist::getInstance()->addToPersonalCollection(d_cityset, new_basename, new_id);
-      if (success)
-        {
-          save_cityset_menuitem->set_sensitive(true);
-          current_save_filename = d_cityset->getConfigurationFile();
-          RecentlyEditedFileList *refl = RecentlyEditedFileList::getInstance();
-          refl->updateEntry(current_save_filename);
-          refl->save();
-          load_cityset(current_save_filename);
-          needs_saving = false;
-          update_window_title();
-        }
-      else
-        {
-          std::string msg;
-          msg = _("Error!  Cityset could not be saved.");
-          Gtk::MessageDialog dialog(*window, msg);
-          dialog.run();
-          dialog.hide();
-        }
-    }
-  else
-    d_cityset->setId(orig_id);
-}
-
-void CitySetWindow::on_save_cityset_activated()
-{
-  if (current_save_filename.empty())
-    current_save_filename = d_cityset->getConfigurationFile();
-
   guint32 suggested_tile_size = d_cityset->calculate_preferred_tile_size();
   if (suggested_tile_size != d_cityset->getTileSize())
     {
-      TileSizeEditorDialog d(d_cityset->getTileSize(), suggested_tile_size);
+      TileSizeEditorDialog d(*window, d_cityset->getTileSize(), suggested_tile_size);
       int response = d.run();
       if (response == Gtk::RESPONSE_ACCEPT)
         d_cityset->setTileSize(d.get_selected_tilesize());
     }
-  bool success = d_cityset->save(autosave, Cityset::file_extension);
-  if (success)
+  Cityset *copy = Cityset::copy (d_cityset);
+  copy->setId(Citysetlist::getNextAvailableId(d_cityset->getId()));
+  CitySetInfoDialog d(*window, copy, 
+                      File::getSetDir(Cityset::file_extension, false), "", 
+                      false, _("Save a Copy of a Cityset"));
+  int response = d.run();
+  if (response == Gtk::RESPONSE_ACCEPT)
     {
-      RecentlyEditedFileList::getInstance()->updateEntry(current_save_filename);
-      RecentlyEditedFileList::getInstance()->save();
-      success = Cityset::copy (autosave, current_save_filename);
-      if (success == true)
+      Glib::ustring new_basename = copy->getBaseName();
+      guint32 new_id = copy->getId();
+      copy->setDirectory(File::getSetDir(Cityset::file_extension, false));
+      guint32 oldid = d_cityset->getId();
+      Glib::ustring oldname = d_cityset->getName();
+      Glib::ustring oldbasename = d_cityset->getBaseName();
+      Glib::ustring olddir = d_cityset->getDirectory();
+
+      Glib::ustring tmpdir = File::get_tmp_file();
+      File::erase(tmpdir);
+      tmpdir += Cityset::file_extension;
+      File::create_dir(tmpdir);
+      d_cityset->setName(copy->getName());
+      File::copy(d_cityset->getConfigurationFile(), 
+                 File::getTempFile (tmpdir, copy->getBaseName() + Cityset::file_extension));
+      d_cityset->setBaseName(copy->getBaseName());
+      d_cityset->setDirectory(tmpdir);
+      d_cityset->setId(copy->getId());
+          
+      current_save_filename = copy->getConfigurationFile();
+      bool ok = Citysetlist::getInstance()->addToPersonalCollection(d_cityset, new_basename, new_id);
+      File::erase(File::getTempFile (tmpdir, copy->getBaseName() + Cityset::file_extension));
+      File::erase_dir(tmpdir);
+      if (ok)
         {
+          save_cityset_menuitem->set_sensitive(true);
+          d_cityset = copy;
           needs_saving = false;
           update_window_title();
           cityset_saved.emit(d_cityset->getId());
         }
+      else
+        {
+          d_cityset->setName(oldname);
+          d_cityset->setBaseName(oldbasename);
+          d_cityset->setId(oldid);
+          d_cityset->setDirectory(olddir);
+          Glib::ustring errmsg = Glib::strerror(errno);
+          Glib::ustring msg;
+          msg = _("Error!  Cityset could not be saved.");
+          msg += "\n" + current_save_filename + "\n" + errmsg;
+          Gtk::MessageDialog dialog(*window, msg);
+          dialog.run();
+          dialog.hide();
+          delete copy;
+        }
     }
-  if (!success)
+  else
+    delete copy;
+}
+
+bool CitySetWindow::save_current_cityset()
+{
+  if (GameMap::getInstance()->getCitysetId() == d_cityset->getId() &&
+      d_cityset->validate() == false)
     {
-      std::string msg;
+      Glib::ustring errmsg = _("Cityset is invalid, and is also the current working cityset.");
+      Glib::ustring msg;
       msg = _("Error!  Cityset could not be saved.");
+      msg += "\n" + current_save_filename + "\n" +
+        errmsg;
+      Gtk::MessageDialog dialog(*window, msg);
+      dialog.run();
+      dialog.hide();
+      return false;
+    }
+  if (current_save_filename.empty())
+    current_save_filename = d_cityset->getConfigurationFile();
+  
+  guint32 suggested_tile_size = d_cityset->calculate_preferred_tile_size();
+  if (suggested_tile_size != d_cityset->getTileSize())
+    {
+      TileSizeEditorDialog d(*window, d_cityset->getTileSize(), suggested_tile_size);
+      int response = d.run();
+      if (response == Gtk::RESPONSE_ACCEPT)
+        d_cityset->setTileSize(d.get_selected_tilesize());
+    }
+
+  bool success = d_cityset->save(current_save_filename, Cityset::file_extension);
+  if (success)
+    {
+      Citysetlist::getInstance()->reload(d_cityset->getId());
+      needs_saving = false;
+      update_window_title();
+      cityset_saved.emit(d_cityset->getId());
+    }
+  else
+    {
+      Glib::ustring errmsg = Glib::strerror(errno);
+      Glib::ustring msg;
+      msg = _("Error!  Cityset could not be saved.");
+      msg += "\n" + current_save_filename + "\n" + errmsg;
       Gtk::MessageDialog dialog(*window, msg);
       dialog.run();
       dialog.hide();
     }
+  return success;
+}
+
+void CitySetWindow::on_save_cityset_activated()
+{
+  save_current_cityset();
 }
 
 void CitySetWindow::on_edit_cityset_info_activated()
 {
-  CitySetInfoDialog d(d_cityset, File::get_dirname(current_save_filename), 
+  CitySetInfoDialog d(*window, d_cityset, File::get_dirname(current_save_filename), 
                       File::get_basename(current_save_filename), true, 
                       _("Edit Cityset Information"));
-  d.set_parent_window(*window);
   int response = d.run();
   if (response == Gtk::RESPONSE_ACCEPT)
     {
@@ -479,14 +478,14 @@ void CitySetWindow::on_help_about_activated()
   Gtk::AboutDialog* dialog;
 
   Glib::RefPtr<Gtk::Builder> xml
-    = Gtk::Builder::create_from_file(get_glade_path() + "/../about-dialog.ui");
+    = Gtk::Builder::create_from_file(File::getGladeFile("about-dialog.ui"));
 
   xml->get_widget("dialog", dialog);
   dialog->set_transient_for(*window);
-  dialog->set_icon_from_file(File::getMiscFile("various/castle_icon.png"));
+  dialog->set_icon_from_file(File::getVariousFile("castle_icon.png"));
 
   dialog->set_version(PACKAGE_VERSION);
-  dialog->set_logo(GraphicsCache::getMiscPicture("castle_icon.png")->to_pixbuf());
+  dialog->set_logo(ImageCache::loadMiscImage("castle_icon.png")->to_pixbuf());
   dialog->show_all();
   dialog->run();
   delete dialog;
@@ -494,27 +493,17 @@ void CitySetWindow::on_help_about_activated()
   return;
 }
 
-bool CitySetWindow::load_cityset(std::string filename)
+bool CitySetWindow::load_cityset(Glib::ustring filename)
 {
-  std::string old_current_save_filename = current_save_filename;
+  Glib::ustring old_current_save_filename = current_save_filename;
   current_save_filename = filename;
-  if (filename != autosave)
-    Cityset::copy(current_save_filename, autosave);
-  else
-    {
-      //go get the real name of the file
-      std::list<RecentlyEditedFile*> files = RecentlyEditedFileList::getInstance()->getFilesWithExtension(Cityset::file_extension);
-      if (files.size() > 0)
-        current_save_filename = files.front()->getFileName();
-    }
-
-  std::string name = File::get_basename(filename);
+  Glib::ustring name = File::get_basename(filename);
 
   bool unsupported_version = false;
-  Cityset *cityset = Cityset::create(autosave, unsupported_version);
+  Cityset *cityset = Cityset::create(filename, unsupported_version);
   if (cityset == NULL)
     {
-      std::string msg;
+      Glib::ustring msg;
       if (unsupported_version)
         msg = _("Error!  The version of cityset is unsupported.");
       else
@@ -525,17 +514,11 @@ bool CitySetWindow::load_cityset(std::string filename)
       dialog.hide();
       return false;
     }
-  if (File::nameEndsWith(current_save_filename, "/autosave" + Cityset::file_extension) == false)
-    {
-      RecentlyEditedFileList::getInstance()->addEntry(current_save_filename);
-      RecentlyEditedFileList::getInstance()->save();
-    }
   if (d_cityset)
     delete d_cityset;
   d_cityset = cityset;
 
   bool broken = false;
-  d_cityset->setBaseName(File::get_basename(autosave));
   d_cityset->instantiateImages(broken);
   update_window_title();
   return true;
@@ -545,22 +528,22 @@ bool CitySetWindow::quit()
 {
   if (needs_saving == true)
     {
-      EditorQuitDialog d;
-      int response = d.run();
-      d.set_parent_window(*window);
-      d.hide();
+      EditorQuitDialog d(*window);
+      int response = d.run_and_hide();
 
       if (response == Gtk::RESPONSE_CANCEL) //we don't want to quit
 	return false;
 
       else if (response == Gtk::RESPONSE_ACCEPT) // save and quit
-	on_save_cityset_activated();
+        {
+          if (save_current_cityset() == false)
+            return false;
+        }
       //else if (Response == Gtk::CLOSE) // don't save just quit
       window->hide();
     }
   else
     window->hide();
-  File::erase(autosave);
   if (d_cityset)
     delete d_cityset;
   return true;
@@ -576,7 +559,7 @@ void CitySetWindow::on_quit_activated()
   quit();
 }
 
-void CitySetWindow::on_city_tile_width_text_changed(const Glib::ustring &s, int* p)
+void CitySetWindow::on_city_tile_width_text_changed()
 {
   city_tile_width_spinbutton->set_value(atoi(city_tile_width_spinbutton->get_text().c_str()));
   on_city_tile_width_changed();
@@ -591,7 +574,7 @@ void CitySetWindow::on_city_tile_width_changed()
   update_window_title();
 }
 
-void CitySetWindow::on_ruin_tile_width_text_changed(const Glib::ustring &s, int* p)
+void CitySetWindow::on_ruin_tile_width_text_changed()
 {
   ruin_tile_width_spinbutton->set_value(atoi(ruin_tile_width_spinbutton->get_text().c_str()));
   on_ruin_tile_width_changed();
@@ -606,7 +589,7 @@ void CitySetWindow::on_ruin_tile_width_changed()
   update_window_title();
 }
 
-void CitySetWindow::on_temple_tile_width_text_changed(const Glib::ustring &s, int* p)
+void CitySetWindow::on_temple_tile_width_text_changed()
 {
   temple_tile_width_spinbutton->set_value(atoi(temple_tile_width_spinbutton->get_text().c_str()));
   on_temple_tile_width_changed();
@@ -623,11 +606,11 @@ void CitySetWindow::on_temple_tile_width_changed()
 
 void CitySetWindow::on_change_citypics_clicked()
 {
-  std::string filename = "";
+  Glib::ustring filename = "";
   if (d_cityset->getCitiesFilename().empty() == false)
     filename = d_cityset->getFileFromConfigurationFile(d_cityset->getCitiesFilename() +".png");
-  ImageEditorDialog d(filename, MAX_PLAYERS + 1);
-  d.set_parent_window(*window);
+  ImageEditorDialog d(*window, filename, MAX_PLAYERS + 1);
+  d.set_title(_("Select a Cities image"));
   int response = d.run();
   if (filename != "")
     File::erase(filename);
@@ -635,12 +618,15 @@ void CitySetWindow::on_change_citypics_clicked()
     {
       if (d.get_selected_filename() != filename)
         {
-          d_cityset->replaceFileInConfigurationFile(d_cityset->getCitiesFilename()+".png", d.get_selected_filename());
-
-          d_cityset->setCitiesFilename
-            (File::get_basename(d.get_selected_filename()));
-          needs_saving = true;
-          update_window_title();
+          Glib::ustring file = File::get_basename(d.get_selected_filename());
+          if (d_cityset->replaceFileInConfigurationFile(d_cityset->getCitiesFilename()+".png", d.get_selected_filename()))
+            {
+              d_cityset->setCitiesFilename (file);
+              needs_saving = true;
+              update_window_title();
+            }
+          else
+            show_add_file_error(*d.get_dialog(), file);
         }
       update_cityset_panel();
     }
@@ -648,11 +634,11 @@ void CitySetWindow::on_change_citypics_clicked()
 
 void CitySetWindow::on_change_razedcitypics_clicked()
 {
-  std::string filename = "";
+  Glib::ustring filename = "";
   if (d_cityset->getRazedCitiesFilename().empty() == false)
     filename = d_cityset->getFileFromConfigurationFile(d_cityset->getRazedCitiesFilename() +".png");
-  ImageEditorDialog d(filename, MAX_PLAYERS);
-  d.set_parent_window(*window);
+  ImageEditorDialog d(*window, filename, MAX_PLAYERS);
+  d.set_title(_("Select a Razed Cities image"));
   int response = d.run();
   if (filename != "")
     File::erase(filename);
@@ -660,11 +646,15 @@ void CitySetWindow::on_change_razedcitypics_clicked()
     {
       if (d.get_selected_filename() != filename)
         {
-          d_cityset->replaceFileInConfigurationFile(d_cityset->getRazedCitiesFilename()+".png", d.get_selected_filename());
-          d_cityset->setRazedCitiesFilename
-            (File::get_basename(d.get_selected_filename()));
-          needs_saving = true;
-          update_window_title();
+          Glib::ustring file = File::get_basename(d.get_selected_filename());
+          if (d_cityset->replaceFileInConfigurationFile(d_cityset->getRazedCitiesFilename()+".png", d.get_selected_filename()))
+            {
+              d_cityset->setRazedCitiesFilename (file);
+              needs_saving = true;
+              update_window_title();
+            }
+          else
+            show_add_file_error(*d.get_dialog(), file);
         }
       update_cityset_panel();
     }
@@ -672,11 +662,11 @@ void CitySetWindow::on_change_razedcitypics_clicked()
 
 void CitySetWindow::on_change_portpic_clicked()
 {
-  std::string filename = "";
+  Glib::ustring filename = "";
   if (d_cityset->getPortFilename().empty() == false)
     filename = d_cityset->getFileFromConfigurationFile(d_cityset->getPortFilename() +".png");
-  ImageEditorDialog d(filename, 1);
-  d.set_parent_window(*window);
+  ImageEditorDialog d(*window, filename, 1);
+  d.set_title(_("Select a Port image"));
   int response = d.run();
   if (filename != "")
     File::erase(filename);
@@ -684,10 +674,15 @@ void CitySetWindow::on_change_portpic_clicked()
     {
       if (d.get_selected_filename() != filename)
         {
-          d_cityset->replaceFileInConfigurationFile(d_cityset->getPortFilename()+".png", d.get_selected_filename());
-          d_cityset->setPortFilename(File::get_basename(d.get_selected_filename()));
-          needs_saving = true;
-          update_window_title();
+          Glib::ustring file = File::get_basename(d.get_selected_filename());
+          if (d_cityset->replaceFileInConfigurationFile(d_cityset->getPortFilename()+".png", d.get_selected_filename()))
+            {
+              d_cityset->setPortFilename(file);
+              needs_saving = true;
+              update_window_title();
+            }
+          else
+            show_add_file_error(*d.get_dialog(), file);
         }
       update_cityset_panel();
     }
@@ -695,11 +690,11 @@ void CitySetWindow::on_change_portpic_clicked()
 
 void CitySetWindow::on_change_signpostpic_clicked()
 {
-  std::string filename = "";
+  Glib::ustring filename = "";
   if (d_cityset->getSignpostFilename().empty() == false)
     filename = d_cityset->getFileFromConfigurationFile(d_cityset->getSignpostFilename() +".png");
-  ImageEditorDialog d(filename, 1);
-  d.set_parent_window(*window);
+  ImageEditorDialog d(*window, filename, 1);
+  d.set_title(_("Select a Signpost image"));
   int response = d.run();
   if (filename != "")
     File::erase(filename);
@@ -707,12 +702,15 @@ void CitySetWindow::on_change_signpostpic_clicked()
     {
       if (d.get_selected_filename() != filename)
         {
-          d_cityset->replaceFileInConfigurationFile(d_cityset->getSignpostFilename()+".png", d.get_selected_filename());
-
-          d_cityset->setSignpostFilename
-            (File::get_basename(d.get_selected_filename()));
-          needs_saving = true;
-          update_window_title();
+          Glib::ustring file = File::get_basename(d.get_selected_filename());
+          if (d_cityset->replaceFileInConfigurationFile(d_cityset->getSignpostFilename()+".png", d.get_selected_filename()))
+            {
+              d_cityset->setSignpostFilename (file);
+              needs_saving = true;
+              update_window_title();
+            }
+          else
+            show_add_file_error(*d.get_dialog(), file);
         }
       update_cityset_panel();
     }
@@ -720,11 +718,11 @@ void CitySetWindow::on_change_signpostpic_clicked()
 
 void CitySetWindow::on_change_ruinpics_clicked()
 {
-  std::string filename = "";
+  Glib::ustring filename = "";
   if (d_cityset->getRuinsFilename().empty() == false)
     filename = d_cityset->getFileFromConfigurationFile(d_cityset->getRuinsFilename() +".png");
-  ImageEditorDialog d(filename, RUIN_TYPES);
-  d.set_parent_window(*window);
+  ImageEditorDialog d(*window, filename, RUIN_TYPES);
+  d.set_title(_("Select a Ruins image"));
   int response = d.run();
   if (filename != "")
     File::erase(filename);
@@ -732,11 +730,15 @@ void CitySetWindow::on_change_ruinpics_clicked()
     {
       if (d.get_selected_filename() != filename)
         {
-          d_cityset->replaceFileInConfigurationFile(d_cityset->getRuinsFilename()+".png", d.get_selected_filename());
-          d_cityset->setRuinsFilename
-            (File::get_basename(d.get_selected_filename()));
-          needs_saving = true;
-          update_window_title();
+          Glib::ustring file = File::get_basename(d.get_selected_filename());
+          if (d_cityset->replaceFileInConfigurationFile(d_cityset->getRuinsFilename()+".png", d.get_selected_filename()))
+            {
+              d_cityset->setRuinsFilename (file);
+              needs_saving = true;
+              update_window_title();
+            }
+          else
+            show_add_file_error(*d.get_dialog(), file);
         }
       update_cityset_panel();
     }
@@ -744,11 +746,11 @@ void CitySetWindow::on_change_ruinpics_clicked()
 
 void CitySetWindow::on_change_templepic_clicked()
 {
-  std::string filename = "";
+  Glib::ustring filename = "";
   if (d_cityset->getTemplesFilename().empty() == false)
     filename = d_cityset->getFileFromConfigurationFile(d_cityset->getTemplesFilename() +".png");
-  ImageEditorDialog d(filename, TEMPLE_TYPES);
-  d.set_parent_window(*window);
+  ImageEditorDialog d(*window, filename, TEMPLE_TYPES);
+  d.set_title(_("Select a Temples image"));
   int response = d.run();
   if (filename != "")
     File::erase(filename);
@@ -756,11 +758,15 @@ void CitySetWindow::on_change_templepic_clicked()
     {
       if (d.get_selected_filename() != filename)
         {
-          d_cityset->replaceFileInConfigurationFile(d_cityset->getTemplesFilename()+".png", d.get_selected_filename());
-          d_cityset->setTemplesFilename
-            (File::get_basename(d.get_selected_filename()));
-          needs_saving = true;
-          update_window_title();
+          Glib::ustring file = File::get_basename(d.get_selected_filename());
+          if (d_cityset->replaceFileInConfigurationFile(d_cityset->getTemplesFilename()+".png", d.get_selected_filename()))
+            {
+              d_cityset->setTemplesFilename (file);
+              needs_saving = true;
+              update_window_title();
+            }
+          else
+            show_add_file_error(*d.get_dialog(), file);
         }
       update_cityset_panel();
     }
@@ -768,11 +774,11 @@ void CitySetWindow::on_change_templepic_clicked()
 
 void CitySetWindow::on_change_towerpics_clicked()
 {
-  std::string filename = "";
+  Glib::ustring filename = "";
   if (d_cityset->getTowersFilename().empty() == false)
     filename = d_cityset->getFileFromConfigurationFile(d_cityset->getTowersFilename() +".png");
-  ImageEditorDialog d(filename, MAX_PLAYERS);
-  d.set_parent_window(*window);
+  ImageEditorDialog d(*window, filename, MAX_PLAYERS);
+  d.set_title(_("Select a Towers image"));
   int response = d.run();
   if (filename != "")
     File::erase(filename);
@@ -780,11 +786,15 @@ void CitySetWindow::on_change_towerpics_clicked()
     {
       if (d.get_selected_filename() != filename)
         {
-          d_cityset->replaceFileInConfigurationFile(d_cityset->getTowersFilename()+".png", d.get_selected_filename());
-          d_cityset->setTowersFilename
-            (File::get_basename(d.get_selected_filename()));
-          needs_saving = true;
-          update_window_title();
+          Glib::ustring file = File::get_basename(d.get_selected_filename());
+          if (d_cityset->replaceFileInConfigurationFile(d_cityset->getTowersFilename()+".png", d.get_selected_filename()))
+            {
+              d_cityset->setTowersFilename (file);
+              needs_saving = true;
+              update_window_title();
+            }
+          else
+            show_add_file_error(*d.get_dialog(), file);
         }
       update_cityset_panel();
     }
@@ -792,7 +802,7 @@ void CitySetWindow::on_change_towerpics_clicked()
 
 void CitySetWindow::update_window_title()
 {
-  std::string title = "";
+  Glib::ustring title = "";
   if (needs_saving)
     title += "*";
   title += File::get_basename(current_save_filename, true);
@@ -800,3 +810,13 @@ void CitySetWindow::update_window_title()
   title += _("LordsAWar! Cityset Editor");
   window->set_title(title);
 }
+
+void CitySetWindow::show_add_file_error(Gtk::Dialog &d, Glib::ustring file)
+{
+  Glib::ustring m = 
+    String::ucompose(_("Couldn't add %1.png to:\n%2"),
+                     file, d_cityset->getConfigurationFile());
+  Gtk::MessageDialog td(d, m);
+  td.run();
+  td.hide();
+}
