@@ -1,4 +1,4 @@
-//  Copyright (C) 2009, 2010 Ben Asselstine
+//  Copyright (C) 2009, 2010, 2014, 2015 Ben Asselstine
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -30,20 +30,18 @@
 class CitySetWindow: public sigc::trackable
 {
  public:
-    CitySetWindow(std::string load_filename = "");
+    CitySetWindow(Glib::ustring load_filename = "");
     ~CitySetWindow();
 
+    Gtk::Window &get_window() { return *window; }
     void show();
     void hide();
 
-    Gtk::Window &get_window() { return *window; }
-
     sigc::signal<void, guint32> cityset_saved;
 
  private:
     Gtk::Window* window;
-    std::string current_save_filename;
-    std::string autosave; //filename
+    Glib::ustring current_save_filename;
     Cityset *d_cityset; //current cityset
     bool needs_saving;
     Gtk::MenuItem *new_cityset_menuitem;
@@ -64,14 +62,15 @@ class CitySetWindow: public sigc::trackable
     Gtk::SpinButton *city_tile_width_spinbutton;
     Gtk::SpinButton *ruin_tile_width_spinbutton;
     Gtk::SpinButton *temple_tile_width_spinbutton;
-    Gtk::Frame *cityset_frame;
+    Gtk::Alignment *cityset_alignment;
 
-    bool on_delete_event(GdkEventAny *e);
+    bool on_delete_event();
 
     void update_cityset_panel();
     void update_cityset_menuitems();
 
-    bool load_cityset(std::string filename);
+    bool load_cityset(Glib::ustring filename);
+    bool save_current_cityset();
 
     //callbacks
     void on_new_cityset_activated();
@@ -85,11 +84,11 @@ class CitySetWindow: public sigc::trackable
     void on_edit_cityset_info_activated();
     void on_help_about_activated();
     void on_city_tile_width_changed();
-    void on_city_tile_width_text_changed(const Glib::ustring &s, int* p);
+    void on_city_tile_width_text_changed();
     void on_ruin_tile_width_changed();
-    void on_ruin_tile_width_text_changed(const Glib::ustring &s, int* p);
+    void on_ruin_tile_width_text_changed();
     void on_temple_tile_width_changed();
-    void on_temple_tile_width_text_changed(const Glib::ustring &s, int* p);
+    void on_temple_tile_width_text_changed();
     void on_change_citypics_clicked();
     void on_change_razedcitypics_clicked();
     void on_change_portpic_clicked();
@@ -98,6 +97,7 @@ class CitySetWindow: public sigc::trackable
     void on_change_templepic_clicked();
     void on_change_towerpics_clicked();
     void update_window_title();
+    void show_add_file_error(Gtk::Dialog &d, Glib::ustring file);
 };
 
 #endif
