@@ -1,4 +1,4 @@
-//  Copyright (C) 2010 Ben Asselstine
+//  Copyright (C) 2010, 2014 Ben Asselstine
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -21,39 +21,12 @@
 
 #include "editor-quit-dialog.h"
 
-#include "glade-helpers.h"
 #include "ucompose.hpp"
 #include "File.h"
 #include "defs.h"
 
 
-EditorQuitDialog::EditorQuitDialog()
+EditorQuitDialog::EditorQuitDialog(Gtk::Window &parent)
+ : LwEditorDialog(parent, "editor-quit-dialog.ui")
 {
-    
-    Glib::RefPtr<Gtk::Builder> xml
-	= Gtk::Builder::create_from_file(get_glade_path()
-				    + "/editor-quit-dialog.ui");
-
-    xml->get_widget("dialog", dialog);
-    dialog->set_icon_from_file(File::getMiscFile("various/castle_icon.png"));
-}
-
-EditorQuitDialog::~EditorQuitDialog()
-{
-  delete dialog;
-}
-void EditorQuitDialog::set_parent_window(Gtk::Window &parent)
-{
-    dialog->set_transient_for(parent);
-}
-
-int EditorQuitDialog::run()
-{
-    dialog->show_all();
-    return dialog->run();
-}
-
-void EditorQuitDialog::hide()
-{
-  dialog->hide();
 }
