@@ -1,4 +1,4 @@
-//  Copyright (C) 2009 Ben Asselstine
+//  Copyright (C) 2009, 2014 Ben Asselstine
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -18,25 +18,21 @@
 #ifndef HERO_EDITOR_DIALOG_H
 #define HERO_EDITOR_DIALOG_H
 
-#include <memory>
-#include <sigc++/trackable.h>
 #include <gtkmm.h>
+#include "lw-editor-dialog.h"
 
 class Hero;
 
 //! Scenario editor.  Change the attributes of a hero.
-class HeroEditorDialog
+class HeroEditorDialog: public LwEditorDialog
 {
  public:
-    HeroEditorDialog(Hero *hero);
-    ~HeroEditorDialog();
-
-    void set_parent_window(Gtk::Window &parent);
+    HeroEditorDialog(Gtk::Window &parent, Hero *hero);
+    ~HeroEditorDialog() {};
 
     void run();
     
  private:
-    Gtk::Dialog* dialog;
     Hero*d_hero;
     Gtk::Entry *name_entry;
     Gtk::RadioButton *male_radiobutton;
