@@ -1,5 +1,5 @@
 //  Copyright (C) 2007 Ole Laursen
-//  Copyright (C) 2007, 2008, 2009, 2010 Ben Asselstine
+//  Copyright (C) 2007, 2008, 2009, 2010, 2012, 2014, 2015 Ben Asselstine
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -27,12 +27,15 @@
 
 #include "map-tip-position.h"
 #include "editorbigmap.h"
-#include "RenamableLocation.h"
 
 class EditorBigMap;
 class SmallMap;
 class GameScenario;
 class CreateScenarioRandomize;
+class City;
+class Signpost;
+class Ruin;
+class RoadEditorTip;
 
 #include "UniquelyIdentified.h"
 
@@ -40,7 +43,7 @@ class CreateScenarioRandomize;
 class MainWindow: public sigc::trackable
 {
  public:
-    MainWindow(std::string load_filename = "");
+    MainWindow(Glib::ustring load_filename = "");
     ~MainWindow();
 
     void show();
@@ -58,7 +61,7 @@ class MainWindow: public sigc::trackable
     GameScenario* game_scenario;
     CreateScenarioRandomize* d_create_scenario_names;
 
-    Gtk::DrawingArea *bigmap_drawingarea;
+    Gtk::Image *bigmap_image;
     Gtk::EventBox *bigmap_eventbox;
     Gtk::CheckMenuItem *fullscreen_menuitem;
     Gtk::MenuItem *new_map_menuitem;
@@ -76,6 +79,7 @@ class MainWindow: public sigc::trackable
     Gtk::MenuItem *edit_armyset_menuitem;
     Gtk::MenuItem *edit_cityset_menuitem;
     Gtk::MenuItem *edit_tileset_menuitem;
+    Gtk::MenuItem *edit_fight_order_menuitem;
     Gtk::MenuItem *toggle_tile_graphics_menuitem;
     Gtk::MenuItem *toggle_grid_menuitem;
     Gtk::MenuItem *smooth_map_menuitem;
@@ -93,25 +97,25 @@ class MainWindow: public sigc::trackable
     Gtk::MenuItem *random_all_signs_menuitem;
     Gtk::MenuItem *random_unnamed_signs_menuitem;
     Gtk::MenuItem *help_about_menuitem;
-    Gtk::HBox *terrain_tile_style_hbox;
-    Gtk::DrawingArea *map_drawingarea;
-    std::string current_save_filename;
+    Gtk::Viewport *terrain_tile_style_viewport;
+    Gtk::Grid *terrain_tile_style_grid;
+    Gtk::Image *smallmap_image;
+    Glib::ustring current_save_filename;
     bool needs_saving;
     Gtk::Table *terrain_type_table;
     Gtk::Label *mouse_position_label;
-    Gtk::RadioButton *pointer_radiobutton;
-    Gtk::Tooltips tooltips;
+    Gtk::RadioToolButton *pointer_radiobutton;
     Gtk::Box *players_hbox;
 
     
     GdkEventButton *button_event;
 
-    bool on_delete_event(GdkEventAny *e);
+    bool on_delete_event();
 
     bool on_bigmap_mouse_button_event(GdkEventButton *e);
     bool on_bigmap_mouse_motion_event(GdkEventMotion *e);
-    bool on_bigmap_key_event(GdkEventKey *e);
-    bool on_bigmap_leave_event(GdkEventCrossing *e);
+    bool on_bigmap_key_event();
+    bool on_bigmap_leave_event();
 
     bool on_smallmap_mouse_button_event(GdkEventButton *e);
     bool on_smallmap_mouse_motion_event(GdkEventMotion *e);
@@ -142,6 +146,7 @@ class MainWindow: public sigc::trackable
     void on_switch_sets_activated();
     void on_edit_items_activated();
     void on_edit_rewards_activated();
+    void on_edit_fight_order_activated();
 
     void on_fullscreen_activated();
     void on_tile_graphics_toggled();
@@ -160,7 +165,7 @@ class MainWindow: public sigc::trackable
     
     struct PointerItem
     {
-	Gtk::RadioButton *button;
+	Gtk::RadioToolButton *button;
 	EditorBigMap::Pointer pointer;
 	int size;
     };
@@ -189,22 +194,24 @@ class MainWindow: public sigc::trackable
 
     void clear_save_file_of_scenario_specific_data();
     Tile::Type get_terrain();
+    int get_pointer_index();
     int get_tile_style_id();
     void setup_pointer_radiobutton(Glib::RefPtr<Gtk::Builder> xml,
-	std::string prefix, std::string image_file,
+	Glib::ustring prefix, Glib::ustring image_file,
 	EditorBigMap::Pointer pointer, int size);
     void setup_terrain_radiobuttons();
 
     void init_maps();
     void set_filled_map(int width, int height, int fill_style, 
-			std::string tileset, std::string shieldset, 
-			std::string cityset, std::string armyset);
+			Glib::ustring tileset, Glib::ustring shieldset, 
+			Glib::ustring cityset, Glib::ustring armyset);
     void set_random_map(int width, int height,
 			int grass, int water, int swamp, int forest,
 			int hills, int mountains,
 			int cities, int ruins, int temples, int signposts,
-			std::string tileset, std::string shieldset,
-			std::string cityset, std::string armyset);
+			Glib::ustring tileset, Glib::ustring shieldset,
+			Glib::ustring cityset, Glib::ustring armyset,
+                        bool generate_roads);
 
     void clear_map_state();
     void init_map_state();
@@ -215,8 +222,9 @@ class MainWindow: public sigc::trackable
     void randomize_ruin(Ruin *ruin);
 
     // map callbacks
-    void on_smallmap_changed(Glib::RefPtr<Gdk::Pixmap> map, Gdk::Rectangle r);
-    void on_bigmap_changed(Glib::RefPtr<Gdk::Pixmap> map);
+    void on_smallmap_changed(Cairo::RefPtr<Cairo::Surface> map);
+    void on_bigmap_changed(Cairo::RefPtr<Cairo::Surface> map);
+    void on_smallmap_water_changed();
     void on_objects_selected(std::vector<UniquelyIdentified *> objects);
     void on_mouse_on_tile(Vector<int> tile);
     
@@ -224,9 +232,11 @@ class MainWindow: public sigc::trackable
 
     void auto_select_appropriate_pointer();
 
-    bool on_bigmap_exposed(GdkEventExpose *event);
-    bool on_smallmap_exposed(GdkEventExpose *event);
+    bool on_bigmap_exposed();
+    bool on_smallmap_exposed();
     void on_bigmap_surface_changed(Gtk::Allocation box);
+    void on_save_as_activated(Gtk::FileChooserDialog *chooser);
+    bool on_bigmap_scrolled(GdkEventScroll* event);
     void redraw();
     void fill_players();
 
@@ -238,10 +248,13 @@ class MainWindow: public sigc::trackable
     std::list<PlayerItem> player_buttons;
     void on_player_toggled(PlayerItem item);
     void update_window_title();
+    void on_bag_selected(Vector<int> pos);
+    void on_road_edited(Vector<int> pos, int new_type);
 
     int d_width;
     int d_height;
-    std::string d_load_filename;// filename given on the command line.
+    Glib::ustring d_load_filename;// filename given on the command line.
+    RoadEditorTip *road_editor_tip;
     
 };
 
