@@ -1,4 +1,4 @@
-//  Copyright (C) 2009, 2010 Ben Asselstine
+//  Copyright (C) 2009, 2010, 2014 Ben Asselstine
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -18,39 +18,34 @@
 #ifndef MASKED_IMAGE_EDITOR_DIALOG_H
 #define MASKED_IMAGE_EDITOR_DIALOG_H
 
-#include <memory>
-#include <map>
-#include <sigc++/trackable.h>
-#include <sigc++/connection.h>
 #include <gtkmm.h>
+#include "shield.h"
+#include "lw-editor-dialog.h"
 
 
-//! general picture editor.  
+//! general masked picture editor.  
 /**
  * This class doesn't actually edit the image, instead it shows the image
  * being edited in each player colour.  The user can pick a new file to be
  * the new image.
  *
  * the shieldset is required to define the mask colours.
+ *
  */
 class Shieldset;
-class MaskedImageEditorDialog: public sigc::trackable
+class MaskedImageEditorDialog: public LwEditorDialog
 {
  public:
-    MaskedImageEditorDialog(std::string filename, Shieldset *shieldset = NULL);
-    ~MaskedImageEditorDialog();
+    MaskedImageEditorDialog(Gtk::Window &parent, Glib::ustring filename, Shieldset *shieldset = NULL);
+    ~MaskedImageEditorDialog() {};
 
-    void set_parent_window(Gtk::Window &parent);
-
-  
-    void set_icon_from_file(std::string name) {dialog->set_icon_from_file(name);};
-    std::string get_selected_filename() {return target_filename;};
+    void set_title(Glib::ustring t) {dialog->set_title(t);};
 
+    Glib::ustring get_selected_filename() {return target_filename;};
     int run();
     
  private:
-    std::string target_filename;
-    Gtk::Dialog* dialog;
+    Glib::ustring target_filename;
     Gtk::FileChooserButton *filechooserbutton;
     Gtk::Image *image_white;
     Gtk::Image *image_green;
@@ -63,8 +58,10 @@ class MaskedImageEditorDialog: public sigc::trackable
     Gtk::Image *image_neutral;
     Shieldset * d_shieldset;
     void on_image_chosen();
-    void show_image(std::string filename);
+    void show_image(Glib::ustring filename);
     void update_panel();
+    void on_add(Gtk::Widget *widget);
+    void on_button_pressed();
 
 };
 
