@@ -1,5 +1,5 @@
 //  Copyright (C) 2007 Ole Laursen
-//  Copyright (C) 2007, 2008, 2009, 2010 Ben Asselstine
+//  Copyright (C) 2007, 2008, 2009, 2010, 2012, 2014, 2015 Ben Asselstine
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -23,8 +23,6 @@
 #include <gtkmm.h>
 
 #include "new-map-dialog.h"
-
-#include "glade-helpers.h"
 #include "defs.h"
 #include "File.h"
 #include "tileset.h"
@@ -36,17 +34,12 @@
 #include "GameMap.h"
 
 
-NewMapDialog::NewMapDialog()
+NewMapDialog::NewMapDialog(Gtk::Window &parent)
+ : LwEditorDialog(parent, "new-map-dialog.ui")
 {
     map_set = false;
     
-    Glib::RefPtr<Gtk::Builder> xml
-	= Gtk::Builder::create_from_file(get_glade_path() + "/new-map-dialog.ui");
-
-    xml->get_widget("dialog", dialog);
-
     xml->get_widget("map_size_combobox", map_size_combobox);
-
     xml->get_widget("custom_size_table", custom_size_table);
     xml->get_widget("width_spinbutton", width_spinbutton);
     xml->get_widget("height_spinbutton", height_spinbutton);
@@ -62,6 +55,7 @@ NewMapDialog::NewMapDialog()
     xml->get_widget("temples_scale", temples_scale);
     xml->get_widget("signposts_scale", signposts_scale);
     xml->get_widget("accept_button", accept_button);
+    xml->get_widget("roads_checkbutton", roads_checkbutton);
 
     // fill in tile themes combobox
     
@@ -79,7 +73,7 @@ NewMapDialog::NewMapDialog()
 	 it++)
       {
 	Glib::ustring s = String::ucompose("%1x%1", *it);
-	tile_size_combobox->append_text(s);
+	tile_size_combobox->append(s);
 	if ((*it) == Tileset::getDefaultTileSize())
 	  default_id = counter;
 	counter++;
@@ -109,13 +103,13 @@ NewMapDialog::NewMapDialog()
     default_id = 0;
     shield_theme_combobox = manage(new Gtk::ComboBoxText);
     Shieldsetlist *sl = Shieldsetlist::getInstance();
-    std::list<std::string> shield_themes = sl->getValidNames();
-    for (std::list<std::string>::iterator i = shield_themes.begin(),
+    std::list<Glib::ustring> shield_themes = sl->getValidNames();
+    for (std::list<Glib::ustring>::iterator i = shield_themes.begin(),
 	 end = shield_themes.end(); i != end; ++i)
       {
 	if (*i == _("Default"))
 	  default_id = counter;
-	shield_theme_combobox->append_text(Glib::filename_to_utf8(*i));
+	shield_theme_combobox->append(Glib::filename_to_utf8(*i));
 	counter++;
       }
 
@@ -136,7 +130,7 @@ NewMapDialog::NewMapDialog()
     add_fill_style(Tile::MOUNTAIN);
     add_fill_style(Tile::SWAMP);
 
-    fill_style_combobox->append_text(_("Random"));
+    fill_style_combobox->append(_("Random"));
     fill_style.push_back(-1);
 
     Gtk::Alignment *alignment;
@@ -164,16 +158,6 @@ NewMapDialog::NewMapDialog()
     height_spinbutton->set_value(MAP_SIZE_TINY_HEIGHT);
 }
 
-NewMapDialog::~NewMapDialog()
-{
-  delete dialog;
-}
-
-void NewMapDialog::set_parent_window(Gtk::Window &parent)
-{
-  dialog->set_transient_for(parent);
-}
-
 void NewMapDialog::run()
 {
   dialog->show_all();
@@ -207,18 +191,18 @@ void NewMapDialog::run()
       assert(row >= 0 && row < int(fill_style.size()));
       map.fill_style = fill_style[row];
 
-      map.tileset = Tilesetlist::getInstance()->getTilesetDir
+      map.tileset = Tilesetlist::getInstance()->getSetDir
 	(Glib::filename_from_utf8(tile_theme_combobox->get_active_text()),
 	 get_active_tile_size());
 
-      map.shieldset = Shieldsetlist::getInstance()->getShieldsetDir
+      map.shieldset = Shieldsetlist::getInstance()->getSetDir
 	(Glib::filename_from_utf8(shield_theme_combobox->get_active_text()));
 
-      map.cityset = Citysetlist::getInstance()->getCitysetDir
+      map.cityset = Citysetlist::getInstance()->getSetDir
 	(Glib::filename_from_utf8(city_theme_combobox->get_active_text()),
 	 get_active_tile_size());
 
-      map.armyset = Armysetlist::getInstance()->getArmysetDir
+      map.armyset = Armysetlist::getInstance()->getSetDir
 	(Glib::filename_from_utf8(army_theme_combobox->get_active_text()),
 	 get_active_tile_size());
 
@@ -234,6 +218,7 @@ void NewMapDialog::run()
 	  map.ruins = int(ruins_scale->get_value());
 	  map.temples = int(temples_scale->get_value());
 	  map.signposts = int(signposts_scale->get_value());
+          map.generate_roads = roads_checkbutton->get_active();
 	}
 
       map_set = true;
@@ -248,6 +233,7 @@ void NewMapDialog::on_fill_style_changed()
   assert(row >= 0 && row < int(fill_style.size()));
   bool random_selected = fill_style[row] == -1;
   random_map_container->set_sensitive(random_selected);
+  roads_checkbutton->set_sensitive(random_selected);
 }
 
 void NewMapDialog::on_map_size_changed()
@@ -287,7 +273,7 @@ void NewMapDialog::add_fill_style(Tile::Type tile_type)
 {
   Tileset *tileset = GameMap::getTileset();
   Tile *tile = (*tileset)[tileset->getIndex(tile_type)];
-  fill_style_combobox->append_text(tile->getName());
+  fill_style_combobox->append(tile->getName());
   fill_style.push_back(tile_type);
 }
 
@@ -301,15 +287,15 @@ void NewMapDialog::on_tile_size_changed()
   guint32 default_id = 0;
   guint32 counter = 0;
 
-  tile_theme_combobox->clear_items();
+  tile_theme_combobox->remove_all();
   Tilesetlist *tl = Tilesetlist::getInstance();
-  std::list<std::string> tile_themes = tl->getValidNames(get_active_tile_size());
-  for (std::list<std::string>::iterator i = tile_themes.begin(),
+  std::list<Glib::ustring> tile_themes = tl->getValidNames(get_active_tile_size());
+  for (std::list<Glib::ustring>::iterator i = tile_themes.begin(),
        end = tile_themes.end(); i != end; ++i)
     {
       if (*i == _("Default"))
 	default_id = counter;
-      tile_theme_combobox->append_text(Glib::filename_to_utf8(*i));
+      tile_theme_combobox->append(Glib::filename_to_utf8(*i));
       counter++;
     }
 
@@ -317,17 +303,17 @@ void NewMapDialog::on_tile_size_changed()
   if (tile_theme_combobox->get_children().size() == 0)
     accept_button->set_sensitive(false);
 
-  army_theme_combobox->clear_items();
+  army_theme_combobox->remove_all();
   Armysetlist *al = Armysetlist::getInstance();
-  std::list<std::string> army_themes = al->getValidNames(get_active_tile_size());
+  std::list<Glib::ustring> army_themes = al->getValidNames(get_active_tile_size());
   counter = 0;
   default_id = 0;
-  for (std::list<std::string>::iterator i = army_themes.begin(),
+  for (std::list<Glib::ustring>::iterator i = army_themes.begin(),
        end = army_themes.end(); i != end; ++i)
     {
       if (*i == _("Default"))
 	default_id = counter;
-      army_theme_combobox->append_text(Glib::filename_to_utf8(*i));
+      army_theme_combobox->append(Glib::filename_to_utf8(*i));
       counter++;
     }
 
@@ -335,17 +321,17 @@ void NewMapDialog::on_tile_size_changed()
   if (army_theme_combobox->get_children().size() == 0)
     accept_button->set_sensitive(false);
 
-  city_theme_combobox->clear_items();
+  city_theme_combobox->remove_all();
   Citysetlist *cl = Citysetlist::getInstance();
-  std::list<std::string> city_themes = cl->getValidNames(get_active_tile_size());
+  std::list<Glib::ustring> city_themes = cl->getValidNames(get_active_tile_size());
   counter = 0;
   default_id = 0;
-  for (std::list<std::string>::iterator i = city_themes.begin(),
+  for (std::list<Glib::ustring>::iterator i = city_themes.begin(),
        end = city_themes.end(); i != end; ++i)
     {
       if (*i == _("Default"))
 	default_id = counter;
-      city_theme_combobox->append_text(Glib::filename_to_utf8(*i));
+      city_theme_combobox->append(Glib::filename_to_utf8(*i));
       counter++;
     }
 
