@@ -1,4 +1,4 @@
-//  Copyright (C) 2008, 2009 Ben Asselstine
+//  Copyright (C) 2008, 2009, 2014 Ben Asselstine
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -23,24 +23,17 @@
 
 #include "select-hidden-ruin-dialog.h"
 
-#include "glade-helpers.h"
-#include "gui/input-helpers.h"
 #include "ucompose.hpp"
 #include "defs.h"
 #include "ruin.h"
 #include "ruinlist.h"
 #include "playerlist.h"
 
-SelectHiddenRuinDialog::SelectHiddenRuinDialog()
+SelectHiddenRuinDialog::SelectHiddenRuinDialog(Gtk::Window &parent)
+ : LwEditorDialog(parent, "select-hidden-ruin-dialog.ui")
 {
     selected_hidden_ruin = 0;
     
-    Glib::RefPtr<Gtk::Builder> xml
-	= Gtk::Builder::create_from_file(get_glade_path()
-				    + "/select-hidden-ruin-dialog.ui");
-
-    xml->get_widget("dialog", dialog);
-    
     xml->get_widget("select_button", select_button);
 
     xml->get_widget("hidden_ruins_treeview", hidden_ruins_treeview);
@@ -69,10 +62,6 @@ SelectHiddenRuinDialog::SelectHiddenRuinDialog()
 
 }
 
-SelectHiddenRuinDialog::~SelectHiddenRuinDialog()
-{
-  delete dialog;
-}
 void SelectHiddenRuinDialog::addHiddenRuin(Ruin *ruin)
 {
     
@@ -84,12 +73,6 @@ void SelectHiddenRuinDialog::addHiddenRuin(Ruin *ruin)
   (*i)[hidden_ruins_columns.ruin] = ruin;
 }
 
-void SelectHiddenRuinDialog::set_parent_window(Gtk::Window &parent)
-{
-    dialog->set_transient_for(parent);
-    //dialog->set_position(Gtk::WIN_POS_CENTER_ON_PARENT);
-}
-
 void SelectHiddenRuinDialog::run()
 {
     dialog->show_all();
