@@ -1,4 +1,4 @@
-//  Copyright (C) 2008, 2009, 2011 Ben Asselstine
+//  Copyright (C) 2008, 2009, 2011, 2014 Ben Asselstine
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -18,28 +18,22 @@
 #ifndef SELECT_ITEM_DIALOG_H
 #define SELECT_ITEM_DIALOG_H
 
-#include <memory>
-#include <vector>
-#include <sigc++/trackable.h>
 #include <gtkmm.h>
+#include "lw-editor-dialog.h"
 
 class ItemProto;
 
 //! Scenario editor.  Select an ItemProto object from the Itemlist.
-class SelectItemDialog: public sigc::trackable
+class SelectItemDialog: public LwEditorDialog
 {
  public:
-    SelectItemDialog();
-    ~SelectItemDialog();
-
-    void set_parent_window(Gtk::Window &parent);
+    SelectItemDialog(Gtk::Window &parent);
+    ~SelectItemDialog() {};
 
     void run();
-
     const ItemProto *get_selected_item(guint32 &id) { id = selected_item_type_id; return selected_item; }
     
  private:
-    Gtk::Dialog* dialog;
     Gtk::Button *select_button;
 
     const ItemProto *selected_item;
