@@ -1,4 +1,4 @@
-//  Copyright (C) 2007, 2008, 2009, 2010 Ben Asselstine
+//  Copyright (C) 2007, 2008, 2009, 2010, 2014 Ben Asselstine
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -23,25 +23,17 @@
 #include "shieldset-info-dialog.h"
 #include "shieldsetlist.h"
 
-#include <string.h>
-#include <stdlib.h>
-#include <ctype.h>
-#include "glade-helpers.h"
 #include "ucompose.hpp"
 #include "defs.h"
 #include "File.h"
 
 
-ShieldSetInfoDialog::ShieldSetInfoDialog(Shieldset *shieldset, std::string dir, std::string file, bool readonly, std::string title)
+ShieldSetInfoDialog::ShieldSetInfoDialog(Gtk::Window &parent, Set *shieldset, Glib::ustring dir, Glib::ustring file, bool readonly, Glib::ustring title)
+ : LwEditorDialog(parent, "shieldset-info-dialog.ui")
 {
   d_shieldset = shieldset;
   d_readonly = readonly;
     
-    Glib::RefPtr<Gtk::Builder> xml
-	= Gtk::Builder::create_from_file(get_glade_path()
-				    + "/shieldset-info-dialog.ui");
-
-    xml->get_widget("dialog", dialog);
     if (title != "")
       dialog->set_title(title);
 
@@ -61,10 +53,10 @@ ShieldSetInfoDialog::ShieldSetInfoDialog(Shieldset *shieldset, std::string dir,
     else
       {
         guint32 num = 0;
-        std::string basename = Shieldsetlist::getInstance()->findFreeBaseName(_("untitled"), 100, num);
+        Glib::ustring basename = Shieldsetlist::getInstance()->findFreeBaseName(_("untitled"), 100, num);
         filename_entry->set_text(basename);
 
-        std::string name = String::ucompose("%1 %2", _("Untitled"), num);
+        Glib::ustring name = String::ucompose("%1 %2", _("Untitled"), num);
         name_entry->set_text(name);
       }
     if (readonly == false)
@@ -102,16 +94,6 @@ void ShieldSetInfoDialog::on_name_changed()
   update_buttons();
 }
 
-ShieldSetInfoDialog::~ShieldSetInfoDialog()
-{
-  delete dialog;
-}
-void ShieldSetInfoDialog::set_parent_window(Gtk::Window &parent)
-{
-    dialog->set_transient_for(parent);
-    //dialog->set_position(Gtk::WIN_POS_CENTER_ON_PARENT);
-}
-
 int ShieldSetInfoDialog::run()
 {
     dialog->show_all();
@@ -139,8 +121,7 @@ void ShieldSetInfoDialog::update_buttons()
       return;
     }
 
-  std::string dir = File::getUserShieldsetDir() + filename_entry->get_text();
-  if (Shieldsetlist::getInstance()->getShieldset(filename_entry->get_text()))
+  if (Shieldsetlist::getInstance()->get(filename_entry->get_text()))
     {
       accept_button->set_sensitive(false);
       status_label->set_markup(String::ucompose("<b>%1</b>", 
