@@ -1,5 +1,5 @@
 //  Copyright (C) 2007 Ole Laursen
-//  Copyright (C) 2007, 2008, 2009 Ben Asselstine
+//  Copyright (C) 2007, 2008, 2009, 2014 Ben Asselstine
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -19,26 +19,23 @@
 #ifndef SIGNPOST_EDITOR_DIALOG_H
 #define SIGNPOST_EDITOR_DIALOG_H
 
-#include <memory>
-#include <sigc++/trackable.h>
 #include <gtkmm.h>
 
+#include "lw-editor-dialog.h"
+
 class Signpost;
 class CreateScenarioRandomize;
 
 //! Scenario editor.  Change the contents of a signpost.
-class SignpostEditorDialog: public sigc::trackable
+class SignpostEditorDialog: public LwEditorDialog
 {
  public:
-    SignpostEditorDialog(Signpost *signpost, CreateScenarioRandomize *randomizer);
-    ~SignpostEditorDialog();
-
-    void set_parent_window(Gtk::Window &parent);
+    SignpostEditorDialog(Gtk::Window &parent, Signpost *signpost, CreateScenarioRandomize *randomizer);
+    ~SignpostEditorDialog() {};
 
     int run();
     
  private:
-    Gtk::Dialog* dialog;
     Gtk::TextView *sign_textview;
     Signpost *signpost;
     Gtk::Button *randomize_button;
