@@ -1,5 +1,5 @@
 //  Copyright (C) 2007 Ole Laursen
-//  Copyright (C) 2007, 2008, 2009 Ben Asselstine
+//  Copyright (C) 2007, 2008, 2009, 2014 Ben Asselstine
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -19,27 +19,23 @@
 #ifndef TEMPLE_EDITOR_DIALOG_H
 #define TEMPLE_EDITOR_DIALOG_H
 
-#include <memory>
-#include <sigc++/trackable.h>
 #include <gtkmm.h>
+#include "lw-editor-dialog.h"
 
 class CreateScenarioRandomize;
 
 class Temple;
 
 //! Scenario editor.  Edits a Temple object.
-class TempleEditorDialog: public sigc::trackable
+class TempleEditorDialog: public LwEditorDialog
 {
  public:
-    TempleEditorDialog(Temple *temple, CreateScenarioRandomize *randomizer);
-    ~TempleEditorDialog();
-
-    void set_parent_window(Gtk::Window &parent);
+    TempleEditorDialog(Gtk::Window &parent, Temple *temple, CreateScenarioRandomize *randomizer);
+    ~TempleEditorDialog() {};
 
     int run();
     
  private:
-    Gtk::Dialog* dialog;
     Gtk::Entry *name_entry;
     Gtk::Entry *description_entry;
     Gtk::SpinButton *type_entry;
