@@ -1,4 +1,4 @@
-//  Copyright (C) 2008, 2009, 2010 Ben Asselstine
+//  Copyright (C) 2008, 2009, 2010, 2014 Ben Asselstine
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -21,17 +21,14 @@
 
 #include "tile-preview-scene.h"
 
-#include "glade-helpers.h"
-#include "gui/image-helpers.h"
-#include "gui/input-helpers.h"
 #include "ucompose.hpp"
 #include "defs.h"
 #include "File.h"
-#include "GraphicsCache.h"
+#include "ImageCache.h"
 
 TilePreviewScene::TilePreviewScene (Tile *tile, Tile *secondary_tile,
 				    guint32 height, guint32 width, 
-				    std::string scene, guint32 tilesize)
+				    Glib::ustring scene, guint32 tilesize)
 {
   struct tile_model model;
   std::list<struct tile_model> tilescene;
@@ -90,7 +87,7 @@ void TilePreviewScene::regenerate()
 	d_view.push_back(tilestyle->getImage()->to_pixbuf());
       else
         {
-          PixMask *img = GraphicsCache::getInstance()->getDefaultTileStylePic(model.type, d_tilesize)->copy();
+          PixMask *img = ImageCache::getInstance()->getDefaultTileStylePic(model.type, d_tilesize)->copy();
           d_view.push_back(img->to_pixbuf());
         }
     }
