@@ -1,4 +1,4 @@
-//  Copyright (C) 2009, 2010 Ben Asselstine
+//  Copyright (C) 2009, 2010, 2014 Ben Asselstine
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -18,45 +18,42 @@
 #ifndef TILESET_EXPLOSION_PICTURE_EDITOR_DIALOG_H
 #define TILESET_EXPLOSION_PICTURE_EDITOR_DIALOG_H
 
-#include <memory>
 #include <map>
-#include <sigc++/trackable.h>
 #include <sigc++/connection.h>
 #include <gtkmm.h>
 #include "tileset.h"
-#include "tile-preview-scene.h"
+#include "lw-editor-dialog.h"
 
+class TilePreviewScene;
 
 //! Tileset explosion picture editor.  
-class TilesetExplosionPictureEditorDialog: public sigc::trackable
+class TilesetExplosionPictureEditorDialog: public LwEditorDialog
 {
  public:
-    TilesetExplosionPictureEditorDialog(Tileset * tileset);
-    ~TilesetExplosionPictureEditorDialog();
+    TilesetExplosionPictureEditorDialog(Gtk::Window &parent, Tileset * tileset);
+    ~TilesetExplosionPictureEditorDialog() {};
 
-    void set_parent_window(Gtk::Window &parent);
-
-    void set_icon_from_file(std::string name) {dialog->set_icon_from_file(name);};
-    std::string get_selected_filename() {return selected_filename;};
+    Glib::ustring get_selected_filename() {return selected_filename;};
     int run();
     
  private:
-    Gtk::Dialog* dialog;
     Gtk::RadioButton *large_explosion_radiobutton;
     Gtk::RadioButton *small_explosion_radiobutton;
     Gtk::FileChooserButton *explosion_filechooserbutton;
     Gtk::Image *scene_image;
     Tileset *d_tileset;
-    std::string selected_filename;
-    std::list<std::string> delfiles;
+    Glib::ustring selected_filename;
+    std::list<Glib::ustring> delfiles;
 
     void on_image_chosen();
     void on_large_toggled();
     void on_small_toggled();
-    void show_explosion_image(std::string filename);
+    void show_explosion_image(Glib::ustring filename);
     void update_panel();
 
-    void update_scene(TilePreviewScene *scene, std::string filename);
+    void update_scene(TilePreviewScene *scene, Glib::ustring filename);
+    void on_add(Gtk::Widget *widget);
+    void on_button_pressed();
 
 };
 
