@@ -1,4 +1,4 @@
-//  Copyright (C) 2007, 2008, 2009, 2010 Ben Asselstine
+//  Copyright (C) 2007, 2008, 2009, 2010, 2014 Ben Asselstine
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -23,25 +23,17 @@
 #include "tileset-info-dialog.h"
 #include "tilesetlist.h"
 
-#include <string.h>
-#include <stdlib.h>
-#include <ctype.h>
-#include "glade-helpers.h"
 #include "ucompose.hpp"
 #include "defs.h"
 #include "File.h"
 
 
-TileSetInfoDialog::TileSetInfoDialog(Tileset *tileset, std::string dir, std::string file, bool readonly, std::string title)
+TileSetInfoDialog::TileSetInfoDialog(Gtk::Window &parent, Set *tileset, Glib::ustring dir, Glib::ustring file, bool readonly, Glib::ustring title)
+ : LwEditorDialog(parent, "tileset-info-dialog.ui")
 {
   d_tileset = tileset;
   d_readonly = readonly;
     
-    Glib::RefPtr<Gtk::Builder> xml
-	= Gtk::Builder::create_from_file(get_glade_path()
-				    + "/tileset-info-dialog.ui");
-
-    xml->get_widget("dialog", dialog);
     if (title != "")
       dialog->set_title(title);
 
@@ -56,16 +48,15 @@ TileSetInfoDialog::TileSetInfoDialog(Tileset *tileset, std::string dir, std::str
 	(sigc::mem_fun(this, &TileSetInfoDialog::on_name_changed));
     
     xml->get_widget("filename_entry", filename_entry);
-    printf("file is `%s'\n", tileset->getBaseName().c_str());
     if (file != "")
       filename_entry->set_text(file);
     else
       {
         guint32 num = 0;
-        std::string basename = Tilesetlist::getInstance()->findFreeBaseName(_("untitled"), 100, num);
+        Glib::ustring basename = Tilesetlist::getInstance()->findFreeBaseName(_("untitled"), 100, num);
         filename_entry->set_text(basename);
 
-        std::string name = String::ucompose("%1 %2", _("Untitled"), num);
+        Glib::ustring name = String::ucompose("%1 %2", _("Untitled"), num);
         name_entry->set_text(name);
       }
     if (readonly == false)
@@ -103,16 +94,6 @@ void TileSetInfoDialog::on_name_changed()
   update_buttons();
 }
 
-TileSetInfoDialog::~TileSetInfoDialog()
-{
-  delete dialog;
-}
-void TileSetInfoDialog::set_parent_window(Gtk::Window &parent)
-{
-    dialog->set_transient_for(parent);
-    //dialog->set_position(Gtk::WIN_POS_CENTER_ON_PARENT);
-}
-
 int TileSetInfoDialog::run()
 {
     dialog->show_all();
@@ -140,8 +121,7 @@ void TileSetInfoDialog::update_buttons()
       return;
     }
 
-  std::string dir = File::getUserTilesetDir() + filename_entry->get_text();
-  if (Tilesetlist::getInstance()->getTileset(filename_entry->get_text()))
+  if (Tilesetlist::getInstance()->get(filename_entry->get_text()))
     {
       accept_button->set_sensitive(false);
       status_label->set_markup(String::ucompose("<b>%1</b>", 
