@@ -1,4 +1,4 @@
-//  Copyright (C) 2010 Ben Asselstine
+//  Copyright (C) 2010, 2014 Ben Asselstine
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -18,26 +18,18 @@
 #ifndef TILESET_SMALLMAP_BUILDING_COLORS_DIALOG_H
 #define TILESET_SMALLMAP_BUILDING_COLORS_DIALOG_H
 
-#include <memory>
-#include <sigc++/trackable.h>
 #include <gtkmm.h>
+#include "lw-editor-dialog.h"
 class Tileset;
 
-class TilesetSmallmapBuildingColorsDialog: public sigc::trackable
+class TilesetSmallmapBuildingColorsDialog: public LwEditorDialog
 {
  public:
-    TilesetSmallmapBuildingColorsDialog(Tileset *tileset);
-    ~TilesetSmallmapBuildingColorsDialog();
+    TilesetSmallmapBuildingColorsDialog(Gtk::Window &parent, Tileset *tileset);
+    ~TilesetSmallmapBuildingColorsDialog() {};
 
-    void set_parent_window(Gtk::Window &parent);
-
-    int run();
-
-    void hide();
-    
  private:
     Tileset *d_tileset;
-    Gtk::Dialog *dialog;
     Gtk::ColorButton *road_colorbutton;
     Gtk::ColorButton *ruin_colorbutton;
     Gtk::ColorButton *temple_colorbutton;
