@@ -1,5 +1,5 @@
 // Copyright (C) 2008 Ole Laursen
-// Copyright (C) 2011 Ben Asselstine
+// Copyright (C) 2011, 2014, 2015 Ben Asselstine
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -19,23 +19,22 @@
 #ifndef GAME_CLIENT_H
 #define GAME_CLIENT_H
 
-#include "config.h"
+#include <config.h>
 
 #include <list>
+#include <glibmm.h>
 #include <memory>
 #include <sigc++/trackable.h>
 #include <sigc++/signal.h>
-class XML_Helper;
 class NetworkAction;
 class NetworkHistory;
 
-#include "network-common.h"
 #include "game-station.h"
 
 class NetworkConnection;
-class GameScenario;
 class Player;
 
+//! A remotely connected user in a networked game.
 class GameClient: public GameStation
 {
 public:
@@ -46,7 +45,7 @@ public:
   //! Deletes the singleton instance.
   static void deleteInstance();
 
-  void start(std::string host, guint32 port, std::string profile_id, std::string nick);
+  void start(Glib::ustring host, guint32 port, Glib::ustring profile_id, Glib::ustring nick);
   void disconnect();
   void request_seat_manifest();
 
@@ -59,9 +58,9 @@ public:
   void stand_up (Player *player);
   void change_name(Player *player, Glib::ustring name);
   void change_type(Player *player, int type);
-  void chat(std::string message);
+  void chat(Glib::ustring message);
 
-  std::string getHost() const{return d_host;};
+  Glib::ustring getHost() const{return d_host;};
   guint32 getPort() const{return d_port;};
 
   void sendRoundOver();
@@ -71,13 +70,14 @@ protected:
   ~GameClient();
 
 private:
-  std::auto_ptr<NetworkConnection> network_connection;
+  NetworkConnection* network_connection;
   int player_id;
 
   void sit_or_stand (Player *player, bool sit);
   void onConnected();
   void onConnectionLost();
-  bool onGotMessage(int type, std::string message);
+  bool onGotMessage(int type, Glib::ustring message);
+  void on_torn_down();
 
   void onActionDone(NetworkAction *action);
   void sendActions();
@@ -85,12 +85,12 @@ private:
   void onHistoryDone(NetworkHistory *history);
   void sendHistories();
 
-  void gotTurnOrder (std::string payload);
+  void gotTurnOrder (Glib::ustring payload);
   void gotKillPlayer(Player *player);
   void gotOffPlayer(Player *player);
 
-  void sat_down(Player *player, std::string nickname);
-  void stood_up(Player *player, std::string nickname);
+  void sat_down(Player *player, Glib::ustring nickname);
+  void stood_up(Player *player, Glib::ustring nickname);
   void name_changed (Player *player, Glib::ustring name);
   void type_changed (Player *player, int type);
 
@@ -99,7 +99,7 @@ private:
   //! A static pointer for the singleton instance.
   static GameClient * s_instance;
   bool d_connected;
-  std::string d_host;
+  Glib::ustring d_host;
   guint32 d_port;
 };
 
