@@ -1,5 +1,5 @@
 // Copyright (C) 2008 Ole Laursen
-// Copyright (C) 2008, 2011 Ben Asselstine
+// Copyright (C) 2008, 2011, 2014, 2015 Ben Asselstine
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -19,9 +19,10 @@
 #include <iostream>
 #include <sstream>
 #include <list>
-
+#include "game-parameters.h"
 #include "game-server.h"
 
+#include "File.h"
 #include "network-server.h"
 #include "game.h"
 #include "xmlhelper.h"
@@ -35,10 +36,10 @@
 #include "real_player.h"
 #include "GameScenarioOptions.h"
 #include "ucompose.hpp"
-#include "game-parameters.h"
 
 class NetworkAction;
 
+//! A helper class for GameServer.  A connected user in a network game.
 struct Participant
 {
   void *conn;
@@ -46,9 +47,9 @@ struct Participant
   std::map<guint32, bool> id_end_turn;
   std::list<NetworkAction *> actions;
   std::list<NetworkHistory *> histories;
-  std::string nickname;
+  Glib::ustring nickname;
   bool departed;
-  std::string profile_id;
+  Glib::ustring profile_id;
 };
 
 GameServer * GameServer::s_instance = 0;
@@ -78,6 +79,7 @@ GameServer::GameServer()
     (sigc::mem_fun(*this, &GameServer::on_player_finished_turn));
   local_player_moved.connect
     (sigc::mem_fun(*this, &GameServer::on_player_finished_turn));
+  d_stop = false;
 }
 
 void GameServer::notifyRoundOver()
@@ -148,7 +150,7 @@ bool GameServer::isListening()
     return false;
 }
 
-void GameServer::start(GameScenario *game_scenario, int port, std::string profile_id, std::string nick)
+void GameServer::start(GameScenario *game_scenario, int port, Glib::ustring profile_id, Glib::ustring nick)
 {
   setGameScenario(game_scenario);
   setNickname(nick);
@@ -243,7 +245,7 @@ bool GameServer::sendRoundStart()
   return nextTurn();
 }
 
-void GameServer::gotChat(void *conn, std::string message)
+void GameServer::gotChat(void *conn, Glib::ustring message)
 {
   Participant *part = findParticipantByConn(conn);
   if (part)
@@ -265,7 +267,7 @@ void GameServer::gotChat(void *conn, std::string message)
 }
 
     
-bool GameServer::onGotMessage(void *conn, int type, std::string payload)
+bool GameServer::onGotMessage(void *conn, int type, Glib::ustring payload)
 {
   //std::cerr << "got message of type " << type << std::endl;
   switch (MessageType(type)) {
@@ -372,6 +374,10 @@ bool GameServer::onGotMessage(void *conn, int type, std::string payload)
 
 void GameServer::onConnectionMade(void *conn)
 {
+  if (conn)
+    {
+      ;
+    }
   remote_participant_connected.emit();
 }
 
@@ -407,8 +413,8 @@ Participant *GameServer::findParticipantByConn(void *conn)
 
 void GameServer::onLocalNonNetworkedActionDone(NetworkAction *action)
 {
-  std::string desc = action->toString();
-  std::cerr << "Game Server got " << Action::actionTypeToString(action->getAction()->getType()) << " "<< desc <<"\n";
+  Glib::ustring desc = action->toString();
+  std::cerr << String::ucompose("Game Server got action: %1 description: %2", Action::actionTypeToString(action->getAction()->getType()), desc) << std::endl;
 
   if (action->getAction()->getType() == Action::END_TURN)
     local_player_moved.emit(action->getOwner());
@@ -421,8 +427,10 @@ void GameServer::onLocalNonNetworkedActionDone(NetworkAction *action)
       (*i)->actions.push_back(new NetworkAction (action->getAction(),
                                                  action->getOwnerId()));
       sendActions(*i);
-      clearNetworkActionlist((*i)->actions);
     }
+  for (std::list<Participant *>::iterator i = participants.begin(),
+       end = participants.end(); i != end; ++i) 
+    clearNetworkActionlist((*i)->actions);
 
   //do it here.
   delete action;
@@ -436,15 +444,17 @@ void GameServer::onActionDone(NetworkAction *action)
   if (p->getType() != Player::NETWORKED)
     onLocalNonNetworkedActionDone(action);
   else
-    ;//we don't want anything to do with actions generated by the local
-  //networked player.
+    {
+      ;//we don't want anything to do with actions generated by the local
+      //networked player.
+    }
 
 }
 
 void GameServer::onLocalNonNetworkedHistoryDone(NetworkHistory *history)
 {
-  std::string desc = history->toString();
-  std::cerr << "Game Server got " << History::historyTypeToString(history->getHistory()->getType())<< " " << desc <<"\n";
+  Glib::ustring desc = history->toString();
+  std::cerr << String::ucompose("Game Server got history: %1 %2", History::historyTypeToString(history->getHistory()->getType()), desc) << std::endl;
 
   if (history->getHistory()->getType() == History::PLAYER_VANQUISHED)
     local_player_died(history->getOwner());
@@ -463,7 +473,7 @@ void GameServer::onLocalNonNetworkedHistoryDone(NetworkHistory *history)
 void GameServer::onLocalNetworkedHistoryDone(NetworkHistory *history)
 {
   //okay we only care about two locally generated history events.
-  std::string desc = history->toString();
+  Glib::ustring desc = history->toString();
 
   for (std::list<Participant *>::iterator i = participants.begin(),
        end = participants.end(); i != end; ++i) 
@@ -473,12 +483,12 @@ void GameServer::onLocalNetworkedHistoryDone(NetworkHistory *history)
       if (history->getHistory()->getType() == History::GOLD_TOTAL ||
           history->getHistory()->getType() == History::SCORE)
         {
-          std::cerr << "Game Server got locally generated networked history event: " << desc <<"\n";
+          std::cerr << String::ucompose("Game Server got locally generated networked history event: %1", desc) << std::endl;
           sendHistories(*i);
         }
       else
         {
-          std::cerr << "Game Server got locally generated networked history event but not sending: " << desc <<"\n";
+          std::cerr << String::ucompose("Game Server got locally generated networked history event but not sending: %1", desc) << std::endl;
         }
 
       clearNetworkHistorylist((*i)->histories);
@@ -495,7 +505,7 @@ void GameServer::onHistoryDone(NetworkHistory *history)
     onLocalNetworkedHistoryDone(history);
 }
 
-void GameServer::notifyJoin(std::string nickname)
+void GameServer::notifyJoin(Glib::ustring nickname)
 {
   remote_participant_joins.emit(nickname);
   for (std::list<Participant *>::iterator i = participants.begin(),
@@ -509,7 +519,7 @@ void GameServer::notifyJoin(std::string nickname)
   gotChatMessage("[server]", nickname + " connected.");
 }
 
-void GameServer::notifyDepart(void *conn, std::string nickname)
+void GameServer::notifyDepart(void *conn, Glib::ustring nickname)
 {
   remote_participant_departs.emit(nickname);
   for (std::list<Participant *>::iterator i = participants.begin(),
@@ -525,7 +535,7 @@ void GameServer::notifyDepart(void *conn, std::string nickname)
   gotChatMessage("", nickname + " disconnected.");
 }
 
-void GameServer::notifySit(Player *player, std::string nickname)
+void GameServer::notifySit(Player *player, Glib::ustring nickname)
 {
   if (!player)
     return;
@@ -635,17 +645,17 @@ Glib::ustring GameServer::make_nickname_unique(Glib::ustring nickname)
   return new_nickname;
 }
 
-void GameServer::join(void *conn, std::string nickname_and_profile_id)
+void GameServer::join(void *conn, Glib::ustring nickname_and_profile_id)
 {
   bool new_participant = false;
   std::cout << "JOIN: " << conn << std::endl;
 
   size_t pos;
   pos = nickname_and_profile_id.rfind(' ');
-  if (pos == std::string::npos)
+  if (pos == Glib::ustring::npos)
     return;
-  std::string nickname = nickname_and_profile_id.substr(0, pos);
-  std::string profile_id = nickname_and_profile_id.substr(pos + 1);
+  Glib::ustring nickname = nickname_and_profile_id.substr(0, pos);
+  Glib::ustring profile_id = nickname_and_profile_id.substr(pos + 1);
   Participant *part = findParticipantByConn(conn);
   if (!part) {
     part = new Participant;
@@ -701,7 +711,7 @@ bool GameServer::player_already_sitting(Player *p)
   return false;
 }
 
-void GameServer::sit(void *conn, Player *player, std::string nickname)
+void GameServer::sit(void *conn, Player *player, Glib::ustring nickname)
 {
   std::cout << "SIT: " << conn << " " << player << std::endl;
 
@@ -755,7 +765,7 @@ void GameServer::change_type(void *conn, Player *player, int type)
   notifyTypeChange(player, type);
 }
 
-void GameServer::notifyStand(Player *player, std::string nickname)
+void GameServer::notifyStand(Player *player, Glib::ustring nickname)
 {
   if (!player)
     return;
@@ -849,7 +859,7 @@ GameServer::remove_from_player_list(std::list<GameParameters::Player> &list, gui
   return false;
 }
 
-void GameServer::stand(void *conn, Player *player, std::string nickname)
+void GameServer::stand(void *conn, Player *player, Glib::ustring nickname)
 {
   std::cout << "STAND: " << conn << " " << player << std::endl;
   if (!player || !conn)
@@ -871,7 +881,7 @@ void GameServer::stand(void *conn, Player *player, std::string nickname)
   notifyStand(player, nickname);
 }
 
-void GameServer::gotRemoteActions(void *conn, const std::string &payload)
+void GameServer::gotRemoteActions(void *conn, const Glib::ustring &payload)
 {
   gotActions(payload);
   for (std::list<Participant *>::iterator i = participants.begin(),
@@ -880,7 +890,7 @@ void GameServer::gotRemoteActions(void *conn, const std::string &payload)
       network_server->send((*i)->conn, MESSAGE_TYPE_SENDING_ACTIONS, payload);
 }
 
-void GameServer::gotRemoteHistory(void *conn, const std::string &payload)
+void GameServer::gotRemoteHistory(void *conn, const Glib::ustring &payload)
 {
   gotHistories(payload);
   for (std::list<Participant *>::iterator i = participants.begin(),
@@ -908,9 +918,7 @@ void GameServer::sendMap(Participant *part)
 
 
   // send the map, and save it to a file somewhere temporarily
-  std::string tmpfile = "lw.XXXX";
-  int fd = Glib::file_open_tmp(tmpfile, "lw.XXXX");
-  close(fd);
+  Glib::ustring tmpfile = File::get_tmp_file();
   File::erase(tmpfile);
   tmpfile += SAVE_EXT;
 
@@ -918,7 +926,7 @@ void GameServer::sendMap(Participant *part)
 
   std::cerr << "sending map" << std::endl;
   network_server->sendFile(part->conn, MESSAGE_TYPE_SENDING_MAP, tmpfile);
-  File::erase (tmpfile);
+  //file get erased in NetworkConnection::sendFileMessage
 
   // unhack the players
   std::vector<Player*>::iterator j = players.begin();
@@ -939,10 +947,7 @@ void GameServer::sendActions(Participant *part)
 
   for (std::list<NetworkAction *>::iterator i = part->actions.begin(),
        end = part->actions.end(); i != end; ++i)
-    {
-  std::cerr << "sending " << Action::actionTypeToString((*i)->getAction()->getType()) << " from " << d_nickname << " (" << Playerlist::getInstance()->getPlayer((*i)->getOwnerId())->getName() << ")" << " to " << part->nickname << std::endl;
     (**i).save(&helper);
-    }
 
   helper.closeTag();
 
@@ -960,8 +965,8 @@ void GameServer::sendHistories(Participant *part)
   for (std::list<NetworkHistory *>::iterator i = part->histories.begin(),
        end = part->histories.end(); i != end; ++i)
     {
-  std::cerr << "sending " << History::historyTypeToString((*i)->getHistory()->getType()) << " from " << d_nickname << " (" << Playerlist::getInstance()->getPlayer((*i)->getOwnerId())->getName() << ")" << " to " << part->nickname << std::endl;
-    (**i).save(&helper);
+      std::cerr << String::ucompose("sending history %1 from person %2 %3 to person %4", History::historyTypeToString((*i)->getHistory()->getType()), d_nickname, Playerlist::getInstance()->getPlayer((*i)->getOwnerId())->getName(), part->nickname) << std::endl;
+      (**i).save(&helper);
     }
 
   helper.closeTag();
@@ -1099,12 +1104,12 @@ void GameServer::type_change (Player *player, int type)
   player_changes_type.emit(player, type);
 }
 
-void GameServer::chat (std::string message)
+void GameServer::chat (Glib::ustring message)
 {
   notifyChat(d_nickname + ":" + message);
 }
 
-void GameServer::notifyChat(std::string message)
+void GameServer::notifyChat(Glib::ustring message)
 {
   gotChatMessage(d_nickname, message);
   for (std::list<Participant *>::iterator i = participants.begin(),
