@@ -1,4 +1,4 @@
-// Copyright (C) 2006, 2007, 2008, 2009, 2010 Ben Asselstine
+// Copyright (C) 2006, 2007, 2008, 2009, 2010, 2014, 2015 Ben Asselstine
 // Copyright (C) 2007, 2008 Ole Laursen
 //
 //  This program is free software; you can redistribute it and/or modify
@@ -16,9 +16,8 @@
 //  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 
 //  02110-1301, USA.
 
-#include "config.h"
+#include <config.h>
 
-#include <algorithm>
 #include <vector>
 #include <assert.h>
 #include <gtkmm.h>
@@ -29,16 +28,12 @@
 
 #include "ucompose.hpp"
 #include "rectangle.h"
-#include "sound.h"
-#include "GraphicsCache.h"
 #include "GameScenario.h"
 #include "NextTurnNetworked.h"
 #include "NextTurnHotseat.h"
-#include "NextTurnPbm.h"
-
+#include "stackreflist.h"
 #include "gamebigmap.h"
 #include "smallmap.h"
-
 #include "army.h"
 #include "fight.h"
 #include "hero.h"
@@ -62,24 +57,24 @@
 #include "action.h"
 #include "game-parameters.h"
 #include "FogMap.h"
-#include "GameMap.h"
 #include "history.h"
-#include "pbm-game-server.h"
 #include "LocationBox.h"
 #include "Backpack.h"
 #include "MapBackpack.h"
 #include "stacktile.h"
-
 #include "herotemplates.h"
 #include "GameScenarioOptions.h"
 #include "ai_fast.h"
 #include "ai_smart.h"
 #include "Sage.h"
 #include "Commentator.h"
+#include "select-city-map.h"
+#include "Item.h"
+#include "rnd.h"
 
 Game *Game::current_game = 0;
 
-//#define debug(x) {cerr<<__FILE__<<": "<<__LINE__<<": "<<x<<flush<<endl;}
+//#define debug(x) {std::cerr<<__FILE__<<": "<<__LINE__<<": "<<x<<std::flush<<std::endl;}
 #define debug(x)
 void Game::addPlayer(Player *p)
 {
@@ -99,12 +94,6 @@ void Game::addPlayer(Player *p)
       connections[p->getId()].push_back
 	(p->snewMedalArmy.connect(sigc::mem_fun(this, &Game::newMedalArmy)));
 
-      connections[p->getId()].push_back
-	(p->streachery.connect
-	 (sigc::bind<0>
-	  (sigc::mem_fun
-	   (stack_considers_treachery, 
-	    &sigc::signal<bool, Player *, Stack *, Player *, Vector<int> >::emit), p)));
       connections[p->getId()].push_back
 	(p->hero_arrives_with_allies.connect
          (sigc::mem_fun
@@ -114,7 +103,7 @@ void Game::addPlayer(Player *p)
 	 (sigc::mem_fun(advice_asked, &sigc::signal<void, float>::emit)));
       connections[p->getId()].push_back
 	(p->smovingStack.connect
-	 (sigc::mem_fun(this, &Game::on_stack_starts_moving)));
+	 (sigc::hide(sigc::mem_fun(this, &Game::on_stack_starts_moving))));
       connections[p->getId()].push_back
 	(p->sstoppingStack.connect
 	 (sigc::mem_fun(this, &Game::on_stack_stopped)));
@@ -123,7 +112,7 @@ void Game::addPlayer(Player *p)
 	 (sigc::mem_fun(this, &Game::on_stack_halted)));
       connections[p->getId()].push_back
 	(p->getStacklist()->sgrouped.connect
-	 (sigc::mem_fun(this, &Game::on_stack_grouped)));
+	 (sigc::hide(sigc::mem_fun(this, &Game::on_stack_grouped))));
       connections[p->getId()].push_back
 	(p->stole_gold.connect
 	 (sigc::mem_fun(stole_gold, &sigc::signal<void, Player*, 
@@ -140,9 +129,31 @@ void Game::addPlayer(Player *p)
 	 (sigc::mem_fun(mp_added_to_hero_stack, &sigc::signal<void, Hero*, guint32>::emit)));
       connections[p->getId()].push_back
 	(p->worms_killed.connect
-	 (sigc::mem_fun(worms_killed, &sigc::signal<void, Hero*, guint32>::emit)));
+	 (sigc::mem_fun(worms_killed, &sigc::signal<void, Hero*, Glib::ustring, guint32>::emit)));
+      connections[p->getId()].push_back
+	(p->bridge_burned.connect
+	 (sigc::mem_fun(bridge_burned, &sigc::signal<void, Hero*>::emit)));
 
+      connections[p->getId()].push_back
+	(p->keeper_captured.connect
+	 (sigc::mem_fun(keeper_captured, &sigc::signal<void, Hero*, Ruin*, Glib::ustring>::emit)));
 
+      connections[p->getId()].push_back
+	(p->monster_summoned.connect
+	 (sigc::mem_fun(monster_summoned, &sigc::signal<void, Hero*, Glib::ustring>::emit)));
+
+      connections[p->getId()].push_back
+	(p->city_diseased.connect
+	 (sigc::mem_fun(city_diseased, &sigc::signal<void, Hero*, Glib::ustring, guint32>::emit)));
+      connections[p->getId()].push_back
+	(p->city_defended.connect
+	 (sigc::mem_fun(city_defended, &sigc::signal<void, Hero*, Glib::ustring, Glib::ustring, guint32>::emit)));
+      connections[p->getId()].push_back
+	(p->city_persuaded.connect
+	 (sigc::mem_fun(city_persuaded, &sigc::signal<void, Hero*, Glib::ustring, guint32>::emit)));
+      connections[p->getId()].push_back
+	(p->stack_teleported.connect
+	 (sigc::mem_fun(stack_teleported, &sigc::signal<void, Hero*, Glib::ustring>::emit)));
     }
       
       
@@ -155,10 +166,10 @@ void Game::addPlayer(Player *p)
     (p->srecruitingHero.connect(sigc::mem_fun(this, &Game::recruitHero)));
   connections[p->getId()].push_back
     (p->svisitingTemple.connect
-     (sigc::mem_fun(this, &Game::stack_searches_temple)));
+     (sigc::hide<0>(sigc::mem_fun(this, &Game::stack_searches_temple))));
   connections[p->getId()].push_back
     (p->ssearchingRuin.connect
-     (sigc::mem_fun(this, &Game::stack_searches_ruin)));
+     (sigc::hide<0>(sigc::mem_fun(this, &Game::stack_searches_ruin))));
   connections[p->getId()].push_back
     (p->getStacklist()->snewpos.connect
      (sigc::mem_fun(this, &Game::stack_arrives_on_tile)));
@@ -175,7 +186,7 @@ void Game::addPlayer(Player *p)
         
   connections[p->getId()].push_back
     (p->schangingStatus.connect 
-	 (sigc::mem_fun(progress_status_changed, &sigc::signal<void, std::string>::emit)));
+	 (sigc::mem_fun(progress_status_changed, &sigc::signal<void, Glib::ustring>::emit)));
         
   connections[p->getId()].push_back
     (p->sbusy.connect (sigc::mem_fun (progress_changed, 
@@ -193,15 +204,8 @@ void Game::addPlayer(Player *p)
   connections[p->getId()].push_back
     (p->ruinfight_started.connect (sigc::mem_fun(*this, &Game::on_ruinfight_started)));
 
-    //(p->ruinfight_started.connect
-     //(sigc::mem_fun
-      //(ruinfight_started, &sigc::signal<void, Stack *, Stack *>::emit)));
   connections[p->getId()].push_back
     (p->ruinfight_finished.connect (sigc::mem_fun(*this, &Game::on_ruinfight_finished)));
-  //connections[p->getId()].push_back
-    //(p->ruinfight_finished.connect
-     //(sigc::mem_fun
-      //(ruinfight_finished, &sigc::signal<void, Fight::Result>::emit)));
   connections[p->getId()].push_back
     (p->cityfight_finished.connect (sigc::mem_fun(*this, &Game::on_city_fight_finished))); 
   if (p->getType() == Player::NETWORKED && p == Playerlist::getActiveplayer())
@@ -210,7 +214,7 @@ void Game::addPlayer(Player *p)
     unlock_inputs();
 }
 
-void Game::on_stack_starts_moving(Stack *stack)
+void Game::on_stack_starts_moving()
 {
   if (Playerlist::getActiveplayer()->getType() == Player::HUMAN)
     lock_inputs();
@@ -231,7 +235,8 @@ void Game::on_stack_halted(Stack *stack)
   bigmap->reset_path_calculator(stack);
   //tell gamebigmap that a stack just stopped
 }
-void Game::on_stack_grouped(Stack *stack, bool grouped)
+
+void Game::on_stack_grouped(Stack *stack)
 {
   bigmap->reset_path_calculator(stack);
   //tell gamebigmap that we just grouped/ungrouped a stack.
@@ -252,9 +257,9 @@ Game::Game(GameScenario* gameScenario, NextTurn *nextTurn)
 		  GameScenario::s_see_opponents_stacks, 
 		  GameScenario::s_military_advisor));
     bigmap->stack_selected.connect(
-	sigc::mem_fun(this, &Game::on_stack_selected));
+	sigc::hide(sigc::mem_fun(this, &Game::on_stack_selected)));
     bigmap->stack_grouped_or_ungrouped.connect(
-	sigc::mem_fun(this, &Game::on_stack_grouped_or_ungrouped));
+	sigc::hide(sigc::mem_fun(this, &Game::on_stack_grouped_or_ungrouped)));
     bigmap->path_set.connect(
 	sigc::mem_fun(this, &Game::update_control_panel));
     bigmap->city_visited.connect(
@@ -280,7 +285,7 @@ Game::Game(GameScenario* gameScenario, NextTurn *nextTurn)
     smallmap->resize();
     smallmap->map_changed.connect(
 	sigc::mem_fun(smallmap_changed,
-		      &sigc::signal<void, Glib::RefPtr<Gdk::Pixmap>, 
+		      &sigc::signal<void, Cairo::RefPtr<Cairo::Surface>, 
 		      Gdk::Rectangle>::emit));
 
     // connect the two maps
@@ -288,7 +293,7 @@ Game::Game(GameScenario* gameScenario, NextTurn *nextTurn)
 	sigc::mem_fun(smallmap.get(), &SmallMap::set_view));
     bigmap->map_changed.connect(
 	sigc::mem_fun(bigmap_changed,
-		      &sigc::signal<void, Glib::RefPtr<Gdk::Pixmap> >::emit));
+		      &sigc::signal<void, Cairo::RefPtr<Cairo::Surface> >::emit));
     smallmap->view_changed.connect(
 	sigc::mem_fun(bigmap.get(), &GameBigMap::set_view));
 
@@ -334,7 +339,7 @@ Game::~Game()
       connections[i].clear();
     }
     delete d_gameScenario;
-    delete d_nextTurn;
+    //delete d_nextTurn;
     
     HeroTemplates::deleteInstance();
 }
@@ -665,11 +670,12 @@ void Game::search_stack(Stack *stack, bool &gotquest, bool &stackdied)
               gotquest = true;
               if (player->isComputer() == false)
                 {
-                  Hero *hero;
                   Army *a = stack->getArmyById(q->getHeroId());
                   if (a)
-                    hero = dynamic_cast<Hero*>(a);
-                  quest_assigned.emit(hero, q);
+                    {
+                      Hero *h = dynamic_cast<Hero*>(a);
+                      quest_assigned.emit(h, q);
+                    }
                 }
             }
 	}
@@ -695,17 +701,26 @@ void Game::on_use_item(Item *item)
   Hero *hero = NULL;
   active->getItemHolder(item, &stack, &hero);
   Player *victim = NULL;
+  City *friendly_city = NULL;
+  City *enemy_city = NULL;
+  City *neutral_city = NULL;
+  City *city = NULL;
 
   //ask the user a series of questions on how to use the item
   if (item->usableOnVictimPlayer())
     victim = select_item_victim_player.emit();
-          
-  // now we act on the answers and execute the usage of the item
-  if (item->usableOnVictimPlayer() == true && victim)
-    active->heroUseItem(hero, item, victim);
+  if (item->usableOnFriendlyCity())
+    friendly_city = 
+      select_city_to_use_item_on.emit(SelectCityMap::FRIENDLY_CITY);
+  if (item->usableOnEnemyCity())
+    enemy_city = select_city_to_use_item_on.emit(SelectCityMap::ENEMY_CITY);
+  if (item->usableOnNeutralCity())
+    neutral_city = select_city_to_use_item_on.emit(SelectCityMap::NEUTRAL_CITY);
+  if (item->usableOnAnyCity())
+    city = select_city_to_use_item_on.emit(SelectCityMap::ANY_CITY);
 
-  if (item->usableOnVictimPlayer() == false)
-    active->heroUseItem(hero, item, NULL);
+  active->heroUseItem(hero, item, victim, friendly_city, enemy_city, 
+                      neutral_city, city);
 }
 
 void Game::search_selected_stack()
@@ -760,14 +775,14 @@ void Game::newMedalArmy(Army* a, int medaltype)
   update_stack_info();
 }
 
-void Game::on_stack_grouped_or_ungrouped(Stack *s)
+void Game::on_stack_grouped_or_ungrouped()
 {
   //this only happens when we double-click on a stack on the bigmap.
   update_stack_info();
   update_control_panel();
 }
 
-void Game::on_stack_selected(Stack* s)
+void Game::on_stack_selected()
 {
   update_stack_info();
   update_control_panel();
@@ -938,7 +953,6 @@ void Game::invading_city(City* city, int gold)
 void Game::lock_inputs()
 {
   // don't accept modifying user input from now on
-  bigmap->reset_zoom();
   bigmap->set_input_locked(true);
   smallmap->set_input_locked(true);
   input_locked = true;
@@ -1007,7 +1021,6 @@ void Game::update_control_panel()
 
   Stack *stack = player->getActivestack();
 
-  can_defend_selected_stack.emit(stack != 0);
   can_park_selected_stack.emit(stack != 0);
   can_deselect_selected_stack.emit(stack != 0);
   can_center_selected_stack.emit(stack != 0);
@@ -1034,6 +1047,8 @@ void Game::update_control_panel()
 
       can_disband_stack.emit(true);
       can_group_ungroup_selected_stack.emit(true);
+      //we can't defend on cities, ruins, temples, ports, or water.
+      can_defend_selected_stack.emit(GameMap::can_defend(stack));
     }
   else
     {
@@ -1043,6 +1058,7 @@ void Game::update_control_panel()
       can_group_ungroup_selected_stack.emit(false);
       can_plant_standard_selected_stack.emit(false);
       can_search_selected_stack.emit(false);
+      can_defend_selected_stack.emit(false);
     }
       
   if (d_gameScenario->getRound() > 1)
@@ -1089,9 +1105,6 @@ void Game::startGame()
 
 void Game::loadGame()
 {
-  //if pbm and human
-  //if pbm and not human, then bail
-
   Player *player = Playerlist::getActiveplayer();
   if (!player)
     {
@@ -1112,41 +1125,10 @@ void Game::loadGame()
       if (player->getType() == Player::HUMAN)
 	d_nextTurn->setContinuingTurn();
     }
-  else if (player->getType() == Player::HUMAN && 
-	   d_gameScenario->getPlayMode() == GameScenario::PLAY_BY_MAIL)
-    {
-      if (player->hasAlreadyInitializedTurn())
-	{
-	  unlock_inputs();
-	  player->setActivestack(0);
-	  center_view_on_city();
-	  update_sidebar_stats();
-	  update_control_panel();
-	  update_stack_info();
-	  game_loaded.emit(player);
-	  player->loadPbmGame();
-	  d_nextTurn->setContinuingTurn();
-	}
-      else
-	{
-	  lock_inputs();
-	}
-    }
   else
     lock_inputs();
 
   d_nextTurn->start();
-#if 0
-  else
-    {
-      lock_inputs();
-      update_sidebar_stats();
-      player->startTurn();
-      d_nextTurn->endTurn();
-      if (Playerlist::getInstance()->countPlayersAlive())
-	update_control_panel();
-    }
-#endif
 }
 
 void Game::stopGame()
@@ -1154,7 +1136,7 @@ void Game::stopGame()
   d_nextTurn->stop();
 }
 
-bool Game::saveGame(std::string file)
+bool Game::saveGame(Glib::ustring file)
 {
   return d_gameScenario->saveGame(file);
 }
@@ -1190,10 +1172,10 @@ void Game::init_turn_for_player(Player* p)
     {
       if (Commentator::getInstance()->hasComment() == true)
         {
-          std::vector<std::string> comments =
+          std::vector<Glib::ustring> comments =
             Commentator::getInstance()->getComments(p);
           if (comments.size() > 0)
-            commentator_comments.emit(comments[rand() % comments.size()]);
+            commentator_comments.emit(comments[Rnd::rand() % comments.size()]);
         }
     }
 
@@ -1356,6 +1338,8 @@ bool Game::maybeTreachery(Stack *stack, Player *them, Vector<int> pos)
           treachery = ai->chooseTreachery (stack, them, pos);
         }
     }
+  else
+    treachery = stack_considers_treachery.emit(stack, them, pos);
   if (treachery == false)
     return false;
   me->proposeDiplomacy (Player::NO_PROPOSAL, them);
@@ -1371,12 +1355,19 @@ bool Game::maybeTreachery(Stack *stack, Player *them, Vector<int> pos)
 
 void Game::nextRound()
 {
-  if (GameServer::getInstance()->isListening())
+  if (d_gameScenario->getPlayMode() == GameScenario::NETWORKED)
     {
-      Playerlist::getInstance()->nextRound
-        (GameScenarioOptions::s_diplomacy, 
-         &GameScenarioOptions::s_surrender_already_offered);
+      if (GameServer::getInstance()->isListening())
+        {
+          Playerlist::getInstance()->nextRound
+            (GameScenarioOptions::s_diplomacy, 
+             &GameScenarioOptions::s_surrender_already_offered);
+        }
     }
+  else
+    Playerlist::getInstance()->nextRound
+      (GameScenarioOptions::s_diplomacy, 
+       &GameScenarioOptions::s_surrender_already_offered);
 }
     
 void Game::on_surrender_offered(Player *recipient)
@@ -1453,15 +1444,6 @@ bool Game::recruitHero(HeroProto *hero, City *city, int gold)
   return retval;
 }
     
-bool Game::saveTurnFile(std::string turnfile)
-{
-  bool broken;
-  //trigger the GameServer to spit out a set of networkactions and networkhistory events for the active player, into a file.
-  if (d_gameScenario->getPlayMode() == GameScenario::PLAY_BY_MAIL)
-    PbmGameServer::getInstance()->endTurn(turnfile, broken);
-  return true;
-}
-
 void Game::inhibitAutosaveRemoval(bool inhibit)
 {
   if (d_gameScenario)
@@ -1501,14 +1483,14 @@ void Game::stack_leaves_tile(Stack *stack, Vector<int> tile)
     {
       if (stack == NULL)
 	{
-	  printf("stack is %p\n", stack);
+	  printf("stack is %p\n", (void*)stack);
 	  printf("WTFFF!!!!!!!!!!!!!!!!!!!!\n");
 	  return;
 	}
     }
 }
-    
-bool Game::stack_searches_ruin(Ruin *ruin, Stack *stack)
+
+bool Game::stack_searches_ruin(Stack *stack)
 {
   bool stack_died = false;
   bool hero_got_quest = false;
@@ -1516,7 +1498,7 @@ bool Game::stack_searches_ruin(Ruin *ruin, Stack *stack)
   return stack_died;
 }
     
-bool Game::stack_searches_temple(Temple *temple, Stack *stack)
+bool Game::stack_searches_temple(Stack *stack)
 {
   bool stack_died = false;
   bool hero_got_quest = false;
