@@ -1,4 +1,4 @@
-// Copyright (C) 2006, 2007, 2008, 2009, 2010 Ben Asselstine
+// Copyright (C) 2006, 2007, 2008, 2009, 2010, 2014, 2015 Ben Asselstine
 // Copyright (C) 2007, 2008 Ole Laursen
 //
 //  This program is free software; you can redistribute it and/or modify
@@ -24,17 +24,14 @@
 #include <sigc++/connection.h>
 #include <memory>
 #include <list>
-#include <string>
 
-#include "rectangle.h"
 #include "sidebar-stats.h"
 #include "map-tip-position.h"
 #include "callback-enums.h"
 #include "army.h"
 #include "fight.h"
-#include "game-parameters.h"
 #include "player.h"
-#include "defs.h"
+#include "select-city-map.h"
 
 class NextTurn;
 class GameBigMap;
@@ -49,11 +46,11 @@ class Ruin;
 class Signpost;
 class Fight;
 class Quest;
-class Stack;
 class Reward;
 class StackTile;
 class Sage;
 
+//! Connects the various game classes with the GameWindow through signals.
 /** Controls a game.
   * 
   * Manages the big and small map, the game scenario and who's turn it is, etc.
@@ -90,13 +87,13 @@ class Game
     void move_all_stacks();
     void end_turn();
     void recalculate_moves_for_stack(Stack *s);
+    void update_sidebar_stats();
 
     void startGame(); // initiate game flow
     void loadGame();
     void stopGame(); // stop game flow, clean up
     // save current game, returns true if successful
-    bool saveGame(std::string file);
-    bool saveTurnFile(std::string file);
+    bool saveGame(Glib::ustring file);
 
     static GameScenario *getScenario();
     GameBigMap &get_bigmap();
@@ -104,10 +101,10 @@ class Game
     
     // signals
     sigc::signal<void, Vector<int> > current_map_position;
-    sigc::signal<void, Glib::RefPtr<Gdk::Pixmap>, Gdk::Rectangle> smallmap_changed;
-    sigc::signal<void, Glib::RefPtr<Gdk::Pixmap> > bigmap_changed;
+    sigc::signal<void, Cairo::RefPtr<Cairo::Surface>, Gdk::Rectangle> smallmap_changed;
+    sigc::signal<void, Cairo::RefPtr<Cairo::Surface> > bigmap_changed;
     sigc::signal<void, SidebarStats> sidebar_stats_changed;
-    sigc::signal<void, std::string> progress_status_changed;
+    sigc::signal<void, Glib::ustring> progress_status_changed;
     sigc::signal<void> progress_changed;
     sigc::signal<void, bool>
 	can_select_next_movable_stack,
@@ -145,7 +142,7 @@ class Game
     sigc::signal<bool, Player *, HeroProto *, City *, int> hero_offers_service;
     sigc::signal<bool, int > enemy_offers_surrender;
     sigc::signal<void, bool> surrender_answered;
-    sigc::signal<bool, Player *, Stack *, Player *, Vector<int> > stack_considers_treachery;
+    sigc::signal<bool, Stack *, Player *, Vector<int> > stack_considers_treachery;
     sigc::signal<bool, Hero *, Temple *, int> temple_searched;
     sigc::signal<void, Hero *, Quest *> quest_assigned;
     sigc::signal<CityDefeatedAction, City *, int> city_defeated;
@@ -164,17 +161,25 @@ class Game
     sigc::signal<void, Player *> game_over;
     sigc::signal<void, Player *> player_died;
     sigc::signal<void> game_stopped;
-    sigc::signal<void, std::string> commentator_comments;
+    sigc::signal<void, Glib::ustring> commentator_comments;
     sigc::signal<void, Stack*, Vector<int> > stack_moves;
     sigc::signal<Item*, std::list<Item*> > select_item;
     sigc::signal<Player*> select_item_victim_player;
+    sigc::signal<City*, SelectCityMap::Type> select_city_to_use_item_on;
 
     //! Results of using items
     sigc::signal<void, Player*, guint32> stole_gold;
     sigc::signal<void, Player*, guint32> sunk_ships;
     sigc::signal<void, Hero*, guint32> bags_picked_up;
     sigc::signal<void, Hero *, guint32> mp_added_to_hero_stack;
-    sigc::signal<void, Hero *, guint32> worms_killed;
+    sigc::signal<void, Hero *, Glib::ustring, guint32> worms_killed;
+    sigc::signal<void, Hero *> bridge_burned;
+    sigc::signal<void, Hero *, Ruin*, Glib::ustring> keeper_captured;
+    sigc::signal<void, Hero *, Glib::ustring> monster_summoned;
+    sigc::signal<void, Hero *, Glib::ustring, guint32> city_diseased;
+    sigc::signal<void, Hero *, Glib::ustring, Glib::ustring, guint32> city_defended;
+    sigc::signal<void, Hero *, Glib::ustring, guint32> city_persuaded;
+    sigc::signal<void, Hero *, Glib::ustring> stack_teleported;
     
     void addPlayer(Player *p);
 
@@ -192,7 +197,6 @@ class Game
     void center_view_on_city();
 
     void update_control_panel();
-    void update_sidebar_stats();
     void update_stack_info();	// emit stack_info_changed
     void clear_stack_info();
 
@@ -204,8 +208,8 @@ class Game
     bool maybeTreachery(Stack *stack, Player *them, Vector<int> pos);
 
     // bigmap callbacks
-    void on_stack_selected(Stack* s);
-    void on_stack_grouped_or_ungrouped(Stack *s);
+    void on_stack_selected();
+    void on_stack_grouped_or_ungrouped();
     void on_city_visted (City* c);
     void on_ruin_queried (Ruin* r, bool brief);
     void on_temple_queried (Temple* t, bool brief);
@@ -217,15 +221,15 @@ class Game
     void on_city_unqueried ();
 
     // smallmap callbacks
-    void on_smallmap_changed(Glib::RefPtr<Gdk::Pixmap> map);
-    void on_bigmap_changed(Glib::RefPtr<Gdk::Pixmap> map);
+    void on_smallmap_changed(Cairo::RefPtr<Cairo::Surface> map);
+    void on_bigmap_changed(Cairo::RefPtr<Cairo::Surface> map);
     
     // misc. callbacks
     void invading_city(City* city, int gold);
     void init_turn_for_player(Player* p);
     void on_player_died(Player *p);
-    bool stack_searches_ruin(Ruin *ruin, Stack *stack);
-    bool stack_searches_temple(Temple *temple, Stack *stack);
+    bool stack_searches_ruin(Stack *stack);
+    bool stack_searches_temple(Stack *stack);
     void on_use_item(Item *item);
     void on_ruinfight_started(Stack *attacker, Stack *defender);
     void on_ruinfight_finished(Fight::Result result);
@@ -249,13 +253,14 @@ class Game
     void select_active_stack();
     bool recruitHero(HeroProto *hero, City *city, int gold);
 
-    void on_stack_grouped(Stack *stack, bool grouped);
+    void on_stack_grouped(Stack *stack);
     void stack_arrives_on_tile(Stack *stack, Vector<int> tile);
     void stack_leaves_tile(Stack *stack, Vector<int> tile);
     void on_stack_halted(Stack *stack);
     void on_stack_stopped();
-    void on_stack_starts_moving(Stack *stack);
+    void on_stack_starts_moving();
 
+    bool ask_if_treachery(Stack *stack, Player *them, Vector<int> pos);
 
     GameScenario* d_gameScenario;
     NextTurn* d_nextTurn;
