@@ -1,5 +1,5 @@
 //  Copyright (C) 2007 Ole Laursen
-//  Copyright (C) 2007, 2008, 2009, 2010, 2011 Ben Asselstine
+//  Copyright (C) 2007, 2008, 2009, 2010, 2011, 2014, 2015 Ben Asselstine
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -19,8 +19,6 @@
 #include <config.h>
 
 #include <assert.h>
-#include <stdlib.h>
-#include <glibmm/timeval.h>
 
 #include "gamebigmap.h"
 
@@ -31,39 +29,24 @@
 #include "ruin.h"
 #include "signpost.h"
 #include "temple.h"
-#include "roadlist.h"
 #include "road.h"
-#include "bridgelist.h"
 #include "bridge.h"
 #include "playerlist.h"
-#include "File.h"
 #include "GameMap.h"
-#include "GraphicsCache.h"
-#include "game.h"
 #include "FogMap.h"
 #include "LocationBox.h"
 #include "Configuration.h"
-#include "gui/image-helpers.h"
 #include "PathCalculator.h"
 #include "stacktile.h"
-#include "action.h"
 #include "tileset.h"
-#include "tilesetlist.h"
 
 #include "timing.h"
 
 
 #include <iostream>
-using namespace std;
-//#define debug(x) {cerr<<__FILE__<<": "<<__LINE__<<": "<<x<<endl<<flush;}
+//#define debug(x) {std::cerr<<__FILE__<<": "<<__LINE__<<": "<<x<<std::endl<<std::flush;}
 #define debug(x)
 
-namespace 
-{
-    // controls speed of selector rotation
-    int selection_timeout = TIMER_BIGMAP_SELECTOR;
-}
-
 GameBigMap::GameBigMap(bool intense_combat, bool see_opponents_production,
 		       bool see_opponents_stacks, bool military_advisor)
 :d_fighting(LocationBox(Vector<int>(-1,-1)))
@@ -76,17 +59,16 @@ GameBigMap::GameBigMap(bool intense_combat, bool see_opponents_production,
 
   current_tile.x = current_tile.y = 0;
   mouse_state = NONE;
-  input_locked = false;
 
   prev_mouse_pos = Vector<int>(0, 0);
 
   // setup timeout
-  selection_timeout_handler = 
-    Timing::instance().register_timer
+  selection_timeout_handler = Timing::instance().register_timer
     (sigc::mem_fun(*this, &GameBigMap::on_selection_timeout),
-     selection_timeout);
+     TIMER_BIGMAP_SELECTOR);
   shift_key_is_down = false;
   control_key_is_down = false;
+  magnification_factor = 1.0;
 }
 
 GameBigMap::~GameBigMap()
@@ -108,7 +90,7 @@ void GameBigMap::select_active_stack()
       assert (Playerlist::getActiveplayer()->getType() == Player::HUMAN);
       //original path was blocked, so let's find a new way there.
       //this shouldn't happen because nextTurn of stack recalculates.
-      //cerr << "original path of stack was blocked\n";
+      //std::cerr << "original path of stack was blocked" << std::endl;
       stack->getPath()->recalculate(stack);
     }
 
@@ -137,7 +119,6 @@ bool GameBigMap::on_selection_timeout()
   return Timing::CONTINUE;
 }
 
-                        
 void GameBigMap::mouse_button_event(MouseButtonEvent e)
 {
   if (input_locked)
@@ -175,21 +156,21 @@ void GameBigMap::mouse_button_event(MouseButtonEvent e)
 
       Stack* stack = Playerlist::getActiveplayer()->getActivestack();
 
-      if (d_cursor == GraphicsCache::HAND)
+      if (d_cursor == ImageCache::HAND)
 	return;
 
       if (stack)
 	{
 	  bool path_already_set = stack->getPath()->size() > 0;
 	  // ask for military advice
-	  if (d_cursor == GraphicsCache::QUESTION)
+	  if (d_cursor == ImageCache::QUESTION)
 	    {
 	      set_shift_key_down (false);
 	      Playerlist::getActiveplayer()->stackFightAdvise
 		(stack, tile, d_intense_combat); 
 	      return;
 	    }
-	  else if (d_cursor == GraphicsCache::RUIN)
+	  else if (d_cursor == ImageCache::RUIN)
 	    {
 	      if (Ruin *r = GameMap::getRuin(tile))
 		{
@@ -208,7 +189,7 @@ void GameBigMap::mouse_button_event(MouseButtonEvent e)
 		}
 	      return;
 	    }
-	  else if (d_cursor == GraphicsCache::ROOK)
+	  else if (d_cursor == ImageCache::ROOK)
 	    {
 	      City* c = GameMap::getCity(tile);
 	      if (c != NULL)
@@ -234,7 +215,7 @@ void GameBigMap::mouse_button_event(MouseButtonEvent e)
 		    }
 		}
 	    }
-	  else if (d_cursor == GraphicsCache::GOTO_ARROW)
+	  else if (d_cursor == ImageCache::GOTO_ARROW)
 	    {
 	      //set in a course, mr crusher.
 	      stack->getPath()->calculate(stack, tile);
@@ -312,7 +293,7 @@ void GameBigMap::mouse_button_event(MouseButtonEvent e)
 
 	  int dist = stack->getPath()->calculate(stack, p);
 	  if (dist == -2)
-	    cerr << "error calculating path!";
+	    std::cerr << "error calculating path!" << std::endl;
 
 	  Vector<int> dest = Vector<int>(-1,-1);
 	  if (!stack->getPath()->empty())
@@ -356,12 +337,12 @@ void GameBigMap::mouse_button_event(MouseButtonEvent e)
 		      if (stack->getPath()->empty() == false && 
 			  stack->getPath()->getMovesExhaustedAtPoint() == 0)
 			deselect = true;
-		      if ((d_cursor == GraphicsCache::SWORD ||
-			   d_cursor == GraphicsCache::HEART) &&
+		      if ((d_cursor == ImageCache::SWORD ||
+			   d_cursor == ImageCache::HEART) &&
 			  stack->canMove() == false)
 			deselect = true;
-		      if ((d_cursor == GraphicsCache::FEET ||
-			   d_cursor == GraphicsCache::SHIP) &&
+		      if ((d_cursor == ImageCache::FEET ||
+			   d_cursor == ImageCache::SHIP) &&
 			  stack->canMove() == false)
 			deselect = true;
 		    }
@@ -384,7 +365,7 @@ void GameBigMap::mouse_button_event(MouseButtonEvent e)
 	{
 	  stack = GameMap::getStack(tile);
 	  if (stack && stack->isFriend(Playerlist::getActiveplayer()) && 
-	      d_cursor == GraphicsCache::TARGET)
+	      d_cursor == ImageCache::TARGET)
 	    {
 	      Playerlist::getActiveplayer()->setActivestack(stack);
 	      select_active_stack();
@@ -392,7 +373,7 @@ void GameBigMap::mouse_button_event(MouseButtonEvent e)
 	  else
 	    {
 	      City* c = GameMap::getCity(tile);
-	      if (c != NULL && d_cursor == GraphicsCache::ROOK)
+	      if (c != NULL && d_cursor == ImageCache::ROOK)
 		{
 		  if (!c->isBurnt())
 		    {
@@ -431,7 +412,7 @@ void GameBigMap::mouse_button_event(MouseButtonEvent e)
       if (mouse_state == DRAGGING_ENDPOINT)
 	{
 	  mouse_state = NONE;
-	  d_cursor = GraphicsCache::FEET;
+	  d_cursor = ImageCache::FEET;
 	  cursor_changed.emit(d_cursor);
 	  path_set.emit();
 	}
@@ -440,7 +421,7 @@ void GameBigMap::mouse_button_event(MouseButtonEvent e)
 	  Stack* stack = Playerlist::getActiveplayer()->getActivestack();
 	  //march a dragged stack!
 	  mouse_state = NONE;
-	  d_cursor = GraphicsCache::FEET;
+	  d_cursor = ImageCache::FEET;
 	  cursor_changed.emit(d_cursor);
 	  //watch out here.  
 	  //we recurse for least amount of code and most programmer confusion.
@@ -463,7 +444,6 @@ void GameBigMap::mouse_button_event(MouseButtonEvent e)
 	mouse_state = NONE;
     }
 
-
   // right mousebutton to get information about things on the map and to
   // unselect the active stack
   else if (e.button == MouseButtonEvent::RIGHT_BUTTON)
@@ -556,50 +536,6 @@ void GameBigMap::mouse_button_event(MouseButtonEvent e)
 	  mouse_state = NONE;
 	}
     }
-  else if (e.button == MouseButtonEvent::WHEEL_UP)
-    {
-      //zoom_in();
-    }
-  else if (e.button == MouseButtonEvent::WHEEL_DOWN)
-    {
-      //zoom_out();
-    }
-}
-
-void GameBigMap::zoom_in()
-{
-  if (input_locked)
-    return;
-  if ((zoom_step / 100.0) + magnification_factor <= max_magnification_factor / 100.0)
-    {
-      int ts = GameMap::getInstance()->getTileSize();
-      Rectangle new_view;
-      double mag = magnification_factor + (zoom_step / 100.0);
-      new_view.w = image.get_width() / (ts * mag) + 1;
-      new_view.h = image.get_height() / (ts * mag) + 1;
-      if (new_view.w <= GameMap::getWidth() && 
-	  new_view.h <= GameMap::getHeight() && 
-	  new_view.w >= 0 && new_view.h >= 0)
-	zoom_view(zoom_step);
-    }
-}
-
-void GameBigMap::zoom_out()
-{
-  if (input_locked)
-    return;
-  if (magnification_factor - (zoom_step / 100.0) >= min_magnification_factor / 100.0)
-    {
-      int ts = GameMap::getInstance()->getTileSize();
-      Rectangle new_view;
-      double mag = magnification_factor - (zoom_step / 100.0);
-      new_view.w = image.get_width() / (ts * mag) + 1;
-      new_view.h = image.get_height() / (ts * mag) + 1;
-      if (new_view.w <= GameMap::getWidth() && 
-	  new_view.h <= GameMap::getHeight() && 
-	  new_view.w >= 0 && new_view.h >= 0)
-	zoom_view(-(const double)zoom_step);
-    }
 }
 
 void GameBigMap::determine_mouse_cursor(Stack *stack, Vector<int> tile)
@@ -608,29 +544,29 @@ void GameBigMap::determine_mouse_cursor(Stack *stack, Vector<int> tile)
   Player *viewing = Playerlist::getViewingplayer();
   if (viewing->getFogMap()->isCompletelyObscuredFogTile(tile))
     {
-      d_cursor = GraphicsCache::HAND;
+      d_cursor = ImageCache::HAND;
     }
   else if (mouse_state == DRAGGING_MAP)
-    d_cursor = GraphicsCache::HAND;
+    d_cursor = ImageCache::HAND;
   else if (stack && 
 	   (mouse_state == DRAGGING_STACK || mouse_state == DRAGGING_ENDPOINT))
     {
-      d_cursor = GraphicsCache::GOTO_ARROW;
+      d_cursor = ImageCache::GOTO_ARROW;
     }
   else if (stack)
     {
-      d_cursor = GraphicsCache::FEET;
+      d_cursor = ImageCache::FEET;
       if (stack->getPos() == tile)
-	d_cursor = GraphicsCache::TARGET;
+	d_cursor = ImageCache::TARGET;
       else
 	{
 	  City *c = GameMap::getCity(tile);
 	  if (c)
 	    {
 	      if (c->getOwner() == active)
-		d_cursor = GraphicsCache::FEET;
+		d_cursor = ImageCache::FEET;
 	      else if (c->isBurnt() == true)
-		d_cursor = GraphicsCache::FEET;
+		d_cursor = ImageCache::FEET;
 	      else
 		{
 		  int delta = abs(tile.x - stack->getPos().x);
@@ -639,7 +575,7 @@ void GameBigMap::determine_mouse_cursor(Stack *stack, Vector<int> tile)
 		  if (delta <= 1)
 		    {
 		      if (is_shift_key_down())
-			d_cursor = GraphicsCache::QUESTION;
+			d_cursor = ImageCache::QUESTION;
 		      else
 			{
 			  Player *me = stack->getOwner();
@@ -647,18 +583,18 @@ void GameBigMap::determine_mouse_cursor(Stack *stack, Vector<int> tile)
 			  bool friendly = (me->getDiplomaticState(them) == 
 					   Player::AT_PEACE);
 			  if (friendly)
-			    d_cursor = GraphicsCache::HEART;
+			    d_cursor = ImageCache::HEART;
 			  else
-			    d_cursor = GraphicsCache::SWORD;
+			    d_cursor = ImageCache::SWORD;
 			}
 		    }
 		  else
 		    {
 		      //can i see other ppl's cities?
 		      if (d_see_opponents_production == true)
-			d_cursor = GraphicsCache::ROOK;
+			d_cursor = ImageCache::ROOK;
 		      else
-			d_cursor = GraphicsCache::HAND;
+			d_cursor = ImageCache::HAND;
 		    }
 		}
 	    }
@@ -674,7 +610,7 @@ void GameBigMap::determine_mouse_cursor(Stack *stack, Vector<int> tile)
 		  if (delta <= 1)
 		    {
 		      if (is_shift_key_down())
-			d_cursor = GraphicsCache::QUESTION;
+			d_cursor = ImageCache::QUESTION;
 		      else
 			{
 			  Player *me = stack->getOwner();
@@ -682,13 +618,13 @@ void GameBigMap::determine_mouse_cursor(Stack *stack, Vector<int> tile)
 			  bool friendly = (me->getDiplomaticState(them) == 
 					   Player::AT_PEACE);
 			  if (friendly)
-			    d_cursor = GraphicsCache::HEART;
+			    d_cursor = ImageCache::HEART;
 			  else
-			    d_cursor = GraphicsCache::SWORD;
+			    d_cursor = ImageCache::SWORD;
 			}
 		    }
 		  else
-		    d_cursor = GraphicsCache::HAND;
+		    d_cursor = ImageCache::HAND;
 		}
 	      else
 		{
@@ -701,38 +637,38 @@ void GameBigMap::determine_mouse_cursor(Stack *stack, Vector<int> tile)
 		    path_calculator = new PathCalculator(stack);
 		  if (path_calculator->isReachable(tile) == false)
 		  //if (moves == 0)
-		    d_cursor = GraphicsCache::HAND;
+		    d_cursor = ImageCache::HAND;
 		  else
 		    {
 		      if (t->getType() == Tile::WATER &&
 			  GameMap::getBridge(tile) == NULL)
 			{
 			  if (stack->isFlying() == true)
-			    d_cursor = GraphicsCache::FEET;
+			    d_cursor = ImageCache::FEET;
 			  else
-			    d_cursor = GraphicsCache::SHIP;
+			    d_cursor = ImageCache::SHIP;
 			}
 		      else
-			d_cursor = GraphicsCache::FEET;
+			d_cursor = ImageCache::FEET;
 		    }
 		}
 	    }
-	  if (d_cursor == GraphicsCache::FEET && is_control_key_down())
-	    d_cursor = GraphicsCache::GOTO_ARROW;
+	  if (d_cursor == ImageCache::FEET && is_control_key_down())
+	    d_cursor = ImageCache::GOTO_ARROW;
 	}
     }
   else
     {
-      d_cursor = GraphicsCache::HAND;
+      d_cursor = ImageCache::HAND;
       Stack *st;
 
       st = GameMap::getStack(tile);
       if (st)
 	{
 	  if (st->getOwner() == active)
-	    d_cursor = GraphicsCache::TARGET;
+	    d_cursor = ImageCache::TARGET;
 	  else
-	    d_cursor = GraphicsCache::HAND;
+	    d_cursor = ImageCache::HAND;
 	}
       else
 	{
@@ -741,22 +677,22 @@ void GameBigMap::determine_mouse_cursor(Stack *stack, Vector<int> tile)
 	    {
 	      City *c = GameMap::getCity(tile);
 	      if (c->isBurnt() == true)
-		d_cursor = GraphicsCache::HAND;
+		d_cursor = ImageCache::HAND;
 	      else if (c->getOwner() == active)
-		d_cursor = GraphicsCache::ROOK;
+		d_cursor = ImageCache::ROOK;
 	      else if (d_see_opponents_production == true)
-		d_cursor = GraphicsCache::ROOK;
+		d_cursor = ImageCache::ROOK;
 	    }
 	  else if (t->getBuilding() == Maptile::RUIN)
 	    {
 	      Ruin *ruin = GameMap::getRuin(tile);
 	      if (ruin->isHidden() == true && ruin->getOwner() == active)
-		d_cursor = GraphicsCache::RUIN;
+		d_cursor = ImageCache::RUIN;
 	      else if (ruin->isHidden() == false)
-		d_cursor = GraphicsCache::RUIN;
+		d_cursor = ImageCache::RUIN;
 	    }
 	  else if (t->getBuilding() == Maptile::TEMPLE)
-	    d_cursor = GraphicsCache::RUIN;
+	    d_cursor = ImageCache::RUIN;
 	}
 
     }
@@ -787,19 +723,19 @@ void GameBigMap::mouse_motion_event(MouseMotionEvent e)
 
   if (e.pressed[MouseMotionEvent::LEFT_BUTTON]
       && (mouse_state == NONE || mouse_state == SHOWING_STACK) && 
-      stack && stack->getPos() == tile && viewing->getFogMap()->isCompletelyObscuredFogTile(tile) == false && d_cursor != GraphicsCache::HAND)
+      stack && stack->getPos() == tile && viewing->getFogMap()->isCompletelyObscuredFogTile(tile) == false && d_cursor != ImageCache::HAND)
     {
       //initial dragging of stack from it's tile
       mouse_state = DRAGGING_STACK;
     }
   else if (e.pressed[MouseMotionEvent::LEFT_BUTTON] && stack &&
-	   d_cursor == GraphicsCache::GOTO_ARROW && mouse_state == NONE)
+	   d_cursor == ImageCache::GOTO_ARROW && mouse_state == NONE)
     {
       //initial dragging of endpoint from it's tile
       mouse_state = DRAGGING_ENDPOINT;
     }
   else if (e.pressed[MouseMotionEvent::LEFT_BUTTON]
-	   && (mouse_state == NONE || mouse_state == DRAGGING_MAP) && d_cursor == GraphicsCache::HAND)
+	   && (mouse_state == NONE || mouse_state == DRAGGING_MAP) && d_cursor == ImageCache::HAND)
     {
       Vector<int> delta = -(e.pos - prev_mouse_pos);
 
@@ -821,7 +757,7 @@ void GameBigMap::mouse_motion_event(MouseMotionEvent e)
 	{
 	  //here we have a case of the new view overlapping with the view.
 	  //why redraw what we've already drawn?
-	  Rectangle old_view = view;
+	  //Rectangle old_view = view;
 	  view.x = new_view.x;
 	  view.y = new_view.y;
 	  view_changed.emit(view);
@@ -868,10 +804,10 @@ void GameBigMap::mouse_motion_event(MouseMotionEvent e)
       mouse_state = NONE;
       determine_mouse_cursor(stack, tile);
       mouse_state = orig_state;
-      if (d_cursor == GraphicsCache::FEET ||
-	  d_cursor == GraphicsCache::SHIP || 
-	  d_cursor == GraphicsCache::GOTO_ARROW || 
-	  d_cursor == GraphicsCache::TARGET)
+      if (d_cursor == ImageCache::FEET ||
+	  d_cursor == ImageCache::SHIP || 
+	  d_cursor == ImageCache::GOTO_ARROW || 
+	  d_cursor == ImageCache::TARGET)
 	{
 	  guint32 moves = 0, turns = 0, left = 0;
 	  Path *new_path = path_calculator->calculate(tile, moves, turns, left,
@@ -895,32 +831,11 @@ void GameBigMap::mouse_motion_event(MouseMotionEvent e)
   last_tile = tile;
 }
 
-void GameBigMap::reset_zoom()
-{
-  magnification_factor = 1.0;
-  screen_size_changed(image);
-  draw(Playerlist::getViewingplayer(), true);
-  view_changed.emit(view);
-}
-
-void GameBigMap::zoom_view(double percent)
-{
-  magnification_factor += percent / 100.0;
-  //call with +2, or -2
-  //Rectangle new_view = view;
-  //new_view.dim += Vector<int>(tiles, tiles);
-  //new_view.pos += Vector<int>(tiles*-1/2, tiles*-1/2);
-  //set_view (new_view);
-  screen_size_changed(image);
-  draw(Playerlist::getViewingplayer(), true);
-  view_changed.emit(view);
-}
-
 void GameBigMap::after_draw()
 {
   if (blank_screen == true)
     return;
-  GraphicsCache *gc = GraphicsCache::getInstance();
+  ImageCache *gc = ImageCache::getInstance();
   int tilesize = GameMap::getInstance()->getTileSize();
 
   Stack* stack = Playerlist::getActiveplayer()->getActivestack();
@@ -945,19 +860,18 @@ void GameBigMap::after_draw()
 	  pos = tile_to_buffer_pos(*it);
 
 	  canMoveThere = (pathcount < stack->getPath()->getMovesExhaustedAtPoint());
+          PixMask *waypoint;
 	  if (canMoveThere)
-            gc->getWaypointPic(0)->blit_centered
-              (buffer, pos + (Vector<int>(tilesize,tilesize)/2));
+            waypoint = gc->getWaypointImage(0);
 	  else
-            gc->getWaypointPic(1)->blit_centered
-              (buffer, pos + (Vector<int>(tilesize,tilesize)/2));
-
+            waypoint = gc->getWaypointImage(1);
+          waypoint->blit_centered (buffer, pos + (Vector<int>(tilesize,tilesize)/2));
 	  pathcount++;
 
 	}
 
       if (mouse_state == DRAGGING_STACK || mouse_state == DRAGGING_ENDPOINT 
-	  || d_cursor == GraphicsCache::GOTO_ARROW)
+	  || d_cursor == ImageCache::GOTO_ARROW)
 	{
 	  Path::iterator it = stack->getPath()->end();
 	  it--;
@@ -990,7 +904,7 @@ void GameBigMap::after_draw()
 
 	  p = tile_to_buffer_pos(p);
 
-	  draw_stack (stack, buffer, buffer_gc);
+	  draw_stack (stack, buffer);
 
 	  PixMask *tmp = NULL;
 	  if (stack->size() > 1)
@@ -1048,31 +962,31 @@ void GameBigMap::set_control_key_down (bool down)
       if (!stack)
 	return;
 
-      if (d_cursor == GraphicsCache::TARGET)
+      if (d_cursor == ImageCache::TARGET)
 	{
 	  City *city = GameMap::getCity(current_tile);
 	  if (city->isBurnt() == false)
 	    {
-	      d_cursor = GraphicsCache::ROOK;
+	      d_cursor = ImageCache::ROOK;
 	      cursor_changed.emit(d_cursor);
 	    }
 	}
-      else if (d_cursor == GraphicsCache::HAND && 
+      else if (d_cursor == ImageCache::HAND && 
 	       d_see_opponents_production == true)
 	{
 	  City *city = GameMap::getCity(current_tile);
 	  if (city->isBurnt() == false)
 	    {
-	      d_cursor = GraphicsCache::ROOK;
+	      d_cursor = ImageCache::ROOK;
 	      cursor_changed.emit(d_cursor);
 	    }
 	}
     }
   else
     {
-      if (d_cursor == GraphicsCache::FEET)
+      if (d_cursor == ImageCache::FEET)
 	{
-	  d_cursor = GraphicsCache::GOTO_ARROW;
+	  d_cursor = ImageCache::GOTO_ARROW;
 	  cursor_changed.emit(d_cursor);
 	}
     }
@@ -1112,42 +1026,42 @@ void GameBigMap::set_shift_key_down (bool down)
     }
   else if (b == Maptile::CITY)
     {
-      if (d_cursor == GraphicsCache::TARGET)
+      if (d_cursor == ImageCache::TARGET)
 	{
-	  d_cursor = GraphicsCache::ROOK;
+	  d_cursor = ImageCache::ROOK;
 	  cursor_changed.emit(d_cursor);
 	}
-      else if (d_cursor == GraphicsCache::FEET)
+      else if (d_cursor == ImageCache::FEET)
 	{
-	  d_cursor = GraphicsCache::ROOK;
+	  d_cursor = ImageCache::ROOK;
 	  cursor_changed.emit(d_cursor);
 	}
-      else if (d_cursor == GraphicsCache::HAND &&
+      else if (d_cursor == ImageCache::HAND &&
 	       d_see_opponents_production == true)
 	{
-	  d_cursor = GraphicsCache::ROOK;
+	  d_cursor = ImageCache::ROOK;
 	  cursor_changed.emit(d_cursor);
 	}
     }
 
   if (active_stack)
     {
-      if (d_cursor == GraphicsCache::SHIP || d_cursor == GraphicsCache::FEET) 
+      if (d_cursor == ImageCache::SHIP || d_cursor == ImageCache::FEET) 
 	{
 	  if (b == Maptile::RUIN || b == Maptile::TEMPLE)
-	    d_cursor = GraphicsCache::RUIN;
+	    d_cursor = ImageCache::RUIN;
 	  else
-	    d_cursor = GraphicsCache::HAND;
+	    d_cursor = ImageCache::HAND;
 	  cursor_changed.emit(d_cursor);
 	}
     }
   else
     {
-      if (d_cursor != GraphicsCache::RUIN)
+      if (d_cursor != ImageCache::RUIN)
 	{
 	  if (b == Maptile::RUIN || b == Maptile::TEMPLE)
 	    {
-	      d_cursor = GraphicsCache::RUIN;
+	      d_cursor = ImageCache::RUIN;
 	      cursor_changed.emit(d_cursor);
 	    }
 	}
@@ -1155,17 +1069,25 @@ void GameBigMap::set_shift_key_down (bool down)
 
   if (d_military_advisor == true)
     {
-      if (active_stack && d_cursor == GraphicsCache::SWORD)
+      if (active_stack && d_cursor == ImageCache::SWORD)
 	{
-	      d_cursor = GraphicsCache::QUESTION;
+	      d_cursor = ImageCache::QUESTION;
 	      cursor_changed.emit(d_cursor);
 	}
     }
 }
-    
+
 void GameBigMap::reset_path_calculator(Stack *s)
 {
   if (path_calculator)
     delete path_calculator;
   path_calculator = new PathCalculator(s);
 }
+
+void GameBigMap::update_mouse_cursor()
+{
+  Vector<int> tile = mouse_pos_to_tile(prev_mouse_pos);
+  current_tile = tile;
+  determine_mouse_cursor (Playerlist::getActiveplayer()->getActivestack(),
+                          current_tile);
+}
