@@ -1,5 +1,5 @@
 //  Copyright (C) 2007 Ole Laursen
-//  Copyright (C) 2007, 2008, 2009 Ben Asselstine
+//  Copyright (C) 2007, 2008, 2009, 2011, 2014 Ben Asselstine
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -23,77 +23,34 @@
 
 #include "army-bonus-dialog.h"
 
-#include "glade-helpers.h"
-#include "image-helpers.h"
-#include "ucompose.hpp"
-#include "defs.h"
-#include "army.h"
 #include "armysetlist.h"
 #include "player.h"
-#include "GraphicsCache.h"
-#include "File.h"
+#include "ImageCache.h"
 
-ArmyBonusDialog::ArmyBonusDialog(Player *p)
+ArmyBonusDialog::ArmyBonusDialog(Gtk::Window &parent, Player *p)
+ :LwDialog(parent, "army-bonus-dialog.ui")
 {
     d_player = p;
-    Glib::RefPtr<Gtk::Builder> xml
-	= Gtk::Builder::create_from_file(get_glade_path()
-				    + "/army-bonus-dialog.ui");
-
-    xml->get_widget("dialog", dialog);
-    decorate(dialog);
-    window_closed.connect(sigc::mem_fun(dialog, &Gtk::Dialog::hide));
-    dialog->set_icon_from_file(File::getMiscFile("various/castle_icon.png"));
 
     armies_list = Gtk::ListStore::create(armies_columns);
     xml->get_widget("treeview", armies_treeview);
     armies_treeview->set_model(armies_list);
     armies_treeview->append_column("", armies_columns.image);
     armies_treeview->append_column("", armies_columns.name);
-    armies_treeview->append_column("Str", armies_columns.str);
-    armies_treeview->append_column("Move", armies_columns.move);
+    armies_treeview->append_column(_("Str"), armies_columns.str);
+    armies_treeview->append_column(_("Move"), armies_columns.move);
     armies_treeview->append_column("", armies_columns.move_image);
-    armies_treeview->append_column("Bonus", armies_columns.bonus);
+    armies_treeview->append_column(_("Bonus"), armies_columns.bonus);
     armies_treeview->set_headers_visible(true);
 
-    guint32 max = Armysetlist::getInstance()->getSize(d_player->getArmyset());
-    for (unsigned int i = 0; i < max; i++)
-      addArmyType(i);
-}
-
-ArmyBonusDialog::~ArmyBonusDialog()
-{
-  delete dialog;
-}
-void ArmyBonusDialog::set_parent_window(Gtk::Window &parent)
-{
-    dialog->set_transient_for(parent);
-    //dialog->set_position(Gtk::WIN_POS_CENTER_ON_PARENT);
-}
-
-void ArmyBonusDialog::hide()
-{
-  dialog->hide();
-}
-
-void ArmyBonusDialog::run()
-{
-    static int width = -1;
-    static int height = -1;
-
-    if (width != -1 && height != -1)
-	dialog->set_default_size(width, height);
-    
-    dialog->show();
-    dialog->run();
-
-    dialog->get_size(width, height);
-
+    Armyset *as=Armysetlist::getInstance()->get(d_player->getArmyset());
+    for (Armyset::iterator i = as->begin(); i != as->end(); ++i)
+      addArmyType((*i)->getId());
 }
 
 void ArmyBonusDialog::addArmyType(guint32 army_type)
 {
-    GraphicsCache *gc = GraphicsCache::getInstance();
+    ImageCache *gc = ImageCache::getInstance();
     Player *p = d_player;
     const ArmyProto *a;
     a = Armysetlist::getInstance()->getArmy(p->getArmyset(), army_type);
@@ -101,15 +58,16 @@ void ArmyBonusDialog::addArmyType(guint32 army_type)
       return; //we don't want to show heroes in this list
     Gtk::TreeIter i = armies_list->append();
     (*i)[armies_columns.name] = a->getName();
-    (*i)[armies_columns.image] = gc->getArmyPic(p->getArmyset(), army_type,
-						p, NULL)->to_pixbuf();
+    (*i)[armies_columns.image] = 
+      gc->getCircledArmyPic(p->getArmyset(), army_type, p, NULL, false,
+                            p->getId(), true)->to_pixbuf();
     (*i)[armies_columns.str] = a->getStrength();
     (*i)[armies_columns.move] = a->getMaxMoves();
     guint32 b = a->getMoveBonus();
     (*i)[armies_columns.move_image] = gc->getMoveBonusPic(b, false)->to_pixbuf();
     (*i)[armies_columns.bonus] = "-";
 
-    std::string s = a->getArmyBonusDescription();
+    Glib::ustring s = a->getArmyBonusDescription();
     if (s == "")
       (*i)[armies_columns.bonus] = "-";
     else
