@@ -1,5 +1,5 @@
 //  Copyright (C) 2007 Ole Laursen
-//  Copyright (C) 2007, 2008, 2009, 2011 Ben Asselstine
+//  Copyright (C) 2007, 2008, 2009, 2011, 2012, 2014, 2015 Ben Asselstine
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -21,9 +21,7 @@
 #include <gtkmm.h>
 
 #include "army-info-tip.h"
-
-#include "glade-helpers.h"
-#include "image-helpers.h"
+#include "builder-cache.h"
 
 #include "ucompose.hpp"
 #include "vector.h"
@@ -31,33 +29,28 @@
 #include "army.h"
 #include "armyprodbase.h"
 #include "armyproto.h"
-#include "GraphicsCache.h"
+#include "ImageCache.h"
 #include "playerlist.h"
 #include "city.h"
-#include "decorated.h"
 #include "File.h"
-
+#include "shield.h"
 
 ArmyInfoTip::ArmyInfoTip(Gtk::Widget *target, const Army *army)
 {
-    Glib::RefPtr<Gtk::Builder> xml
-	= Gtk::Builder::create_from_file(get_glade_path()
-				    + "/army-info-window.ui");
+    Glib::RefPtr<Gtk::Builder> xml = BuilderCache::get("army-info-window.ui");
 
     xml->get_widget("window", window);
-    Decorated decorator;
-    decorator.decorate(window, GraphicsCache::GAME_BACKGROUND, 200);
-
     Gtk::Image *army_image;
     xml->get_widget("army_image", army_image);
     Player *p;
     int armyset;
     p = army->getOwner();
     armyset = army->getArmyset();
-    GraphicsCache *gc = GraphicsCache::getInstance();
+    ImageCache *gc = ImageCache::getInstance();
     army_image->property_pixbuf() = 
-      gc->getArmyPic(armyset, army->getTypeId(), p, 
-		     army->getMedalBonuses())->to_pixbuf();
+      gc->getCircledArmyPic(armyset, army->getTypeId(), p, 
+                            army->getMedalBonuses(), false, Shield::NEUTRAL, 
+                            true)->to_pixbuf();
 
     // fill in terrain image
     Gtk::Image *terrain_image;
@@ -87,7 +80,7 @@ ArmyInfoTip::ArmyInfoTip(Gtk::Widget *target, const Army *army)
     window->get_child()->show();
     Vector<int> pos(0, 0);
     target->get_window()->get_origin(pos.x, pos.y);
-    if (target->has_no_window())
+    if (target->get_has_window() == false)
       {
 	Gtk::Allocation a = target->get_allocation();
 	pos.x += a.get_x();
@@ -105,22 +98,18 @@ ArmyInfoTip::ArmyInfoTip(Gtk::Widget *target, const Army *army)
 ArmyInfoTip::ArmyInfoTip(Gtk::Widget *target, const ArmyProdBase *army, 
 			 City *city)
 {
-    Glib::RefPtr<Gtk::Builder> xml
-	= Gtk::Builder::create_from_file(get_glade_path()
-				    + "/army-info-window.ui");
+    Glib::RefPtr<Gtk::Builder> xml = BuilderCache::get("army-info-window.ui");
 
     xml->get_widget("window", window);
-    Decorated decorator;
-    decorator.decorate(window,GraphicsCache::GAME_BACKGROUND, 200);
-
     Gtk::Image *army_image;
     xml->get_widget("army_image", army_image);
     Player *p = city->getOwner();
     int armyset;
     armyset = army->getArmyset();
-    GraphicsCache *gc = GraphicsCache::getInstance();
-    army_image->property_pixbuf() = gc->getArmyPic(armyset, army->getTypeId(), 
-						   p, NULL)->to_pixbuf();
+    ImageCache *gc = ImageCache::getInstance();
+    army_image->property_pixbuf() = 
+      gc->getCircledArmyPic(armyset, army->getTypeId(), p, NULL, false,
+                            Shield::NEUTRAL, true)->to_pixbuf();
 
     // fill in terrain image
     Gtk::Image *terrain_image;
@@ -150,7 +139,7 @@ ArmyInfoTip::ArmyInfoTip(Gtk::Widget *target, const ArmyProdBase *army,
     window->get_child()->show();
     Vector<int> pos(0, 0);
     target->get_window()->get_origin(pos.x, pos.y);
-    if (target->has_no_window())
+    if (target->get_has_window() == false)
       {
 	Gtk::Allocation a = target->get_allocation();
 	pos.x += a.get_x();
@@ -167,22 +156,18 @@ ArmyInfoTip::ArmyInfoTip(Gtk::Widget *target, const ArmyProdBase *army,
 
 ArmyInfoTip::ArmyInfoTip(Gtk::Widget *target, const ArmyProto *army)
 {
-    Glib::RefPtr<Gtk::Builder> xml
-	= Gtk::Builder::create_from_file(get_glade_path()
-				    + "/army-info-window.ui");
+    Glib::RefPtr<Gtk::Builder> xml = BuilderCache::get("army-info-window.ui");
 
     xml->get_widget("window", window);
-    Decorated decorator;
-    decorator.decorate(window, GraphicsCache::GAME_BACKGROUND, 200);
-
     Gtk::Image *army_image;
     xml->get_widget("army_image", army_image);
     Player *p = Playerlist::getInstance()->getActiveplayer();
     int armyset;
     armyset = army->getArmyset();
-    GraphicsCache *gc = GraphicsCache::getInstance();
-    army_image->property_pixbuf() = gc->getArmyPic(armyset, army->getTypeId(), 
-						   p, NULL)->to_pixbuf();
+    ImageCache *gc = ImageCache::getInstance();
+    army_image->property_pixbuf() = 
+      gc->getCircledArmyPic(armyset, army->getId(), p, NULL, false,
+                            Shield::NEUTRAL, true)->to_pixbuf();
 
     // fill in terrain image
     Gtk::Image *terrain_image;
@@ -212,7 +197,7 @@ ArmyInfoTip::ArmyInfoTip(Gtk::Widget *target, const ArmyProto *army)
     window->get_child()->show();
     Vector<int> pos(0, 0);
     target->get_window()->get_origin(pos.x, pos.y);
-    if (target->has_no_window())
+    if (target->get_has_window() == false)
       {
 	Gtk::Allocation a = target->get_allocation();
 	pos.x += a.get_x();
