@@ -1,5 +1,5 @@
 //  Copyright (C) 2007 Ole Laursen
-//  Copyright (C) 2007, 2008, 2009 Ben Asselstine
+//  Copyright (C) 2007, 2008, 2009, 2011, 2014, 2015 Ben Asselstine
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -24,34 +24,24 @@
 
 #include "buy-production-dialog.h"
 
-#include "glade-helpers.h"
-#include "image-helpers.h"
 #include "input-helpers.h"
 #include "ucompose.hpp"
 #include "defs.h"
 #include "army.h"
 #include "city.h"
-#include "GraphicsCache.h"
+#include "ImageCache.h"
 #include "armysetlist.h"
 #include "playerlist.h"
-#include "File.h"
+#include "shield.h"
 
-BuyProductionDialog::BuyProductionDialog(City *c)
+BuyProductionDialog::BuyProductionDialog(Gtk::Window &parent, City *c)
+ : LwDialog(parent, "buy-production-dialog.ui")
 {
   army_info_tip = NULL;
-    GraphicsCache *gc = GraphicsCache::getInstance();
+    ImageCache *gc = ImageCache::getInstance();
     city = c;
     selected_army = NO_ARMY_SELECTED;
     
-    Glib::RefPtr<Gtk::Builder> xml
-	= Gtk::Builder::create_from_file(get_glade_path()
-				    + "/buy-production-dialog.ui");
-
-    xml->get_widget("dialog", dialog);
-    decorate(dialog);
-    window_closed.connect(sigc::mem_fun(dialog, &Gtk::Dialog::hide));
-    dialog->set_icon_from_file(File::getMiscFile("various/castle_icon.png"));
-    
     xml->get_widget("production_info_label1", production_info_label1);
     xml->get_widget("production_info_label2", production_info_label2);
     xml->get_widget("buy_button", buy_button);
@@ -63,9 +53,10 @@ BuyProductionDialog::BuyProductionDialog(City *c)
 
     Player *p = Playerlist::getInstance()->getActiveplayer();
     // fill in purchasable armies
-    for (unsigned int j = 0; j < al->getSize(p->getArmyset()); j++)
+    Armyset *as = al->get(p->getArmyset());
+    for (Armyset::iterator j = as->begin(); j != as->end(); ++j)
       {
-        const ArmyProto *a = al->getArmy (p->getArmyset(), j);
+        const ArmyProto *a = al->getArmy (p->getArmyset(), (*j)->getId());
         if (a->getNewProductionCost() > 0)
           purchasables.push_back(a);
       }
@@ -77,14 +68,16 @@ BuyProductionDialog::BuyProductionDialog(City *c)
 	Gtk::ToggleButton *toggle = manage(new Gtk::ToggleButton);
 	
 	bool greyed_out = false;
+        guint32 selected = Shield::NEUTRAL;
 	if (city->hasProductionBase(purchasables[i]) == true)
-	  greyed_out = true;
-	else if ((int)purchasables[i]->getNewProductionCost() > 
+	  selected = p->getId();
+	if ((int)purchasables[i]->getNewProductionCost() > 
 		 c->getOwner()->getGold())
 	  greyed_out = true;
 	Glib::RefPtr<Gdk::Pixbuf> pix
-	    = gc->getArmyPic(p->getArmyset(), purchasables[i]->getTypeId(), p, 
-			     NULL, greyed_out)->to_pixbuf();
+	    = gc->getCircledArmyPic(p->getArmyset(), 
+                                    purchasables[i]->getId(), p, NULL, 
+                                    greyed_out, selected, true)->to_pixbuf();
 	
 	Gtk::Image *image = new Gtk::Image();
 	image->property_pixbuf() = pix;
@@ -117,12 +110,6 @@ BuyProductionDialog::~BuyProductionDialog()
 {
   if (army_info_tip)
     delete army_info_tip;
-  delete dialog;
-}
-void BuyProductionDialog::set_parent_window(Gtk::Window &parent)
-{
-    dialog->set_transient_for(parent);
-    //dialog->set_position(Gtk::WIN_POS_CENTER_ON_PARENT);
 }
 
 void BuyProductionDialog::hide()
@@ -177,9 +164,7 @@ void BuyProductionDialog::fill_in_production_info()
 	s1 += "\n";
 	s1 += String::ucompose(_("Duration: %1"), a->getProduction());
 	s1 += "\n";
-	// note to translators: %1 is melee strength, %2 is ranged strength
-	s1 += String::ucompose(_("Strength: %1"),
-			      a->getStrength());
+	s1 += String::ucompose(_("Strength: %1"), a->getStrength());
 	
 	// fill in second column
 	s2 += String::ucompose(_("Cost: %1"), a->getNewProductionCost());
@@ -254,4 +239,3 @@ bool BuyProductionDialog::on_production_button_event(GdkEventButton *e, Gtk::Tog
     
     return false;
 }
-
