@@ -1,5 +1,5 @@
 //  Copyright (C) 2007 Ole Laursen
-//  Copyright (C) 2007, 2008, 2009 Ben Asselstine
+//  Copyright (C) 2007, 2008, 2009, 2014 Ben Asselstine
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -19,34 +19,28 @@
 #ifndef BUY_PRODUCTION_DIALOG_H
 #define BUY_PRODUCTION_DIALOG_H
 
-#include <memory>
-#include <vector>
-#include <sigc++/trackable.h>
 #include <gtkmm.h>
 
 #include "army-info-tip.h"
-
 #include "armyproto.h"
+#include "lw-dialog.h"
+
 class City;
 
-#include "decorated.h"
 // dialog for buying a production slot for a city
-class BuyProductionDialog: public Decorated
+class BuyProductionDialog: public LwDialog
 {
  public:
-    BuyProductionDialog(City *city);
+    BuyProductionDialog(Gtk::Window &parent, City *city);
     ~BuyProductionDialog();
 
-    void set_parent_window(Gtk::Window &parent);
-
     void run();
     void hide();
 
     enum { NO_ARMY_SELECTED = -1 };
-    int get_selected_army() { return selected_army == NO_ARMY_SELECTED ? NO_ARMY_SELECTED : purchasables[selected_army]->getTypeId(); }
+    int get_selected_army() { return selected_army == NO_ARMY_SELECTED ? int(NO_ARMY_SELECTED) : purchasables[selected_army]->getId(); }
     
  private:
-    Gtk::Dialog* dialog;
     ArmyInfoTip* army_info_tip;
     Gtk::Label *production_info_label1;
     Gtk::Label *production_info_label2;
