@@ -1,4 +1,4 @@
-//  Copyright (C) 2009, 2011 Ben Asselstine
+//  Copyright (C) 2009, 2011, 2012, 2014, 2015 Ben Asselstine
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -16,35 +16,25 @@
 //  02110-1301, USA.
 
 #include <config.h>
-
 #include <gtkmm.h>
 
 #include "city-info-tip.h"
-
-#include "glade-helpers.h"
-#include "image-helpers.h"
-
+#include "builder-cache.h"
 #include "ucompose.hpp"
 #include "vector.h"
 #include "defs.h"
-#include "GraphicsCache.h"
+#include "ImageCache.h"
 #include "playerlist.h"
 #include "city.h"
-#include "decorated.h"
 #include "File.h"
 
 
 CityInfoTip::CityInfoTip(Gtk::Widget *target, MapTipPosition mpos, const City *city)
 {
-    GraphicsCache *gc = GraphicsCache::getInstance();
-    Glib::RefPtr<Gtk::Builder> xml
-	= Gtk::Builder::create_from_file(get_glade_path()
-				    + "/city-info-window.ui");
+    ImageCache *gc = ImageCache::getInstance();
+    Glib::RefPtr<Gtk::Builder> xml = BuilderCache::get("city-info-window.ui");
 
     xml->get_widget("window", window);
-    Decorated decorator;
-    decorator.decorate(window, GraphicsCache::GAME_BACKGROUND, 200);
-
     Gtk::Image *left_shield_image;
     xml->get_widget("left_shield_image", left_shield_image);
     Gtk::Image *right_shield_image;
@@ -67,7 +57,7 @@ CityInfoTip::CityInfoTip(Gtk::Widget *target, MapTipPosition mpos, const City *c
     xml->get_widget("razed_label", razed_label);
 
     if (city->isBurnt() == true)
-      razed_label->set_markup("<b>" + std::string(_("Razed!")) + "</b>");
+      razed_label->set_markup("<b>" + Glib::ustring(_("Razed!")) + "</b>");
       
     name_label->set_markup("<b>" + city->getName() + "</b>");
     left_shield_image->property_pixbuf() = 
@@ -75,9 +65,9 @@ CityInfoTip::CityInfoTip(Gtk::Widget *target, MapTipPosition mpos, const City *c
     right_shield_image->property_pixbuf() = 
       gc->getShieldPic(1, city->getOwner())->to_pixbuf();
     income_image->property_file() = 
-      File::getMiscFile("various/smallincome.png");
+      File::getVariousFile("smallincome.png");
     defense_image->property_file() = 
-      File::getMiscFile("various/smalldefense.png");
+      File::getVariousFile("smalldefense.png");
     income_label->set_markup(String::ucompose("<b>%1</b>",
 					      city->getGold()));
     defense_label->set_markup(String::ucompose("<b>%1</b>",
@@ -103,7 +93,7 @@ CityInfoTip::CityInfoTip(Gtk::Widget *target, MapTipPosition mpos, const City *c
     window->get_child()->show();
     Vector<int> p(0, 0);
     target->get_window()->get_origin(p.x, p.y);
-    if (target->has_no_window())
+    if (target->get_has_window() == false)
     {
 	Gtk::Allocation a = target->get_allocation();
 	p.x += a.get_x();
