@@ -1,5 +1,5 @@
 //  Copyright (C) 2007 Ole Laursen
-//  Copyright (C) 2007, 2008, 2009, 2011 Ben Asselstine
+//  Copyright (C) 2007, 2008, 2009, 2011, 2014, 2015 Ben Asselstine
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -25,8 +25,6 @@
 
 #include "city-window.h"
 
-#include "glade-helpers.h"
-#include "image-helpers.h"
 #include "input-helpers.h"
 #include "ucompose.hpp"
 #include "defs.h"
@@ -34,28 +32,20 @@
 #include "armyprodbase.h"
 #include "player.h"
 #include "city.h"
-#include "GraphicsCache.h"
-#include "armysetlist.h"
+#include "ImageCache.h"
 #include "buy-production-dialog.h"
 #include "destination-dialog.h"
-#include "GameMap.h"
 #include "citylist.h"
 #include "playerlist.h"
-#include "File.h"
 
-CityWindow::CityWindow(City *c, bool razing_possible, 
+CityWindow::CityWindow(Gtk::Window &parent, City *c, bool razing_possible, 
 		       bool see_opponents_production)
+ : LwDialog(parent, "city-window.ui")
 {
   army_info_tip = NULL;
     city = c;
     
-    Glib::RefPtr<Gtk::Builder> xml
-	= Gtk::Builder::create_from_file(get_glade_path() + "/city-window.ui");
-
-    xml->get_widget("dialog", dialog);
-    decorate(dialog);
-    window_closed.connect(sigc::mem_fun(dialog, &Gtk::Dialog::hide));
-    set_title(c->getName());
+    dialog->set_title(c->getName());
     
     xml->get_widget("map_image", map_image);
 
@@ -112,6 +102,7 @@ CityWindow::CityWindow(City *c, bool razing_possible,
 
     d_razing_possible = razing_possible;
     fill_in_city_info();
+
     fill_in_production_toggles();
 
     ignore_toggles = false;
@@ -119,7 +110,6 @@ CityWindow::CityWindow(City *c, bool razing_possible,
 
 CityWindow::~CityWindow()
 {
-  delete dialog;
   delete prodmap;
   if (army_info_tip != NULL)
     delete army_info_tip;
@@ -139,12 +129,6 @@ bool CityWindow::on_map_mouse_button_event(GdkEventButton *e)
     return true;
 }
 
-void CityWindow::set_parent_window(Gtk::Window &parent)
-{
-    dialog->set_transient_for(parent);
-    //dialog->set_position(Gtk::WIN_POS_CENTER_ON_PARENT);
-}
-
 void CityWindow::hide()
 {
   dialog->hide();
@@ -160,8 +144,7 @@ void CityWindow::run()
 
 void CityWindow::fill_in_city_info()
 {
-    
-    set_title(city->getName());
+    dialog->set_title(city->getName());
 
     // fill in status label
     Glib::ustring s;
@@ -180,37 +163,14 @@ void CityWindow::fill_in_city_info()
 
 void CityWindow::fill_in_production_toggles()
 {
-    Player *player = city->getOwner();
-    unsigned int as = player->getArmyset();
     int production_index = city->getActiveProductionSlot();
-    int type;
-    Glib::RefPtr<Gdk::Pixbuf> pic;
-    GraphicsCache *gc = GraphicsCache::getInstance();
 
-    Glib::RefPtr<Gdk::Pixbuf> s
-	= GraphicsCache::getInstance()->getArmyPic(as, 0, player, NULL)->to_pixbuf();
-    Glib::RefPtr<Gdk::Pixbuf> empty_pic
-	= Gdk::Pixbuf::create(Gdk::COLORSPACE_RGB, true, 8, s->get_width(), s->get_height());
-    empty_pic->fill(0x00000000);
-    
     ignore_toggles = true;
     for (unsigned int i = 0; i < city->getMaxNoOfProductionBases(); i++)
     {
 	Gtk::ToggleButton *toggle = production_toggles[i];
 	toggle->foreach(sigc::mem_fun(toggle, &Gtk::Container::remove));
-
-	type = city->getArmytype(i);
-        if (type != -1)
-            // use GraphicsCache to load army pics because of player-specific
-            // colors
-            pic = gc->getArmyPic(as, type, player, NULL)->to_pixbuf();
-	else
-	    pic = empty_pic;
-	
-	Gtk::Image *image = new Gtk::Image();
-	image->property_pixbuf() = pic;
-	toggle->add(*manage(image));
-
+        update_toggle_picture(i);
 	toggle->set_active((int)i == production_index);
 	toggle->show_all();
     }
@@ -218,7 +178,33 @@ void CityWindow::fill_in_production_toggles()
 
     on_hold_button->set_sensitive(production_index != -1);
     fill_in_production_info();
+}
 
+void CityWindow::update_toggle_picture(int slot)
+{
+  Player *player = city->getOwner();
+  unsigned int as = player->getArmyset();
+  ImageCache *gc = ImageCache::getInstance();
+  Gtk::ToggleButton *toggle = production_toggles[slot];
+  Glib::RefPtr<Gdk::Pixbuf> pic;
+  if (city->getArmytype(slot) == -1)
+    pic = 
+      gc->getCircledArmyPic(as, 0, player, NULL, false, 
+                            Shield::NEUTRAL, false)->to_pixbuf();
+  else
+    {
+      int type = city->getArmytype(slot);
+      pic =
+        gc->getCircledArmyPic (as, type, player, NULL, false,
+                               slot == city->getActiveProductionSlot() ? 
+                               player->getId(): int(Shield::NEUTRAL), 
+                               true)->to_pixbuf();
+    }
+  Gtk::Image *image = new Gtk::Image();
+  image->property_pixbuf() = pic;
+  toggle->remove();
+  toggle->add(*manage(image));
+  toggle->show_all();
 }
 
 void CityWindow::on_production_toggled(Gtk::ToggleButton *toggle)
@@ -250,6 +236,8 @@ void CityWindow::on_production_toggled(Gtk::ToggleButton *toggle)
 
     on_hold_button->set_sensitive(!is_empty);
     
+    for (unsigned int i = 0; i < production_toggles.size(); ++i) 
+      update_toggle_picture(i);
     fill_in_production_info();
 }
 
@@ -258,13 +246,10 @@ void CityWindow::fill_in_production_info()
     Player *player = city->getOwner();
     unsigned int as = player->getArmyset();
     Glib::RefPtr<Gdk::Pixbuf> pic;
-    GraphicsCache *gc = GraphicsCache::getInstance();
+    ImageCache *gc = ImageCache::getInstance();
     int slot = city->getActiveProductionSlot();
-    Glib::RefPtr<Gdk::Pixbuf> s
-	= GraphicsCache::getInstance()->getArmyPic(as, 0, player, NULL)->to_pixbuf();
-    Glib::RefPtr<Gdk::Pixbuf> empty_pic
-	= Gdk::Pixbuf::create(Gdk::COLORSPACE_RGB, true, 8, s->get_width(), s->get_height());
-    empty_pic->fill(0x00000000);
+    Glib::RefPtr<Gdk::Pixbuf> empty_pic =
+      ImageCache::getInstance()->getCircledArmyPic(as, 0, player, NULL, false, Shield::NEUTRAL, false)->to_pixbuf();
     
     Glib::ustring s1, s2, s3;
     Glib::ustring s4 = _("Current:");
@@ -303,9 +288,10 @@ void CityWindow::fill_in_production_info()
             Citylist *cl = Citylist::getInstance();
             City *dest = cl->getNearestFriendlyCity(city->getVectoring(), 4);
             s3 += String::ucompose(_(", then to %1"), 
-                                   dest ? dest->getName() : "Standard");
+                                   dest ? dest->getName() : _("Standard"));
           }
-      pic = gc->getArmyPic(as, a->getTypeId(), player, NULL)->to_pixbuf();
+      pic = gc->getCircledArmyPic(as, a->getTypeId(), player, NULL, false,
+                                  Shield::NEUTRAL, true)->to_pixbuf();
     }
     
     current_image->property_pixbuf() = pic;
@@ -316,21 +302,19 @@ void CityWindow::fill_in_production_info()
 
     if (city->getOwner () != Playerlist::getActiveplayer())
       {
-        turns_left_label->hide();
-        current_label->hide();
-        current_image->hide();
+        turns_left_label->set_text("");
+        current_label->set_text("");
+        pic->fill(0x00000000);
+        current_image->property_pixbuf() = pic;
         buy_button->set_sensitive(false);
         raze_button->set_sensitive(false);
         rename_button->set_sensitive(false);
         destination_button->set_sensitive(false);
         on_hold_button->set_sensitive(false);
         for (unsigned int i = 0; i < production_toggles.size(); ++i) 
-          {
-           // production_toggles[i]->set_sensitive(false);
-	    production_toggles[i]->set_active(false);
-          }
-        production_info_label1->hide();
-        production_info_label2->hide();
+          production_toggles[i]->set_active(false);
+        production_info_label1->set_text("");
+        production_info_label2->set_text("");
       }
     else
       {
@@ -342,8 +326,6 @@ void CityWindow::fill_in_production_info()
         rename_button->set_sensitive(true);
         destination_button->set_sensitive(true);
         on_hold_button->set_sensitive(true);
-        //for (unsigned int i = 0; i < production_toggles.size(); ++i) 
-          //production_toggles[i]->set_sensitive(true);
         production_info_label1->show();
         production_info_label2->show();
       }
@@ -401,8 +383,7 @@ void CityWindow::on_on_hold_clicked() //stop button
 
 void CityWindow::on_buy_clicked()
 {
-    BuyProductionDialog d(city);
-    d.set_parent_window(*dialog);
+    BuyProductionDialog d(*dialog, city);
     d.run();
 
     int army = d.get_selected_army();
@@ -430,45 +411,37 @@ void CityWindow::on_buy_clicked()
 
 void CityWindow::on_destination_clicked()
 {
-    DestinationDialog d(city, &d_see_all);
-    d.set_parent_window(*dialog);
+    DestinationDialog d(*dialog, city, &d_see_all);
     d.run();
     fill_in_production_info();
     prodmap->draw(Playerlist::getActiveplayer());
 }
 
-void CityWindow::on_map_changed(Glib::RefPtr<Gdk::Pixmap> map)
+void CityWindow::on_map_changed(Cairo::RefPtr<Cairo::Surface> map)
 {
-    map_image->property_pixmap() = map;
+  Glib::RefPtr<Gdk::Pixbuf> pixbuf = 
+    Gdk::Pixbuf::create(map, 0, 0, prodmap->get_width(), prodmap->get_height());
+  map_image->property_pixbuf() = pixbuf;
 }
 
 void CityWindow::on_rename_clicked ()
 {
-    Gtk::Dialog* subdialog;
-    
-    Glib::RefPtr<Gtk::Builder> renamexml
-	= Gtk::Builder::create_from_file(get_glade_path() + "/city-rename-dialog.ui");
-	
-    renamexml->get_widget("dialog", subdialog);
-    Decorated decorator;
-    decorator.decorate(subdialog);
-    decorator.window_closed.connect(sigc::mem_fun(subdialog, &Gtk::Dialog::hide));
-    subdialog->set_transient_for(*dialog);
+  LwDialog subdialog(*dialog, "city-rename-dialog.ui");
     
     Glib::ustring s = _("Rename City");
 
+    Glib::RefPtr<Gtk::Builder> renamexml = subdialog.get_builder();
     Gtk::Label *l;
     renamexml->get_widget("label", l);
     Gtk::Entry *e;
     renamexml->get_widget("name_entry", e);
 
-    decorator.set_title(s);
+    subdialog.set_title(s);
     s = _("Type the new name for this city:");
     l->set_text(s);
 
     e->set_text(city->getName());
-    subdialog->show_all();
-    int response = subdialog->run();
+    int response = subdialog.run_and_hide();
 
     if (response == Gtk::RESPONSE_ACCEPT)		// changed city name
       {
@@ -477,8 +450,6 @@ void CityWindow::on_rename_clicked ()
             (city, String::utrim(e->get_text()));
         fill_in_city_info();
       }
-    subdialog->hide();
-    delete subdialog;
   return;
 }
 
@@ -489,35 +460,22 @@ void CityWindow::on_raze_clicked ()
 
 bool CityWindow::on_raze_clicked (City *city, Gtk::Dialog *parent)
 {
-    Gtk::Dialog* subdialog;
-    
-    Glib::RefPtr<Gtk::Builder> razexml
-	= Gtk::Builder::create_from_file(get_glade_path() + "/city-raze-dialog.ui");
-	
-    razexml->get_widget("dialog", subdialog);
-    Decorated decorator;
-    decorator.decorate(subdialog);
-    decorator.window_closed.connect(sigc::mem_fun(subdialog, &Gtk::Dialog::hide));
-    subdialog->set_transient_for(*parent);
-    subdialog->set_icon_from_file(File::getMiscFile("various/castle_icon.png"));
+  LwDialog subdialog (*parent, "city-raze-dialog.ui");
     
     Glib::ustring s = _("Raze City");
+    Glib::RefPtr<Gtk::Builder> razexml = subdialog.get_builder();
 
     Gtk::Label *l;
     razexml->get_widget("label", l);
 
-    decorator.set_title(s);
+    subdialog.set_title(s);
     s = String::ucompose(_("Are you sure that you want to raze %1?"), 
                            city->getName());
     s += "\n";
     s += _("You won't be popular!");
     l->set_text(s);
 
-    subdialog->show_all();
-    int response = subdialog->run();
-    subdialog->hide();
-    delete subdialog;
-
+    int response = subdialog.run_and_hide();
     if (response == Gtk::RESPONSE_ACCEPT) // burn it to the ground ralphie boy!
       {
 	Playerlist::getActiveplayer()->cityRaze(city);
