@@ -1,4 +1,4 @@
-//  Copyright (C) 2008, 2009 Ben Asselstine
+//  Copyright (C) 2008, 2009, 2011, 2014 Ben Asselstine
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -23,28 +23,17 @@
 #include "diplomacy-dialog.h"
 #include "diplomacy-report-dialog.h"
 
-#include "glade-helpers.h"
-#include "image-helpers.h"
-#include "input-helpers.h"
-#include "ucompose.hpp"
 #include "defs.h"
-#include "File.h"
-#include "GraphicsCache.h"
+#include "ImageCache.h"
 #include "playerlist.h"
 #include "player.h"
 
-DiplomacyDialog::DiplomacyDialog(Player *player)
+DiplomacyDialog::DiplomacyDialog(Gtk::Window &parent, Player *player)
+ : LwDialog(parent, "diplomacy-dialog.ui")
 {
-  GraphicsCache *gc = GraphicsCache::getInstance();
+  ImageCache *gc = ImageCache::getInstance();
   Playerlist *pl = Playerlist::getInstance();
   d_player = player;
-  Glib::RefPtr<Gtk::Builder> xml
-    = Gtk::Builder::create_from_file(get_glade_path() + "/diplomacy-dialog.ui");
-
-  xml->get_widget("dialog", dialog);
-  decorate(dialog);
-  window_closed.connect(sigc::mem_fun(dialog, &Gtk::Dialog::hide));
-
   xml->get_widget("proposals_table", d_proposals_table);
   xml->get_widget("offers_table", d_offers_table);
   xml->get_widget("player_label", d_player_label);
@@ -69,7 +58,6 @@ DiplomacyDialog::DiplomacyDialog(Player *player)
       Glib::RefPtr<Gdk::Pixbuf> pixbuf= gc->getShieldPic(2, p)->to_pixbuf();
       Gtk::Image *im = new Gtk::Image();
       im->property_pixbuf() = pixbuf;
-      //im->set_padding(11, 0);
       d_proposals_table->attach(*manage(im), i + 0, i + 1, 0, 1, 
 				Gtk::SHRINK, Gtk::SHRINK);
       i++;
@@ -152,6 +140,7 @@ DiplomacyDialog::DiplomacyDialog(Player *player)
       Gtk::Image *im3 = new Gtk::Image();
       im3->property_pixbuf() = 
 	gc->getDiplomacyPic(1, Player::AT_PEACE)->to_pixbuf();
+      radio1->set_tooltip_text(_("Propose peace"));
       radio1->add(*manage(im3));
       radio1->set_mode(false);
       Gtk::RadioButtonGroup group = radio1->get_group();
@@ -172,6 +161,7 @@ DiplomacyDialog::DiplomacyDialog(Player *player)
       Gtk::Image *im4 = new Gtk::Image();
       im4->property_pixbuf() = 
 	gc->getDiplomacyPic(1, Player::AT_WAR_IN_FIELD)->to_pixbuf();
+      radio2->set_tooltip_text(_("Propose war on armies not in cities"));
       radio2->add(*manage(im4));
       radio2->set_mode(false);
       radio2->set_group(group);
@@ -192,6 +182,7 @@ DiplomacyDialog::DiplomacyDialog(Player *player)
       Gtk::Image *im5 = new Gtk::Image();
       im5->property_pixbuf() = 
 	gc->getDiplomacyPic(1, Player::AT_WAR)->to_pixbuf();
+      radio3->set_tooltip_text(_("Propose war"));
       radio3->add(*manage(im5));
       radio3->set_mode(false);
       radio3->set_group(group);
@@ -210,10 +201,6 @@ DiplomacyDialog::DiplomacyDialog(Player *player)
     }
 }
 
-DiplomacyDialog::~DiplomacyDialog()
-{
-  delete dialog;
-}
 void DiplomacyDialog::on_proposal_toggled (Gtk::ToggleButton *toggle, 
 					   Player *player, 
 					   Player::DiplomaticProposal proposal)
@@ -222,28 +209,8 @@ void DiplomacyDialog::on_proposal_toggled (Gtk::ToggleButton *toggle,
     d_player->proposeDiplomacy (proposal, player);
 }
 
-void DiplomacyDialog::set_parent_window(Gtk::Window &parent)
-{
-  dialog->set_transient_for(parent);
-  //dialog->set_position(Gtk::WIN_POS_CENTER_ON_PARENT);
-
-}
-
 void DiplomacyDialog::on_report_clicked()
 {
-  DiplomacyReportDialog d(d_player);
-  d.set_parent_window(*dialog);
-  d.run();
-  d.hide();
+  DiplomacyReportDialog d(*dialog, d_player);
+  d.run_and_hide();
 }
-
-void DiplomacyDialog::hide()
-{
-  dialog->hide();
-}
-void DiplomacyDialog::run()
-{
-  dialog->show_all();
-  dialog->run();
-}
-
