@@ -1,4 +1,4 @@
-//  Copyright (C) 2008, 2009 Ben Asselstine
+//  Copyright (C) 2008, 2009, 2014 Ben Asselstine
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -18,30 +18,20 @@
 #ifndef DIPLOMACY_DIALOG_H
 #define DIPLOMACY_DIALOG_H
 
-#include <memory>
-#include <vector>
-#include <sigc++/trackable.h>
 #include <gtkmm.h>
 
 #include "player.h"
+#include "lw-dialog.h"
 
-#include "decorated.h"
 //
 //
-class DiplomacyDialog: public Decorated
+class DiplomacyDialog: public LwDialog
 {
  public:
-    DiplomacyDialog(Player *player);
-    ~DiplomacyDialog();
+    DiplomacyDialog(Gtk::Window &parent, Player *player);
+    ~DiplomacyDialog() {};
 
-    void set_parent_window(Gtk::Window &parent);
-
-    void run();
-    void hide();
-    
  private:
-    Gtk::Dialog* dialog;
-
     void on_report_clicked();
     void on_proposal_toggled (Gtk::ToggleButton *toggle, Player *p, Player::DiplomaticProposal proposal);
 
@@ -52,7 +42,6 @@ class DiplomacyDialog: public Decorated
     Gtk::Label *d_player_label;
     Gtk::Image *d_player_shield_image;
     Gtk::Button *d_report_button;
-
 };
 
 #endif
