@@ -1,4 +1,4 @@
-//  Copyright (C) 2008, 2009 Ben Asselstine
+//  Copyright (C) 2008, 2009, 2014 Ben Asselstine
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -22,29 +22,17 @@
 
 #include "diplomacy-report-dialog.h"
 
-#include "glade-helpers.h"
-#include "image-helpers.h"
-#include "input-helpers.h"
-#include "ucompose.hpp"
 #include "defs.h"
-#include "File.h"
-#include "GraphicsCache.h"
+#include "ImageCache.h"
 #include "playerlist.h"
 #include "player.h"
 
-DiplomacyReportDialog::DiplomacyReportDialog(Player *player)
+DiplomacyReportDialog::DiplomacyReportDialog(Gtk::Window &parent, Player *player)
+ : LwDialog(parent, "diplomacy-report-dialog.ui")
 {
-  GraphicsCache *gc = GraphicsCache::getInstance();
+  ImageCache *gc = ImageCache::getInstance();
   Playerlist *pl = Playerlist::getInstance();
   d_player = player;
-  Glib::RefPtr<Gtk::Builder> xml
-    = Gtk::Builder::create_from_file(get_glade_path() + 
-				"/diplomacy-report-dialog.ui");
-
-  xml->get_widget("dialog", dialog);
-  decorate(dialog);
-  window_closed.connect(sigc::mem_fun(dialog, &Gtk::Dialog::hide));
-
   xml->get_widget("diplomacy_table", d_table);
 
   int order[MAX_PLAYERS];
@@ -96,25 +84,3 @@ DiplomacyReportDialog::DiplomacyReportDialog(Player *player)
 	}
     }
 }
-
-DiplomacyReportDialog::~DiplomacyReportDialog()
-{
-  delete dialog;
-}
-void DiplomacyReportDialog::set_parent_window(Gtk::Window &parent)
-{
-  dialog->set_transient_for(parent);
-  //dialog->set_position(Gtk::WIN_POS_CENTER_ON_PARENT);
-}
-
-void DiplomacyReportDialog::hide()
-{
-  dialog->hide();
-}
-
-void DiplomacyReportDialog::run()
-{
-  dialog->show_all();
-  dialog->run();
-}
-
