@@ -1,5 +1,5 @@
 //  Copyright (C) 2007, 2008, Ole Laursen
-//  Copyright (C) 2007, 2008, 2009, 2010 Ben Asselstine
+//  Copyright (C) 2007, 2008, 2009, 2010, 2012, 2014, 2015 Ben Asselstine
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -25,13 +25,12 @@
 #include "driver.h"
 
 #include <iostream>
-#include "error-utils.h"
 #include "main.h"
 #include "splash-window.h"
 #include "game-window.h"
 #include "game-lobby-dialog.h"
 #include "defs.h"
-#include "GraphicsCache.h"
+#include "ImageCache.h"
 #include "GameScenario.h"
 #include "CreateScenario.h"
 #include "counter.h"
@@ -44,18 +43,14 @@
 #include "xmlhelper.h"
 #include "Configuration.h"
 #include "ucompose.hpp"
-#include "sound.h"
+#include "snd.h"
 #include "timed-message-dialog.h"
 #include "game-preferences-dialog.h"
 
 #include "game-client.h"
-#include "pbm-game-client.h"
 #include "game-server.h"
-#include "pbm-game-server.h"
 #include "NextTurnHotseat.h"
-#include "NextTurnPbm.h"
 #include "NextTurnNetworked.h"
-#include "pbm/pbm.h"
 #include "recently-played-game-list.h"
 #include "recently-played-game.h"
 #include "Backpack.h"
@@ -72,8 +67,12 @@
 #include "profilelist.h"
 #include "gamelist-client.h"
 #include "gamehost-client.h"
+#include "tilesetlist.h"
+#include "citysetlist.h"
+#include "herotemplates.h"
+#include "new-network-game-dialog.h"
 
-Driver::Driver(std::string load_filename)
+Driver::Driver(Glib::ustring load_filename)
 {
     game_window = NULL;
     game_lobby_dialog = NULL;
@@ -87,8 +86,6 @@ Driver::Driver(std::string load_filename)
 	sigc::mem_fun(*this, &Driver::on_new_hosted_network_game_requested));
     splash_window->new_remote_network_game_requested.connect(
 	sigc::mem_fun(*this, &Driver::on_new_remote_network_game_requested));
-    splash_window->new_pbm_game_requested.connect(
-	sigc::mem_fun(*this, &Driver::on_new_pbm_game_requested));
     splash_window->load_requested.connect(
 	sigc::mem_fun(*this, &Driver::on_load_requested));
     splash_window->quit_requested.connect(
@@ -99,7 +96,7 @@ Driver::Driver(std::string load_filename)
     //here are the command-line options that don't bring up the splash screen:
     if (Main::instance().start_stress_test) 
       {
-	Sound::deleteInstance();
+	Snd::deleteInstance();
 	stress_test();
 	exit(0);
       }
@@ -111,7 +108,7 @@ Driver::Driver(std::string load_filename)
     if (Main::instance().start_headless_server)
       {
         GameScenario *game_scenario = NULL;
-        std::string path = load_filename;
+        Glib::ustring path = load_filename;
         if (load_filename.empty() == true)
           {
             GameParameters g;
@@ -119,7 +116,7 @@ Driver::Driver(std::string load_filename)
             game_scenario = create_new_scenario(g, GameScenario::NETWORKED);
             if (!game_scenario)
               {
-                cerr << "Error: could not load randomly generated map." << 
+                std::cerr << "Error: could not load randomly generated map." << 
                   std::endl;
                 exit(1);
               }
@@ -133,7 +130,7 @@ Driver::Driver(std::string load_filename)
               game_scenario = load_game(load_filename);
             if (!game_scenario || broken)
               {
-                cerr << "Error: could not load file " << 
+                std::cerr << "Error: could not load file " << 
                   load_filename << std::endl;
                 exit(1);
               }
@@ -160,7 +157,7 @@ void Driver::serve (GameScenario *game_scenario)
   if (Main::instance().port)
     port = Main::instance().port;
   game_server->port_in_use.connect(sigc::mem_fun(*this, &Driver::on_could_not_bind_to_port_for_headless_server));
-  std::string id = "";
+  Glib::ustring id = "";
   if (Profilelist::getInstance()->empty() == false)
     id = Profilelist::getInstance()->front()->getId();
   game_server->start(game_scenario, port, id, "admin");
@@ -184,10 +181,10 @@ void Driver::serve (GameScenario *game_scenario)
   game_server->notifyClientsGameMayBeginNow();
   Game *game = new Game(game_scenario, next_turn);
   if (game)
-    game_server->player_sits.connect(sigc::mem_fun(this, &Driver::on_client_sits_down_in_headless_server_game));
+    game_server->player_sits.connect(sigc::hide(sigc::hide(sigc::mem_fun(this, &Driver::on_client_sits_down_in_headless_server_game))));
 }
 
-void Driver::on_client_sits_down_in_headless_server_game(Player *p, std::string nick)
+void Driver::on_client_sits_down_in_headless_server_game()
 {
   static unsigned int count;
   count++;
@@ -330,44 +327,9 @@ void Driver::run()
       g.see_opponents_production = true;
       on_new_game_requested(g);
     }
-  else if (Main::instance().turn_filename != "") 
-    {
-      PbmGameClient *pbm_game_client = PbmGameClient::getInstance();
-      GameScenario *game_scenario = load_game(d_load_filename);
-      if (game_scenario == NULL)
-	return;
-      //now apply the actions in the turn file
-      bool broken;
-
-      //if the active player isn't a network player than don't do anything
-      if (Playerlist::getActiveplayer()->getType() != Player::NETWORKED)
-	return;
-      //load the file, and decode them as we go.
-      XML_Helper helper(Main::instance().turn_filename, std::ios::in, 
-			Configuration::s_zipfiles);
-      NextTurnPbm *nextTurn;
-      nextTurn = new NextTurnPbm(game_scenario->getTurnmode(),
-			      game_scenario->s_random_turns);
-      broken = pbm_game_client->loadWithHelper (helper, 
-						Playerlist::getActiveplayer());
-      helper.close();
-      if (!broken)
-	{
-	  game_scenario->saveGame(d_load_filename);
-	  if (Playerlist::getActiveplayer()->getType() != Player::NETWORKED)
-	    {
-	      if (splash_window)
-		splash_window->hide();
-
-	      init_game_window();
-	      game_window->show();
-	      game_window->load_game(game_scenario, nextTurn);
-	    }
-	}
-    }
   else if (Main::instance().start_robots != 0) 
     {
-      Sound::deleteInstance();
+      Snd::deleteInstance();
       lordsawaromatic("127.0.0.1", LORDSAWAR_PORT, Player::AI_FAST,
 		      Main::instance().start_robots);
     }
@@ -379,10 +341,9 @@ void Driver::run()
 	  GameParameters g;
 	  g.map_path = d_load_filename;
 	  size_t found = d_load_filename.find(".map");
-	  if (found != std::string::npos)
+	  if (found != Glib::ustring::npos)
 	    {
-	      GamePreferencesDialog d(d_load_filename, GameScenario::HOTSEAT);
-	      d.set_parent_window(*splash_window->get_window());
+	      GamePreferencesDialog d(*splash_window->get_window(), d_load_filename, GameScenario::HOTSEAT);
 	      d.game_started.connect(sigc::mem_fun
 				     (*this, &Driver::on_new_game_requested));
 	      d.run();
@@ -390,7 +351,7 @@ void Driver::run()
 	  else
 	    {
 	      found = d_load_filename.find(SAVE_EXT);
-	      if (found != std::string::npos)
+	      if (found != Glib::ustring::npos)
 		on_load_requested(d_load_filename);
 	      else
 		on_new_game_requested(g);
@@ -446,7 +407,7 @@ void Driver::on_hosted_player_says_game_may_begin()
   game_server->notifyClientsGameMayBeginNow();
 }
 
-void Driver::on_hosted_player_chat(std::string message)
+void Driver::on_hosted_player_chat(Glib::ustring message)
 {
   GameServer *game_server = GameServer::getInstance();
   game_server->chat(message);
@@ -476,7 +437,7 @@ void Driver::on_client_player_changed_type(Player *player, int type)
   game_client->change_type(player, type);
 }
 
-void Driver::on_client_player_chat(std::string message)
+void Driver::on_client_player_chat(Glib::ustring message)
 {
   GameClient *game_client = GameClient::getInstance();
   game_client->chat(message);
@@ -488,7 +449,7 @@ GameScenario *Driver::create_new_scenario(GameParameters &g, GameScenario::PlayM
   if (g.map_path.empty()) 
     {
       // construct new random scenario if we're not going to load the game
-      std::string path = 
+      Glib::ustring path = 
         NewRandomMapDialog::create_and_dump_scenario("random.map", g, NULL);
       g.map_path = path;
     }
@@ -496,7 +457,7 @@ GameScenario *Driver::create_new_scenario(GameParameters &g, GameScenario::PlayM
     update_uuid = true;
 
   bool broken = false;
-						 
+
   GameScenario* game_scenario = new GameScenario(g.map_path, broken);
   if (broken)
     return NULL;
@@ -544,12 +505,11 @@ void Driver::on_connected_to_gamelist_server_for_advertising(GameScenario *game_
     new RecentlyPlayedNetworkedGame(game_scenario, p);
   g->setNumberOfPlayers(Playerlist::getInstance()->countPlayersAlive());
   gsc->received_advertising_response.connect
-    (sigc::mem_fun(*this, &Driver::on_advertising_response_received));
+    (sigc::hide(sigc::mem_fun(*this, &Driver::on_advertising_response_received)));
   gsc->request_advertising(g);
 }
 
-void Driver::on_advertising_response_received(std::string scenario_id, 
-                                              std::string err)
+void Driver::on_advertising_response_received(Glib::ustring scenario_id)
 {
   d_advertised_scenario_id = scenario_id;
   return;
@@ -578,19 +538,18 @@ void Driver::on_could_not_connect_to_gamehost_server()
         (*splash_window->get_window(),
          String::ucompose(_("Could not connect to gamehost server:\n%1:%2"),
                           ghc->getHost(), ghc->getPort()), 0);
-      dialog.run();
-      dialog.hide();
+      dialog.run_and_hide();
     }
 }
 
 void Driver::on_connected_to_gamehost_server_for_hosting_request (GameScenario *game_scenario)
 {
   GamehostClient *ghc = GamehostClient::getInstance();
-  ghc->received_host_response.connect(sigc::bind(sigc::mem_fun(*this, &Driver::on_got_game_host_response), game_scenario));
+  ghc->received_host_response.connect(sigc::bind(sigc::hide<0>(sigc::mem_fun(*this, &Driver::on_got_game_host_response)), game_scenario));
   ghc->request_game_host (game_scenario->getId());
 }
 
-void Driver::on_got_game_host_response(std::string scenario_id, std::string err, GameScenario *game_scenario)
+void Driver::on_got_game_host_response(Glib::ustring err, GameScenario *game_scenario)
 {
   if (err != "")
     {
@@ -601,23 +560,22 @@ void Driver::on_got_game_host_response(std::string scenario_id, std::string err,
             String::ucompose(_("Gamehost Server Error: %1"), err);
           TimedMessageDialog dialog(*splash_window->get_window(), s, 0);
           dialog.set_title(_("Server Failure"));
-          dialog.run();
-          dialog.hide();
+          dialog.run_and_hide();
         }
       return;
     }
   GamehostClient *ghc = GamehostClient::getInstance();
-  ghc->received_map_response.connect(sigc::mem_fun(*this, &Driver::on_remote_game_hosted));
+  ghc->received_map_response.connect(sigc::hide<0>(sigc::mem_fun(*this, &Driver::on_remote_game_hosted)));
   if (download_window)
     delete download_window;
   download_window = new NewNetworkGameDownloadWindow(_("Uploading."));
   download_window->pulse();
   upload_heartbeat_conn = Glib::signal_timeout().connect
-    (bind_return(sigc::mem_fun(*this, &Driver::upload_heartbeat), true), 1 * 1000);
+    (sigc::mem_fun(*this, &Driver::upload_heartbeat), 1 * 1000);
   ghc->send_map(game_scenario);
 }
 
-void Driver::on_remote_game_hosted(std::string scenario_id, guint32 port, std::string err)
+void Driver::on_remote_game_hosted(guint32 port, Glib::ustring err)
 {
   upload_heartbeat_conn.disconnect();
   if (download_window)
@@ -634,8 +592,7 @@ void Driver::on_remote_game_hosted(std::string scenario_id, guint32 port, std::s
             String::ucompose(_("Gamehost Server Error: %1"), err);
           TimedMessageDialog dialog(*splash_window->get_window(), s, 0);
           dialog.set_title(_("Server Failure"));
-          dialog.run();
-          dialog.hide();
+          dialog.run_and_hide();
         }
     return;
     }
@@ -660,15 +617,34 @@ void Driver::on_new_hosted_network_game_requested(GameParameters g, int port,
       {
 	TimedMessageDialog dialog(*splash_window->get_window(),
 				  _("Corrupted saved game file."), 0);
-	dialog.run();
-	dialog.hide();
+	dialog.run_and_hide();
 	return;
       }
-    if (remotely_hosted)
-      {
-        remotely_serve (game_scenario, p);
-        return;
-      }
+    on_load_hosted_network_game_requested(game_scenario, port, p, advertised,
+                                          remotely_hosted);
+}
+void Driver::on_game_ended_and_load_network_game(Glib::ustring filename, int port, Profile *p, bool advertised, bool remotely_hosted)
+{
+  on_game_ended();
+  bool broken = false;
+  GameScenario* game_scenario = new GameScenario(filename, broken);
+  splash_window->hide();
+  if (game_window)
+    delete game_window;
+  game_window = NULL;
+  on_load_hosted_network_game_requested(game_scenario, port, p, advertised, remotely_hosted);
+}
+
+void Driver::on_load_hosted_network_game_requested(GameScenario *game_scenario,
+                                                   int port, Profile *p, 
+                                                   bool advertised,
+                                                   bool remotely_hosted)
+{
+  if (remotely_hosted)
+    {
+      remotely_serve (game_scenario, p);
+      return;
+    }
 
   GameServer *game_server = GameServer::getInstance();
   game_server->port_in_use.connect(sigc::mem_fun(*this, &Driver::on_could_not_bind_to_port));
@@ -691,14 +667,14 @@ void Driver::on_new_hosted_network_game_requested(GameParameters g, int port,
   next_turn->srequestAbort.connect(sigc::mem_fun(game_server, &GameServer::on_turn_aborted));
   if (game_lobby_dialog)
     delete game_lobby_dialog;
-  game_lobby_dialog = new GameLobbyDialog(game_scenario, next_turn, 
-					      game_server, true);
+  game_lobby_dialog = new GameLobbyDialog(*splash_window->get_window(), 
+                                          game_scenario, next_turn, 
+                                          game_server, true);
   game_server->get_next_player.connect(sigc::mem_fun(next_turn, &NextTurnNetworked::next));
   game_server->round_ends.connect
     (sigc::mem_fun(*this, &Driver::on_keep_network_play_going));
   Playerlist::getInstance()->splayerDead.connect
     (sigc::mem_fun(GameServer::getInstance(), &GameServer::sendKillPlayer));
-  game_lobby_dialog->set_parent_window(*splash_window->get_window());
   game_lobby_dialog->player_sat_down.connect
     (sigc::mem_fun(this, &Driver::on_hosted_player_sat_down));
   game_lobby_dialog->player_stood_up.connect
@@ -716,24 +692,25 @@ void Driver::on_new_hosted_network_game_requested(GameParameters g, int port,
   game_lobby_dialog->show();
   bool response = game_lobby_dialog->run();
   game_lobby_dialog->hide();
-    
+
   if (response == false)
     {
-      if (d_advertised_scenario_id != "")
-        unadvertise_game (d_advertised_scenario_id, p);
-      GameServer::deleteInstance();
-
+      on_game_ended();
       delete game_scenario;
-      if (splash_window)
-        splash_window->show();
     }
 }
 
-  
 void Driver::on_server_went_away()
 {
   upload_heartbeat_conn.disconnect();
   heartbeat_conn.disconnect();
+  if (game_window)
+    {
+      //hide() isn't sufficient here because we need to close any open dialogs.
+      //they're all set as "destroy with parent" so deleting works.
+      delete game_window;
+      game_window = NULL;
+    }
   if (game_lobby_dialog)
     game_lobby_dialog->hide();
   if (download_window)
@@ -743,8 +720,7 @@ void Driver::on_server_went_away()
   TimedMessageDialog dialog(*splash_window->get_window(), 
 			    _("Server went away."), 0);
   dialog.set_title(_("Disconnected"));
-  dialog.run();
-  dialog.hide();
+  dialog.run_and_hide();
   GameClient::deleteInstance();
 }
 
@@ -762,12 +738,11 @@ void Driver::on_client_could_not_connect()
     (*splash_window->get_window(), 
      String::ucompose(_("Could not connect to server:\n%1 %2"),
                         gc->getHost(), gc->getPort()), 0);
-  dialog.run();
-  dialog.hide();
+  dialog.run_and_hide();
   GameClient::deleteInstance();
 }
 
-void Driver::on_new_remote_network_game_requested(std::string host, unsigned short port, Profile *p)
+void Driver::on_new_remote_network_game_requested(Glib::ustring host, unsigned short port, Profile *p)
 {
   if (splash_window)
     splash_window->hide();
@@ -780,7 +755,7 @@ void Driver::on_new_remote_network_game_requested(std::string host, unsigned sho
     (sigc::mem_fun(this, &Driver::on_server_went_away));
   game_client->client_could_not_connect.connect
     (sigc::mem_fun(this, &Driver::on_client_could_not_connect));
-  game_scenario_received.connect
+  recv_conn = game_scenario_received.connect
     (sigc::bind(sigc::mem_fun(this, &Driver::on_game_scenario_received), p));
   if (download_window)
     delete download_window;
@@ -788,53 +763,50 @@ void Driver::on_new_remote_network_game_requested(std::string host, unsigned sho
   download_window->pulse();
   game_client->start(host, port, p->getId(), p->getNickname());
   heartbeat_conn = Glib::signal_timeout().connect
-    (bind_return(sigc::mem_fun(*this, &Driver::heartbeat), true), 1 * 1000);
+    (sigc::mem_fun(*this, &Driver::heartbeat), 1 * 1000);
 
 }
 
-void Driver::upload_heartbeat()
+bool Driver::upload_heartbeat()
 {
   if (download_window)
     download_window->pulse();
+  return true;
 }
 
-void Driver::heartbeat()
+bool Driver::heartbeat()
 {
-  static bool already_done = false;
-  if (already_done)
-    return;
   if (game_scenario_downloaded == "")
     {
       if (download_window)
 	download_window->pulse();
-      return;
+      return true;
     }
   
   game_scenario_received.emit(game_scenario_downloaded);
-  already_done = true;
+  return false;
 }
 
-void Driver::on_game_scenario_received(std::string path, Profile *p)
+void Driver::on_game_scenario_received(Glib::ustring path, Profile *p)
 {
+  recv_conn.disconnect();
   heartbeat_conn.disconnect();
   if (download_window)
     download_window->hide();
   GameScenario *game_scenario = load_game(path);
   GameClient *game_client = GameClient::getInstance();
-  std::string host = game_client->getHost();
+  Glib::ustring host = game_client->getHost();
   guint32 port = game_client->getPort();
   RecentlyPlayedGameList::getInstance()->addNetworkedEntry(game_scenario, p, host, port);
   RecentlyPlayedGameList::getInstance()->save();
 
   NextTurnNetworked *next_turn = new NextTurnNetworked(game_scenario->getTurnmode(), game_scenario->s_random_turns);
   game_client->start_player_turn.connect(sigc::mem_fun(next_turn, &NextTurnNetworked::start_player));
-  //game_client->round_begins.connect(sigc::mem_fun(next_turn, &NextTurnNetworked::start));
   game_client->round_ends.connect(sigc::mem_fun(next_turn, &NextTurnNetworked::finishRound));
   if (game_lobby_dialog)
     delete game_lobby_dialog;
-  game_lobby_dialog = new GameLobbyDialog(game_scenario, next_turn, 
+  game_lobby_dialog = new GameLobbyDialog(*splash_window->get_window(), game_scenario, next_turn, 
 					      GameClient::getInstance(), false);
-  game_lobby_dialog->set_parent_window(*splash_window->get_window());
   game_lobby_dialog->player_sat_down.connect
     (sigc::mem_fun(this, &Driver::on_client_player_sat_down));
   game_lobby_dialog->player_stood_up.connect
@@ -849,21 +821,16 @@ void Driver::on_game_scenario_received(std::string path, Profile *p)
     (sigc::mem_fun(this, &Driver::start_network_game_requested));
   game_lobby_dialog->show();
   bool response = game_lobby_dialog->run();
-  game_lobby_dialog->hide();
 
   if (response == false)
     {
       GameClient::getInstance()->disconnect();
       on_game_ended();
-
-      if (splash_window)
-	splash_window->show();
-    
-      GameClient::deleteInstance();
       delete game_scenario;
     }
 }
-void Driver::on_game_scenario_downloaded(std::string path)
+
+void Driver::on_game_scenario_downloaded(Glib::ustring path)
 {
   game_scenario_downloaded = path;
   //emitting the signal doesn't work.
@@ -880,26 +847,24 @@ void Driver::on_new_game_requested(GameParameters g)
       {
 	TimedMessageDialog dialog(*splash_window->get_window(),
 				  _("Corrupted saved game file."), 0);
-	dialog.run();
-	dialog.hide();
+	dialog.run_and_hide();
 	splash_window->show();
 	return;
       }
 
-    std::list<std::string> e, w;
+    std::list<Glib::ustring> e, w;
     if (g.map_path != "" && game_scenario->validate(e, w) == false)
       {
 	TimedMessageDialog dialog
 	  (*splash_window->get_window(), 
 	   _("Invalid map file.\n" 
 	     "Please validate it in the scenario editor."), 0);
-	std::list<std::string>::iterator it = e.begin();
+	std::list<Glib::ustring>::iterator it = e.begin();
 	for (; it != e.end(); it++)
 	  {
 	    printf ("error: %s\n", (*it).c_str());
 	  }
-	dialog.run();
-	dialog.hide();
+	dialog.run_and_hide();
 	splash_window->show();
 	return;
       }
@@ -915,7 +880,7 @@ void Driver::on_new_game_requested(GameParameters g)
     game_window->new_game(game_scenario, next_turn);
 }
 
-void Driver::on_load_requested(std::string filename)
+void Driver::on_load_requested(Glib::ustring filename)
 {
     if (splash_window)
 	splash_window->hide();
@@ -927,30 +892,29 @@ void Driver::on_load_requested(std::string filename)
 	return;
       }
 
-    init_game_window();
-    
-    game_window->show();
     if (game_scenario->getPlayMode() == GameScenario::HOTSEAT)
-      game_window->load_game
-	(game_scenario, new NextTurnHotseat(game_scenario->getTurnmode(),
-					    game_scenario->s_random_turns));
-    else if (game_scenario->getPlayMode() == GameScenario::PLAY_BY_MAIL)
       {
-	PbmGameServer::getInstance()->start();
-	game_window->load_game
-	  (game_scenario, new NextTurnPbm(game_scenario->getTurnmode(),
-					  game_scenario->s_random_turns));
+        init_game_window();
+
+        game_window->show();
+        game_window->load_game
+          (game_scenario, new NextTurnHotseat(game_scenario->getTurnmode(),
+                                              game_scenario->s_random_turns));
       }
     else if (game_scenario->getPlayMode() == GameScenario::NETWORKED)
       {
-	TimedMessageDialog dialog(*splash_window->get_window(),
-				  _("Can't load networked game from file."), 0);
-	dialog.run();
-	dialog.hide();
-        //well isn't this interesting.  we lack a game station.
-      //game_window->load_game
-	//(game_scenario, new NextTurnNetworked(game_scenario->getTurnmode(),
-					      //game_scenario->s_random_turns));
+        NewNetworkGameDialog nngd(*splash_window->get_window(), true);
+        bool retval = nngd.run();
+        nngd.hide();
+        if (retval == false)
+          splash_window->show();
+        else
+          {
+            on_load_hosted_network_game_requested(game_scenario, LORDSAWAR_PORT,
+						  nngd.getProfile(), 
+                                                  nngd.isAdvertised(),
+                                                  nngd.isRemotelyHosted());
+          }
       }
 }
 
@@ -967,11 +931,17 @@ void Driver::on_quit_requested()
 
 void Driver::on_game_ended()
 {
-    if (game_lobby_dialog)
-      game_lobby_dialog->clean_up_players();
+  if (game_lobby_dialog)
+    {
+      delete game_lobby_dialog;
+      game_lobby_dialog = NULL;
+    }
+
   if (game_window)
     {
       game_window->hide();
+      delete game_window;
+      game_window = NULL;
     }
 
   if (GameServer::getInstance()->isListening() &&
@@ -984,16 +954,20 @@ void Driver::on_game_ended()
     }
 
   GameClient::deleteInstance();
-  PbmGameClient::deleteInstance();
   GameServer::deleteInstance();
-  PbmGameServer::deleteInstance();
 
-  GraphicsCache::deleteInstance();
+  ImageCache::deleteInstance();
   GamehostClient::getInstance()->disconnect();
   GamehostClient::deleteInstance();
   GamelistClient::getInstance()->disconnect();
   GamelistClient::deleteInstance();
 
+  Armysetlist::deleteInstance();
+  Shieldsetlist::deleteInstance();
+  Tilesetlist::deleteInstance();
+  Citysetlist::deleteInstance();
+  HeroTemplates::deleteInstance();
+
   splash_window->show();
 }
 
@@ -1017,6 +991,8 @@ void Driver::init_game_window()
 	sigc::mem_fun(*this, &Driver::on_show_lobby_requested));
     game_window->quit_requested.connect(
 	sigc::mem_fun(*this, &Driver::on_quit_requested));
+    game_window->load_hosted_network_game.connect(
+	sigc::mem_fun(*this, &Driver::on_game_ended_and_load_network_game));
 
     //make the width+height suitable for the screen size.
     Glib::RefPtr<Gdk::Screen> screen = Gdk::Display::get_default()->get_default_screen();
@@ -1029,7 +1005,7 @@ void Driver::init_game_window()
 
 }
 
-GameScenario *Driver::load_game(std::string file_path)
+GameScenario *Driver::load_game(Glib::ustring file_path)
 {
     bool broken = false;
     GameScenario* game_scenario = new GameScenario(file_path, broken);
@@ -1038,116 +1014,17 @@ GameScenario *Driver::load_game(std::string file_path)
       {
 	TimedMessageDialog dialog(*splash_window->get_window(),
 				  _("Corrupted saved game file."), 0);
-	dialog.run();
-	dialog.hide();
+	dialog.run_and_hide();
 	return NULL;
       }
     return game_scenario;
 }
 
-void Driver::on_new_pbm_game_requested(GameParameters g)
-{
-  std::string filename;
-  std::string temp_filename = File::getSavePath() + "pbmtmp" + SAVE_EXT;
-      
-  GameScenario *game_scenario = 
-    create_new_scenario(g, GameScenario::PLAY_BY_MAIL);
-  if (game_scenario == NULL)
-    {
-      TimedMessageDialog dialog(*splash_window->get_window(),
-				_("Corrupted saved game file."), 0);
-	dialog.run();
-	dialog.hide();
-      return;
-    }
-  game_scenario->saveGame(temp_filename);
-  std::string player_name = Playerlist::getActiveplayer()->getName();
-  delete game_scenario;
-  pbm play_by_mail;
-  play_by_mail.init(temp_filename);
-
-  Gtk::FileChooserDialog chooser(*splash_window->get_window(), _("Save the scenario and mail it to the first player"),
-				 Gtk::FILE_CHOOSER_ACTION_SAVE);
-  Gtk::FileFilter sav_filter;
-  sav_filter.add_pattern("*" + SAVE_EXT);
-  chooser.set_filter(sav_filter);
-  chooser.set_current_folder(Glib::get_home_dir());
-
-  chooser.add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
-  chooser.add_button(Gtk::Stock::SAVE, Gtk::RESPONSE_ACCEPT);
-  chooser.set_default_response(Gtk::RESPONSE_ACCEPT);
-
-  chooser.show_all();
-  int res = chooser.run();
-  chooser.hide();
-
-  if (res == Gtk::RESPONSE_ACCEPT)
-    {
-      std::string filename = chooser.get_filename();
-
-      remove (filename.c_str());
-      if (rename(temp_filename.c_str(), filename.c_str()))
-	{
-	  char* err = strerror(errno);
-	  std::cerr <<_("Error while trying to rename the temporary file to ")
-	    << filename << "\n";
-	  std::cerr <<_("Error: ") <<err <<std::endl;
-	}
-    }
-  Glib::ustring s = String::ucompose(_("Now send the saved-game file to %1"),
-				     player_name);
-  TimedMessageDialog dialog(*splash_window->get_window(), s, 0);
-  dialog.run();
-  dialog.hide();
-  return;
-}
-
 void Driver::stressTestNextRound()
 {
   static int count = 1;
-  /*
-  static time_t prev_round_start = time(NULL);
-  if (count == 1)
-    {
-      FILE * fileptr = fopen("/tmp/crapola.csv", "w");
-      fclose(fileptr);
-    }
-  time_t now = time(NULL);
-  */
   count++;
   printf ("starting round %d!\n", count);
-  /*
-  FILE * fileptr = fopen("/tmp/crapola.csv", "a");
-  int total_fights = Playerlist::getInstance()->countFightsThisTurn();
-  int total_moves = Playerlist::getInstance()->countMovesThisTurn();
-  fprintf(fileptr, "%d, %d, %d, %d, %d, %d, %d, %d, %d, %d", count, 
-          now - prev_round_start,
-          Stacklist::getNoOfStacks(), 
-          Stacklist::getNoOfArmies(), 
-          total_fights,
-          total_moves,
-          Ruinlist::getInstance()->countUnexploredRuins(Playerlist::getInstance()->getNeutral()),
-          Playerlist::getInstance()->countBlessings(),
-          Citylist::getInstance()->countCities(Playerlist::getInstance()->getNeutral()), 
-          Playerlist::getInstance()->countPlayersAlive());
-  //if (Playerlist::getInstance()->countPlayersAlive() == 2)
-    {
-      Playerlist *pl = Playerlist::getInstance();
-      for (Playerlist::iterator it = pl->begin(); it != pl->end(); it++)
-        {
-          if ((*it)->isDead() == true)
-            continue;
-          if ((*it) == pl->getNeutral())
-            continue;
-          fprintf(fileptr,", %d: (%d %d)", (*it)->getId(), Citylist::getInstance()->countCities(*it), (*it)->getHeroes().size() + (*it)->countAllies());
-        }
-    }
-  fprintf(fileptr,"\n");
-  fclose(fileptr);
-      
-  prev_round_start = now;
-  sleep (1);
-  */
 }
 
 void Driver::stress_test()
@@ -1206,7 +1083,7 @@ void Driver::stress_test()
   g.see_opponents_production = true;
       
   bool broken = false;
-  std::string path;
+  Glib::ustring path;
   path = NewRandomMapDialog::create_and_dump_scenario("random.map", g, NULL);
   g.map_path = path;
 
@@ -1234,16 +1111,18 @@ void Driver::stress_test()
   nextTurn->start();
   //next turn and game_Scenario get deleted inside game.
   size_t mins = (time(NULL) - start) / 60;
-  printf("duration: %d mins, turns: %d ", mins, game_scenario->getRound());
+  printf("duration: %lu mins, turns: %d ", mins, game_scenario->getRound());
   fflush(stdout);
   printf("winner type: %s\n", Player::playerTypeToString(Player::Type(Playerlist::getInstance()->getFirstLiving()->getType())).c_str());
-  Glib::ustring s = String::ucompose("/tmp/run-seed-%1", 
-                                     Main::instance().random_number_seed);
+  Glib::ustring s = 
+    File::getTempFile(Glib::get_tmp_dir(), 
+                      String::ucompose ("run-seed-%1", 
+                                        Main::instance().random_number_seed));
   game_scenario->saveGame(s + SAVE_EXT);
 
 }
 	
-void Driver::lordsawaromatic(std::string host, unsigned short port, Player::Type type, int num_players)
+void Driver::lordsawaromatic(Glib::ustring host, unsigned short port, Player::Type type, int num_players)
 {
   GameClient *game_client = GameClient::getInstance();
   game_client->game_scenario_received.connect
@@ -1257,19 +1136,18 @@ void Driver::lordsawaromatic(std::string host, unsigned short port, Player::Type
   game_scenario_received.connect
     (sigc::mem_fun(this, &Driver::on_game_scenario_received_for_robots));
   game_client->setNickname("robot");
-  std::string id = "";
+  Glib::ustring id = "";
   if (Profilelist::getInstance()->empty() == false)
     id = Profilelist::getInstance()->front()->getId();
   game_client->start(host, port, id, "robot");
   heartbeat_conn = Glib::signal_timeout().connect
-    (bind_return(sigc::mem_fun(*this, &Driver::heartbeat), true), 1 * 1000);
+    (sigc::mem_fun(*this, &Driver::heartbeat), 1 * 1000);
   robot_player_type = type;
   number_of_robots = num_players;
 }
 
-void Driver::on_game_scenario_received_for_robots(std::string path)
+void Driver::on_game_scenario_received_for_robots(Glib::ustring path)
 {
-
   heartbeat_conn.disconnect();
   GameScenario *game_scenario = load_game(path);
   if (!game_scenario)
@@ -1380,7 +1258,7 @@ void Driver::on_keep_network_play_going()
     
 void Driver::on_could_not_bind_to_port_for_headless_server(int port)
 {
-  cerr << "Could not bind to port " << port << std::endl;
+  std::cerr << "Could not bind to port " << port << std::endl;
   exit(1);
 }
 
@@ -1391,11 +1269,10 @@ void Driver::on_could_not_bind_to_port (int port)
   Glib::ustring s = String::ucompose(_("Could not bind to port %1"), port);
   TimedMessageDialog dialog(*splash_window->get_window(), s, 0);
   dialog.set_title(_("Server Failure"));
-  dialog.run();
-  dialog.hide();
+  dialog.run_and_hide();
 }
 
-void Driver::unadvertise_game(std::string scenario_id, Profile *p)
+void Driver::unadvertise_game(Glib::ustring scenario_id, Profile *p)
 {
   GamelistClient *gsc = GamelistClient ::getInstance();
   gsc->client_connected.connect
@@ -1404,16 +1281,15 @@ void Driver::unadvertise_game(std::string scenario_id, Profile *p)
              Configuration::s_gamelist_server_port, p);
 }
 
-void Driver::on_connected_to_gamelist_server_for_advertising_removal(std::string scenario_id)
+void Driver::on_connected_to_gamelist_server_for_advertising_removal(Glib::ustring scenario_id)
 {
   GamelistClient *gsc = GamelistClient::getInstance();
   gsc->received_advertising_removal_response.connect
-    (sigc::mem_fun(*this, &Driver::on_advertising_removal_response_received));
+    (sigc::hide(sigc::hide(sigc::mem_fun(*this, &Driver::on_advertising_removal_response_received))));
   gsc->request_advertising_removal(scenario_id);
 }
 
-void Driver::on_advertising_removal_response_received(std::string scenario_id, 
-                                                      std::string err)
+void Driver::on_advertising_removal_response_received()
 {
   d_advertised_scenario_id = "";
   return;
