@@ -1,5 +1,5 @@
 //  Copyright (C) 2007, 2008, Ole Laursen
-//  Copyright (C) 2007, 2008, 2009 Ben Asselstine
+//  Copyright (C) 2007, 2008, 2009, 2014, 2015 Ben Asselstine
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -36,11 +36,9 @@ class Army;
 class FightWindow: public sigc::trackable
 {
  public:
-    FightWindow(Fight &fight);
+    FightWindow(Gtk::Window &parent, Fight &fight);
     ~FightWindow();
 
-    void set_parent_window(Gtk::Window &parent);
-
     void hide();
     void run(bool *quick);
     
@@ -52,7 +50,6 @@ class FightWindow: public sigc::trackable
     {
 	Army *army;
 	int hp;
-	Gtk::ProgressBar *bar;
         Gtk::EventBox *box;
 	Gtk::Image *image;
         bool exploding;
@@ -71,17 +68,12 @@ class FightWindow: public sigc::trackable
     int round;
     actions_type::iterator action_iterator;
     
-    // determine the max no. of rows in each column
-    int compute_max_rows(const armies_type &attackers,
-			 const armies_type &defenders);
-    
     // add an army to the window
     void add_army(Army *army, int initial_hp,
                   std::vector<Gtk::HBox *> &hboxes,
-		  Gtk::VBox *vbox,
-		  int current_no, int max_rows, Gtk::AlignmentEnum alignment);
+		  Gtk::VBox *vbox, int current_no);
 
-    void on_key_release_event(GdkEventKey* event);
+    void on_key_release_event();
 
     bool do_round();
     bool d_quick;
