@@ -1,4 +1,4 @@
-//  Copyright (C) 2008, 2009, 2011 Ben Asselstine
+//  Copyright (C) 2008, 2009, 2011, 2014, 2015 Ben Asselstine
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -23,16 +23,13 @@
 
 #include "game-lobby-dialog.h"
 
-#include "glade-helpers.h"
-#include "image-helpers.h"
-#include "input-helpers.h"
 #include "ucompose.hpp"
 #include "defs.h"
 #include "File.h"
 #include "citylist.h"
 #include "playerlist.h"
 #include "game-options-dialog.h"
-#include "GraphicsCache.h"
+#include "ImageCache.h"
 #include "network_player.h"
 #include "game-client.h"
 #include "game-server.h"
@@ -41,6 +38,8 @@
 #include "recently-played-game-list.h"
 #include "game-parameters.h"
 
+#define SITTING _("Yes")
+#define NOT_SITTING _("No")
 namespace
 {
     Glib::ustring player_type_to_string(guint32 type)
@@ -71,8 +70,7 @@ void GameLobbyDialog::update_city_map()
     }
   else
     {
-      map_image->property_file() = 
-	File::getMiscFile("various/city_occupied.png");
+      map_image->property_file() = File::getVariousFile("city_occupied.png");
     }
 }
 
@@ -87,13 +85,6 @@ void GameLobbyDialog::initDialog(GameScenario *gamescenario,
   d_next_turn = next_turn;
   d_play_message_received = false;
   citymap = NULL;
-    Glib::RefPtr<Gtk::Builder> xml
-	= Gtk::Builder::create_from_file(get_glade_path()
-				    + "/game-lobby-dialog.ui");
-
-    xml->get_widget("dialog", dialog);
-    decorate(dialog);
-    window_closed.connect(sigc::mem_fun(dialog, &Gtk::Dialog::hide));
 
     xml->get_widget("player_treeview", player_treeview);
     player_treeview->get_selection()->signal_changed().connect
@@ -127,13 +118,13 @@ void GameLobbyDialog::initDialog(GameScenario *gamescenario,
 	(sigc::mem_fun(*this, &GameLobbyDialog::on_show_options_clicked));
 
     game_station->remote_player_moved.connect
-      (sigc::mem_fun(*this, &GameLobbyDialog::on_remote_player_ends_turn));
+      (sigc::hide(sigc::mem_fun(*this, &GameLobbyDialog::on_remote_player_ends_turn)));
     game_station->remote_player_starts_move.connect
-      (sigc::mem_fun(*this, &GameLobbyDialog::on_remote_player_starts_turn));
+      (sigc::hide(sigc::mem_fun(*this, &GameLobbyDialog::on_remote_player_starts_turn)));
     game_station->local_player_moved.connect
-      (sigc::mem_fun(*this, &GameLobbyDialog::on_local_player_ends_turn));
+      (sigc::hide(sigc::mem_fun(*this, &GameLobbyDialog::on_local_player_ends_turn)));
     game_station->local_player_starts_move.connect
-      (sigc::mem_fun(*this, &GameLobbyDialog::on_local_player_starts_turn));
+      (sigc::hide(sigc::mem_fun(*this, &GameLobbyDialog::on_local_player_starts_turn)));
     game_station->remote_participant_joins.connect
       (sigc::mem_fun(*this, &GameLobbyDialog::on_remote_participant_joins));
     game_station->remote_participant_departs.connect
@@ -141,7 +132,7 @@ void GameLobbyDialog::initDialog(GameScenario *gamescenario,
     game_station->player_sits.connect
       (sigc::mem_fun(*this, &GameLobbyDialog::on_player_sits));
     game_station->player_stands.connect
-      (sigc::mem_fun(*this, &GameLobbyDialog::on_player_stands));
+      (sigc::hide(sigc::mem_fun(*this, &GameLobbyDialog::on_player_stands)));
     game_station->player_changes_name.connect
       (sigc::mem_fun(*this, &GameLobbyDialog::on_player_changes_name));
     game_station->player_changes_type.connect
@@ -149,7 +140,7 @@ void GameLobbyDialog::initDialog(GameScenario *gamescenario,
     game_station->remote_player_named.connect
       (sigc::mem_fun(*this, &GameLobbyDialog::on_remote_player_changes_name));
     game_station->chat_message_received.connect
-      (sigc::mem_fun(*this, &GameLobbyDialog::on_chatted));
+      (sigc::hide<0>(sigc::mem_fun(*this, &GameLobbyDialog::on_chatted)));
     game_station->playerlist_reorder_received.connect
       (sigc::mem_fun(*this, &GameLobbyDialog::on_reorder_playerlist));
     game_station->round_begins.connect
@@ -171,7 +162,6 @@ void GameLobbyDialog::initDialog(GameScenario *gamescenario,
     people_list = Gtk::ListStore::create(people_columns);
     // setup the player settings
     people_treeview->set_model(people_list);
-
 }
 
 void GameLobbyDialog::update_buttons()
@@ -236,10 +226,20 @@ GameLobbyDialog::update_player_details()
   player_treeview->append_column("", player_columns.shield);
 
   //the sitting toggle
+  player_sitting_list = Gtk::ListStore::create(player_sitting_columns);
+  Gtk::TreeModel::iterator j;
+  j = player_sitting_list->append();
+  (*j)[player_sitting_columns.sitting] = NOT_SITTING;
+  j = player_sitting_list->append();
+  (*j)[player_sitting_columns.sitting] = SITTING;
+  sitting_renderer.property_model() = player_sitting_list;
+  sitting_renderer.property_text_column() = 0;
+  sitting_renderer.property_has_entry() = false;
+  sitting_renderer.property_editable() = true;
+
   sitting_renderer.property_mode() = Gtk::CELL_RENDERER_MODE_EDITABLE;
-  sitting_renderer.property_activatable() = true;
-  sitting_renderer.signal_editing_started().connect
-    (sigc::mem_fun(*this, &GameLobbyDialog::on_sitting_changed));
+  sitting_renderer.signal_edited()
+    .connect(sigc::mem_fun(*this, &GameLobbyDialog::on_sitting_edited));
   sitting_column.set_cell_data_func
     (sitting_renderer, sigc::mem_fun(*this, &GameLobbyDialog::cell_data_sitting));
   player_treeview->append_column(sitting_column);
@@ -287,9 +287,9 @@ GameLobbyDialog::update_player_details()
   Playerlist *pl = Playerlist::getInstance();
 
   guint32 count = 0;
-  for (Playerlist::iterator i = pl->begin(), end = pl->end(); i != end; ++i)
+  for (Playerlist::iterator k = pl->begin(), end = pl->end(); k != end; ++k)
     {
-      Player *player = *i;
+      Player *player = *k;
       if (player == pl->getNeutral())
 	continue;
       if (player->isDead())
@@ -301,10 +301,14 @@ GameLobbyDialog::update_player_details()
   update_turn_indicator();
 }
 
-void GameLobbyDialog::on_sitting_changed(Gtk::CellEditable *editable,
-					 const Glib::ustring &path)
+void GameLobbyDialog::on_sitting_edited(const Glib::ustring &path,
+                                        const Glib::ustring &new_text)
 {
   Gtk::TreeModel::iterator iter = player_treeview->get_model()->get_iter(path);
+  if (new_text == SITTING && (*iter)[player_columns.sitting] == true)
+    return;
+  if (new_text == NOT_SITTING && (*iter)[player_columns.sitting] == false)
+    return;
   Playerlist *pl = Playerlist::getInstance();
   Player *player;
   //maybe we can't make other ppl stand up
@@ -313,7 +317,6 @@ void GameLobbyDialog::on_sitting_changed(Gtk::CellEditable *editable,
       d_has_ops == false)
     return;
 
-  sitting_renderer.set_sensitive(false);
   player = pl->getPlayer((*iter)[player_columns.player_id]);
   d_player_id_of_sit_or_stand_request = player->getId();
 
@@ -323,11 +326,13 @@ void GameLobbyDialog::on_sitting_changed(Gtk::CellEditable *editable,
     player_stood_up.emit(player);
 }
 
-GameLobbyDialog::GameLobbyDialog(GameScenario *game_scenario, 
+GameLobbyDialog::GameLobbyDialog(Gtk::Window &parent,
+                                 GameScenario *game_scenario, 
 				 NextTurnNetworked *next_turn, 
 				 GameStation *game_station,
 				 bool has_ops)
-:name_column(_("Name"), name_renderer),
+ : LwDialog(parent, "game-lobby-dialog.ui"),
+    name_column(_("Name"), name_renderer),
     type_column(_("Type"), type_renderer),
     sitting_column(_("Controlled"), sitting_renderer)
 {
@@ -351,7 +356,7 @@ GameLobbyDialog::~GameLobbyDialog()
 {
   if (citymap)
     delete citymap;
-  delete dialog;
+  clean_up_players();
 }
 
 void GameLobbyDialog::update_scenario_details()
@@ -367,12 +372,6 @@ void GameLobbyDialog::update_scenario_details()
   update_city_map();
 }
 
-void GameLobbyDialog::set_parent_window(Gtk::Window &parent)
-{
-  dialog->set_transient_for(parent);
-  //dialog->set_position(Gtk::WIN_POS_CENTER_ON_PARENT);
-}
-
 void GameLobbyDialog::hide()
 {
   dialog->hide();
@@ -384,14 +383,16 @@ void GameLobbyDialog::show()
   return;
 }
 
-void GameLobbyDialog::on_map_changed(Glib::RefPtr<Gdk::Pixmap> map)
+void GameLobbyDialog::on_map_changed(Cairo::RefPtr<Cairo::Surface> map)
 {
-  map_image->property_pixmap() = map;
+  Glib::RefPtr<Gdk::Pixbuf> pixbuf = 
+    Gdk::Pixbuf::create(map, 0, 0, citymap->get_width(), citymap->get_height());
+  map_image->property_pixbuf() = pixbuf;
 }
 
 void GameLobbyDialog::on_show_options_clicked()
 {
-  GameOptionsDialog gd(true);
+  GameOptionsDialog gd(*dialog, true);
   gd.run();
 }
 
@@ -459,13 +460,16 @@ void GameLobbyDialog::on_name_edited(const Glib::ustring &path,
 void GameLobbyDialog::cell_data_sitting(Gtk::CellRenderer *renderer,
 					const Gtk::TreeIter& i)
 {
-  dynamic_cast<Gtk::CellRendererToggle*>(renderer)->set_active((*i)[player_columns.sitting]);
+  if ((*i)[player_columns.sitting] == true)
+    dynamic_cast<Gtk::CellRendererText*>(renderer)->property_text() = SITTING;
+  else
+    dynamic_cast<Gtk::CellRendererText*>(renderer)->property_text() = NOT_SITTING;
 }
 
 void GameLobbyDialog::add_player(guint32 order, const Glib::ustring &type,
 				 const Glib::ustring &name, Player *player)
 {
-  GraphicsCache *gc = GraphicsCache::getInstance();
+  ImageCache *gc = ImageCache::getInstance();
   Gtk::TreeIter i = player_list->append();
   (*i)[player_columns.order] = order;
   (*i)[player_columns.shield] = gc->getShieldPic(1, player)->to_pixbuf();
@@ -497,13 +501,13 @@ void GameLobbyDialog::on_player_selected()
 
 }
 
-void GameLobbyDialog::on_remote_participant_joins(std::string nickname)
+void GameLobbyDialog::on_remote_participant_joins(Glib::ustring nickname)
 {
   Gtk::TreeIter j = people_list->append();
   (*j)[people_columns.nickname] = nickname;
 }
 
-void GameLobbyDialog::on_remote_participant_departs(std::string nickname)
+void GameLobbyDialog::on_remote_participant_departs(Glib::ustring nickname)
 {
   //iterate through and remove the nickname
   Gtk::TreeNodeChildren rows = people_list->children();
@@ -569,7 +573,7 @@ void GameLobbyDialog::on_player_changes_type(Player *p, int type)
     }
 }
 
-void GameLobbyDialog::on_player_sits(Player *p, std::string nickname)
+void GameLobbyDialog::on_player_sits(Player *p, Glib::ustring nickname)
 {
   if (!p)
     return;
@@ -595,7 +599,7 @@ void GameLobbyDialog::on_player_sits(Player *p, std::string nickname)
     }
 }
 
-void GameLobbyDialog::on_player_stands(Player *p, std::string nickname)
+void GameLobbyDialog::on_player_stands(Player *p)
 {
   if (!p)
     return;
@@ -615,7 +619,7 @@ void GameLobbyDialog::on_player_stands(Player *p, std::string nickname)
     }
 }
 
-void GameLobbyDialog::on_local_player_ends_turn(Player *p)
+void GameLobbyDialog::on_local_player_ends_turn()
 {
   update_turn_indicator();
   update_scenario_details();
@@ -623,7 +627,7 @@ void GameLobbyDialog::on_local_player_ends_turn(Player *p)
 
 void GameLobbyDialog::update_turn_indicator()
 {
-  GraphicsCache *gc = GraphicsCache::getInstance();
+  ImageCache *gc = ImageCache::getInstance();
   Gtk::TreeModel::Children kids = player_list->children();
   for (Gtk::TreeModel::Children::iterator i = kids.begin(); 
        i != kids.end(); i++)
@@ -633,7 +637,7 @@ void GameLobbyDialog::update_turn_indicator()
       if (active)
         {
           if (row[player_columns.player_id] == active->getId())
-            (*i)[player_columns.turn] = gc->getCursorPic(GraphicsCache::SWORD)->to_pixbuf();
+            (*i)[player_columns.turn] = gc->getCursorPic(ImageCache::SWORD)->to_pixbuf();
           else
             {
               Glib::RefPtr<Gdk::Pixbuf> empty_pic
@@ -645,19 +649,19 @@ void GameLobbyDialog::update_turn_indicator()
     }
 }
 
-void GameLobbyDialog::on_remote_player_starts_turn(Player *p)
+void GameLobbyDialog::on_remote_player_starts_turn()
 {
   update_turn_indicator();
   update_scenario_details();
 }
 
-void GameLobbyDialog::on_local_player_starts_turn(Player *p)
+void GameLobbyDialog::on_local_player_starts_turn()
 {
   update_turn_indicator();
   update_scenario_details();
 }
 
-void GameLobbyDialog::on_remote_player_ends_turn(Player *p)
+void GameLobbyDialog::on_remote_player_ends_turn()
 {
   if (GameServer::getInstance()->isListening() == false)
     {
@@ -735,11 +739,12 @@ void GameLobbyDialog::on_chat_key_pressed(GdkEventKey *event)
   return;
 }
 
-void GameLobbyDialog::on_chatted(std::string nickname, std::string message)
+void GameLobbyDialog::on_chatted(Glib::ustring message)
 {
-  std::string new_text;
+  Glib::ustring new_text;
   new_text = chat_textview->get_buffer()->get_text() + "\n" + message;
   chat_textview->get_buffer()->set_text(new_text);
+  while (g_main_context_iteration(NULL, FALSE)); //doEvents
   chat_scrolledwindow->get_vadjustment()->set_value(chat_scrolledwindow->get_vadjustment()->get_upper());
 }
 
@@ -891,5 +896,4 @@ void GameLobbyDialog::sort_player_list_by_turn_order()
 void GameLobbyDialog::clean_up_players()
 {
   player_list->clear();
-  //player_list.reset();
 }
