@@ -1,5 +1,5 @@
 //  Copyright (C) 2007 Ole Laursen
-//  Copyright (C) 2007, 2008, 2009 Ben Asselstine
+//  Copyright (C) 2007, 2008, 2009, 2014, 2015 Ben Asselstine
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -21,38 +21,32 @@
 
 #include <memory>
 #include <vector>
-#include <sigc++/signal.h>
 #include <gtkmm.h>
 
 #include "game-parameters.h"
 #include "game-options-dialog.h"
 #include "GameScenario.h"
+#include "lw-dialog.h"
 
-#include "decorated.h"
 class XML_Helper;
 
 // dialog for choosing parameters for starting a new game
-class GamePreferencesDialog: public Decorated
+class GamePreferencesDialog: public LwDialog
 {
  public:
-    GamePreferencesDialog(std::string filename, GameScenario::PlayMode mode);
+    GamePreferencesDialog(Gtk::Window &parent, Glib::ustring filename, GameScenario::PlayMode mode);
     ~GamePreferencesDialog();
 
-    void set_title(std::string title);
-
-    void set_parent_window(Gtk::Window &parent);
-
     sigc::signal<void, GameParameters> game_started;
     
-    bool run(std::string nickname = "guest");
+    bool run(Glib::ustring nickname = "guest");
     void hide();
     
  private:
-    void init(std::string filename);
-    Gtk::Dialog* dialog;
+    void init(Glib::ustring filename);
     GameScenario::PlayMode mode;
 
-    Gtk::VBox *dialog_vbox;
+    Gtk::Box *dialog_vbox;
     Gtk::ProgressBar *progressbar;
     Gtk::Button *start_game_button;
     Gtk::Button *edit_options_button;
@@ -88,8 +82,7 @@ class GamePreferencesDialog: public Decorated
     void update_difficulty_rating();
     void update_shields();
     void update_buttons();
-    Glib::RefPtr<Gdk::Pixbuf> getShieldPic(guint32 type, guint32 owner);
-    std::string d_filename;
+    Glib::ustring d_filename;
     guint32 d_shieldset;
 };
 
