@@ -1,5 +1,5 @@
 //  Copyright (C) 2007 Ole Laursen
-//  Copyright (C) 2007, 2008, 2009, 2010 Ben Asselstine
+//  Copyright (C) 2007, 2008, 2009, 2010, 2012, 2014, 2015 Ben Asselstine
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -17,41 +17,31 @@
 //  02110-1301, USA.
 
 #include <config.h>
-
 #include <iomanip>
 
 #include <gtkmm.h>
 #include <sigc++/functors/mem_fun.h>
-#include <vector>
 
 #include "hero-dialog.h"
 
-#include "glade-helpers.h"
-#include "image-helpers.h"
 #include "input-helpers.h"
 #include "ucompose.hpp"
 #include "defs.h"
 #include "hero.h"
 #include "Item.h"
 #include "GameMap.h"
-#include "GraphicsCache.h"
 #include "Backpack.h"
 #include "MapBackpack.h"
 #include "history.h"
+#include "playerlist.h"
 
-HeroDialog::HeroDialog(Hero *h, Vector<int> p)
+HeroDialog::HeroDialog(Gtk::Window &parent, Hero *h, Vector<int> p)
+ : LwDialog(parent, "hero-dialog.ui")
 {
   inhibit_hero_changed = false;
     hero = h;
     pos = p;
     
-    Glib::RefPtr<Gtk::Builder> xml
-	= Gtk::Builder::create_from_file(get_glade_path()
-				    + "/hero-dialog.ui");
-
-    xml->get_widget("dialog", dialog);
-    decorate(dialog);
-    window_closed.connect(sigc::mem_fun(dialog, &Gtk::Dialog::hide));
     xml->get_widget("map_image", map_image);
 
     std::list<Hero*> heroes;
@@ -61,9 +51,9 @@ HeroDialog::HeroDialog(Hero *h, Vector<int> p)
       heroesmap->setSelectedHero(hero);
     else
       {
-	Player *p = Playerlist::getActiveplayer();
+	Player *player = Playerlist::getActiveplayer();
 	hero = *heroes.begin();
-	pos = p->getPositionOfArmyById(hero->getId());
+	pos = player->getPositionOfArmyById(hero->getId());
 	heroesmap->setSelectedHero(hero);
       }
     heroesmap->map_changed.connect(
@@ -141,8 +131,8 @@ HeroDialog::HeroDialog(Hero *h, Vector<int> p)
 HeroDialog::~HeroDialog()
 {
   delete heroesmap;
-  delete dialog;
 }
+
 void HeroDialog::addHistoryEvent(History *history)
 {
   Glib::ustring s = "";
@@ -226,12 +216,6 @@ void HeroDialog::addHistoryEvent(History *history)
   (*i)[events_columns.history] = history;
 }
 
-void HeroDialog::set_parent_window(Gtk::Window &parent)
-{
-    dialog->set_transient_for(parent);
-    //dialog->set_position(Gtk::WIN_POS_CENTER_ON_PARENT);
-}
-
 void HeroDialog::hide()
 {
   dialog->hide();
@@ -457,9 +441,12 @@ void HeroDialog::fill_in_info_labels()
     info_label2->set_text(s);
 }
 
-void HeroDialog::on_map_changed(Glib::RefPtr<Gdk::Pixmap> map)
+void HeroDialog::on_map_changed(Cairo::RefPtr<Cairo::Surface> map)
 {
-    map_image->property_pixmap() = map;
+  Glib::RefPtr<Gdk::Pixbuf> pixbuf = 
+    Gdk::Pixbuf::create(map, 0, 0, 
+                        heroesmap->get_width(), heroesmap->get_height());
+  map_image->property_pixbuf() = pixbuf;
 }
 
 bool HeroDialog::on_map_mouse_button_event(GdkEventButton *e)
@@ -479,7 +466,7 @@ bool HeroDialog::on_map_mouse_button_event(GdkEventButton *e)
 
 void HeroDialog::show_hero()
 {
-    set_title(hero->getName());
+    dialog->set_title(hero->getName());
 
     fill_in_info_labels();
     std::list<History* > events;
