@@ -1,5 +1,5 @@
 //  Copyright (C) 2007 Ole Laursen
-//  Copyright (C) 2007, 2008, 2009 Ben Asselstine
+//  Copyright (C) 2007, 2008, 2009, 2014 Ben Asselstine
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -19,33 +19,27 @@
 #ifndef HERO_DIALOG_H
 #define HERO_DIALOG_H
 
-#include <memory>
-#include <sigc++/trackable.h>
 #include <gtkmm.h>
 
 #include "vector.h"
 #include "heroesmap.h"
+#include "lw-dialog.h"
 
 class Hero;
 class Item;
 class History;
 
-#include "decorated.h"
-
 // dialog for showing info about a hero, esp. about the hero's items
-class HeroDialog: public Decorated
+class HeroDialog: public LwDialog
 {
  public:
-    HeroDialog(Hero *hero, Vector<int> pos);
+    HeroDialog(Gtk::Window &parent, Hero *hero, Vector<int> pos);
     ~HeroDialog();
 
-    void set_parent_window(Gtk::Window &parent);
-
     void run();
     void hide();
     
  private:
-    Gtk::Dialog* dialog;
     HeroesMap* heroesmap;
 
     Hero *hero;
@@ -77,7 +71,7 @@ class HeroDialog: public Decorated
 	ItemColumns() 
         { add(image); add(name); add(attributes); add(status); add(item); }
 	
-	Gtk::TreeModelColumn<Glib::RefPtr<Gdk::Pixmap> > image;
+	Gtk::TreeModelColumn<Glib::RefPtr<Gdk::Pixbuf> > image;
 	Gtk::TreeModelColumn<Glib::ustring> name;
 	Gtk::TreeModelColumn<Glib::ustring> attributes;
 	Gtk::TreeModelColumn<Glib::ustring> status;
@@ -109,7 +103,7 @@ class HeroDialog: public Decorated
     void addHistoryEvent(History *event);
     void fill_in_info_labels();
 
-    void on_map_changed(Glib::RefPtr<Gdk::Pixmap> map);
+    void on_map_changed(Cairo::RefPtr<Cairo::Surface> map);
     bool on_map_mouse_button_event(GdkEventButton *e);
     void show_hero();
     void update_buttons();
