@@ -1,5 +1,5 @@
 //  Copyright (C) 2007 Ole Laursen
-//  Copyright (C) 2007, 2008, 2009 Ben Asselstine
+//  Copyright (C) 2007, 2008, 2009, 2012, 2014, 2015 Ben Asselstine
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -19,46 +19,40 @@
 #ifndef HERO_OFFER_DIALOG_H
 #define HERO_OFFER_DIALOG_H
 
-#include <memory>
-#include <vector>
-#include <sigc++/trackable.h>
 #include <gtkmm.h>
 
 #include "heromap.h"
 #include "player.h"
 #include "hero.h"
 #include "heroproto.h"
-
-#include "decorated.h"
+#include "lw-dialog.h"
 
 //! dialog for accepting/rejecting a hero
-class HeroOfferDialog: public Decorated
+class HeroOfferDialog: public LwDialog
 {
  public:
-    HeroOfferDialog(Player *player, HeroProto *hero, City *city, int gold);
+    HeroOfferDialog(Gtk::Window &parent, Player *player, HeroProto *hero, City *city, int gold);
     ~HeroOfferDialog();
 
-    void set_parent_window(Gtk::Window &parent);
-
     bool run();
     void hide();
     
  private:
-    Gtk::Dialog* dialog;
     //! The smallmap that shows where the Hero is emerging.
     HeroMap* heromap;
 
     Gtk::Image *map_image;
     Gtk::Image *hero_image;
     Gtk::RadioButton *male_radiobutton;
+    Gtk::RadioButton *female_radiobutton;
     Gtk::Entry *name_entry;
     Gtk::Button *accept_button;
     
     HeroProto *hero;
     City *city;
 
-    void on_male_toggled();
-    void on_map_changed(Glib::RefPtr<Gdk::Pixmap> map);
+    void on_toggled();
+    void on_map_changed(Cairo::RefPtr<Cairo::Surface> map);
     void update_buttons();
     void on_name_changed();
 
