@@ -1,4 +1,4 @@
-//  Copyright (C) 2007, 2008, 2009 Ben Asselstine
+//  Copyright (C) 2007, 2008, 2009, 2012, 2014, 2015 Ben Asselstine
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -18,9 +18,7 @@
 #ifndef HISTORY_REPORT_DIALOG_H
 #define HISTORY_REPORT_DIALOG_H
 
-#include <memory>
 #include <vector>
-#include <sigc++/trackable.h>
 #include <gtkmm.h>
 
 #include "line-chart.h"
@@ -29,18 +27,17 @@
 #include "historymap.h"
 #include "history.h"
 #include "player.h"
-#include "decorated.h"
+#include "lw-dialog.h"
 
 class Citylist;
 class City;
-
 class Player;
 
-class HistoryReportDialog: public Decorated
+class HistoryReportDialog: public LwDialog
 {
  public:
     enum HistoryReportType {CITY = 0, RUIN, EVENTS, GOLD, WINNING};
-    HistoryReportDialog(Player *p, HistoryReportType type);
+    HistoryReportDialog(Gtk::Window &parent, Player *p, HistoryReportType type);
     ~HistoryReportDialog();
 
     void generatePastCitylists(); //data for map
@@ -50,13 +47,11 @@ class HistoryReportDialog: public Decorated
     void generatePastGoldCounts(); //data for chart
     void generatePastWinningCounts(); //data for chart
     void generatePastEventlists(); //data for events list
-    void set_parent_window(Gtk::Window &parent);
 
     void run();
     void hide();
     
  private:
-    Gtk::Dialog* dialog;
     HistoryMap* historymap;
 
     Player *d_player;
@@ -85,19 +80,16 @@ class HistoryReportDialog: public Decorated
 
     Gtk::Image *map_image;
   
-    std::list<Gdk::Color> d_colours; //player colours
+    std::list<Gdk::RGBA> d_colours; //player colours
     
     Gtk::VBox *events_list_box;
 
     void addHistoryEvent(NetworkHistory *event);
-    void on_close_button();
-    void on_map_changed(Glib::RefPtr<Gdk::Pixmap> map);
-    void on_turn_changed(Gtk::Scale *scale);
+    void on_map_changed(Cairo::RefPtr<Cairo::Surface> map);
+    void on_turn_changed();
     void fill_in_turn_info(guint32 turn);
-    void on_switch_page(GtkNotebookPage *page, guint number);
+    void on_switch_page();
     void update_window_title();
-
-    bool closing;
 };
 
 #endif
