@@ -1,5 +1,5 @@
 //  Copyright (C) 2007 Ole Laursen
-//  Copyright (C) 2007, 2008, 2009 Ben Asselstine
+//  Copyright (C) 2007, 2008, 2009, 2014, 2015 Ben Asselstine
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -19,29 +19,23 @@
 #ifndef LOAD_SCENARIOS_DIALOG_H
 #define LOAD_SCENARIOS_DIALOG_H
 
-#include <memory>
-#include <string>
-#include <sigc++/trackable.h>
-#include <sigc++/signal.h>
 #include <gtkmm.h>
 
-#include "decorated.h"
+#include "lw-dialog.h"
+
 // dialog for choosing a scenario
-class LoadScenarioDialog: public Decorated
+class LoadScenarioDialog: public LwDialog
 {
  public:
-    LoadScenarioDialog();
-    ~LoadScenarioDialog();
-
-    void set_parent_window(Gtk::Window &parent);
+    LoadScenarioDialog(Gtk::Window &parent);
+    ~LoadScenarioDialog() {};
 
     void run();
     void hide();
 
-    std::string get_scenario_filename();
+    Glib::ustring get_scenario_filename();
 
  private:
-    Gtk::Dialog* dialog;
     Gtk::Button *load_button;
     Gtk::TextView *description_textview;
     Gtk::Label *num_players_label;
@@ -56,18 +50,19 @@ class LoadScenarioDialog: public Decorated
         { add(name); add(filename); }
 	
 	Gtk::TreeModelColumn<Glib::ustring> name;
-	Gtk::TreeModelColumn<std::string> filename;
+	Gtk::TreeModelColumn<Glib::ustring> filename;
     };
     const ScenariosColumns scenarios_columns;
     Glib::RefPtr<Gtk::ListStore> scenarios_list;
     
-    std::string selected_filename;
+    Glib::ustring selected_filename;
     
     void on_selection_changed();
-    void add_scenario(std::string filename);
+    void add_scenario(Glib::ustring filename);
     void on_add_scenario_clicked();
     void on_remove_scenario_clicked();
     int copy_file (Glib::ustring from, Glib::ustring to);
+    void on_scenario_activated();
 };
 
 #endif
