@@ -1,5 +1,5 @@
 //  Copyright (C) 2007 Ole Laursen
-//  Copyright (C) 2007, 2008, 2009, 2010 Ben Asselstine
+//  Copyright (C) 2007, 2008, 2009, 2010, 2014 Ben Asselstine
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -16,7 +16,6 @@
 //  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 
 //  02110-1301, USA.
 
-#include <assert.h>
 #include <memory>
 #include <iostream>
 #include <assert.h>
@@ -24,6 +23,8 @@
 #include <gtkmm.h>
 #include <sigc++/trackable.h>
 #include <sigc++/functors/mem_fun.h>
+#include <time.h>
+#include "rnd.h"
 
 #include "main.h"
 
@@ -105,10 +106,14 @@ bool Main::iterate_main_loop()
 
 void Main::start_main_loop()
 {
+  if (random_number_seed)
+    Rnd::set_seed(random_number_seed);
+  else
+    Rnd::set_seed(time(NULL));
+
     if (Configuration::s_decorated)
       {
-      Gtk::RC::add_default_file(File::getMiscFile("gtkrc"));
-	Gtk::RC::reparse_all(Gtk::Settings::get_default(), true);
+        ;
       }
 
     try
