@@ -20,11 +20,10 @@
 #define GUI_MAIN_H
 
 #include <sigc++/trackable.h>
-#include <boost/noncopyable.hpp>
 
 // initialize the GUI and run the main loop; only one instance is ever
 // constructed so Main::instance is a convenience for retrieving it
-class Main: public sigc::trackable, public boost::noncopyable
+class Main: public sigc::trackable
 {
  public:
     Main(int &argc, char **&argv);
@@ -40,11 +39,12 @@ class Main: public sigc::trackable, public boost::noncopyable
     bool start_stress_test;
     int start_robots;
     bool start_test_scenario;
-    std::string load_filename;
-    std::string turn_filename;
+    Glib::ustring load_filename;
+    Glib::ustring turn_filename;
     guint32 random_number_seed;
     bool start_headless_server;
     guint32 port;
+    Glib::Rand rnd;
     
  private:
     struct Impl;
