@@ -1,4 +1,4 @@
-//  Copyright (C) 2008 Ben Asselstine
+//  Copyright (C) 2008, 2014, 2015 Ben Asselstine
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -16,31 +16,22 @@
 //  02110-1301, USA.
 #include "new-network-game-download-window.h"
 #include "defs.h"
-#include "File.h"
-#include "decorated.h"
 
 NewNetworkGameDownloadWindow::NewNetworkGameDownloadWindow(Glib::ustring title)
 : m_vbox(false,10)
 {
+  set_position(Gtk::WIN_POS_CENTER);
   add(m_vbox);
   m_vbox.set_border_width(10);
   m_vbox.pack_start(m_label);
   m_vbox.pack_start(m_pbar);
  
-  Decorated decorator;
-  decorator.decorate(this);
-  decorator.window_closed.connect(sigc::mem_fun(this, &Gtk::Dialog::hide));
   if (title == "")
     title = _("Downloading.");
   set_title(title);
-  set_icon_from_file(File::getMiscFile("various/castle_icon.png"));
   show_all();
 }
 
-NewNetworkGameDownloadWindow::~NewNetworkGameDownloadWindow()
-{
-}
-
 void NewNetworkGameDownloadWindow::pulse()
 {
   m_pbar.pulse();
