@@ -1,4 +1,4 @@
-//  Copyright (C) 2008 Ben Asselstine
+//  Copyright (C) 2008, 2014 Ben Asselstine
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -14,16 +14,14 @@
 //  along with this program; if not, write to the Free Software
 //  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 
 //  02110-1301, USA.
-#include <boost/noncopyable.hpp>
 #include "game-parameters.h"
 #include <gtkmm.h>
 
-
-class NewNetworkGameDownloadWindow : public Gtk::Window, boost::noncopyable
+class NewNetworkGameDownloadWindow : public Gtk::Window
 {
   public:
     NewNetworkGameDownloadWindow(Glib::ustring title = "");
-    virtual ~NewNetworkGameDownloadWindow();
+    virtual ~NewNetworkGameDownloadWindow() {};
     void pulse();
 
   private:
