@@ -1,5 +1,5 @@
 //  Copyright (C) 2007 Ole Laursen
-//  Copyright (C) 2007, 2008, 2009 Ben Asselstine
+//  Copyright (C) 2007, 2008, 2009, 2014 Ben Asselstine
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -20,32 +20,30 @@
 #define PREFERENCES_DIALOG_H
 
 #include <map>
-#include <memory>
-#include <sigc++/trackable.h>
 #include <sigc++/signal.h>
 #include <gtkmm.h>
 
 #include "vector.h"
+#include "lw-dialog.h"
 
-#include "decorated.h"
 class Game;
 class Player;
 // dialog for showing sound and game preferences
-class PreferencesDialog: public Decorated
+class PreferencesDialog: public LwDialog
 {
  public:
-    PreferencesDialog(bool readonly);
-    ~PreferencesDialog();
-
-    void set_parent_window(Gtk::Window &parent);
+    PreferencesDialog(Gtk::Window &parent, bool readonly);
+    ~PreferencesDialog() {};
 
     void run(Game *game);
     void hide();
 
+    sigc::signal<void, guint32> ui_form_factor_changed;
+
  private:
-    Gtk::Dialog* dialog;
     Gtk::CheckButton *commentator_checkbutton;
     Gtk::Scale *speed_scale;
+    Gtk::ComboBox *ui_combobox;
     Gtk::CheckButton *play_music_checkbutton;
     Gtk::Scale *music_volume_scale;
     Gtk::Box *music_volume_hbox;
@@ -58,6 +56,7 @@ class PreferencesDialog: public Decorated
     void on_speed_changed();
     void on_observe_toggled(Gtk::CheckButton *button);
     void on_type_changed(Gtk::ComboBoxText *combo);
+    void on_ui_form_factor_changed();
 
     typedef std::map<Player*, Gtk::ComboBoxText*> PlayerTypeMap;
     PlayerTypeMap player_types;
