@@ -1,4 +1,4 @@
-//  Copyright (C) 2007, 2008, 2009 Ben Asselstine
+//  Copyright (C) 2007, 2008, 2009, 2012, 2014 Ben Asselstine
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -18,31 +18,24 @@
 #ifndef QUEST_ASSIGNED_DIALOG_H
 #define QUEST_ASSIGNED_DIALOG_H
 
-#include <memory>
-#include <vector>
-#include <sigc++/trackable.h>
 #include <gtkmm.h>
 
 #include "questmap.h"
 #include "Quest.h"
 #include "hero.h"
+#include "lw-dialog.h"
 
-
-#include "decorated.h"
 // dialog for depicting a quest
-class QuestAssignedDialog: public Decorated
+class QuestAssignedDialog: public LwDialog
 {
  public:
-    QuestAssignedDialog(Hero *hero, Quest *quest);
+    QuestAssignedDialog(Gtk::Window &parent, Hero *hero, Quest *quest);
     ~QuestAssignedDialog();
 
-    void set_parent_window(Gtk::Window &parent);
-
     void run();
     void hide();
     
  private:
-    Gtk::Dialog* dialog;
     QuestMap* questmap;
 
     Gtk::Image *map_image;
@@ -51,7 +44,7 @@ class QuestAssignedDialog: public Decorated
     Hero *hero;
     Quest *quest;
 
-    void on_map_changed(Glib::RefPtr<Gdk::Pixmap> map);
+    void on_map_changed(Cairo::RefPtr<Cairo::Surface> map);
 };
 
 #endif
