@@ -1,4 +1,4 @@
-//  Copyright (C) 2007, 2008, 2009 Ben Asselstine
+//  Copyright (C) 2007, 2008, 2009, 2012, 2014 Ben Asselstine
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -20,30 +20,21 @@
 #include <gtkmm.h>
 #include <sigc++/functors/mem_fun.h>
 
+#include "SightMap.h"
 #include "quest-completed-dialog.h"
 
-#include "glade-helpers.h"
-#include "image-helpers.h"
-#include "input-helpers.h"
 #include "ucompose.hpp"
 #include "hero.h"
 #include "defs.h"
 #include "ruin.h"
-#include "GameMap.h"
+#include "Item.h"
 
-QuestCompletedDialog::QuestCompletedDialog(Quest *q, Reward *r)
+QuestCompletedDialog::QuestCompletedDialog(Gtk::Window &parent, Quest *q, Reward *r)
+ : LwDialog(parent, "quest-assigned-dialog.ui")
 {
   reward = r;
   quest = q;
     
-  Glib::RefPtr<Gtk::Builder> xml
-      = Gtk::Builder::create_from_file(get_glade_path()
-				  + "/quest-assigned-dialog.ui");
-
-    xml->get_widget("dialog", dialog);
-    decorate(dialog);
-    window_closed.connect(sigc::mem_fun(dialog, &Gtk::Dialog::hide));
-
     xml->get_widget("map_image", map_image);
 
     questmap = new QuestMap(quest);
@@ -53,16 +44,15 @@ QuestCompletedDialog::QuestCompletedDialog(Quest *q, Reward *r)
     Gtk::EventBox *map_eventbox;
     xml->get_widget("map_eventbox", map_eventbox);
 
-    set_title(String::ucompose(_("Quest for %1"), 
-			       quest->getHero()->getName()));
+    dialog->set_title(String::ucompose(_("Quest for %1"), 
+                                       quest->getHero()->getName()));
 
     xml->get_widget("label", label);
-    Glib::ustring s;
-    s += String::ucompose(_("%1 completed the quest!"),
-			  quest->getHero()->getName());
+    Glib::ustring s = String::ucompose(_("%1 completed the quest!"),
+                                       quest->getHero()->getName());
     s += "\n\n";
     // add messages from the quest
-    std::queue<std::string> msgs;
+    std::queue<Glib::ustring> msgs;
     quest->getSuccessMsg(msgs);
     while (!msgs.empty())
     {
@@ -89,13 +79,13 @@ QuestCompletedDialog::QuestCompletedDialog(Quest *q, Reward *r)
     else if (reward->getType() == Reward::ITEM)
       {
 	Item *item = dynamic_cast<Reward_Item*>(reward)->getItem();
-	s += String::ucompose("You have been rewarded with the %1.", 
+	s += String::ucompose(_("You have been rewarded with the %1."), 
 			      item->getName());
       }
     else if (reward->getType() == Reward::RUIN)
       {
 	Ruin *ruin = dynamic_cast<Reward_Ruin*>(reward)->getRuin();
-	s += String::ucompose("You are shown the site of %1\n", 
+	s += String::ucompose(_("You are shown the site of %1\n"), 
 			      ruin->getName());
 	questmap->set_target(ruin->getPos());
 	if (ruin->getReward() == NULL)
@@ -125,14 +115,8 @@ QuestCompletedDialog::QuestCompletedDialog(Quest *q, Reward *r)
 
 QuestCompletedDialog::~QuestCompletedDialog()
 {
-  delete dialog;
   delete questmap;
 }
-void QuestCompletedDialog::set_parent_window(Gtk::Window &parent)
-{
-  dialog->set_transient_for(parent);
-  //dialog->set_position(Gtk::WIN_POS_CENTER_ON_PARENT);
-}
 
 void QuestCompletedDialog::hide()
 {
@@ -148,8 +132,11 @@ void QuestCompletedDialog::run()
   dialog->run();
 }
 
-void QuestCompletedDialog::on_map_changed(Glib::RefPtr<Gdk::Pixmap> map)
+void QuestCompletedDialog::on_map_changed(Cairo::RefPtr<Cairo::Surface> map)
 {
-  map_image->property_pixmap() = map;
+  Glib::RefPtr<Gdk::Pixbuf> pixbuf = 
+    Gdk::Pixbuf::create(map, 0, 0, 
+                        questmap->get_width(), questmap->get_height());
+    map_image->property_pixbuf() = pixbuf;
 }
 
