@@ -1,4 +1,4 @@
-//  Copyright (C) 2007, 2008, 2009 Ben Asselstine
+//  Copyright (C) 2007, 2008, 2009, 2012, 2014, 2015 Ben Asselstine
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -20,28 +20,19 @@
 #include <gtkmm.h>
 #include <sigc++/functors/mem_fun.h>
 
+#include "SightMap.h"
 #include "quest-report-dialog.h"
 
-#include "glade-helpers.h"
-#include "image-helpers.h"
-#include "input-helpers.h"
 #include "ucompose.hpp"
 #include "hero.h"
 #include "defs.h"
-#include "GameMap.h"
+#include "playerlist.h"
 
-QuestReportDialog::QuestReportDialog(std::vector<Quest *>q, Hero *hero)
+QuestReportDialog::QuestReportDialog(Gtk::Window &parent, std::vector<Quest *>q, Hero *hero)
+ : LwDialog(parent, "quest-report-dialog.ui")
 {
   quests = q;
 
-  Glib::RefPtr<Gtk::Builder> xml
-    = Gtk::Builder::create_from_file(get_glade_path()
-                                     + "/quest-report-dialog.ui");
-
-  xml->get_widget("dialog", dialog);
-  decorate(dialog);
-  window_closed.connect(sigc::mem_fun(dialog, &Gtk::Dialog::hide));
-
   xml->get_widget("map_image", map_image);
 
   questmap = NULL;
@@ -80,11 +71,11 @@ QuestReportDialog::QuestReportDialog(std::vector<Quest *>q, Hero *hero)
     fill_quest_info(NULL);
 }
 
-void QuestReportDialog::add_questing_hero(Quest *quest, Hero *h)
+void QuestReportDialog::add_questing_hero(Quest *q, Hero *h)
 {
     Gtk::TreeIter i = heroes_list->append();
     (*i)[heroes_columns.hero_name] = h->getName();
-    (*i)[heroes_columns.quest] = quest;
+    (*i)[heroes_columns.quest] = q;
 }
 
 void QuestReportDialog::fill_quest_info(Quest *q)
@@ -96,7 +87,7 @@ void QuestReportDialog::fill_quest_info(Quest *q)
   questmap = new QuestMap(q);
   questmap->map_changed.connect
     (sigc::mem_fun(this, &QuestReportDialog::on_map_changed));
-  if (dialog->is_realized() == true)
+  if (dialog->get_realized() == true)
     {
       questmap->resize();
       questmap->draw(Playerlist::getActiveplayer());
@@ -104,7 +95,7 @@ void QuestReportDialog::fill_quest_info(Quest *q)
 
   if (q)
     {
-      set_title(String::ucompose(_("Quest for %1"), q->getHero()->getName()));
+      dialog->set_title(String::ucompose(_("Quest for %1"), q->getHero()->getName()));
 
       s = q->getDescription();
       s += "\n\n";
@@ -113,7 +104,7 @@ void QuestReportDialog::fill_quest_info(Quest *q)
     }
   else
     {
-      set_title(_("No Quest"));
+      dialog->set_title(_("No Quest"));
       int num = rand() % 3;
       switch (num)
         {
@@ -139,27 +130,22 @@ void QuestReportDialog::on_hero_changed()
   if (iterrow)
     {
       Gtk::TreeModel::Row row = *iterrow;
-      Quest *quest = row[heroes_columns.quest];
-      fill_quest_info(quest);
+      Quest *q = row[heroes_columns.quest];
+      fill_quest_info(q);
     }
 
 }
+
 QuestReportDialog::~QuestReportDialog()
 {
-  delete dialog;
   delete questmap;
 }
 
-void QuestReportDialog::set_parent_window(Gtk::Window &parent)
-{
-    dialog->set_transient_for(parent);
-    //dialog->set_position(Gtk::WIN_POS_CENTER_ON_PARENT);
-}
-
 void QuestReportDialog::hide()
 {
   dialog->hide();
 }
+
 void QuestReportDialog::run()
 {
   questmap->resize();
@@ -174,8 +160,11 @@ void QuestReportDialog::run()
   dialog->run();
 }
 
-void QuestReportDialog::on_map_changed(Glib::RefPtr<Gdk::Pixmap> map)
+void QuestReportDialog::on_map_changed(Cairo::RefPtr<Cairo::Surface> map)
 {
-  map_image->property_pixmap() = map;
+  Glib::RefPtr<Gdk::Pixbuf> pixbuf = 
+    Gdk::Pixbuf::create(map, 0, 0, 
+                        questmap->get_width(), questmap->get_height());
+    map_image->property_pixbuf() = pixbuf;
 }
 
