@@ -1,4 +1,4 @@
-//  Copyright (C) 2007, 2008, 2009 Ben Asselstine
+//  Copyright (C) 2007, 2008, 2009, 2012, 2014 Ben Asselstine
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -18,36 +18,29 @@
 #ifndef RUIN_REPORT_DIALOG_H
 #define RUIN_REPORT_DIALOG_H
 
-#include <memory>
-#include <vector>
-#include <sigc++/trackable.h>
 #include <gtkmm.h>
 
 #include "ruinmap.h"
 #include "player.h"
+#include "lw-dialog.h"
 
-
-#include "decorated.h"
 // dialog for showing all ruins and temples
 // the stack parameter is used as a starting position for showing ruins
-class RuinReportDialog: public Decorated
+class RuinReportDialog: public LwDialog
 {
  public:
-    RuinReportDialog(Vector<int> pos);
+    RuinReportDialog(Gtk::Window &parent, Vector<int> pos);
     ~RuinReportDialog();
 
-    void set_parent_window(Gtk::Window &parent);
-
     void hide();
     void run();
     
  private:
-    Gtk::Dialog* dialog;
     RuinMap* ruinmap;
 
     Gtk::Image *map_image;
     
-    void on_map_changed(Glib::RefPtr<Gdk::Pixmap> map);
+    void on_map_changed(Cairo::RefPtr<Cairo::Surface> map);
     bool on_map_mouse_button_event(GdkEventButton *e);
     void fill_in_ruin_info();
     Gtk::Label *name_label;
