@@ -1,4 +1,4 @@
-//  Copyright (C) 2007, 2008, 2009 Ben Asselstine
+//  Copyright (C) 2007, 2008, 2009, 2012, 2014 Ben Asselstine
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -20,30 +20,25 @@
 
 #include <memory>
 #include <vector>
-#include <sigc++/trackable.h>
 #include <gtkmm.h>
 
 #include "ruinmap.h"
 #include "player.h"
+#include "lw-dialog.h"
 
-
-#include "decorated.h"
 class Reward_Ruin;
 
 // dialog for visiting a sage
-class RuinRewardedDialog: public Decorated
+class RuinRewardedDialog: public LwDialog
 {
  public:
-    RuinRewardedDialog(Reward_Ruin *reward);
+    RuinRewardedDialog(Gtk::Window &parent, Reward_Ruin *reward);
     ~RuinRewardedDialog();
 
-    void set_parent_window(Gtk::Window &parent);
-
     void hide();
     void run();
     
  private:
-    Gtk::Dialog* dialog;
     RuinMap* ruinmap;
 
     Gtk::Image *map_image;
@@ -51,7 +46,7 @@ class RuinRewardedDialog: public Decorated
     
     Reward_Ruin *d_reward;
 
-    void on_map_changed(Glib::RefPtr<Gdk::Pixmap> map);
+    void on_map_changed(Cairo::RefPtr<Cairo::Surface> map);
 };
 
 #endif
