@@ -1,5 +1,5 @@
 //  Copyright (C) 2007 Ole Laursen
-//  Copyright (C) 2007, 2008, 2009, 2011 Ben Asselstine
+//  Copyright (C) 2007, 2008, 2009, 2011, 2014, 2015 Ben Asselstine
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -25,9 +25,8 @@
 #include <sigc++/signal.h>
 
 #include "game-parameters.h"
+#include "PixMask.h"
 
-
-#include "decorated.h"
 class Profile;
 /** The opening window of the game
   * 
@@ -37,7 +36,7 @@ class Profile;
   * The splash screen is also responsible for launching dialogs in response to
   * the menu choices.
   */
-class SplashWindow: public Decorated
+class SplashWindow: public sigc::trackable
 {
  public:
     SplashWindow();
@@ -49,11 +48,10 @@ class SplashWindow: public Decorated
         
     Gtk::Window *get_window() {return window;}
 
-    sigc::signal<void, std::string, unsigned short, Profile*> new_remote_network_game_requested;
+    sigc::signal<void, Glib::ustring, unsigned short, Profile*> new_remote_network_game_requested;
     sigc::signal<void, GameParameters, int, Profile*, bool, bool > new_hosted_network_game_requested;
-    sigc::signal<void, GameParameters> new_pbm_game_requested;
     sigc::signal<void, GameParameters> new_game_requested;
-    sigc::signal<void, std::string> load_requested;
+    sigc::signal<void, Glib::ustring> load_requested;
     sigc::signal<void> quit_requested;
 
  private:
@@ -63,17 +61,15 @@ class SplashWindow: public Decorated
     Gtk::Button *load_scenario_button;
     Gtk::Button *quit_button;
     Gtk::Button *new_network_game_button;
-    Gtk::Button *new_pbm_game_button;
     Gtk::Button *preferences_button;
     Gtk::VBox *button_box; //crash button box
+    Gtk::HBox *main_box; //crash button box
 	    
-    bool on_delete_event(GdkEventAny *e);
-    void on_window_closed();
+    bool on_delete_event();
   
-    std::string network_game_nickname;
+    Glib::ustring network_game_nickname;
     
     void on_new_network_game_clicked();
-    void on_new_pbm_game_clicked();
     void on_load_game_clicked();
     void on_load_scenario_clicked();
     void on_preferences_clicked();
@@ -82,22 +78,10 @@ class SplashWindow: public Decorated
 	
     void on_game_started(GameParameters g);
     void on_network_game_created(GameParameters g, Profile *profile, bool advertised, bool remotely_hosted);
-    void on_pbm_game_created(GameParameters g);
-    void on_network_game_selected(std::string ip, unsigned short port, Profile  *profile);
-
-#if 0
-    //! Separate network input thread
-    static void *networkThread(void *arg);
-
-    //! Callback for arrival of network data
-    bool networkInput();
-
-    //! Network data processing
-    void networkData();
-#endif
-        
- public:
-    //void on_sdl_surface_changed();
+    void on_network_game_selected(Glib::ustring ip, unsigned short port, Profile  *profile);
+    bool on_draw(const ::Cairo::RefPtr< ::Cairo::Context >& cr);
+  
+    PixMask *bg;
 };
 
 #endif
