@@ -1,4 +1,4 @@
-//  Copyright (C) 2008, 2009 Ben Asselstine
+//  Copyright (C) 2008, 2009, 2011, 2014 Ben Asselstine
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -18,9 +18,6 @@
 #ifndef STACK_INFO_DIALOG_H
 #define STACK_INFO_DIALOG_H
 
-#include <memory>
-#include <sigc++/trackable.h>
-#include <sigc++/signal.h>
 #include <gtkmm.h>
 #include <list>
 
@@ -30,24 +27,18 @@ class Stack;
 class Army;
 
 #include "vector.h"
-#include "decorated.h"
+#include "lw-dialog.h"
 
 // dialog for showing hero information
-class StackInfoDialog: public Decorated
+class StackInfoDialog: public LwDialog
 {
  public:
-    StackInfoDialog(Vector<int> pos);
+    StackInfoDialog(Gtk::Window &parent, Vector<int> pos);
     ~StackInfoDialog();
 
-    void set_parent_window(Gtk::Window &parent);
-
-    void run();
     Stack * get_selected_stack() {return currently_selected_stack;};
-    void hide();
 
  private:
-    Gtk::Dialog* dialog;
-
     Vector<int> tile;
     Gtk::Table *stack_table;
 
@@ -59,7 +50,7 @@ class StackInfoDialog: public Decorated
     Gtk::Button *ungroup_button;
     Stack *currently_selected_stack;
 
-    void addArmy (bool first, Stack *s, Army *a, guint32 modified_strength, int idx);
+    void addArmy (bool first, Stack *s, Army *a, guint32 modified_strength, int idx, guint32 colour_id);
     void addStack(Stack *s, guint32 &idx);
     void on_group_clicked();
     void on_ungroup_clicked();
