@@ -1,4 +1,4 @@
-//  Copyright (C) 2007, 2008, 2009 Ben Asselstine
+//  Copyright (C) 2007, 2008, 2009, 2014 Ben Asselstine
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -20,26 +20,18 @@
 
 #include <memory>
 #include <vector>
-#include <sigc++/trackable.h>
 #include <gtkmm.h>
 
-
-#include "decorated.h"
+#include "lw-dialog.h"
 
 // dialog for showing the refusal of surrender
-class SurrenderRefusedDialog: public Decorated
+class SurrenderRefusedDialog: public LwDialog
 {
  public:
-    SurrenderRefusedDialog();
-    ~SurrenderRefusedDialog();
+    SurrenderRefusedDialog(Gtk::Window &parent);
+    ~SurrenderRefusedDialog() {};
 
-    void set_parent_window(Gtk::Window &parent);
-
-    void run();
-    void hide();
-    
  private:
-    Gtk::Dialog* dialog;
     Gtk::Image *image;
 
 };
