@@ -1,4 +1,4 @@
-//  Copyright (C) 2008 Ben Asselstine
+//  Copyright (C) 2008, 2014, 2015 Ben Asselstine
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -23,12 +23,10 @@
 #include "timed-message-dialog.h"
 
 #include "ucompose.hpp"
-#include "decorated.h"
 #include "defs.h"
 #include "timing.h"
-#include "decorated.h"
 
-TimedMessageDialog::TimedMessageDialog(Gtk::Window &parent, std::string message, int timeout, int grace)
+TimedMessageDialog::TimedMessageDialog(Gtk::Window &parent, Glib::ustring message, int timeout, int grace)
 {
   d_timeout = timeout;
   d_timer_count = 0;
@@ -39,14 +37,11 @@ TimedMessageDialog::TimedMessageDialog(Gtk::Window &parent, std::string message,
   //window.reset(&dialog);
   window->set_message(message);
   window->signal_response().connect
-       (sigc::mem_fun(*this, &TimedMessageDialog::on_response));
-  Decorated decorator;
-  decorator.decorate(window);
-  decorator.window_closed.connect(sigc::mem_fun(window, &Gtk::Dialog::hide));
+       (sigc::hide(sigc::mem_fun(*this, &TimedMessageDialog::on_response)));
   window->set_transient_for(parent);
 }
 
-void TimedMessageDialog::on_response(int id)
+void TimedMessageDialog::on_response()
 {
   window->hide();
   main_loop->quit();
@@ -57,17 +52,7 @@ TimedMessageDialog::~TimedMessageDialog()
   delete window;
 }
 
-void TimedMessageDialog::show_all()
-{
-    window->show_all();
-}
-
-void TimedMessageDialog::hide()
-{
-  window->hide();
-}
-
-void TimedMessageDialog::run()
+void TimedMessageDialog::run_and_hide()
 {
   if (d_timeout > 0)
     {
@@ -78,6 +63,7 @@ void TimedMessageDialog::run()
     window->show_all();
     main_loop = Glib::MainLoop::create();
     main_loop->run();
+  window->hide();
 }
 
 bool TimedMessageDialog::tick()
@@ -111,7 +97,7 @@ bool TimedMessageDialog::tick()
   return Timing::STOP;
 }
     
-void TimedMessageDialog::set_title(std::string title)
+void TimedMessageDialog::set_title(Glib::ustring title)
 {
   window->set_title(title);
 }
