@@ -1,4 +1,4 @@
-//  Copyright (C) 2009 Ben Asselstine
+//  Copyright (C) 2009, 2012, 2014 Ben Asselstine
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -41,6 +41,9 @@ class HeroesMap : public OverviewMap
       */
      HeroesMap(std::list<Hero*> heroes);
 
+     //! Destructor.
+     ~HeroesMap() {};
+
      //! Realize the mouse click.
      void mouse_button_event(MouseButtonEvent e);
 
@@ -52,7 +55,7 @@ class HeroesMap : public OverviewMap
     /**
      * Classes that use HeroesMap must catch this signal to display the map.
      */
-    sigc::signal<void, Glib::RefPtr<Gdk::Pixmap> > map_changed;
+    sigc::signal<void, Cairo::RefPtr<Cairo::Surface> > map_changed;
 
     //! Emitted when a hero is clicked on.
     sigc::signal<void, Hero* > hero_selected;
