@@ -1,4 +1,4 @@
-// Copyright (C) 2008 Ben Asselstine
+// Copyright (C) 2008, 2014 Ben Asselstine
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -19,15 +19,15 @@
 #define NETWORK_ACTION_H
 
 #include "action.h"
-#include "playerlist.h"
 #include "OwnerId.h"
 
+//! Just like Action, but explicitly associated with a Player.
 class NetworkAction: public OwnerId
 {
 public:
 	
     //! The xml tag of this object in a network stream.
-    static std::string d_tag; 
+    static Glib::ustring d_tag; 
 
      //! Default constructor.
      NetworkAction(Action *action, guint32 owner);
@@ -39,7 +39,7 @@ public:
      ~NetworkAction();
 
      //! Returns debug information.
-     std::string toString() const;
+     Glib::ustring toString() const;
 
      //!Saving the network action to an xml stream.
      bool save(XML_Helper* helper) const;
