@@ -1,5 +1,5 @@
 // Copyright (C) 2006, 2007 Ulf Lorenz
-// Copyright (C) 2006, 2007, 2008, 2009, 2010 Ben Asselstine
+// Copyright (C) 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2014, 2015 Ben Asselstine
 // Copyright (C) 2007 Ole Laursen
 //
 //  This program is free software; you can redistribute it and/or modify
@@ -17,13 +17,11 @@
 //  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 
 //  02110-1301, USA.
 
+#include <config.h>
 #include <assert.h>
-#include <math.h>
-#include <stdlib.h>
-#include "gui/image-helpers.h"
+#include <cairomm/context.h>
 
 #include "overviewmap.h"
-#include "stacklist.h"
 #include "citylist.h"
 #include "ruinlist.h"
 #include "templelist.h"
@@ -33,10 +31,13 @@
 #include "playerlist.h"
 #include "player.h"
 #include "GameMap.h"
-#include "GraphicsCache.h"
+#include "ImageCache.h"
 #include "FogMap.h"
 #include "GameScenarioOptions.h"
-#include "tilesetlist.h"
+#include "Configuration.h"
+#include "shieldset.h"
+#include "tileset.h"
+#include "rnd.h"
 
 OverviewMap::OverviewMap()
 {
@@ -45,10 +46,6 @@ OverviewMap::OverviewMap()
   pixels_per_tile = 2.0;
 }
 
-OverviewMap::~OverviewMap()
-{
-}
-
 bool OverviewMap::isShadowed(Tile::Type type, int i, int j)
 {
   GameMap *gm = GameMap::getInstance();
@@ -92,7 +89,11 @@ bool OverviewMap::isShadowed(Tile::Type type, int i, int j)
 static int 
 prand(int i, int j)
 {
-  return (rand () % 3);
+  if (i || j)
+    {
+      ;
+    }
+  return (Rnd::rand () % 3);
 }
 
 static int 
@@ -111,8 +112,8 @@ drand(int i, int j)
 }
 
 
-void OverviewMap::choose_surface(bool front, Glib::RefPtr<Gdk::Pixmap> &surf,
-				 Glib::RefPtr<Gdk::GC> &gc)
+void OverviewMap::choose_surface(bool front, Cairo::RefPtr<Cairo::Surface> &surf,
+				 Cairo::RefPtr<Cairo::Context> &gc)
 {
   if (front)
     {
@@ -125,181 +126,195 @@ void OverviewMap::choose_surface(bool front, Glib::RefPtr<Gdk::Pixmap> &surf,
       surf = static_surface;
     }
 }
+
 void
-OverviewMap::draw_pixel(Glib::RefPtr<Gdk::Pixmap> surf, Glib::RefPtr<Gdk::GC> gc, int x, int y, const Gdk::Color color)
+OverviewMap::draw_pixel(Cairo::RefPtr<Cairo::Context> gc, int x, int y, const Gdk::RGBA color)
 {
-  gc->set_rgb_fg_color(color);
-  surf->draw_point(gc, x, y);
+  gc->set_source_rgba(color.get_red(), color.get_green(), color.get_blue(), color.get_alpha());
+  gc->rectangle(x, y, 1, 1);
+  gc->fill();
   return;
 }
 
 void
-OverviewMap::draw_filled_rect(int x, int y, int width, int height, const Gdk::Color color)
+OverviewMap::draw_filled_rect(int x, int y, int width, int height, const Gdk::RGBA color)
 {
   draw_filled_rect(true, x, y, width, height, color);
 }
 
 void
-OverviewMap::draw_filled_rect(bool front, int x, int y, int width, int height, const Gdk::Color color)
+OverviewMap::draw_filled_rect(bool front, int x, int y, int width, int height, const Gdk::RGBA color)
 {
-  Glib::RefPtr<Gdk::Pixmap> surf;
-  Glib::RefPtr<Gdk::GC> gc;
+  Cairo::RefPtr<Cairo::Surface> surf;
+  Cairo::RefPtr<Cairo::Context> gc;
   choose_surface (front, surf, gc);
-  gc->set_rgb_fg_color(color);
-  surf->draw_rectangle(gc, true, x, y, width, height);
+  gc->set_source_rgba(color.get_red(), color.get_green(), color.get_blue(), color.get_alpha());
+  gc->rectangle(x, y, width, height);
+  gc->fill();
 }
 
 void
-OverviewMap::draw_line(int src_x, int src_y, int dst_x, int dst_y, const Gdk::Color color)
+OverviewMap::draw_line(int src_x, int src_y, int dst_x, int dst_y, const Gdk::RGBA color)
 {
   draw_line(true, src_x, src_y, dst_x, dst_y, color);
 }
 
 void
-OverviewMap::draw_line(bool front, int src_x, int src_y, int dst_x, int dst_y, Gdk::Color color)
+OverviewMap::draw_line(bool front, int src_x, int src_y, int dst_x, int dst_y, Gdk::RGBA color)
 {
-  Glib::RefPtr<Gdk::Pixmap> surf;
-  Glib::RefPtr<Gdk::GC> gc;
+  Cairo::RefPtr<Cairo::Surface> surf;
+  Cairo::RefPtr<Cairo::Context> gc;
   choose_surface (front, surf, gc);
-  gc->set_rgb_fg_color(color);
-  surf->draw_line(gc, src_x, src_y, dst_x, dst_y);
+  gc->set_source_rgba(color.get_red(), color.get_green(), color.get_blue(), color.get_alpha());
+  gc->move_to(src_x, src_y);
+  gc->line_to(dst_x, dst_y);
 }
 
 void
-OverviewMap::draw_rect(int x, int y, int width, int height, const Gdk::Color color)
+OverviewMap::draw_rect(int x, int y, int width, int height, const Gdk::RGBA color)
 {
   draw_rect (true, x, y, width, height, color);
 }
 
 void
-OverviewMap::draw_rect(bool front, int x, int y, int width, int height, const Gdk::Color color)
+OverviewMap::draw_rect(bool front, int x, int y, int width, int height, const Gdk::RGBA color)
 {
-  Glib::RefPtr<Gdk::Pixmap> surf;
-  Glib::RefPtr<Gdk::GC> gc;
+  Cairo::RefPtr<Cairo::Surface> surf;
+  Cairo::RefPtr<Cairo::Context> gc;
   choose_surface (front, surf, gc);
-  gc->set_rgb_fg_color(color);
-  surf->draw_rectangle(gc, false, x, y, width, height);
+  gc->set_source_rgba(color.get_red(), color.get_green(), color.get_blue(), color.get_alpha());
+  //gc->rectangle(x, y, width, height);
+   gc->move_to(x, y);
+    gc->rel_line_to(width, 0);
+     gc->rel_line_to(0, height);
+      gc->rel_line_to(-width, 0);
+      gc->rel_line_to(0, -height);
+      gc->set_line_width(1.0);
+      gc->stroke();
 }
 
-void OverviewMap::draw_terrain_tile(Glib::RefPtr<Gdk::Pixmap> surf,
-				    Glib::RefPtr<Gdk::GC> gc,
+void OverviewMap::draw_terrain_tile(Cairo::RefPtr<Cairo::Context> gc,
 				    SmallTile::Pattern pattern,
-				    Gdk::Color first, 
-				    Gdk::Color second,
-				    Gdk::Color third,
+				    Gdk::RGBA first, 
+				    Gdk::RGBA second,
+				    Gdk::RGBA third,
 				    int i, int j, bool shadowed)
 {
 
   switch (pattern)
     {
       case SmallTile::SOLID:
-        draw_pixel(surf, gc, i, j, first);
+        draw_pixel(gc, i, j, first);
         break;
       case SmallTile::STIPPLED:
         {
           if ((i+j) % 2 == 0)
-            draw_pixel(surf, gc, i, j, first);
+            draw_pixel(gc, i, j, first);
           else
-            draw_pixel(surf, gc, i, j, second);
+            draw_pixel(gc, i, j, second);
         }
         break;
       case SmallTile::RANDOMIZED:
         {
           int num = prand(i, j) % 3;
           if (num == 0)
-            draw_pixel(surf, gc, i, j, first);
+            draw_pixel(gc, i, j, first);
           else if (num == 1)
-            draw_pixel(surf, gc, i, j, second);
+            draw_pixel(gc, i, j, second);
           else
-            draw_pixel(surf, gc, i, j, third);
+            draw_pixel(gc, i, j, third);
         }
         break;
       case SmallTile::DIAGONAL:
         {
           int num = drand(i, j) % 3;
           if (num == 0)
-            draw_pixel(surf, gc, i, j, first);
+            draw_pixel(gc, i, j, first);
           else if (num == 1)
-            draw_pixel(surf, gc, i, j, second);
+            draw_pixel(gc, i, j, second);
           else
-            draw_pixel(surf, gc, i, j, third);
+            draw_pixel(gc, i, j, third);
         }
         break;
       case SmallTile::CROSSHATCH:
         {
           int num = crand(i, j) % 3;
           if (num == 0)
-            draw_pixel(surf, gc, i, j, first);
+            draw_pixel(gc, i, j, first);
           else if (num == 1)
-            draw_pixel(surf, gc , i, j, second);
+            draw_pixel(gc , i, j, second);
           else
-            draw_pixel(surf, gc, i, j, third);
+            draw_pixel(gc, i, j, third);
         }
         break;
       case SmallTile::SUNKEN:
         if (shadowed == false)
-          draw_pixel(surf, gc, i, j, first);
+          draw_pixel(gc, i, j, first);
         else
           {
-            draw_pixel(surf, gc, i, j, second);
+            draw_pixel(gc, i, j, second);
           }
         break;
       case SmallTile::SUNKEN_STRIPED:
         if (shadowed == false)
 	  {
-	    if (j % 1 == 0)
-	      draw_pixel(surf, gc, i, j, first);
+	    if (j % 2 == 0)
+	      draw_pixel(gc, i, j, first);
 	    else
-	      draw_pixel(surf, gc, i, j, third);
+	      draw_pixel(gc, i, j, third);
 	  }
         else
           {
-            draw_pixel(surf, gc, i, j, second);
+            draw_pixel(gc, i, j, second);
           }
         break;
       case SmallTile::TABLECLOTH:
           {
             if (i % 4 == 0 && j % 4 == 0)
-              draw_pixel(surf, gc, i, j, first);
+              draw_pixel(gc, i, j, first);
             else if (i % 4 == 0 && j % 4 == 1)
-              draw_pixel(surf, gc, i, j, second);
+              draw_pixel(gc, i, j, second);
             else if (i % 4 == 0 && j % 4 == 2)
-              draw_pixel(surf, gc, i, j, first);
+              draw_pixel(gc, i, j, first);
             else if (i % 4 == 0 && j % 4 == 3)
-              draw_pixel(surf, gc, i, j, second);
+              draw_pixel(gc, i, j, second);
             else if (i % 4 == 1 && j % 4 == 0)
-              draw_pixel(surf, gc, i, j, second);
+              draw_pixel(gc, i, j, second);
             else if (i % 4 == 1 && j % 4 == 1)
-              draw_pixel(surf, gc, i, j, third);
+              draw_pixel(gc, i, j, third);
             else if (i % 4 == 1 && j % 4 == 2)
-              draw_pixel(surf, gc, i, j, second);
+              draw_pixel(gc, i, j, second);
             else if (i % 4 == 1 && j % 4 == 3)
-              draw_pixel(surf, gc, i, j, third);
+              draw_pixel(gc, i, j, third);
             else if (i % 4 == 2 && j % 4 == 0)
-              draw_pixel(surf, gc, i, j, first);
+              draw_pixel(gc, i, j, first);
             else if (i % 4 == 2 && j % 4 == 1)
-              draw_pixel(surf, gc, i, j, second);
+              draw_pixel(gc, i, j, second);
             else if (i % 4 == 2 && j % 4 == 2)
-              draw_pixel(surf, gc, i, j, first);
+              draw_pixel(gc, i, j, first);
             else if (i % 4 == 2 && j % 4 == 3)
-              draw_pixel(surf, gc, i, j, second);
+              draw_pixel(gc, i, j, second);
             else if (i % 4 == 3 && j % 4 == 0)
-              draw_pixel(surf, gc, i, j, second);
+              draw_pixel(gc, i, j, second);
             else if (i % 4 == 3 && j % 4 == 1)
-              draw_pixel(surf, gc, i, j, third);
+              draw_pixel(gc, i, j, third);
             else if (i % 4 == 3 && j % 4 == 2)
-              draw_pixel(surf, gc, i, j, second);
+              draw_pixel(gc, i, j, second);
             else if (i % 4 == 3 && j % 4 == 3)
-              draw_pixel(surf, gc, i, j, third);
+              draw_pixel(gc, i, j, third);
           }
         break;
+      case SmallTile::SUNKEN_RADIAL:
+        if (shadowed == true)
+          draw_pixel(gc, i, j, third);
+        break;
     }
 }
 
 void OverviewMap::draw_terrain_tile(Maptile *t, int i, int j)
 {
   bool shadowed = isShadowed(t->getType(), i, j);
-  draw_terrain_tile (static_surface, static_surface_gc, t->getPattern(), 
+  draw_terrain_tile (static_surface_gc, t->getPattern(), 
 		     t->getColor(), 
 		     t->getSecondColor(), 
 		     t->getThirdColor(),
@@ -308,14 +323,25 @@ void OverviewMap::draw_terrain_tile(Maptile *t, int i, int j)
 
 int OverviewMap::calculatePixelsPerTile(int width, int height)
 {
+  int pixels = 2;
   if (width <= (int)MAP_SIZE_TINY_WIDTH && 
       height <= (int)MAP_SIZE_TINY_HEIGHT)
-    return 4;
+    pixels = 4;
   else if (width <= (int)MAP_SIZE_SMALL_WIDTH && 
 	   height <= (int)MAP_SIZE_SMALL_HEIGHT)
-    return 3;
+    pixels = 3;
   else
-    return 2;
+    pixels = 2;
+  switch (Configuration::UiFormFactor(Configuration::s_ui_form_factor))
+    {
+    case Configuration::UI_FORM_FACTOR_DESKTOP:
+    case Configuration::UI_FORM_FACTOR_NETBOOK:
+      break;
+    case Configuration::UI_FORM_FACTOR_LARGE_SCREEN:
+      pixels++;
+      break;
+    }
+  return pixels;
 }
 
 int OverviewMap::calculatePixelsPerTile()
@@ -330,9 +356,25 @@ void OverviewMap::resize()
          GameMap::calculateTilesPerOverviewMapTile());
 }
 
+Vector<int> OverviewMap::calculate_smallmap_size()
+{
+  Vector<int> max_dimensions = GameMap::get_dim() * calculatePixelsPerTile();
+  Vector<int> d;
+  double p = max_dimensions.x / double(GameMap::get_dim().x);
+  d.x = max_dimensions.x;
+  d.y = int(round(GameMap::get_dim().y * p));
+  if (d.y > max_dimensions.y)
+    {
+	p = max_dimensions.y / double(GameMap::get_dim().y);
+	d.x = int(round(GameMap::get_dim().x * p));
+	d.y = max_dimensions.y;
+    }
+  return d;
+}
+
 void OverviewMap::resize(Vector<int> max_dimensions, float scale)
 {
-  surface.reset();
+  surface.clear();
 
     // calculate the width and height relations between pixels and maptiles
     Vector<int> bigmap_dim = GameMap::get_dim();
@@ -352,13 +394,18 @@ void OverviewMap::resize(Vector<int> max_dimensions, float scale)
     }
     map_tiles_per_tile = scale;
 
-    
-    static_surface = Gdk::Pixmap::create(Glib::RefPtr<Gdk::Drawable>(0), d.x, d.y, 24);
-    static_surface_gc = Gdk::GC::create(static_surface);
+    Cairo::RefPtr<Cairo::Surface> empty = Cairo::ImageSurface::create (Cairo::FORMAT_ARGB32, d.x, d.y);
+    static_surface = Cairo::Surface::create(empty, Cairo::CONTENT_COLOR_ALPHA, d.x, d.y);
+    static_surface_gc = Cairo::Context::create(static_surface);
 
+    Tileset *ts = GameMap::getTileset();
+    Tile *tile = ts->getFirstTile(SmallTile::SUNKEN_RADIAL);
+    if (tile)
+      draw_radial_gradient(tile->getSmallTile()->getColor(),
+                           tile->getSmallTile()->getSecondColor(), d.x, d.y);
     draw_terrain_tiles(Rectangle(0, 0, d.x, d.y));
-    surface = Gdk::Pixmap::create(static_surface, d.x, d.y, 24);
-    surface_gc = Gdk::GC::create(surface);
+    surface = Cairo::Surface::create(empty, Cairo::CONTENT_COLOR_ALPHA, d.x, d.y);
+    surface_gc = Cairo::Context::create(surface);
 
 }
 
@@ -375,9 +422,8 @@ void OverviewMap::redraw_tiles(Rectangle tiles)
 	Vector<int> dim(int(round(tiles.w * pixels_per_tile)),
 			int(round(tiles.h * pixels_per_tile)));
 
-	int width;
-	int height;
-	static_surface->get_size(width, height);
+	int width = get_width();
+	int height = get_height();
 	// ensure we're within bounds
 	pos = clip(Vector<int>(0, 0), pos,
 		   Vector<int>(width, height) - Vector<int>(1, 1));
@@ -416,18 +462,14 @@ Maptile* OverviewMap::getTile(int x, int y)
           favoured_tile = gm->getTile(pos);
         else if (gm->getTerrainType(pos) == Tile::MOUNTAIN)
           favoured_tile = gm->getTile(pos);
-        else if (gm->getTerrainType(pos) == Tile::VOID)
-          favoured_tile = gm->getTile(pos);
       }
   return favoured_tile;
 }
 
 void OverviewMap::draw_terrain_tiles(Rectangle r)
 {
-    unsigned int oldrand = rand();
-    srand(0);
     Tileset *ts = GameMap::getTileset();
-    Gdk::Color rd = ts->getRoadColor();
+    Gdk::RGBA rd = ts->getRoadColor();
     for (int i = r.x; i < r.x + r.w; i+=int(map_tiles_per_tile))
       for (int j = r.y; j < r.y + r.h; j+=int(map_tiles_per_tile))
         {
@@ -436,11 +478,10 @@ void OverviewMap::draw_terrain_tiles(Rectangle r)
           Maptile *mtile = getTile(x,y);
 
           if (mtile->isRoadTerrain())
-            draw_pixel(static_surface, static_surface_gc, i, j, rd);
+            draw_pixel(static_surface_gc, i, j, rd);
           else
             draw_terrain_tile (mtile, i, j);
         }
-    srand (oldrand);
 }
 
 void OverviewMap::after_draw()
@@ -460,16 +501,13 @@ void OverviewMap::draw(Player *player)
     // of the surface when drawing. I will implcitely assume this during this
     // function.
     
-    //Gdk::Color black = Gdk::Color();
-    //black.set_rgb_p(0,0,0);
-    //surface_gc->set_rgb_fg_color(black);
-    //surface->draw_rectangle(surface_gc, true, 0, 0, -1, -1);
-    //Glib::RefPtr<Gdk::Pixbuf> buf = Gdk::Pixbuf::create(static_surface, 0, 0, -1, -1);
-    surface->draw_drawable(surface_gc, static_surface, 0, 0, 0, 0, -1, -1);
+    //put the static surface on the surface
+    surface_gc->set_source(static_surface, 0, 0);
+    surface_gc->paint();
 
     // Draw ruins as a white dot
 	
-    Gdk::Color ruindotcolor = ts->getRuinColor();
+    Gdk::RGBA ruindotcolor = ts->getRuinColor();
     for (Ruinlist::iterator it = Ruinlist::getInstance()->begin();
         it != Ruinlist::getInstance()->end(); it++)
     {
@@ -486,7 +524,7 @@ void OverviewMap::draw(Player *player)
     }
 
     // Draw temples as a white dot
-    Gdk::Color templedotcolor = ts->getTempleColor();
+    Gdk::RGBA templedotcolor = ts->getTempleColor();
     for (Templelist::iterator it = Templelist::getInstance()->begin();
         it != Templelist::getInstance()->end(); it++)
     {
@@ -500,18 +538,19 @@ void OverviewMap::draw(Player *player)
     }
 
     //fog it up
-    for (int i = 0; i < GameMap::getWidth(); i++)
-        for (int j = 0; j < GameMap::getHeight(); j++)
-        {
-          Vector <int> pos;
-          pos.x = i;
-          pos.y = j;
-          if (Playerlist::getViewingplayer()->getFogMap()->isFogged(pos) == true)
+    if (GameScenarioOptions::s_hidden_map == true)
+      {
+        for (int i = 0; i < GameMap::getWidth(); i++)
+          for (int j = 0; j < GameMap::getHeight(); j++)
             {
-              pos = mapToSurface(pos);
-              draw_filled_rect(true, pos.x, pos.y, size, size, FOG_COLOUR);
+              Vector <int> pos = Vector<int>(i, j);
+              if (Playerlist::getViewingplayer()->getFogMap()->isFogged(pos) == true)
+                {
+                  pos = mapToSurface(pos);
+                  draw_filled_rect(true, pos.x, pos.y, size, size, FOG_COLOUR);
+                }
             }
-        }
+      }
 
     //the idea here is that we want to show what happens when an AI-owned
     //stack moves through our area.  so we block the view of computer
@@ -519,26 +558,24 @@ void OverviewMap::draw(Player *player)
     if (Playerlist::getViewingplayer()->getType() != Player::HUMAN &&
        	GameScenarioOptions::s_hidden_map == true)
       {
-	int width = 0;
-	int height = 0;
-	surface->get_size(width, height);
+	int width = get_width();
+	int height = get_height();
 	draw_filled_rect(true, 0, 0, width, height, FOG_COLOUR);
       }
 
   if (blank_screen)
     {
-      int width = 0;
-      int height = 0;
-      surface->get_size(width, height);
-      surface_gc->set_rgb_fg_color(FOG_COLOUR);
-      surface->draw_rectangle(surface_gc, true, 0,0,width, height);
+      int width = get_width();
+      int height = get_height();
+      surface_gc->set_source_rgba(FOG_COLOUR.get_red(), FOG_COLOUR.get_green(), FOG_COLOUR.get_blue(), FOG_COLOUR.get_alpha());
+      surface_gc->rectangle(0,0,width, height);
+      surface_gc->fill();
     }
     // let derived classes do their job
     after_draw();
-
 }
 
-Glib::RefPtr<Gdk::Pixmap> OverviewMap::get_surface()
+Cairo::RefPtr<Cairo::Surface> OverviewMap::get_surface()
 {
     return surface;
 }
@@ -590,7 +627,17 @@ Vector<int> OverviewMap::mapToSurface(Vector<int> pos)
 
 void OverviewMap::draw_cities (bool all_razed)
 {
-  GraphicsCache *gc = GraphicsCache::getInstance();
+  int csize = 0;
+  switch (Configuration::UiFormFactor(Configuration::s_ui_form_factor))
+    {
+    case Configuration::UI_FORM_FACTOR_DESKTOP:
+    case Configuration::UI_FORM_FACTOR_NETBOOK:
+      break;
+    case Configuration::UI_FORM_FACTOR_LARGE_SCREEN:
+      csize = 1;
+      break;
+    }
+  ImageCache *gc = ImageCache::getInstance();
 
   // Draw all cities as shields over the city location, in the colors of
   // the players.
@@ -602,14 +649,13 @@ void OverviewMap::draw_cities (bool all_razed)
       if (c->isVisible(Playerlist::getViewingplayer()) == false)
         continue;
       if (c->isBurnt() == true || all_razed == true)
-        tmp = gc->getSmallRuinedCityPic();
+        tmp = gc->getSmallRuinedCityImage();
       else
-        tmp = gc->getShieldPic(0, c->getOwner());
+        tmp = gc->getShieldPic(csize, c->getOwner());
   
       Vector<int> pos = c->getPos();
       pos = mapToSurface(pos);
       tmp->blit_centered(surface, pos);
-
   }
 }
 
@@ -621,18 +667,18 @@ void OverviewMap::blank(bool on)
 
 void OverviewMap::draw_hero(Vector<int> pos, bool white)
 {
-    GraphicsCache *gc = GraphicsCache::getInstance();
+    ImageCache *gc = ImageCache::getInstance();
     // draw the hero picture over top of the host city
 
     Vector<int> start = mapToSurface(pos);
 
     start += Vector<int>(int(pixels_per_tile/2), int(pixels_per_tile/2));
 
-    PixMask *heropic = gc->getSmallHeroPic(white);
+    PixMask *heropic = gc->getSmallHeroImage(white);
     heropic->blit_centered(surface, start);
 }
 
-void OverviewMap::draw_target_box(Vector<int> pos, const Gdk::Color c)
+void OverviewMap::draw_target_box(Vector<int> pos, const Gdk::RGBA c)
 {
   Vector<int> start = mapToSurface(pos);
   start += Vector<int>(int(pixels_per_tile/2), int(pixels_per_tile/2));
@@ -646,3 +692,52 @@ void OverviewMap::draw_target_box(Vector<int> pos, const Gdk::Color c)
   draw_filled_rect(start.x - (xsize / 2), start.y - (ysize / 2), 
 		   xsize, ysize, c);
 }
+
+void OverviewMap::draw_square_around_city(City *c, Gdk::RGBA colour)
+{
+  Vector<int> start = c->getPos();
+  start = mapToSurface(start);
+  Shieldset *ss = GameMap::getShieldset();
+  guint32 width = ss->getSmallWidth();
+  guint32 height = ss->getSmallHeight();
+  start -= Vector<int>(width,height)/2;
+  Vector<int> end = start + Vector<int>(width,height);
+  draw_rect (start.x-3, start.y-3, end.x-start.x+4, end.y-start.y+4, colour);
+}
+
+void OverviewMap::draw_radial_gradient(Cairo::RefPtr<Cairo::Surface> surface, Gdk::RGBA inner, Gdk::RGBA outer, int width, int height)
+{
+  double ired = inner.get_red();
+  double igreen = inner.get_green();
+  double iblue = inner.get_blue();
+  double ored = outer.get_red();
+  double ogreen = outer.get_green();
+  double oblue = outer.get_blue();
+  Cairo::RefPtr<Cairo::Context> cr = Cairo::Context::create(surface);
+  double max = (double) width;
+  if ((double)height > max)
+    max = (double)width;
+  double xcenter = (double)width / 2.0;
+  double ycenter = (double)height / 2.0;
+  Cairo::RefPtr<Cairo::RadialGradient> gradient = 
+    Cairo::RadialGradient::create(xcenter, ycenter, 1, xcenter, ycenter, max);
+  gradient->add_color_stop_rgb(0, ired, igreen, iblue);
+  gradient->add_color_stop_rgb(1.0, ored, ogreen, oblue);
+  cr->set_source(gradient);
+  cr->paint();
+}
+
+void OverviewMap::draw_radial_gradient(Gdk::RGBA inner, Gdk::RGBA outer, int width, int height)
+{
+  return draw_radial_gradient(static_surface, inner, outer, width, height);
+}
+
+int OverviewMap::get_width()
+{
+  return GameMap::get_dim().x / map_tiles_per_tile * pixels_per_tile;
+}
+
+int OverviewMap::get_height()
+{
+  return GameMap::get_dim().y / map_tiles_per_tile * pixels_per_tile;
+}
