@@ -41,7 +41,7 @@ class Path : public std::list<Vector<int> >
 {
     public:
 	//! The xml tag of this object in a saved-game file.
-	static std::string d_tag; 
+	static Glib::ustring d_tag; 
 
         //! Default constructor.
         Path();
@@ -134,9 +134,6 @@ class Path : public std::list<Vector<int> >
 
         // Data
 
-	//! A cached copy of a Stack object's movement bonus.
-        guint32 d_bonus;
-
 	//! The point in the path that can't be reached.
 	guint32 d_moves_exhausted_at_point;
 
