@@ -3,7 +3,7 @@
 // Copyright (C) 2003 Marek Publicewicz
 // Copyright (C) 2004 John Farrell
 // Copyright (C) 2005 Bryan Duff
-// Copyright (C) 2006, 2007, 2008, 2009, 2010, 2011 Ben Asselstine
+// Copyright (C) 2006, 2007, 2008, 2009, 2010, 2011, 2014, 2015 Ben Asselstine
 // Copyright (C) 2007, 2008 Ole Laursen
 //
 //  This program is free software; you can redistribute it and/or modify
@@ -24,7 +24,6 @@
 #ifndef PLAYER_H
 #define PLAYER_H
 
-#include <string>
 #include <list>
 #include <vector>
 #include <sigc++/trackable.h>
@@ -121,7 +120,7 @@ class Player: public sigc::trackable
 {
     public:
 	//! The xml tag of this object in a saved-game file.
-	static std::string d_tag; 
+	static Glib::ustring d_tag; 
 
         //! The available player types.
         enum Type {
@@ -174,7 +173,7 @@ class Player: public sigc::trackable
 	 *                     the next free Id it used.
          */
 	//! Default constructor.
-        Player (std::string name, guint32 armyset, Gdk::Color color, int width,
+        Player (Glib::ustring name, guint32 armyset, Gdk::RGBA color, int width,
 		int height, Type type, int player_no = -1);
 
         //! Copy constructor.
@@ -190,7 +189,7 @@ class Player: public sigc::trackable
 	// Set Methods
 
         //! Change the player's name.
-        void setName(std::string name){d_name = name;}
+        void setName(Glib::ustring name){d_name = name;}
         
         //! Change the player's Armyset.
         void setArmyset(guint32 armyset){d_armyset = armyset;}
@@ -199,7 +198,7 @@ class Player: public sigc::trackable
         void setType(Type type) {d_type = type;}
 
         //! Change the player's colour.
-        void setColor(Gdk::Color c);
+        void setColor(Gdk::RGBA c);
 
         //! Makes a player unable to die, even when having no units or cities.
         void setMortality(bool ismortal) {d_immortal = !ismortal;}
@@ -211,7 +210,7 @@ class Player: public sigc::trackable
 	void setDiplomaticRank (guint32 rank) {d_diplomatic_rank = rank;};
 
 	//! Set the rank as a name.
-	void setDiplomaticTitle (std::string title) {d_diplomatic_title = title;};
+	void setDiplomaticTitle (Glib::ustring title) {d_diplomatic_title = title;};
 	//! Set if this player will be seen as it moves through visible terrain.
 	void setObservable(bool observable) {d_observable = observable;};
 
@@ -272,16 +271,16 @@ class Player: public sigc::trackable
 	guint32 getDiplomaticRank () const {return d_diplomatic_rank;};
 
 	//! What rank do we have?  As a name.
-	std::string getDiplomaticTitle() const {return d_diplomatic_title;};
+	Glib::ustring getDiplomaticTitle() const {return d_diplomatic_title;};
 
         //! Returns the colour of the player.
-	Gdk::Color getColor() const {return d_color;}
+	Gdk::RGBA getColor() const {return d_color;}
 
         //! Returns the amount of gold pieces the player has in the treasury.
         int getGold() const {return d_gold;}
 
         //! Returns the name of the player.
-        std::string getName() const;
+        Glib::ustring getName() const;
 
 	//! Returns the player's current score.
         guint32 getScore() const;
@@ -313,27 +312,21 @@ class Player: public sigc::trackable
         //! Returns a list of the player's actions to show in a report.
 	std::list<Action *> getReportableActions() const;
 
-        //! Returns the number of times we fought so far this turn.
-        int countFightsThisTurn() const;
-        //! Returns the number of times we moved a stack this turn.
-        int countMovesThisTurn() const;
         //! Returns number of cities that were too poor to produce this turn.
         int countDestituteCitiesThisTurn() const;
 
-        //! Returns the battle actions for this turn.
-        std::list<Action *> getFightsThisTurn() const;
-        //! Returns the movement actions for this turn.
+        //! Returns all actions for this turn.
         std::list<Action *> getMovesThisTurn() const;
 
+        //! Returns the first city conquered that is still ours and not razed.
+        City *getFirstCity() const;
+
         //! Remove every Action from the list of the player's actions.
         void clearActionlist();
 
         //! Show debugging information for the player's Action list.
         void dumpActionlist() const;
 
-	//! Wrap the player's actions for transport over the network.
-	void saveNetworkActions(XML_Helper *helper) const;
-
 	//! Check to see if it's our turn.
 	bool hasAlreadyInitializedTurn() const;
 
@@ -382,9 +375,6 @@ class Player: public sigc::trackable
 	//! Return the grand total of the player's armies.
 	guint32 countArmies() const;
 
-        //! Return the number of the player's armies that are awardable.
-	guint32 countAllies() const;
-
 	//! Return the player's currently selected stack.
 	Stack * getActivestack() const;
 
@@ -612,7 +602,7 @@ class Player: public sigc::trackable
          * @return False on error, true otherwise.
 	 */
         //! Change the text on a signpost.
-        bool signpostChange(Signpost *signpost, std::string message);
+        bool signpostChange(Signpost *signpost, Glib::ustring message);
 
 
 
@@ -714,7 +704,7 @@ class Player: public sigc::trackable
 	bool heroPickupAllItems(Hero *h, Vector<int> pos);
 
         //! Have the given hero use the given item, on the given player.
-        bool heroUseItem(Hero *h, Item *item, Player *player);
+        bool heroUseItem(Hero *h, Item *item, Player *player, City *friendly_city, City *enemy_city, City *neutral_city, City *city);
 
 	/**
 	 * Completing a Quest entails that the Hero is going to receive a
@@ -806,7 +796,7 @@ class Player: public sigc::trackable
          * @return False on error, true otherwise.
 	 */
         //! Callback to rename a city.
-        bool cityRename(City *city, std::string name);
+        bool cityRename(City *city, Glib::ustring name);
 
         /** 
 	 * Callback to initiate vectoring new units from a player's City to 
@@ -1097,7 +1087,7 @@ class Player: public sigc::trackable
         Reward* giveQuestReward(Quest *quest, Stack *stack);
 
 	//! Give the player a new name.
-	void rename (std::string name);
+	void rename (Glib::ustring name);
 
         //! have a hero show up, or not.
         bool maybeRecruitHero ();
@@ -1167,24 +1157,6 @@ class Player: public sigc::trackable
 	bool AI_maybePickUpItems (Stack *s, int dist, bool &picked_up,
 				  bool &stack_died);
 
-	/** 
-	 * Callback to decide if we perform treachery on a friendly player.
-	 *
-	 * For human players this method presents a dialog for the user
-	 * to confirm if treachery should happen or not.  For computer
-	 * players this method implements the decision to perform treachery
-	 * or not.
-	 *
-	 * @param stack             My stack considering the treachery.
-	 * @param player            The friendly player.
-	 * @param pos               The place on the map being targetted.
-	 * @param state             The state we end up in if we decide yes.
-	 *
-	 * @return True if we decided to be treacherous.  False otherwise.
-	 */
-	//! Decision callback for if we should perform trechery or not.
-	bool treachery (Stack *stack, Player *player, Vector <int> pos);
-
 	/**
 	 * Callback to have the Player resign.  This entails disbanding
 	 * all of the player's stacks and then razing all of the player's 
@@ -1383,7 +1355,7 @@ class Player: public sigc::trackable
         sigc::signal<void> schangingStats;
 
 	//! Emitted whenever a computer player does something of note.
-        sigc::signal<void, std::string> schangingStatus;
+        sigc::signal<void, Glib::ustring> schangingStatus;
 
 	//! Emitted whenever any player does anything at all.
 	sigc::signal<void> sbusy;
@@ -1450,7 +1422,7 @@ class Player: public sigc::trackable
         sigc::signal<bool, Stack *, Player *, Vector<int> > streacheryStack;
 
 	//! Signal raised when a human player is deciding.
-        sigc::signal<bool, Stack *, Player *, Vector<int> > streachery;
+        //sigc::signal<bool, Stack *, Player *, Vector<int> > streachery;
 
         //! Player would like to end the turn.
         sigc::signal<void> ending_turn;
@@ -1470,9 +1442,15 @@ class Player: public sigc::trackable
         sigc::signal<void, Player*, guint32> sunk_ships;
         sigc::signal<void, Hero *, guint32> bags_picked_up;
         sigc::signal<void, Hero *, guint32> mp_added_to_hero_stack;
-        sigc::signal<void, Hero *, guint32> worms_killed;
+        sigc::signal<void, Hero *, Glib::ustring, guint32> worms_killed;
+        sigc::signal<void, Hero *> bridge_burned;
+        sigc::signal<void, Hero *, Ruin*, Glib::ustring> keeper_captured;
+        sigc::signal<void, Hero *, Glib::ustring> monster_summoned;
+        sigc::signal<void, Hero *, Glib::ustring, guint32> city_diseased;
+        sigc::signal<void, Hero *, Glib::ustring, Glib::ustring, guint32> city_defended;
+        sigc::signal<void, Hero *, Glib::ustring, guint32> city_persuaded;
+        sigc::signal<void, Hero *, Glib::ustring> stack_teleported;
         
-	void loadPbmGame();
 	//! Check the history to see if we ever conquered the given city.
 
 
@@ -1482,8 +1460,8 @@ class Player: public sigc::trackable
 	
 	// Static Methods
 
-	static std::string playerTypeToString(const Player::Type type);
-	static Player::Type playerTypeFromString(const std::string str);
+	static Glib::ustring playerTypeToString(const Player::Type type);
+	static Player::Type playerTypeFromString(const Glib::ustring str);
 	//! is it safe to vector from the given city?
 	static bool safeFromAttack(City *c, guint32 safe_mp, guint32 min_defenders);
         /** 
@@ -1500,8 +1478,8 @@ class Player: public sigc::trackable
          * @param type     The player's type (Player::Type).
          */
 	//! Create a player.
-        static Player* create(std::string name, guint32 armyset, 
-			      Gdk::Color color, int width, int height, 
+        static Player* create(Glib::ustring name, guint32 armyset, 
+			      Gdk::RGBA color, int width, int height, 
 			      Type type);
         
         /** 
@@ -1554,10 +1532,10 @@ class Player: public sigc::trackable
 	/**
 	 * Mask portions of images are shaded in this colour.
 	 */
-	Gdk::Color d_color;
+	Gdk::RGBA d_color;
 
 	//! The name of the Player.
-        std::string d_name;
+        Glib::ustring d_name;
 
 	//! The ArmySet of the Player.
         guint32 d_armyset;
@@ -1611,7 +1589,7 @@ class Player: public sigc::trackable
 	guint32 d_diplomatic_rank;
 
 	//! The title that goes along with the diplomatic rank.
-	std::string d_diplomatic_title;
+	Glib::ustring d_diplomatic_title;
 
 	//! The proposals that this Player is making this turn.
 	DiplomaticProposal d_diplomatic_proposal[MAX_PLAYERS];
@@ -1646,7 +1624,7 @@ class Player: public sigc::trackable
         void doGiveReward(Stack *s, Reward *reward, StackReflist *stacks);
         void doHeroDropItem(Hero *hero, Item *item, Vector<int> pos, bool &splash);
 	bool doHeroDropAllItems(Hero *h, Vector<int> pos, bool &splash);
-        bool doHeroUseItem(Hero *h, Item *item, Player *victim);
+        bool doHeroUseItem(Hero *h, Item *item, Player *victim, City *friendly_city, City *enemy_city, City *neutral_city, City *city);
         void doHeroPickupItem(Hero *hero, Item *item, Vector<int> pos);
         bool doHeroPickupAllItems(Hero *h, Vector<int> pos);
         void doHeroGainsLevel(Hero *hero, Army::Stat stat);
@@ -1654,8 +1632,8 @@ class Player: public sigc::trackable
         void doStacksReset();
         void doRuinsReset();
         void doCollectTaxesAndPayUpkeep();
-        void doSignpostChange(Signpost *signpost, std::string message);
-        void doCityRename(City *c, std::string name);
+        void doSignpostChange(Signpost *signpost, Glib::ustring message);
+        void doCityRename(City *c, Glib::ustring name);
         void doVectorFromCity(City * c, Vector<int> dest);
         void doSetFightOrder(std::list<guint32> order);
         void doResign(std::list<History*> &history);
@@ -1665,7 +1643,7 @@ class Player: public sigc::trackable
         void doConquerCity(City *city);
 	void doLootCity(Player *looted, guint32 added, guint32 subtracted);
         Hero* doRecruitHero(HeroProto* hero, City *city, int cost, int alliesCount, const ArmyProto *ally, StackReflist *stacks);
-        void doRename(std::string name);
+        void doRename(Glib::ustring name);
 	void doKill();
         void doStackDefend(Stack *stack);
         void doStackUndefend(Stack *stack);
@@ -1723,11 +1701,10 @@ class Player: public sigc::trackable
 
 	void pruneActionlist();
 	static void pruneActionlist(std::list<Action*> actions);
-	std::list<History*> getHistoryForThisTurn() const;
 	    
     private:
         //! Loads the subdata of a player (actions and stacklist)
-        bool load(std::string tag, XML_Helper* helper);
+        bool load(Glib::ustring tag, XML_Helper* helper);
 
         /**
 	 * Returns all heroes in the given list of stacks.
@@ -1765,8 +1742,6 @@ class Player: public sigc::trackable
         void handleDeadHeroes(std::list<Stack*> &stacks, std::list<History*> &history);
         History* handleDeadHero(Hero *h, Maptile *tile, Vector<int> pos);
 
-        void healInjuredArmies(std::list<Stack*> &stacks);
-
         void handleDeadArmiesForQuests(std::list<Stack*> &stacks, std::vector<guint32> &culprits);
         
         /** 
@@ -1818,6 +1793,7 @@ class Player: public sigc::trackable
         bool computerSearch(Stack *s, MoveResult *r);
 };
 
+Fight::Result ruinfight (Stack **attacker, Stack **defender);
 #endif // PLAYER_H
 
 // End of file
