@@ -2,7 +2,7 @@
 // Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006 Ulf Lorenz
 // Copyright (C) 2004 John Farrell
 // Copyright (C) 2005 Andrea Paternesi
-// Copyright (C) 2007, 2008, 2009, 2010 Ben Asselstine
+// Copyright (C) 2007, 2008, 2009, 2010, 2014, 2015 Ben Asselstine
 // Copyright (C) 2007 Ole Laursen
 //
 //  This program is free software; you can redistribute it and/or modify
@@ -20,21 +20,17 @@
 //  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 
 //  02110-1301, USA.
 
-#include "config.h"
+#include <config.h>
 #include <sstream>
-#include <algorithm>
 #include <sigc++/functors/mem_fun.h>
 
 #include "playerlist.h"
 #include "armysetlist.h"
-
 #include "citylist.h"
 #include "ruinlist.h"
 #include "vectoredunitlist.h"
-#include "stacklist.h"
 #include "xmlhelper.h"
 #include "history.h"
-#include "citylist.h"
 #include "stacklist.h"
 #include "FogMap.h"
 #include "real_player.h"
@@ -45,13 +41,14 @@
 #include "GameMap.h"
 #include "shieldset.h"
 #include "shieldsetlist.h"
+#include "ucompose.hpp"
+#include "stack.h"
+#include "rnd.h"
 
-using namespace std;
-
-//#define debug(x) {cerr<<__FILE__<<": "<<__LINE__<<": "<<x<<endl<<flush;}
+//#define debug(x) {std::cerr<<__FILE__<<": "<<__LINE__<<": "<<x<<std::endl<<std::flush;}
 #define debug(x)
 
-std::string Playerlist::d_tag = "playerlist";
+Glib::ustring Playerlist::d_tag = "playerlist";
 Playerlist* Playerlist::s_instance = 0;
 Player* Playerlist::d_activeplayer = 0;
 Player* Playerlist::viewingplayer = 0;
@@ -158,9 +155,7 @@ void Playerlist::nextPlayer()
     iterator it;
 
     if (!d_activeplayer)
-    {
-        it = begin();
-    }
+      it = begin();
     else
     {
         for (it = begin(); it != end(); ++it)
@@ -190,13 +185,11 @@ void Playerlist::nextPlayer()
     debug("got player: " <<d_activeplayer->getName())
 }
 
-Player* Playerlist::getPlayer(string name) const
+Player* Playerlist::getPlayer(Glib::ustring name) const
 {
     debug("getPlayer()");
     for (const_iterator it = begin(); it != end(); ++it)
-    {
-        if ((*it)->getName() == name) return (*it);
-    }
+      if ((*it)->getName() == name) return (*it);
     return 0;
 }
 
@@ -253,7 +246,8 @@ void Playerlist::add(Player *player)
   push_back(player);
   d_id[player->getId()] = player;
 }
-bool Playerlist::load(string tag, XML_Helper* helper)
+
+bool Playerlist::load(Glib::ustring tag, XML_Helper* helper)
 {
     static guint32 active = 0;
     static guint32 neutral = 0;
@@ -295,12 +289,6 @@ Playerlist::iterator Playerlist::flErase(Playerlist::iterator it)
     return erase (it);
 }
 
-struct rankable_t
-{
-  guint32 score;
-  Player *player;
-};
-
 bool compareDiplomaticScores (const struct rankable_t lhs,
 			      const struct rankable_t rhs)
 {
@@ -381,7 +369,7 @@ void Playerlist::calculateDiplomaticRankings()
   unsigned int numAlive = countPlayersAlive ();
   // okay, we take the first numAlive titles
 
-  std::vector<std::string> available_titles;
+  std::vector<Glib::ustring> available_titles;
   for (i = numAlive; i < MAX_PLAYERS ; i++)
     {
       for (unsigned int j = 0; j < MAX_PLAYERS; j++)
@@ -390,10 +378,10 @@ void Playerlist::calculateDiplomaticRankings()
             used_titles[j] = 1;
 	}
     }
-  for (unsigned int i = 0; i < MAX_PLAYERS; i++)
+  for (unsigned int j = 0; j < MAX_PLAYERS; j++)
     {
-      if (used_titles[i] != 1)
-	available_titles.push_back (get_title(i));
+      if (used_titles[j] != 1)
+	available_titles.push_back (get_title(j));
     }
 
   for (const_iterator it = begin (); it != end (); it++)
@@ -554,9 +542,13 @@ void Playerlist::swap(Player *old_player, Player *new_player)
 bool Playerlist::randomly(const Player *lhs, const Player *rhs)  
 {
   Playerlist *pl = Playerlist::getInstance();
+  if (rhs)
+    {
+      ;
+    }
   if (lhs == pl->getNeutral())
     return false;
-  if (rand() % 2 == 0)
+  if (Rnd::rand() % 2 == 0)
     return true;
   else
     return false;
@@ -643,7 +635,7 @@ void Playerlist::syncPlayer(GameParameters::Player player)
 	  //was off, now it's still off.
 	  break;
 	default:
-	  cerr << "could not make player with type " << player.type;
+          std::cerr << String::ucompose("could not make player with type %1", player.type) << std::endl;
 	  exit (1);
 	  break;
 	}
@@ -702,7 +694,7 @@ void Playerlist::syncPlayer(GameParameters::Player player)
 	}
       break;
     default:
-      cerr << "could not sync player with type " << player.type;
+      std::cerr << String::ucompose("could not sync player with type %1", player.type) << std::endl;
       exit (1);
       break;
     }
@@ -715,28 +707,29 @@ void Playerlist::syncPlayer(GameParameters::Player player)
 
 void Playerlist::syncPlayers(std::vector<GameParameters::Player> players)
 {
-
   std::vector<GameParameters::Player>::const_iterator i = players.begin();
   for (; i != players.end(); i++)
     syncPlayer(*i);
-
 }
 	
 guint32 Playerlist::turnHumansIntoNetworkPlayers()
 {
   guint32 count = 0;
+  std::list<Player*> p;
   for (iterator i = begin(); i != end(); i++)
     {
       if ((*i)->getType() == Player::HUMAN)
 	{
           count++;
 	  NetworkPlayer *new_p = new NetworkPlayer(**i);
+          p.push_back(*i);
 	  swap((*i), new_p);
-	  //delete *i; fixme
 	  i = begin();
 	  continue;
 	}
     }
+  for (std::list<Player*>::iterator j = p.begin(); j != p.end(); j++)
+    delete *j;
   return count;
 }
 
@@ -826,7 +819,6 @@ bool Playerlist::inGivenOrder(const Player *lhs, const Player *rhs)
   return lhs_rank < rhs_rank;
 }
 
-
 void Playerlist::reorder(std::list<guint32> order)
 {
   given_turn_order = order;
@@ -854,9 +846,7 @@ void Playerlist::surrender()
   for (iterator it = begin(); it != end(); it++)
     {
       if ((*it)->getType() != Player::HUMAN)
-	{
-	  (*it)->setSurrendered(true);
-	}
+        (*it)->setSurrendered(true);
     }
 }
 
@@ -883,22 +873,6 @@ void Playerlist::setWinningPlayer(Player *winner)
   d_activeplayer = winner;
 }
 
-int Playerlist::countFightsThisTurn() const
-{
-  int count = 0;
-  for (const_iterator it = begin(); it != end(); it++)
-    count += (*it)->countFightsThisTurn();
-  return count;
-}
-
-int Playerlist::countMovesThisTurn() const
-{
-  int count = 0;
-  for (const_iterator it = begin(); it != end(); it++)
-    count += (*it)->countMovesThisTurn();
-  return count;
-}
-
 Player *Playerlist::getWinningPlayer() const
 {
   guint32 best_score = 0;
@@ -923,8 +897,7 @@ bool Playerlist::hasArmyset(guint32 id) const
 {
   for (const_iterator it = begin(); it != end(); it++)
     {
-      Player *p = (*it);
-      if (p->getArmyset() == id)
+      if ((*it)->getArmyset() == id)
         return true;
     }
   return false;
@@ -933,18 +906,7 @@ bool Playerlist::hasArmyset(guint32 id) const
 void Playerlist::setNewColours(Shieldset *shieldset)
 {
   for (iterator it = begin(); it != end(); it++)
-    {
-      Player *p = (*it);
-      p->setColor(shieldset->getColor(p->getId()));
-    }
-}
-
-guint32 Playerlist::countBlessings() const
-{
-  guint32 count = 0;
-  for (const_iterator it = begin(); it != end(); it++)
-    count += (*it)->getStacklist()->countBlessingsOnArmyUnits();
-  return count;
+    (*it)->setColor(shieldset->getColor((*it)->getId()));
 }
 
 void Playerlist::clearAllActions()
@@ -964,3 +926,17 @@ void Playerlist::syncNeutral()
   swap(d_neutral, p);
   d_neutral = p;
 }
+
+Stack *Playerlist::getStackById(guint32 id) const
+{
+  //this method checks each stacklist for the given stack id.
+  //the stack ids in each stacklist are in a std::map, so that's how the
+  //stack pointers are gotten.
+  for (const_iterator j = begin(), jend = end(); j != jend; ++j) 
+    {
+      Stack *s = (*j)->getStacklist()->getStackById(id);
+      if (s)
+        return s;
+    }
+  return NULL;
+}
