@@ -1,6 +1,6 @@
 // Copyright (C) 2000, 2001, 2002, 2003 Michael Bartl
 // Copyright (C) 2001, 2002, 2003, 2004, 2005 Ulf Lorenz
-// Copyright (C) 2007, 2008, 2009 Ben Asselstine
+// Copyright (C) 2007, 2008, 2009, 2014, 2015 Ben Asselstine
 // Copyright (C) 2007 Ole Laursen
 //
 //  This program is free software; you can redistribute it and/or modify
@@ -23,7 +23,6 @@
 
 #include <list>
 #include <map>
-#include <string>
 #include <vector>
 #include <sigc++/trackable.h>
 #include <sigc++/signal.h>
@@ -31,6 +30,7 @@
 
 class History;
 class Shieldset;
+class Stack;
 
 #include "player.h"
 
@@ -47,7 +47,7 @@ class Playerlist : public std::list<Player*>, public sigc::trackable
     public:
 
 	//! The xml tag of this object in a saved-game file.
-	static std::string d_tag; 
+	static Glib::ustring d_tag; 
 
 	// Set Methods
 
@@ -65,7 +65,7 @@ class Playerlist : public std::list<Player*>, public sigc::trackable
         //! Returns the neutral player.
         Player* getNeutral() const {return d_neutral;}
 
-        //! Return the player with the highest score.
+        //! The game is over and this is the winner.
         Player *getWinningPlayer() const;
 
 
@@ -203,7 +203,7 @@ class Playerlist : public std::list<Player*>, public sigc::trackable
 	 * @return A pointer to the Player if it is found, or NULL if it isn't.
 	 */
 	//! Lookup a Player by it's name.
-        Player* getPlayer(std::string name) const;
+        Player* getPlayer(Glib::ustring name) const;
 
 	/**
 	 * Scan the list of players for a Player with a given Id.
@@ -236,12 +236,12 @@ class Playerlist : public std::list<Player*>, public sigc::trackable
 	//! Return the number of players left alive, not including neutral.
 	guint32 countPlayersAlive() const;
 
-        //! counts blessings of all army units for all players.
-        guint32 countBlessings() const;
-
 	//! Return the list of activities that the given hero has accomplished.
 	std::list<History *>getHistoryForHeroId(guint32 id) const;
 
+        //! Return a stack belonging to any player with the given id.
+        Stack *getStackById(guint32 id) const;
+
 	/** 
 	  \brief Check to see if this is the end of the round or not.
 	  */
@@ -293,9 +293,6 @@ class Playerlist : public std::list<Player*>, public sigc::trackable
         //! Returns the viewing player (the Player who is looking at maps).
 	static Player *getViewingplayer() {return viewingplayer;}
 
-
-        int countFightsThisTurn() const;
-        int countMovesThisTurn() const;
     protected:
 
 	//! Default constructor.
@@ -309,7 +306,7 @@ class Playerlist : public std::list<Player*>, public sigc::trackable
         
     private:
         //! Callback for loading the playerlist from an opened saved-game file.
-        bool load(std::string, XML_Helper* helper);
+        bool load(Glib::ustring, XML_Helper* helper);
 
 	//! Comparison function to assist in sorting the list of players.
 	static bool randomly(const Player *lhs, const Player *rhs);
@@ -347,4 +344,12 @@ class Playerlist : public std::list<Player*>, public sigc::trackable
         
 };
 
+//! A helper struct in Playerlist to help with sorting Player objects by score.
+struct rankable_t
+{
+  guint32 score;
+  Player *player;
+};
+
+bool compareDiplomaticScores (const struct rankable_t lhs, const struct rankable_t rhs);
 #endif // PLAYERLIST_H
