@@ -1,4 +1,4 @@
-//  Copyright (C) 2007, 2008, 2009, 2010 Ben Asselstine
+//  Copyright (C) 2007, 2008, 2009, 2010, 2014 Ben Asselstine
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -15,19 +15,17 @@
 //  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 
 //  02110-1301, USA.
 
+#include <config.h>
 #include "questmap.h"
 
-#include "gui/image-helpers.h"
 #include "Quest.h"
 #include "QuestsManager.h"
-#include "playerlist.h"
-#include "GraphicsCache.h"
+#include "ImageCache.h"
 #include "stacklist.h"
-#include "playerlist.h"
 #include "player.h"
 #include "maptile.h"
-#include "citylist.h"
 #include "GameMap.h"
+#include "hero.h"
 
 QuestMap::QuestMap(Quest *q)
 {
@@ -36,10 +34,9 @@ QuestMap::QuestMap(Quest *q)
     d_target.y = -1;
 }
 
-
 void QuestMap::draw_stacks(Player *p, std::list< Vector<int> > targets)
 {
-  Gdk::Color cross_color = p->getColor();
+  Gdk::RGBA cross_color = p->getColor();
   int size = int(pixels_per_tile) > 1 ? int(pixels_per_tile) : 1;
         
   for (std::list< Vector<int> >::iterator it= targets.begin(); it != targets.end(); it++)
@@ -105,7 +102,7 @@ void QuestMap::draw_target(Vector<int> start, Vector<int> target)
 
 void QuestMap::after_draw()
 {
-  GraphicsCache *gc = GraphicsCache::getInstance();
+  ImageCache *gc = ImageCache::getInstance();
   if (!quest)
     {
       draw_cities(true);
@@ -155,7 +152,7 @@ void QuestMap::after_draw()
 
   start += Vector<int>(int (pixels_per_tile / 2), int (pixels_per_tile / 2));
 
-  PixMask *heropic = gc->getSmallHeroPic (true);
+  PixMask *heropic = gc->getSmallHeroImage(true);
   heropic->blit_centered(surface, start);
   map_changed.emit(surface);
 }
