@@ -1,4 +1,4 @@
-// Copyright (C) 2007, 2008, 2009 Ben Asselstine
+// Copyright (C) 2007, 2008, 2009, 2012, 2014 Ben Asselstine
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -24,8 +24,6 @@
 #include "input-events.h"
 #include "NamedLocation.h"
 
-class Ruin;
-
 //! Draw the ruins and temples onto a miniature map graphic.
 /** 
   * This method draws Ruin and Temple objects onto a miniature map graphic.
@@ -44,6 +42,9 @@ class RuinMap : public OverviewMap
       */
      RuinMap(NamedLocation *ruin);
 
+     //! Destructor.
+     ~RuinMap() {};
+
      // Set Methods
 
      //! Change the Ruin or Temple object that is currently selected.
@@ -71,7 +72,7 @@ class RuinMap : public OverviewMap
      /**
       * Classes that use RuinMap must catch this signal to display the map.
       */
-     sigc::signal<void, Glib::RefPtr<Gdk::Pixmap> > map_changed;
+     sigc::signal<void, Cairo::RefPtr<Cairo::Surface> > map_changed;
 
  private:
      //! Draw the Ruin objects on the map.
