@@ -1,4 +1,4 @@
-//  Copyright (C) 2008, 2009, 2010, 2011 Ben Asselstine
+//  Copyright (C) 2008, 2009, 2010, 2011, 2014, 2015 Ben Asselstine
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -16,7 +16,6 @@
 //  02110-1301, USA.
 
 #include <iostream>
-#include <expat.h>
 #include "rectangle.h"
 #include <sigc++/functors/mem_fun.h>
 
@@ -26,30 +25,37 @@
 #include "File.h"
 #include "Configuration.h"
 #include "tarhelper.h"
+#include "file-compat.h"
+#include "ucompose.hpp"
+#include "xmlhelper.h"
 
-using namespace std;
+Glib::ustring Shieldset::d_tag = "shieldset";
+Glib::ustring Shieldset::file_extension = SHIELDSET_EXT;
 
-std::string Shieldset::d_tag = "shieldset";
-std::string Shieldset::file_extension = SHIELDSET_EXT;
-
-#define debug(x) {cerr<<__FILE__<<": "<<__LINE__<<": "<<x<<endl<<flush;}
+#define debug(x) {std::cerr<<__FILE__<<": "<<__LINE__<<": "<<x<<std::endl<<std::flush;}
 //#define debug(x)
 
-Shieldset::Shieldset(guint32 id, std::string name)
-	: d_id(id), d_name(name), d_copyright(""), d_license(""), d_info(""), 
-	d_basename("")
+Shieldset::Shieldset(guint32 id, Glib::ustring name)
+ : Set(SHIELDSET_EXT, id, name, 0), d_small_height(0), d_small_width(0), 
+    d_medium_height(0), d_medium_width(0), d_large_height(0), d_large_width(0)
 {
 }
 
-Shieldset::Shieldset(XML_Helper *helper, std::string directory)
-	: d_basename("")
+Shieldset::Shieldset(const Shieldset& s)
+ : std::list<Shield*>(), sigc::trackable(s), Set(s), 
+    d_small_height(s.d_small_height), d_small_width(s.d_small_width),
+    d_medium_height(s.d_medium_height), d_medium_width(s.d_medium_width),
+    d_large_height(s.d_large_height), d_large_width(s.d_large_width)
+{
+  for (const_iterator it = s.begin(); it != s.end(); it++)
+    push_back(new Shield(*(*it)));
+}
+
+Shieldset::Shieldset(XML_Helper *helper, Glib::ustring directory)
+ : Set(SHIELDSET_EXT, helper)
 {
   setDirectory(directory);
-  helper->getData(d_id, "id");
-  helper->getData(d_name, "name");
-  helper->getData(d_copyright, "copyright");
-  helper->getData(d_license, "license");
-  helper->getData(d_info, "info");
+  setTileSize(0);
   helper->getData(d_small_width, "small_width");
   helper->getData(d_small_height, "small_height");
   helper->getData(d_medium_width, "medium_width");
@@ -84,17 +90,17 @@ ShieldStyle * Shieldset::lookupShieldByTypeAndColour(guint32 type, guint32 colou
   return NULL;
 }
 
-Gdk::Color Shieldset::getColor(guint32 owner) const
+Gdk::RGBA Shieldset::getColor(guint32 owner) const
 {
   for (const_iterator it = begin(); it != end(); it++)
     {
       if ((*it)->getOwner() == owner)
 	return (*it)->getColor();
     }
-  return Gdk::Color("black");
+  return Gdk::RGBA("black");
 }
 
-bool Shieldset::loadShield(std::string tag, XML_Helper* helper)
+bool Shieldset::loadShield(Glib::ustring tag, XML_Helper* helper)
 {
   if (tag == Shield::d_tag)
     {
@@ -111,11 +117,11 @@ bool Shieldset::loadShield(std::string tag, XML_Helper* helper)
   return false;
 }
 
-
+//! Helper class for making a new Shieldset object from a shieldset file.
 class ShieldsetLoader
 {
 public:
-    ShieldsetLoader(std::string filename, bool &broken, bool &unsupported)
+    ShieldsetLoader(Glib::ustring filename, bool &broken, bool &unsupported)
       {
         unsupported_version = false;
 	shieldset = NULL;
@@ -126,26 +132,25 @@ public:
         Tar_Helper t(filename, std::ios::in, broken);
         if (broken)
           return;
-        std::string lwsfilename = 
+        Glib::ustring lwsfilename = 
           t.getFirstFile(Shieldset::file_extension, broken);
         if (broken)
           return;
-	XML_Helper helper(lwsfilename, ios::in, false);
+	XML_Helper helper(lwsfilename, std::ios::in);
 	helper.registerTag(Shieldset::d_tag, sigc::mem_fun((*this), &ShieldsetLoader::load));
-	if (!helper.parse())
+	if (!helper.parseXML())
 	  {
             unsupported = unsupported_version;
-	    std::cerr << "Error, while loading an shieldset. Shieldset File: ";
-	    std::cerr << filename << std::endl <<std::flush;
+            std::cerr << String::ucompose(_("Error!  can't load shieldet `%1'."), filename) << std::endl;
 	    if (shieldset != NULL)
 	      delete shieldset;
 	    shieldset = NULL;
 	  }
-        File::erase(lwsfilename);
         helper.close();
+        File::erase(lwsfilename);
         t.Close();
       };
-    bool load(std::string tag, XML_Helper* helper)
+    bool load(Glib::ustring tag, XML_Helper* helper)
       {
 	if (tag == Shieldset::d_tag)
 	  {
@@ -163,13 +168,13 @@ public:
 	  }
 	return false;
       };
-    std::string dir;
-    std::string file;
+    Glib::ustring dir;
+    Glib::ustring file;
     Shieldset *shieldset;
     bool unsupported_version;
 };
 
-Shieldset *Shieldset::create(std::string filename, bool &unsupported_version)
+Shieldset *Shieldset::create(Glib::ustring filename, bool &unsupported_version)
 {
   bool broken = false;
   ShieldsetLoader d(filename, broken, unsupported_version);
@@ -178,73 +183,29 @@ Shieldset *Shieldset::create(std::string filename, bool &unsupported_version)
   return d.shieldset;
 }
 
-void Shieldset::getFilenames(std::list<std::string> &files) const
+void Shieldset::getFilenames(std::list<Glib::ustring> &files) const
 {
   for (const_iterator it = begin(); it != end(); it++)
     for (Shield::const_iterator i = (*it)->begin(); i != (*it)->end(); i++)
       {
-	std::string file = (*i)->getImageName();
+	Glib::ustring file = (*i)->getImageName();
 	if (std::find(files.begin(), files.end(), file) == files.end())
 	  files.push_back(file);
       }
 }
 
-bool Shieldset::save(std::string filename, std::string extension) const
+bool Shieldset::save(Glib::ustring filename, Glib::ustring ext) const
 {
   bool broken = false;
-  std::string goodfilename = File::add_ext_if_necessary(filename, extension);
-  std::string tmpfile = "lw.XXXX";
-  int fd = Glib::file_open_tmp(tmpfile, "lw.XXXX");
-  close (fd);
-  XML_Helper helper(tmpfile, std::ios::out, Configuration::s_zipfiles);
+  Glib::ustring goodfilename = File::add_ext_if_necessary(filename, ext);
+  Glib::ustring tmpfile = File::get_tmp_file();
+  XML_Helper helper(tmpfile, std::ios::out);
   helper.begin(LORDSAWAR_SHIELDSET_VERSION);
   broken = !save(&helper);
   helper.close();
   if (broken == true)
     return false;
-  std::string tmptar = tmpfile + ".tar";
-  Tar_Helper t(tmptar, std::ios::out, broken);
-  if (broken == true)
-    return false;
-  t.saveFile(tmpfile, File::get_basename(goodfilename, true));
-  //now the images, go get 'em from the tarball we were made from.
-  std::list<std::string> delfiles;
-  Tar_Helper orig(getConfigurationFile(), std::ios::in, broken);
-  if (broken == false)
-    {
-      std::list<std::string> files = orig.getFilenamesWithExtension(".png");
-      for (std::list<std::string>::iterator it = files.begin(); 
-           it != files.end(); it++)
-        {
-          std::string pngfile = orig.getFile(*it, broken);
-          if (broken == false)
-            {
-              t.saveFile(pngfile);
-              delfiles.push_back(pngfile);
-            }
-          else
-            break;
-        }
-      orig.Close();
-    }
-  else
-    {
-      FILE *fileptr = fopen (getConfigurationFile().c_str(), "r");
-      if (fileptr)
-        fclose (fileptr);
-      else
-        broken = false;
-    }
-  t.Close();
-  for (std::list<std::string>::iterator it = delfiles.begin(); it != delfiles.end(); it++)
-    File::erase(*it);
-  File::erase(tmpfile);
-  if (broken == false)
-    {
-      if (File::copy(tmptar, goodfilename) == 0)
-        File::erase(tmptar);
-    }
-
+  broken = saveTar(tmpfile, tmpfile + ".tar", goodfilename);
   return !broken;
 }
 
@@ -253,11 +214,7 @@ bool Shieldset::save(XML_Helper *helper) const
   bool retval = true;
 
   retval &= helper->openTag(d_tag);
-  retval &= helper->saveData("id", d_id);
-  retval &= helper->saveData("name", d_name);
-  retval &= helper->saveData("copyright", d_copyright);
-  retval &= helper->saveData("license", d_license);
-  retval &= helper->saveData("info", d_info);
+  retval &= Set::save(helper);
   retval &= helper->saveData("small_width", d_small_width);
   retval &= helper->saveData("small_height", d_small_height);
   retval &= helper->saveData("medium_width", d_medium_width);
@@ -272,6 +229,7 @@ bool Shieldset::save(XML_Helper *helper) const
 
 void Shieldset::instantiateImages(bool &broken)
 {
+  uninstantiateImages();
   for (iterator it = begin(); it != end(); it++)
     (*it)->instantiateImages(this, broken);
 }
@@ -282,31 +240,6 @@ void Shieldset::uninstantiateImages()
     (*it)->uninstantiateImages();
 }
 
-std::string Shieldset::getConfigurationFile() const
-{
-  return getDirectory() + d_basename + file_extension;
-}
-
-std::list<std::string> Shieldset::scanUserCollection()
-{
-  return File::scanForFiles(File::getUserShieldsetDir(), file_extension);
-}
-
-std::list<std::string> Shieldset::scanSystemCollection()
-{
-  std::list<std::string> retlist = File::scanForFiles(File::getShieldsetDir(), 
-                                                      file_extension);
-  if (retlist.empty())
-    {
-      std::cerr << "Couldn't find any shieldsets (*" << file_extension << 
-        ") in : " << File::getShieldsetDir() << std::endl;
-      std::cerr << "Please check the path settings in ~/.lordsawarrc" << std::endl;
-      exit(-1);
-    }
-  return retlist;
-}
-
-	
 bool Shieldset::validate() const
 {
   bool valid = true;
@@ -317,6 +250,12 @@ bool Shieldset::validate() const
       if (validateShieldImages(Shield::Colour(i)) == false)
 	return false;
     }
+  if (d_small_width == 0 || d_small_height == 0)
+    return false;
+  if (d_medium_width == 0 || d_medium_height == 0)
+    return false;
+  if (d_large_width == 0 || d_large_height == 0)
+    return false;
   return valid;
 }
 
@@ -391,45 +330,13 @@ void Shieldset::reload(bool &broken)
       uninstantiateImages();
       for (iterator it = begin(); it != end(); it++)
         delete *it;
-      std::string basename = d_basename;
+      Glib::ustring basename = getBaseName();
       *this = *d.shieldset;
       instantiateImages(broken);
-      d_basename = basename;
+      setBaseName(basename);
     }
 }
 
-std::string Shieldset::getFileFromConfigurationFile(std::string file)
-{
-  bool broken = false;
-  Tar_Helper t(getConfigurationFile(), std::ios::in, broken);
-  if (broken == false)
-    {
-      std::string filename = t.getFile(file, broken);
-      t.Close();
-  
-      if (broken == false)
-        return filename;
-    }
-  return "";
-}
-
-bool Shieldset::replaceFileInConfigurationFile(std::string file, std::string new_file)
-{
-  bool broken = false;
-  Tar_Helper t(getConfigurationFile(), std::ios::in, broken);
-  if (broken == false)
-    {
-      broken = t.replaceFile(file, new_file);
-      t.Close();
-    }
-  return broken;
-}
-
-bool Shieldset::copy(std::string src, std::string dest)
-{
-  return Tar_Helper::copy(src, dest);
-}
-
 guint32 Shieldset::countEmptyImageNames() const
 {
   guint32 count = 0;
@@ -444,8 +351,69 @@ guint32 Shieldset::countEmptyImageNames() const
   return count;
 }
 
-void Shieldset::clean_tmp_dir() const
+bool Shieldset::upgrade(Glib::ustring filename, Glib::ustring old_version, Glib::ustring new_version)
 {
-  return Tar_Helper::clean_tmp_dir(getConfigurationFile());
+  return FileCompat::getInstance()->upgrade(filename, old_version, new_version,
+                                            FileCompat::SHIELDSET, d_tag);
+}
+
+void Shieldset::support_backward_compatibility()
+{
+  FileCompat::getInstance()->support_type(FileCompat::SHIELDSET, 
+                                          file_extension, d_tag, true);
+  FileCompat::getInstance()->support_version
+    (FileCompat::SHIELDSET, "0.2.0", LORDSAWAR_SHIELDSET_VERSION,
+     sigc::ptr_fun(&Shieldset::upgrade));
+}
+
+Shieldset* Shieldset::copy(const Shieldset *shieldset)
+{
+  if (!shieldset)
+    return NULL;
+  return new Shieldset(*shieldset);
+}
+
+void Shieldset::setHeightsAndWidthsFromImages()
+{
+  d_small_width = 0;
+  d_small_height = 0;
+  d_medium_width = 0;
+  d_medium_height = 0;
+  d_large_width = 0;
+  d_large_height = 0;
+  for (iterator it = begin(); it != end(); it++)
+    for (Shield::iterator i = (*it)->begin(); i != (*it)->end(); i++)
+      {
+        PixMask *image = (*i)->getImage();
+        if (image == NULL)
+          continue;
+        guint32 height = 0;
+        if (image->get_height() > 0)
+          height = image->get_unscaled_height();
+        guint32 width = 0;
+        if (image->get_unscaled_width() > 0)
+          width = image->get_width();
+        if ((*i)->getType() == ShieldStyle::SMALL)
+          {
+            if (width > d_small_width)
+              d_small_width = width;
+            if (height > d_small_height)
+              d_small_height = height;
+          }
+        else if ((*i)->getType() == ShieldStyle::MEDIUM)
+          {
+            if (width > d_medium_width)
+              d_medium_width = width;
+            if (height > d_medium_height)
+              d_medium_height = height;
+          }
+        else if ((*i)->getType() == ShieldStyle::LARGE)
+          {
+            if (width > d_large_width)
+              d_large_width = width;
+            if (height > d_large_height)
+              d_large_height = height;
+          }
+      }
 }
 //End of file
