@@ -1,4 +1,4 @@
-//  Copyright (C) 2008, 2009, 2011 Ben Asselstine
+//  Copyright (C) 2008, 2009, 2011, 2014, 2015 Ben Asselstine
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -19,12 +19,10 @@
 #define SHIELDSTYLE_H
 
 #include <gtkmm.h>
-#include <string>
 #include <sigc++/trackable.h>
 #include "PixMask.h"
 
 
-class Player;
 class XML_Helper;
 class Shieldset;
 
@@ -44,7 +42,7 @@ class ShieldStyle : public sigc::trackable
     public:
 
 	//! The xml tag of this object in a shieldset configuration file.
-	static std::string d_tag; 
+	static Glib::ustring d_tag; 
 
 	//! The size of the shield.
 	enum Type {
@@ -66,12 +64,14 @@ class ShieldStyle : public sigc::trackable
          */
         ShieldStyle(XML_Helper* helper);
 
+        //! Copy constructor.
+        ShieldStyle(const ShieldStyle& s);
 
 	//! Default constructor.
 	ShieldStyle(ShieldStyle::Type type);
         
 	//! Destructor.
-        virtual ~ShieldStyle();
+        ~ShieldStyle();
 
         
         // Get Methods
@@ -86,7 +86,7 @@ class ShieldStyle : public sigc::trackable
 	PixMask* getMask() const {return d_mask;}
 
 	//! Returns the basename of the picture's filename.
-	std::string getImageName() const {return d_image_name;}
+	Glib::ustring getImageName() const {return d_image_name;}
 
 
         // Set Methods
@@ -98,13 +98,13 @@ class ShieldStyle : public sigc::trackable
         void setMask(PixMask* mask) {d_mask = mask;}
 
 	//! Set the basename of the shield picture's filename.
-	void setImageName(std::string name) {d_image_name = name;}
+	void setImageName(Glib::ustring name) {d_image_name = name;}
 
 
 	// Methods that operate on class data and modify the class.
 
 	//! Load the images for this shieldstyle from the given file.
-	void instantiateImages(std::string filename, Shieldset *s, bool &broke);
+	void instantiateImages(Glib::ustring filename, Shieldset *s, bool &broke);
 
 	//! Destroy the images associated with this shieldstyle.
 	void uninstantiateImages();
@@ -119,10 +119,12 @@ class ShieldStyle : public sigc::trackable
 	// Static Methods
 	
 	//! Convert a ShieldStyle::Type enumerated value to a string.
-	static std::string shieldStyleTypeToString(const ShieldStyle::Type type);
+	static Glib::ustring shieldStyleTypeToString(const ShieldStyle::Type type);
+        //! Convret a ShieldStyle::Type to a suitable string for display.
+        static Glib::ustring shieldStyleTypeToFriendlyName(const ShieldStyle::Type type);
 
 	//! Convert a ShieldStyle::Type string to an enumerated value.
-	static ShieldStyle::Type shieldStyleTypeFromString(const std::string str);
+	static ShieldStyle::Type shieldStyleTypeFromString(const Glib::ustring str);
     protected:
 
 	//! The size of the shield. (small, medium, or large)
@@ -149,7 +151,7 @@ class ShieldStyle : public sigc::trackable
 	 * The filename does not have a path, and the filename does
 	 * not have an extension (e.g. .png).
 	 */
-	std::string d_image_name;
+	Glib::ustring d_image_name;
 };
 
 #endif // SHIELDSTYLE_H
