@@ -2,7 +2,7 @@
 // Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006 Ulf Lorenz
 // Copyright (C) 2004, 2005 Andrea Paternesi
 // Copyright (C) 2004 John Farrell
-// Copyright (C) 2007, 2008, 2009, 2010, 2011 Ben Asselstine
+// Copyright (C) 2007, 2008, 2009, 2010, 2011, 2014, 2015 Ben Asselstine
 // Copyright (C) 2007, 2008 Ole Laursen
 //
 //  This program is free software; you can redistribute it and/or modify
@@ -20,12 +20,11 @@
 //  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 
 //  02110-1301, USA.
 
-#include "config.h"
+#include <config.h>
 #include "signal.h"
 #include <sigc++/functors/mem_fun.h>
 #include <sigc++/adaptors/bind.h>
 #include <assert.h>
-#include <algorithm>
 
 #include "stacklist.h"
 #include "stack.h"
@@ -39,13 +38,11 @@
 #include "LocationList.h"
 #include "GameMap.h"
 #include "stacktile.h"
-#include "Item.h"
 #include "stackreflist.h"
 
-std::string Stacklist::d_tag = "stacklist";
-using namespace std;
+Glib::ustring Stacklist::d_tag = "stacklist";
 
-//#define debug(x) {cerr<<__FILE__<<": "<<__LINE__<<": "<<x<<endl<<flush;}
+//#define debug(x) {std::cerr<<__FILE__<<": "<<__LINE__<<": "<<x<<std::endl<<std::flush;}
 #define debug(x)
 
 Vector<int> Stacklist::getPosition(guint32 id)
@@ -113,7 +110,7 @@ bool Stacklist::check()
         std::list<Stack*> f = GameMap::getFriendlyStacks((*it)->getPos());
         if (f.size() > 1)
           {
-	      fprintf (stderr, "%d stacks found on %d,%d\n", f.size(),
+	      fprintf (stderr, "%lu stacks found on %d,%d\n", f.size(),
                        (*it)->getPos().x, (*it)->getPos().y);
               for (std::list<Stack*>::iterator t = f.begin(); t != f.end(); t++)
                 {
@@ -121,7 +118,7 @@ bool Stacklist::check()
                   if (stack)
                     {
                     printf("stack id: %d\n", stack->getId());
-                    printf("\tsize is %d\n", stack->size());
+                    printf("\tsize is %lu\n", stack->size());
                     }
                   else
                     printf("null stack\n");
@@ -155,11 +152,11 @@ void Stacklist::nextTurn()
     //check();
 }
 
-vector<Stack*> Stacklist::getDefendersInCity(const City *city)
+std::vector<Stack*> Stacklist::getDefendersInCity(const City *city)
 {
     debug("getDefendersInCity()");
 
-    vector<Stack*> stackvector;
+    std::vector<Stack*> stackvector;
     Vector<int> pos = city->getPos();
 
     for (unsigned int i = pos.x; i < pos.x + city->getSize(); i++)
@@ -184,9 +181,7 @@ unsigned int Stacklist::getNoOfStacks()
 
     for (Playerlist::iterator pit = Playerlist::getInstance()->begin();
         pit != Playerlist::getInstance()->end(); pit++)
-    {
-        mysize += (*pit)->getStacklist()->size();
-    }
+      mysize += (*pit)->getStacklist()->size();
 
     return mysize;
 }
@@ -197,9 +192,7 @@ unsigned int Stacklist::getNoOfArmies()
 
     for (Playerlist::iterator pit = Playerlist::getInstance()->begin();
         pit != Playerlist::getInstance()->end(); pit++)
-    {
-        mysize += (*pit)->getStacklist()->countArmies();
-    }
+      mysize += (*pit)->getStacklist()->countArmies();
 
     return mysize;
 }
@@ -214,18 +207,6 @@ unsigned int Stacklist::countArmies() const
     return mysize;
 }
 
-unsigned int Stacklist::countAllies() const
-{
-    unsigned int mysize = 0;
-
-    for (const_iterator it = begin(); it != end(); it++)
-      {
-        mysize += (*it)->countAllies();
-      }
-
-    return mysize;
-}
-
 Stacklist::Stacklist()
     :d_activestack(0)
 {
@@ -235,16 +216,13 @@ Stacklist::Stacklist(Stacklist *stacklist)
     :d_activestack(0)
 {
     for (iterator it = stacklist->begin(); it != stacklist->end(); it++)
-    {
-        add(new Stack(**it));
-    }
+      add(new Stack(**it));
 }
 
 Stacklist::Stacklist(XML_Helper* helper)
     :d_activestack(0)
 {
     helper->registerTag(Stack::d_tag, sigc::mem_fun((*this), &Stacklist::load));
-
     load(Stacklist::d_tag, helper);
 }
 
@@ -254,15 +232,11 @@ Stacklist::~Stacklist()
   for (ConnectionMap::iterator it = d_connections.begin(); 
        it != d_connections.end(); it++)
     {
-      std::list<sigc::connection> list = (*it).second;
-      for (std::list<sigc::connection>::iterator lit = list.begin(); lit != list.end(); lit++)
+      std::list<sigc::connection> l = (*it).second;
+      for (std::list<sigc::connection>::iterator lit = l.begin(); lit != l.end(); lit++)
 	(*lit).disconnect();
     }
-  for (Stacklist::iterator it = begin(); it != end(); it++)
-    {
-      it = flErase(it);
-    }
-
+  flClear();
 }
 
 Stack* Stacklist::getNextMovable() const
@@ -322,8 +296,7 @@ Stack *Stacklist::getArmyStackById(guint32 army) const
   for (Stacklist::const_iterator i = begin(), e = end(); i != e; ++i)
     if ((*i)->getArmyById(army))
       return *i;
-  
-  return 0;
+  return NULL;
 }
 
 void Stacklist::flClear()
@@ -331,9 +304,7 @@ void Stacklist::flClear()
     d_activestack = 0;
 
     for (iterator it = begin(); it != end(); it++)
-    {
-        delete (*it);
-    }
+      delete (*it);
 
     clear();
 }
@@ -404,7 +375,7 @@ bool Stacklist::enoughMoves() const
     return false;
 }
 
-bool Stacklist::load(string tag, XML_Helper* helper)
+bool Stacklist::load(Glib::ustring tag, XML_Helper* helper)
 {
     static guint32 active = 0;
     
@@ -502,9 +473,7 @@ Hero *Stacklist::getNearestHero(Vector<int> pos, int dist) const
   std::list<Hero*> heroes = getHeroes();
   LocationList<Location*> hero_locales;
   for (std::list<Hero*>::iterator it = heroes.begin(); it != heroes.end(); it++)
-    {
-      hero_locales.push_back(new Location(getPosition((*it)->getId()), 1));
-    }
+    hero_locales.push_back(new Location(getPosition((*it)->getId()), 1));
   Location *hero_locale = hero_locales.getNearestObjectBefore(pos, dist);
   if (hero_locale)
     {
@@ -563,8 +532,8 @@ void Stacklist::on_stack_died (Stack *stack)
   ConnectionMap::iterator it = d_connections.find(stack);
   if (it != d_connections.end())
     {
-      std::list<sigc::connection> list = (*it).second;
-      for (std::list<sigc::connection>::iterator lit = list.begin(); lit != list.end(); lit++)
+      std::list<sigc::connection> l = (*it).second;
+      for (std::list<sigc::connection>::iterator lit = l.begin(); lit != l.end(); lit++)
 	(*lit).disconnect();
     }
   d_id.erase(d_id.find(stack->getId()));
@@ -588,8 +557,7 @@ void Stacklist::setActivestack(Stack* activestack)
 void Stacklist::drainAllMovement()
 {
   for (iterator it = begin(); it != end(); it++)
-    for (Stack::iterator ait = (*it)->begin(); ait != (*it)->end(); ait++)
-      (*ait)->decrementMoves((*ait)->getMoves());
+    (*it)->drainMovement();
 }
 
 void Stacklist::changeOwnership(Player *old_owner, Player *new_owner)
@@ -705,13 +673,4 @@ bool Stacklist::getItemHolder(Item *item, Stack **stack, Hero **hero) const
     }
   return false;
 }
-
-guint32 Stacklist::countBlessingsOnArmyUnits() const
-{
-  guint32 count = 0;
-  for (const_iterator it = begin(); it != end(); it++)
-    count += (*it)->countBlessings();
-  return count;
-}
-
 // End of file
