@@ -1,6 +1,6 @@
 // Copyright (C) 2003 Michael Bartl
 // Copyright (C) 2003, 2004, 2005, 2006 Ulf Lorenz
-// Copyright (C) 2007, 2008, 2009, 2010, 2011 Ben Asselstine
+// Copyright (C) 2007, 2008, 2009, 2010, 2011, 2014, 2015 Ben Asselstine
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -19,6 +19,7 @@
 
 #include <sigc++/functors/mem_fun.h>
 #include <string.h>
+#include <iostream>
 
 #include "tileset.h"
 
@@ -27,39 +28,36 @@
 #include "SmallTile.h"
 #include "xmlhelper.h"
 #include "gui/image-helpers.h"
-#include "GraphicsCache.h"
+#include "ImageCache.h"
 #include "tilesetlist.h"
 #include "tarhelper.h"
 #include "Configuration.h"
+#include "file-compat.h"
+#include "ucompose.hpp"
 
-using namespace std;
-
-#include <iostream>
-//#define debug(x) {cerr<<__FILE__<<": "<<__LINE__<<": "<<x<<endl<<flush;}
+//#define debug(x) {std::cerr<<__FILE__<<": "<<__LINE__<<": "<<x<<std::endl<<std::flush;}
 #define debug(x)
 
 #define DEFAULT_TILE_SIZE 40
 
-std::string Tileset::d_tag = "tileset";
-std::string Tileset::d_road_smallmap_tag = "road_smallmap";
-std::string Tileset::d_ruin_smallmap_tag = "ruin_smallmap";
-std::string Tileset::d_temple_smallmap_tag = "temple_smallmap";
-std::string Tileset::file_extension = TILESET_EXT;
+Glib::ustring Tileset::d_tag = "tileset";
+Glib::ustring Tileset::d_road_smallmap_tag = "road_smallmap";
+Glib::ustring Tileset::d_ruin_smallmap_tag = "ruin_smallmap";
+Glib::ustring Tileset::d_temple_smallmap_tag = "temple_smallmap";
+Glib::ustring Tileset::file_extension = TILESET_EXT;
 
-Tileset::Tileset(guint32 id, std::string name)
-	: Set(), d_name(name), d_copyright(""), d_license(""), d_id(id), 
-	d_tileSize(DEFAULT_TILE_SIZE), d_basename("")
+Tileset::Tileset(guint32 id, Glib::ustring name)
+	: Set(TILESET_EXT, id, name, DEFAULT_TILE_SIZE)
 {
-  d_info = "";
   d_large_selector = "";
   d_small_selector = "";
   d_fog = "";
   d_roads = "";
   d_bridges = "";
   d_flags = "";
-  d_road_color.set_rgb_p(0,0,0);
-  d_ruin_color.set_rgb_p(100,100,100);
-  d_temple_color.set_rgb_p(100,100,100);
+  d_road_color.set_rgba(0,0,0);
+  d_ruin_color.set_rgba(100,100,100);
+  d_temple_color.set_rgba(100,100,100);
   for (unsigned int i = 0; i < ROAD_TYPES; i++)
     roadpic[i] = NULL;
   for (unsigned int i = 0; i < BRIDGE_TYPES; i++)
@@ -79,16 +77,101 @@ Tileset::Tileset(guint32 id, std::string name)
     fogpic[i] = NULL;
 }
 
-Tileset::Tileset(XML_Helper *helper, std::string directory)
-	:Set()
+Tileset::Tileset (const Tileset& t)
+  : sigc::trackable(t), std::vector<Tile*>(), Set(t)
+{
+  d_large_selector = t.d_large_selector;
+  d_small_selector = t.d_small_selector;
+  d_fog = t.d_fog;
+  d_roads = t.d_roads;
+  d_bridges = t.d_bridges;
+  d_flags = t.d_flags;
+  d_road_color = t.d_road_color;
+  d_ruin_color = t.d_ruin_color;
+  d_temple_color = t.d_temple_color;
+  for (unsigned int i = 0; i < ROAD_TYPES; i++)
+    {
+      if (t.roadpic[i])
+        roadpic[i] = t.roadpic[i]->copy();
+      else
+        roadpic[i] = NULL;
+    }
+  for (unsigned int i = 0; i < BRIDGE_TYPES; i++)
+    {
+      if (t.bridgepic[i])
+        bridgepic[i] = t.bridgepic[i]->copy();
+      else
+        bridgepic[i] = NULL;
+    }
+  for (unsigned int i = 0; i < FLAG_TYPES; i++)
+    {
+      if (t.flagpic[i])
+        flagpic[i] = t.flagpic[i]->copy();
+      else
+        flagpic[i] = NULL;
+    }
+  for (unsigned int i = 0; i < FLAG_TYPES; i++)
+    {
+      if (t.flagmask[i])
+        flagmask[i] = t.flagmask[i]->copy();
+      else
+        flagmask[i] = NULL;
+    }
+  number_of_selector_frames = t.number_of_selector_frames;
+  std::vector<PixMask*> s1 = std::vector<PixMask*>(number_of_selector_frames);
+  for (unsigned int i = 0; i < number_of_selector_frames; i++)
+    s1[i] = t.selector[i]->copy();
+  selector = s1;
+  std::vector<PixMask*> s2 = std::vector<PixMask*>(number_of_selector_frames);
+  for (unsigned int i = 0; i < number_of_selector_frames; i++)
+    s2[i] = t.selectormask[i]->copy();
+  selectormask = s2;
+
+  number_of_small_selector_frames = t.number_of_small_selector_frames;
+  std::vector<PixMask*> s3 = std::vector<PixMask*>(number_of_small_selector_frames);
+  for (unsigned int i = 0; i < number_of_small_selector_frames; i++)
+    s3[i] = t.smallselector[i]->copy();
+  smallselector = s3;
+  std::vector<PixMask*> s4 = std::vector<PixMask*>(number_of_small_selector_frames);
+  for (unsigned int i = 0; i < number_of_small_selector_frames; i++)
+    s4[i] = t.smallselectormask[i]->copy();
+  smallselectormask = s4;
+
+  if (t.explosion != NULL)
+    explosion = t.explosion->copy();
+  else
+    explosion = NULL;
+
+  for (unsigned int i = 0; i < FOG_TYPES; i++)
+    {
+      if (t.fogpic[i] != NULL)
+        fogpic[i] = t.fogpic[i]->copy();
+      else
+        fogpic[i] = NULL;
+    }
+
+  for (Tileset::const_iterator i = t.begin(); i != t.end(); ++i)
+    push_back(new Tile(*(*i)));
+
+  for (Tileset::const_iterator i = begin(); i != end(); ++i)
+    {
+      for (std::list<TileStyleSet*>::const_iterator j = (*i)->begin(); j != (*i)->end(); j++)
+	{
+	  for (std::vector<TileStyle*>::const_iterator k = (*j)->begin(); k != (*j)->end(); k++)
+            {
+              d_tilestyles[(*k)->getId()] = *k;
+            }
+        }
+    }
+}
+
+Tileset::Tileset(XML_Helper *helper, Glib::ustring directory)
+	:Set(TILESET_EXT, helper)
 {
   setDirectory(directory);
-  helper->getData(d_id, "id"); 
-  helper->getData(d_name, "name"); 
-  helper->getData(d_copyright, "copyright"); 
-  helper->getData(d_license, "license"); 
-  helper->getData(d_info, "info");
-  helper->getData(d_tileSize, "tilesize");
+  guint32 ts;
+  helper->getData(ts, "tilesize");
+  setTileSize(ts);
   helper->getData(d_large_selector, "large_selector");
   helper->getData(d_small_selector, "small_selector");
   helper->getData(d_explosion, "explosion");
@@ -127,6 +210,7 @@ Tileset::~Tileset()
   uninstantiateImages();
   for (unsigned int i=0; i < size(); i++)
     delete (*this)[i];
+  clear();
   clean_tmp_dir();
 }
 
@@ -140,7 +224,7 @@ int Tileset::getIndex(Tile::Type type) const
   return -1;
 }
 
-bool Tileset::loadTile(string tag, XML_Helper* helper)
+bool Tileset::loadTile(Glib::ustring tag, XML_Helper* helper)
 {
   debug("loadTile()")
 
@@ -219,12 +303,8 @@ bool Tileset::save(XML_Helper *helper) const
   bool retval = true;
 
   retval &= helper->openTag(d_tag);
-  retval &= helper->saveData("id", d_id);
-  retval &= helper->saveData("name", d_name);
-  retval &= helper->saveData("copyright", d_copyright);
-  retval &= helper->saveData("license", d_license);
-  retval &= helper->saveData("info", d_info);
-  retval &= helper->saveData("tilesize", d_tileSize);
+  retval &= Set::save(helper);
+  retval &= helper->saveData("tilesize", getTileSize());
   retval &= helper->saveData("large_selector", d_large_selector);
   retval &= helper->saveData("small_selector", d_small_selector);
   retval &= helper->saveData("explosion", d_explosion);
@@ -248,62 +328,18 @@ bool Tileset::save(XML_Helper *helper) const
   return retval;
 }
 
-bool Tileset::save(std::string filename, std::string extension) const
+bool Tileset::save(Glib::ustring filename, Glib::ustring ext) const
 {
   bool broken = false;
-  std::string goodfilename = File::add_ext_if_necessary(filename, extension);
-  std::string tmpfile = "lw.XXXX";
-  int fd = Glib::file_open_tmp(tmpfile, "lw.XXXX");
-  close (fd);
-  XML_Helper helper(tmpfile, std::ios::out, Configuration::s_zipfiles);
+  Glib::ustring goodfilename = File::add_ext_if_necessary(filename, ext);
+  Glib::ustring tmpfile = File::get_tmp_file();
+  XML_Helper helper(tmpfile, std::ios::out);
   helper.begin(LORDSAWAR_TILESET_VERSION);
   broken = !save(&helper);
   helper.close();
   if (broken == true)
     return false;
-  std::string tmptar = tmpfile + ".tar";
-  Tar_Helper t(tmptar, std::ios::out, broken);
-  if (broken == true)
-    return false;
-  t.saveFile(tmpfile, File::get_basename(goodfilename, true));
-  //now the images, go get 'em from the tarball we were made from.
-  std::list<std::string> delfiles;
-  Tar_Helper orig(getConfigurationFile(), std::ios::in, broken);
-  if (broken == false)
-    {
-      std::list<std::string> files = orig.getFilenamesWithExtension(".png");
-      for (std::list<std::string>::iterator it = files.begin(); 
-           it != files.end(); it++)
-        {
-          std::string pngfile = orig.getFile(*it, broken);
-          if (broken == false)
-            {
-              t.saveFile(pngfile);
-              delfiles.push_back(pngfile);
-            }
-          else
-            break;
-        }
-      orig.Close();
-    }
-  else
-    {
-      FILE *fileptr = fopen (getConfigurationFile().c_str(), "r");
-      if (fileptr)
-        fclose (fileptr);
-      else
-        broken = false;
-    }
-  t.Close();
-  for (std::list<std::string>::iterator it = delfiles.begin(); it != delfiles.end(); it++)
-    File::erase(*it);
-  File::erase(tmpfile);
-  if (broken == false)
-    {
-      if (File::copy(tmptar, goodfilename) == 0)
-        File::erase(tmptar);
-    }
-
+  broken = saveTar(tmpfile, tmpfile + ".tar", goodfilename);
   return !broken;
 }
 
@@ -347,11 +383,6 @@ int Tileset::getLargestTileStyleId() const
   return largest;
 }
 
-void Tileset::setBaseName(std::string bname)
-{
-  d_basename = bname;
-}
-
 guint32 Tileset::getDefaultTileSize()
 {
   return DEFAULT_TILE_SIZE;
@@ -362,21 +393,30 @@ bool Tileset::validate() const
   if (size() == 0)
     return false;
   for (Tileset::const_iterator i = begin(); i != end(); i++)
-    {
-      if ((*i)->validate() == false)
-	{
-	  fprintf (stderr, "`%s' tile fails validation in %s\n", 
-		   (*i)->getName().c_str(), getConfigurationFile().c_str());
-	  return false;
-	}
-    }
+    if ((*i)->validate() == false)
+      return false;
+  if (countTilesWithPattern(SmallTile::SUNKEN_RADIAL) > 1)
+    return false;
+  if (getIndex(Tile::GRASS) == -1)
+    return false;
+  if (getIndex(Tile::WATER) == -1)
+    return false;
+  if (getIndex(Tile::FOREST) == -1)
+    return false;
+  if (getIndex(Tile::HILLS) == -1)
+    return false;
+  if (getIndex(Tile::MOUNTAIN) == -1)
+    return false;
+  if (getIndex(Tile::SWAMP) == -1)
+    return false;
   return true;
 }
 
+//! Helper class for making a new Tileset object from a tileset file.
 class TilesetLoader
 {
 public:
-    TilesetLoader(std::string filename, bool &broken, bool &unsupported)
+    TilesetLoader(Glib::ustring filename, bool &broken, bool &unsupported)
       {
         unsupported_version = false;
 	tileset = NULL;
@@ -387,26 +427,25 @@ public:
         Tar_Helper t(filename, std::ios::in, broken);
         if (broken)
           return;
-        std::string lwtfilename = 
+        Glib::ustring lwtfilename = 
           t.getFirstFile(Tileset::file_extension, broken);
         if (broken)
           return;
-	XML_Helper helper(lwtfilename, ios::in, false);
+	XML_Helper helper(lwtfilename, std::ios::in);
 	helper.registerTag(Tileset::d_tag, sigc::mem_fun((*this), &TilesetLoader::load));
-	if (!helper.parse())
+	if (!helper.parseXML())
 	  {
             unsupported = unsupported_version;
-	    std::cerr << "Error, while loading a tileset. Tileset File: ";
-	    std::cerr << filename << std::endl <<std::flush;
+            std::cerr << String::ucompose(_("Error!  can't load tileset `%1'."), filename) << std::endl;
 	    if (tileset != NULL)
 	      delete tileset;
 	    tileset = NULL;
 	  }
-        File::erase(lwtfilename);
         helper.close();
+        File::erase(lwtfilename);
         t.Close();
       };
-    bool load(std::string tag, XML_Helper* helper)
+    bool load(Glib::ustring tag, XML_Helper* helper)
       {
 	if (tag == Tileset::d_tag)
 	  {
@@ -424,14 +463,13 @@ public:
 	  }
 	return false;
       };
-    std::string dir;
-    std::string file;
+    Glib::ustring dir;
+    Glib::ustring file;
     Tileset *tileset;
     bool unsupported_version;
 };
 
-
-Tileset *Tileset::create(std::string file, bool &unsupported_version)
+Tileset *Tileset::create(Glib::ustring file, bool &unsupported_version)
 {
   bool broken = false;
   TilesetLoader d(file, broken, unsupported_version);
@@ -440,14 +478,14 @@ Tileset *Tileset::create(std::string file, bool &unsupported_version)
   return d.tileset;
 }
 
-void Tileset::getFilenames(std::list<std::string> &files)
+void Tileset::getFilenames(std::list<Glib::ustring> &files)
 {
   for (iterator it = begin(); it != end(); it++)
     {
       Tile *t = *it;
       for (Tile::iterator sit = t->begin(); sit != t->end(); sit++)
 	{
-	  std::string file = (*sit)->getName();
+	  Glib::ustring file = (*sit)->getName();
 	  if (std::find(files.begin(), files.end(), file) == files.end())
 	    files.push_back(file);
 	}
@@ -466,84 +504,72 @@ void Tileset::uninstantiateImages()
   for (iterator it = begin(); it != end(); it++)
     (*it)->uninstantiateImages();
 
-  if (getExplosionImage() != NULL)
-    {
-      delete getExplosionImage();
-      setExplosionImage(NULL);
-    }
   for (unsigned int i = 0; i < ROAD_TYPES; i++)
     {
-      if (getRoadImage(i) != NULL)
-	{
-	  delete getRoadImage(i);
-	  setRoadImage(i, NULL);
-	}
+      if (roadpic[i] != NULL)
+        delete roadpic[i];
+      roadpic[i] = NULL;
     }
+
   for (unsigned int i = 0; i < BRIDGE_TYPES; i++)
     {
-      if (getBridgeImage(i) != NULL)
-	{
-	  delete getBridgeImage(i);
-	  setBridgeImage(i, NULL);
-	}
+      if (bridgepic[i] != NULL)
+        delete bridgepic[i];
+      bridgepic[i] = NULL;
     }
-  for (unsigned int i = 0; i < FOG_TYPES; i++)
+
+  for (unsigned int i = 0; i < FLAG_TYPES; i++)
     {
-      if (getFogImage(i) != NULL)
-	{
-	  delete getFogImage(i);
-	  setFogImage(i, NULL);
-	}
+      if (flagpic[i] != NULL)
+        delete flagpic[i];
+      if (flagmask[i] != NULL)
+        delete flagmask[i];
+      flagpic[i] = NULL;
+      flagmask[i] = NULL;
     }
+
   for (unsigned int i = 0; i < getNumberOfSelectorFrames(); i++)
     {
-      if (getSelectorImage(i) != NULL)
-	{
-	  delete getSelectorImage(i);
-	  setSelectorImage(i, NULL);
-	}
+      if (selector[i] != NULL)
+        delete selector[i];
+      if (selectormask[i] != NULL)
+        delete selectormask[i];
+      selector[i] = NULL;
+      selectormask[i] = NULL;
     }
+
   for (unsigned int i = 0; i < getNumberOfSmallSelectorFrames(); i++)
     {
-      if (getSmallSelectorImage(i) != NULL)
-	{
-	  delete getSmallSelectorImage(i);
-	  setSmallSelectorImage(i, NULL);
-	}
+      if (smallselector[i] != NULL)
+        delete smallselector[i];
+      if (smallselectormask[i] != NULL)
+        delete smallselectormask[i];
+      smallselector[i] = NULL;
+      smallselectormask[i] = NULL;
     }
-  for (unsigned int i = 0; i < getNumberOfSmallSelectorFrames(); i++)
+
+  if (explosion != NULL)
+    delete explosion;
+      
+  explosion = NULL;
+
+  for (unsigned int i = 0; i < FOG_TYPES; i++)
     {
-      if (getSmallSelectorMask(i) != NULL)
+      if (fogpic[i] != NULL)
 	{
-	  delete getSmallSelectorMask(i);
-	  setSmallSelectorMask(i, NULL);
-	}
-    }
-  for (unsigned int i = 0; i < FLAG_TYPES; i++)
-    {
-      if (getFlagImage(i) != NULL)
-	{
-	  delete getFlagImage(i);
-	  setFlagImage(i, NULL);
-	}
-    }
-  for (unsigned int i = 0; i < FLAG_TYPES; i++)
-    {
-      if (getFlagMask(i) != NULL)
-	{
-	  delete getFlagMask(i);
-	  setFlagMask(i, NULL);
+	  delete fogpic[i];
+          fogpic[i] = NULL;
 	}
     }
 }
 
-void Tileset::instantiateImages(std::string explosion_filename,
-				std::string roads_filename,
-				std::string bridges_filename,
-				std::string fog_filename,
-				std::string flags_filename,
-				std::string selector_filename,
-				std::string small_selector_filename,
+void Tileset::instantiateImages(Glib::ustring explosion_filename,
+				Glib::ustring roads_filename,
+				Glib::ustring bridges_filename,
+				Glib::ustring fog_filename,
+				Glib::ustring flags_filename,
+				Glib::ustring selector_filename,
+				Glib::ustring small_selector_filename,
                                 bool &broken)
 {
   if (explosion_filename.empty() == false && !broken)
@@ -557,8 +583,8 @@ void Tileset::instantiateImages(std::string explosion_filename,
         {
           for (unsigned int i = 0; i < ROAD_TYPES ; i++)
             {
-              if (roadpics[i]->get_width() != (int)d_tileSize)
-                PixMask::scale(roadpics[i], d_tileSize, d_tileSize);
+              if (roadpics[i]->get_width() != (int)getTileSize())
+                PixMask::scale(roadpics[i], getTileSize(), getTileSize());
               setRoadImage(i, roadpics[i]);
             }
         }
@@ -573,8 +599,8 @@ void Tileset::instantiateImages(std::string explosion_filename,
         {
           for (unsigned int i = 0; i < BRIDGE_TYPES ; i++)
             {
-              if (bridgepics[i]->get_width() != (int)d_tileSize)
-                PixMask::scale(bridgepics[i], d_tileSize, d_tileSize);
+              if (bridgepics[i]->get_width() != (int)getTileSize())
+                PixMask::scale(bridgepics[i], getTileSize(), getTileSize());
               setBridgeImage(i, bridgepics[i]);
             }
         }
@@ -588,8 +614,8 @@ void Tileset::instantiateImages(std::string explosion_filename,
         {
           for (unsigned int i = 0; i < FOG_TYPES ; i++)
             {
-              if (fogpics[i]->get_width() != (int)d_tileSize)
-                PixMask::scale(fogpics[i], d_tileSize, d_tileSize);
+              if (fogpics[i]->get_width() != (int)getTileSize())
+                PixMask::scale(fogpics[i], getTileSize(), getTileSize());
               setFogImage(i, fogpics[i]);
             }
         }
@@ -600,8 +626,7 @@ void Tileset::instantiateImages(std::string explosion_filename,
       std::vector<PixMask* > flagpics;
       std::vector<PixMask* > maskpics;
       bool success;
-      success = GraphicsCache::loadFlagImages (flags_filename, d_tileSize, 
-                                               flagpics, maskpics);
+      success = FlagPixMaskCacheItem::loadFlagImages (flags_filename, getTileSize(), flagpics, maskpics);
       if (success)
         {
           for (unsigned int i = 0; i < flagpics.size(); i++)
@@ -618,8 +643,8 @@ void Tileset::instantiateImages(std::string explosion_filename,
   if (selector_filename.empty() == false && !broken)
     {
       bool success;
-      success = GraphicsCache::loadSelectorImages (selector_filename, 
-                                                   d_tileSize, 
+      success = SelectorPixMaskCacheItem::loadSelectorImages (selector_filename, 
+                                                   getTileSize(), 
                                                    images, masks);
       if (success)
         {
@@ -639,8 +664,8 @@ void Tileset::instantiateImages(std::string explosion_filename,
   if (small_selector_filename.empty() == false && !broken)
     {
       bool success;
-      success = GraphicsCache::loadSelectorImages (small_selector_filename, 
-                                                   d_tileSize, images, masks);
+      success = SelectorPixMaskCacheItem::loadSelectorImages (small_selector_filename, 
+                                                   getTileSize(), images, masks);
       if (success)
         {
           setNumberOfSmallSelectorFrames(images.size());
@@ -657,7 +682,7 @@ void Tileset::instantiateImages(std::string explosion_filename,
 
 void Tileset::instantiateImages(bool &broken)
 {
-  int size = getTileSize();
+  int siz = getTileSize();
   debug("Loading images for tileset " << getName());
   uninstantiateImages();
   broken = false;
@@ -667,15 +692,15 @@ void Tileset::instantiateImages(bool &broken)
   for (iterator it = begin(); it != end(); it++)
     {
       if (!broken)
-        (*it)->instantiateImages(size, &t, broken);
+        (*it)->instantiateImages(siz, &t, broken);
     }
-  std::string explosion_filename = "";
-  std::string roads_filename = "";
-  std::string bridges_filename = "";
-  std::string fog_filename = "";
-  std::string flags_filename = "";
-  std::string selector_filename = "";
-  std::string small_selector_filename = "";
+  Glib::ustring explosion_filename = "";
+  Glib::ustring roads_filename = "";
+  Glib::ustring bridges_filename = "";
+  Glib::ustring fog_filename = "";
+  Glib::ustring flags_filename = "";
+  Glib::ustring selector_filename = "";
+  Glib::ustring small_selector_filename = "";
 
   if (getExplosionFilename().empty() == false && !broken)
     explosion_filename = t.getFile(getExplosionFilename() + ".png", broken);
@@ -714,31 +739,6 @@ void Tileset::instantiateImages(bool &broken)
   return;
 }
 
-std::string Tileset::getConfigurationFile() const
-{
-  return getDirectory() + d_basename + file_extension;
-}
-
-std::list<std::string> Tileset::scanUserCollection()
-{
-  return File::scanForFiles(File::getUserTilesetDir(), file_extension);
-}
-
-std::list<std::string> Tileset::scanSystemCollection()
-{
-  std::list<std::string> retlist = File::scanForFiles(File::getTilesetDir(), 
-                                                      file_extension);
-  if (retlist.empty())
-    {
-      std::cerr << "Couldn't find any tilesets (*" << file_extension << 
-        ") in : " << File::getTilesetDir() << std::endl;
-      std::cerr << "Please check the path settings in ~/.lordsawarrc" << std::endl;
-      exit(-1);
-    }
-
-  return retlist;
-}
-
 TileStyle *Tileset::getTileStyle(guint32 id) const
 {
   TileStyleIdMap::const_iterator it = d_tilestyles.find(id);
@@ -759,44 +759,13 @@ void Tileset::reload(bool &broken)
       uninstantiateImages();
       for (iterator it = begin(); it != end(); it++)
         delete *it;
-      std::string basename = d_basename;
+      Glib::ustring basename = getBaseName();
       *this = *d.tileset;
       instantiateImages(broken);
-      d_basename = basename;
+      setBaseName(basename);
     }
 }
 
-std::string Tileset::getFileFromConfigurationFile(std::string file)
-{
-  bool broken = false;
-  Tar_Helper t(getConfigurationFile(), std::ios::in, broken);
-  if (broken == false)
-    {
-      std::string filename = t.getFile(file, broken);
-      t.Close();
-  
-      if (broken == false)
-        return filename;
-    }
-  return "";
-}
-
-bool Tileset::addFileInConfigurationFile(std::string new_file)
-{
-  return replaceFileInConfigurationFile("", new_file);
-}
-bool Tileset::replaceFileInConfigurationFile(std::string file, std::string new_file)
-{
-  bool broken = false;
-  Tar_Helper t(getConfigurationFile(), std::ios::in, broken);
-  if (broken == false)
-    {
-      broken = t.replaceFile(file, new_file);
-      t.Close();
-    }
-  return broken;
-}
-
 guint32 Tileset::calculate_preferred_tile_size() const
 {
   guint32 tilesize = 0;
@@ -808,9 +777,9 @@ guint32 Tileset::calculate_preferred_tile_size() const
     sizecounts[bridgepic[0]->get_unscaled_width()]++;
   if (flagpic[0])
     sizecounts[flagpic[0]->get_unscaled_width()]++;
-  if (selector[0])
+  if (selector.empty() == false)
     sizecounts[selector[0]->get_unscaled_width()]++;
-  if (smallselector[0])
+  if (smallselector.empty() == false)
     sizecounts[smallselector[0]->get_unscaled_width()]++;
   if (fogpic[0])
     sizecounts[fogpic[0]->get_unscaled_width()]++;
@@ -842,33 +811,25 @@ guint32 Tileset::calculate_preferred_tile_size() const
   return tilesize;
 }
 
-bool Tileset::copy(std::string src, std::string dest)
-{
-  return Tar_Helper::copy(src, dest);
-}
-        
-bool Tileset::addTileStyleSet(Tile *tile, std::string filename)
+bool Tileset::addTileStyleSet(Tile *tile, Glib::ustring filename)
 {
   bool success = true;
   TileStyle::Type tilestyle_type;
-  if (tile->getType() == Tile::GRASS)
-    tilestyle_type = TileStyle::LONE;
-  else
-    tilestyle_type = TileStyle::UNKNOWN;
+  tilestyle_type = TileStyle::UNKNOWN;
   TileStyleSet *set = 
-    new TileStyleSet(filename, d_tileSize, success, tilestyle_type);
+    new TileStyleSet(filename, getTileSize(), success, tilestyle_type);
   if (!success)
     {
       delete set;
       return success;
     }
-  guint32 first_free_id = getFreeTileStyleId();
-  for (TileStyleSet::iterator it = set->begin(); it != set->end(); it++)
-    (*it)->setId(first_free_id);
-
   tile->push_back(set);
-  for (TileStyleSet::iterator it = ++set->begin(); it != set->end(); it++)
-    (*it)->setId(getFreeTileStyleId());
+  for (TileStyleSet::iterator it = set->begin(); it != set->end(); it++)
+    {
+      guint32 tile_style_id = getFreeTileStyleId();
+      d_tilestyles[tile_style_id] = (*it);
+      (*it)->setId(tile_style_id);
+    }
   return success;
 }
 
@@ -892,8 +853,58 @@ bool Tileset::getTileStyle(guint32 id, Tile **tile, TileStyleSet **set, TileStyl
   return false;
 }
 
-void Tileset::clean_tmp_dir() const
+bool Tileset::upgrade(Glib::ustring filename, Glib::ustring old_version, Glib::ustring new_version)
 {
-  return Tar_Helper::clean_tmp_dir(getConfigurationFile());
+  return FileCompat::getInstance()->upgrade(filename, old_version, new_version,
+                                            FileCompat::TILESET, d_tag);
+}
+
+void Tileset::support_backward_compatibility()
+{
+  FileCompat::getInstance()->support_type(FileCompat::TILESET, file_extension, 
+                                          d_tag, true);
+  FileCompat::getInstance()->support_version
+    (FileCompat::TILESET, "0.2.0", LORDSAWAR_TILESET_VERSION,
+     sigc::ptr_fun(&Tileset::upgrade));
+}
+
+Tile *Tileset::getFirstTile(SmallTile::Pattern pattern) const
+{
+  for (const_iterator i = begin(); i != end(); i++)
+    if ((*i)->getSmallTile()->getPattern() == pattern)
+      return *i;
+    return NULL;
+}
+
+Tileset* Tileset::copy(const Tileset *tileset)
+{
+  if (!tileset)
+    return NULL;
+  return new Tileset(*tileset);
+}
+
+void Tileset::populateWithDefaultTiles()
+{
+  uninstantiateImages();
+  for (unsigned int i=0; i < size(); i++)
+    delete (*this)[i];
+  clear();
+  push_back(Tile::get_default_grass());
+  push_back(Tile::get_default_water());
+  push_back(Tile::get_default_forest());
+  push_back(Tile::get_default_hills());
+  push_back(Tile::get_default_mountains());
+  push_back(Tile::get_default_swamp());
+}
+
+int Tileset::countTilesWithPattern(SmallTile::Pattern pattern) const
+{
+  int radial_count = 0;
+  for (Tileset::const_iterator i = begin(); i != end(); i++)
+    {
+      if ((*i)->getSmallTile()->getPattern() == pattern)
+        radial_count++;
+    }
+  return radial_count;
 }
 //End of file
