@@ -1,4 +1,4 @@
-// Copyright (C) 2007, 2008, 2009, 2010 Ben Asselstine
+// Copyright (C) 2007, 2008, 2009, 2010, 2014, 2015 Ben Asselstine
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -20,9 +20,10 @@
 
 #include <gtkmm.h>
 
-#include "xmlhelper.h"
 #include "PixMask.h"
 
+class XML_Helper;
+
 //! Describes the look of a Tile.
 /**
   * A TileStyle describes a single look of a Tile.  It is responsible for
@@ -44,7 +45,7 @@ class TileStyle
 {
     public:
 	//! The xml tag of this object in a tileset configuration file.
-	static std::string d_tag; 
+	static Glib::ustring d_tag; 
 
         //! Describe terrain tile styles.
 	  /**
@@ -261,12 +262,15 @@ ooooooo
            */
 	  BOTTOMLEFTTOTOPRIGHTDIAGONAL = 15,
 	  OTHER = 16,
-          UNKNOWN = 17,
+          UNKNOWN = 17
 	};
 
 	//! Default constructor.
 	TileStyle();
 
+        //! Copy constructor.
+        TileStyle(const TileStyle& t);
+
         //! Make a new tilestyle from an id, and a type.
         TileStyle(guint32 id, TileStyle::Type type);
 
@@ -295,7 +299,7 @@ ooooooo
 	PixMask* getImage() const {return d_image;}
 
 	//! Get the name of the current style.
-	std::string getTypeName() const;
+	Glib::ustring getTypeName() const;
 
 
 	// Set Methods
@@ -312,6 +316,8 @@ ooooooo
 	//! Set the image for the tilestyle.
 	void setImage(PixMask* image) {d_image = image;};
 
+        //! Free up the image associated with this tilestyle.
+        void uninstantiateImage();
 
 	// Methods that operate on the class data but do not modify the class.
 
@@ -324,10 +330,10 @@ ooooooo
 	// Static Methods
 
 	//! Get the name of the TileStyle::Type in string form.
-	static std::string getTypeName(Type type);
+	static Glib::ustring getTypeName(Type type);
 
 	//! Return the style type enumeration given the type name.
-	static TileStyle::Type typeNameToType(std::string name);
+	static TileStyle::Type typeNameToType(Glib::ustring name);
 
         //! Return how many digits the hex number should be for an id this big.
         /**
@@ -341,7 +347,7 @@ ooooooo
          * wants to save a series of tile style ids with the same width (in 
          * characters).
          */
-        static std::string idToString(guint32 id, guint32 digits = 0);
+        static Glib::ustring idToString(guint32 id, guint32 digits = 0);
 
     private:
         // DATA
