@@ -21,18 +21,17 @@
 #include <sigc++/slot.h>
 #include <sigc++/connection.h>
 #include <sigc++/signal.h>
-#include <boost/noncopyable.hpp>
 
 //! A simple timing framework.
 /**
   * Main function is register_timer. The timer_registered signal hook is used
   * to do the actual work.
   */
-class Timing: public boost::noncopyable
+class Timing
 {
  public:
     static Timing &instance();
-    ~Timing();
+    ~Timing() {};
 
     enum { STOP = false, CONTINUE = true };
     typedef sigc::slot<bool> timer_slot;
