@@ -1,4 +1,4 @@
-//  Copyright (C) 2009 Ben Asselstine
+//  Copyright (C) 2009, 2014 Ben Asselstine
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
 //  the Free Software Foundation; either version 3 of the License, or
@@ -20,13 +20,13 @@
 #include <cmath>
 #include <stdlib.h>
 
+extern int max_vector_width;
 //! A simple 2d point structure.
 /**
  * Implements a lot of overloaded operators to ease calculations.
  * x is the axis in the left/right direction.
  * y is the axis in the up/down direction.
  */
-extern int max_vector_width;
 template <typename T>
 struct Vector
 {
@@ -87,7 +87,7 @@ struct Vector
 	//size_t size = v.x * max_vector_width + v.y;
 	//return size;
       //};
-   T toIndex() {return y*max_vector_width+x; }
+   int toIndex() {return y*max_vector_width+x; }
 };
 
 template <typename T>
@@ -146,6 +146,14 @@ inline bool operator <(Vector<T> lhs, Vector<T> rhs)
   return r < l;
 }
 
+template <typename T>
+inline bool operator >(Vector<T> lhs, Vector<T> rhs)
+{
+  T l = lhs.y * max_vector_width + lhs.x;
+  T r = rhs.y * max_vector_width + rhs.x;
+  return r > l;
+}
+
 template <>
 inline bool operator ==<double>(Vector<double> lhs, Vector<double> rhs)
 {
