@@ -1,4 +1,4 @@
-//  Copyright (C) 2007, 2008, 2009 Ben Asselstine
+//  Copyright (C) 2007, 2008, 2009, 2014, 2015 Ben Asselstine
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -16,8 +16,6 @@
 //  02110-1301, USA.
 
 #include "vectoredunit.h"
-#include <stdlib.h>
-#include <string.h>
 #include <xmlhelper.h>
 
 #include "armysetlist.h"
@@ -28,7 +26,7 @@
 #include "action.h"
 #include "MapBackpack.h"
 
-std::string VectoredUnit::d_tag = "vectoredunit";
+Glib::ustring VectoredUnit::d_tag = "vectoredunit";
 
 VectoredUnit::VectoredUnit(Vector<int> pos, Vector<int> dest, ArmyProdBase *army, int duration, Player *player)
     :Ownable(player), LocationBox(pos), d_destination(dest), 
@@ -41,8 +39,8 @@ VectoredUnit::VectoredUnit(Vector<int> pos, Vector<int> dest, ArmyProdBase *army
 }
 
 VectoredUnit::VectoredUnit(const VectoredUnit& v)
-    :Ownable(v), LocationBox(v), d_destination(v.d_destination),
-     d_duration(v.d_duration)
+    :Ownable(v), LocationBox(v), sigc::trackable(v),
+    d_destination(v.d_destination), d_duration(v.d_duration)
 {
   if (v.d_army)
     d_army = new ArmyProdBase(*v.d_army);
@@ -68,7 +66,7 @@ VectoredUnit::~VectoredUnit()
 bool VectoredUnit::save(XML_Helper* helper) const
 {
     bool retval = true;
-    std::string name = "";
+    Glib::ustring name = "";
 
     retval &= helper->openTag(VectoredUnit::d_tag);
     retval &= helper->saveData("x", getPos().x);
