@@ -1,4 +1,4 @@
-//  Copyright (C) 2007, 2008, 2009 Ben Asselstine
+//  Copyright (C) 2007, 2008, 2009, 2014 Ben Asselstine
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -25,8 +25,8 @@
 #include "player.h"
 #include "GameMap.h"
 
-std::string VectoredUnitlist::d_tag = "vectoredunitlist";
-//#define debug(x) {cerr<<__FILE__<<": "<<__LINE__<<": "<<x<<endl<<flush;}
+Glib::ustring VectoredUnitlist::d_tag = "vectoredunitlist";
+//#define debug(x) {std::cerr<<__FILE__<<": "<<__LINE__<<": "<<x<<std::endl<<std::flush;}
 #define debug(x)
 
 VectoredUnitlist* VectoredUnitlist::s_instance = 0;
@@ -86,7 +86,7 @@ bool VectoredUnitlist::save(XML_Helper* helper) const
     return retval;
 }
 
-bool VectoredUnitlist::load(std::string tag, XML_Helper* helper)
+bool VectoredUnitlist::load(Glib::ustring tag, XML_Helper* helper)
 {
   if (tag == VectoredUnit::d_tag)
     {
